#ifndef ARITHM_FUNCTIONS_H
#define ARITHM_FUNCTIONS_H
#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS


namespace arithm
{
  using std::abs;
  using std::sqrt;
  template<class T>
  inline T numerator(const T& a) { return a; };
  template<class T>
  inline T denumerator( const T& a ) { return T(1); };
};

template < typename FT,
	   typename T > inline
FT to_FT(const T& a, const T& b = 1) { return FT(a, b); }

/// FROM NUMBERTRAITS 
  
  /** 
   * \brief default @gcd@ function defined as 1.
   **/
  template < typename T >  inline
  T gcd(const T& a, const T& b) { return T(1); }
  
  /**
   * \brief default @lcm@ function defined as (a*b)/gcd(a,b)
   **/
  template < typename T >  inline
  T lcm(const T& a, const T& b) { return ((a * b) / gcd(a, b)); } ;
  /** 
   * \brief default @compare@ function (use operator<)
   **/
  template < typename T >
  int compare(const T& a, const T& b)
  { return (b<a)?1:((a<b)?-1:0); };
  
  template < typename T >
  double to_double( const T & a )
    { 
      double tmp;
      let::assign(tmp,a);
      return tmp;
    };
  
  /** 
 * \brief Computes the numerator of a rational number
 * 
 * \param a 
 * 
 * \return if @a@ is rational then it returns @numerator(a)@ else it returns @a@ 
 *
 * \sa denominator
 */
  template < typename RT,
           typename FT >  inline
  RT numerator(const FT& a) { return a.numerator(); }
/** 
 * \brief Computes the denominator of a rational number
 * 
 * \param a 
 * 
 * \return if @a@ is rational then it returns @denominator(a)@ else it returns @1@ 
 *
 * \sa numerator
 */
template < typename RT, typename FT >  inline
RT denominator(const FT& a)
{ return a.denominator(); }


template < typename XT, typename T >  inline
XT to_XT(const T& a) { return to_double(a); }


__END_NAMESPACE_SYNAPS

#include <synaps/arithm/let.h>
#include <synaps/arithm/sign.h>

#endif
