<TeXmacs|1.0.4>

<style|generic>

<\body>
  <section|Reference counting>

  \ Because objects management in the C++ programming language defaults to
  \ <em|value semantics>, we have to take explicit actions to avoid copying
  large objects -- the time spent in copying large objects may be shown to be
  non negligible under usual circumstances. A popular way to solve that
  memory management problem is to use reference counting coupled with a
  copy-on-write strategy.\ 

  Concretely, each object (of type <verbatim|VectDse>, \ <verbatim|MatrDse>,
  <verbatim|MatrStr>, \ <verbatim|MatrSps>, <verbatim|UPolDse> or
  <verbatim|MPol>, used as template-arguments for our containers) is attached
  with a counter whose purpose is to keep track of the number of references
  to its associated object. An assignment to a new object will increment the
  counter and copy the actual data's address. A modification of the shared
  object will induce a copy of the data leading to a new shared object with
  counter 0. The object's destructor either decrements the counter (if it is
  positive) or else effectively destroys the data, thereby freeing its
  storage.\ 

  The overall scheme is implemented by the class
  <verbatim|shared_object\<less\>R\<gtr\>>. Therefore, returning such objects
  by value in a function as in\ 

  <\quote-env>
    <\code>
      VectDse\<less\>C\<gtr\> F(...) { VectDse\<less\>C\<gtr\> W= ...; return
      W; }
    </code>
  </quote-env>

  does not induce a performance penalty. Indeed a copy and destruction of
  <verbatim|W> will in this case just increment or decrement a counter.
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|toc-1|<tuple|<uninit>|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Reference
      counting> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-1><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>