/********************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 *   $Id: ReferTo.h,v 1.1 2005/07/11 11:23:12 mourrain Exp $
 ********************************************************************/
#ifndef SYNAPS_UTIL_REFERTO_H
#define SYNAPS_UTIL_REFERTO_H
#include <synaps/init.h>
#include <synaps/base/shared_object.h>
/********************************************************************/
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
/*! Allows us to access to the data when we don't use shared object. */
template<class R>
inline       R & Rep(R & r)       {return r;}

/*! Allows us to access to the data when we don't use shared object. */
template<class R>
inline const R & Rep(const R & r) {return r;}

/*! Allows us to access to the data when we use shared object. */
template<class R>
inline       R & Rep(shared_object<R> & r)       {return *r;}

/*! Allows us to access to the data when we use shared object. */
template<class R>
inline const R & Rep(const shared_object<R> & r) {return *r;}

//--------------------------------------------------------------------
//! Trait class which get the template parameter of a shared_object.
/*!
  This structure allows us to get the value type when
  we don't use shared object.\n
  We have to implement this structure in order to let the user choose
  as he want if he wants or not to use shared object.
*/
template<class T>
struct ReferTo
{
  /*! It represents the value type we want to recuperate. */
  typedef T value_type;
};

//! The structure for obtain the template parameter.
/*!
  This structure allows us to recuperate the value type when
  we use shared object.\n
  We have to implement this structure in order to let the user choose
  as he want if he wants or not to use shared object.
*/
template<class T>
struct ReferTo<shared_object<T> >
{
  /*! It represents the value type we want to recuperate. */
  typedef T value_type;
};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
