/*********************************************************************
*      This file is part of the source code of SYNAPS library.       *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: $Id: maple.H,v 1.1 2005/07/11 09:32:55 mourrain Exp $
**********************************************************************/
#ifndef synaps_inout_maple_H_
#define synaps_inout_maple_H_
//--------------------------------------------------------------------
#include <iostream>
#include <complex>
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
/// Namespace for output in maple format.
namespace maple
{
  /// Maple output stream.
  class ostream {
  public:
    ostream(std::ostream& os): chl(os) { };
    std::ostream & chl;
  };

  /// Generic print function for maple ostream.
  template<class OSTREAM, class X>
  OSTREAM& print(OSTREAM & os, const X & x) {os.chl<<x; return(os);}

  /// Specialised print function for maple ostream and complex numbers.
  template<class OSTREAM, class C>
  OSTREAM & print(OSTREAM& os, const std::complex<C> & c) {
    print(os, c.real()); 
    if(c.imag() !=0) {
      os<<"+I* ";
      print(os,c.imag());
    }
    return(os);
  }

}//namespace maple
//----------------------------------------------------------------------

//----------------------------------------------------------------------
maple::ostream& operator<< (maple::ostream& os, std::ostream& s(std::ostream &)) 
{
 maple::print(os,";"); maple::print(os,s); return os;
}
//----------------------------------------------------------------------
template<class T>
maple::ostream & operator<< (maple::ostream& os, T t)
{
  using namespace maple;
  print(os,t); return(os);
}
//----------------------------------------------------------------------
/// The maple stream provided by the class @maple::ostream@.
maple::ostream cmaple(std::cout);
//----------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
#endif //synaps_inout_maple_H_
