/*********************************************************************
*      This file is part of the source code of SYNAPS library.       *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
**********************************************************************/
#ifndef synaps_base_lisp_H
#define synaps_base_lisp_H
//--------------------------------------------------------------------
#include <iostream>
#include <synaps/init.h>
//--------------------------------------------------------------------
namespace std {template<class C> class complex;}
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
/// Namespace for output in lisp format.
namespace lisp
{
  /// Lisp output stream.
  class ostream {
  public:
    ostream(std::ostream& os): chl(os) { };
    std::ostream & chl;

  };
}
//   template<class OSTREAM, class X>
//   OSTREAM& print_lisp(OSTREAM & os, X x) {os.chl<<x; return(os);}
template<class OSTREAM, class C>
OSTREAM & print_lisp(OSTREAM& os, const std::complex<C> & c) {
  os << "(+ ";
  print_lisp(os, c.real()); 
  if(c.imag() !=0) {
    os<<" (* ";
    print_lisp(os,c.imag());
    os<<" i)";
  }
  os <<")";
  return(os);
}

namespace lisp {
// using std::endl;
/// Generic lisp output operator, based on the function @print_lisp@.
//   lisp::ostream & operator<< (lisp::ostream& os, T t)
//   {
//     print_lisp(os,t); return(os);
//   }
  inline ostream& operator <<(ostream& os, std::ostream& x(std::ostream &))
  {
    os.chl<<x; return os; 
  }

  template<class X>
  ostream& operator <<(ostream& os, const X& x) 
  { 
    print_lisp(os.chl,x); return os; 
  }
}
//--------------------------------------------------------------------

template<class OSTREAM, class X>
OSTREAM& print_lisp(OSTREAM& os, const X& x) 
{ 
  os<<x; return os; 
}

//----------------------------------------------------------------------
namespace UPOLDAR 
{
  template<class OSTREAM, class POL> 
  OSTREAM& lisp(OSTREAM& os, const POL&  P) 
  {
    print_lisp(os,"(+");
    int i, len= degree(P);
    for (i=len; i>=0; i--) {
      print_lisp(os," (* ");
      print_lisp(os,P[i]);
      print_lisp(os," (^ x ");
      print_lisp(os,i);
      print_lisp(os,"))");
    }
    print_lisp(os,")");
    return os;
  }
}
//----------------------------------------------------------------------
template<class C,class R> struct UPolDse;
//----------------------------------------------------------------------
/// Lisp output operator for a univariate polynomial.
template<class OSTREAM, class C, class R>
OSTREAM& print_lisp(OSTREAM& os, const UPolDse<C,R>& p) 
{
  return UPOLDAR::lisp(os,p.rep());
}
//----------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
#endif //synaps_base_lisp_H
