<TeXmacs|1.0.6>

<style|generic>

<\body>
  <section|Polynomials>

  <subsection|Univariate polynomials>

  Univariate polynomials are derived from the vectors. They have a
  multiplication, a degree and are print function using the variable x. Here
  is an example of a definition of univariate \ polynomials, with
  coefficients modulo <with|mode|math|31>:\ 

  <\code>
    \ typedef UPolDse\<less\>Z\<less\>31\<gtr\> \<gtr\> upol_t;
  </code>

  The internal representation is an array of all the coefficients of the
  polynomials, up to its degree. This type can be used, once the following
  files have been included:

  <\code>
    #include \<less\>synaps/arithm/Zp.h\<gtr\> //modular arithmetic

    #include \<less\>synaps/upol.h\<gtr\> //univariate polynomials
  </code>

  A polynomial can be constructed from an array of coefficients or parsed
  from a string:

  <\code>
    \ upol_t p("x^3-34*x+23");
  </code>

  which yields the polynomial:

  <\equation*>
    p \ \<assign\> \ x<rsub|><rsup|3> +28*x<rsub|>+23
  </equation*>

  For univariate polynomials, the variables <with|mode|math|x<rsub|0>> and
  <with|mode|math|x> are equal. The natural operations of polynomials are
  available. For instance,

  <\code>
    \ upol_t q= p*p;

    \ p += upol_t("4*x+32");

    \ upol_t r = q%p;

    \ Z\<less\>31\<gtr\> v =q(Z\<less\>31\<gtr\>(1)), w= q[2];
  </code>

  yields:

  <\eqnarray*>
    <tformat|<table|<row|<cell|q>|<cell|\<assign\> >|<cell|x<rsup|6>
    +28*x<rsup|4>+15*x<rsup|3>+9*x<rsup|2> +17*
    x+2>>|<row|<cell|p>|<cell|=>|<cell|x<rsup|3>+x+24>>|<row|<cell|r>|<cell|\<assign\>>|<cell|16*x<rsup|2>
    + 8*x +1>>|<row|<cell|v>|<cell|\<assign\>>|<cell|7
    >>|<row|<cell|w>|<cell|\<assign\>>|<cell|9>>>>
  </eqnarray*>

  Notice that the operator <samp|%> compute the remainder of the Euclidean
  division if the leading term is invertible (here it is the case since 31 is
  prime). The quotient is given by the operator <samp|/>. For more details on
  the available operators see <verbatim|synaps/upol/UPolDse.h>.

  File: <verbatim|step3-1.cc><inactive|>

  <\exercise>
    Build a function which compute the companion matrix of a univariate
    polynomial with <samp|double> coefficients and which output its
    eigenvalues (using the internal representation
    <samp|lapack::rep2d\<less\>double\<gtr\>>)
    <hlink|<with|mode|math|\<boxdot\>>|step3-exo1.cpp>\ 
  </exercise>

  <\exercise>
    Build a function <samp|Wilkinson> which given <with|mode|math|n> computes
    <with|mode|math|W<rsub|n>(x)=<big|prod><rsub|1><rsup|n>(x-i) \<in\>
    \<bbb-Q\>[x]><hlink|<with|mode|math| \<boxdot\>>|step3-exo2.cpp>\ 
  </exercise>

  Instead of using the monomial basis, one might be interested by using
  coefficients in another basis, eg. the Bernstein basis. In order to
  manipulate such representations, one has only to change the container:

  <\code>
    \ typedef UPolDse\<less\>double, bezier::rep1d\<less\>double\<gtr\>
    \<gtr\> upolb_t;
  </code>

  and to provide the needed functions (access and specialisation of the
  algebraic operations) on this container:

  <\code>
    #include \<less\>synaps/upol/bezier/rep1d.h\<gtr\>
  </code>

  Here is an example of computation, where the internal representation is the
  array of coefficients of the polynomial in the Bernstein basis on the
  interval <with|mode|math|[0,1]>.

  <\code>
    \ upolb_t b1("x^2+1"), b2("x"), b3;

    \ VECTOR::print(cout,b1)\<less\>\<less\>endl; //b1 printed as a vector of
    coefficients

    \ VECTOR::print(cout,b2)\<less\>\<less\>endl; //b2 printed as a vector of
    coefficients

    \ b3 = b1*b2;

    \ VECTOR::print(cout,b3)\<less\>\<less\>endl; //b3 printed as a vector of
    coefficients
  </code>

  It yields:

  <\eqnarray*>
    <tformat|<table|<row|<cell|>|<cell|[1,1,2]>|<cell|(coefficients of
    b<rsub|1> in the Bernstein basis on [0,1])>>|<row|<cell|>|<cell|[0,1]>|<cell|(coefficients
    of b<rsub|2> in the Bernstein basis on
    [0,1])>>|<row|<cell|>|<cell|b<rsub|3> \ \<assign\>x<rsup|3>+x
    >|<cell|>>|<row|<cell|>|<cell|[0,0.333333,0.666667,2]>|<cell|(coefficients
    of b<rsub|3> in the Bernstein basis on [0,1])>>>>
  </eqnarray*>

  Notice that even if the coefficients of <with|mode|math|b<rsub|1>> and
  <with|mode|math|b<rsub|2>> are integers, the coefficients of
  <with|mode|math|b<rsub|3>> are not. <strong|When we use the Bernstein
  representation, the coefficient type of <samp|UPolDse> should be a field
  type>.

  File: <verbatim|step3-2.cc>

  <subsection|Multivariate distributed polynomials>

  Multivariate polynomials are represented by a list of sorted monomials,
  which are defined by \ for the coefficients of type <samp|C> and exponents
  of type <samp|E>. The order in which the monomials are stored in the list
  is a parameter of the multivariate polynomial type. A usual declaration of
  multivariate polynomials with big integer is:

  <\code>
    \ typedef MPol\<less\>QQ\<gtr\> mpol_t;
  </code>

  This type can be used, once the following files have been included:

  <\code>
    #include \<less\>synaps/arithm/gmp.h\<gtr\> //big numbers

    #include \<less\>synaps/mpol.h\<gtr\> \ \ \ \ \ \ //multivariate
    polynomials
  </code>

  The previous definition corresponds to the actual type:

  <\code>
    \ typedef MPol\<less\>QQ,DegLex,std::list\<less\>Monom\<less\>ZZ,dynamicexp\<less\>unsigned\<gtr\>
    \<gtr\> \<gtr\> \<gtr\> mpol_t;
  </code>

  where\ 

  <\itemize-dot>
    <item><samp|DegLex> is the class implementing the degree-lexicographic
    order,

    <item><samp|std::list\<less\>Monom\<less\>QQ,dynamicexp\<less\>unsigned\<gtr\>
    \<gtr\> \<gtr\>> is the container type, for the multivariate polynomials.

    <item><samp|Monom\<less\>QQ,dynamicexp\<less\>unsigned\<gtr\> \<gtr\>> is
    the monomial type, implemented as a coefficient of type <samp|QQ>
    (rational numbers) and an array of exponents
    <samp|dynamicexp\<less\>unsigned\<gtr\>>.
  </itemize-dot>

  The multivariate polynomials <samp|MPol\<less\>C,O,R\<gtr\>> are
  parameterised by the coefficient type <samp|C>, the order class <samp|O>
  and the \ container type <samp|R>. For more details on this class, see
  <verbatim|synaps/mpol/MPol.h><hlink||../html/structMPol.html>.

  They share, with the univariate polynomials, algebraic operators such as
  addition, multiplication, ... They can also be constructed from strings.
  The following examples

  <\code>
    \ mpol_t a("x1^2*x0+2*x1+3;"), b("x1^2+123*x2;");\ 

    \ a *= b;\ 
  </code>

  yields

  <\eqnarray*>
    <tformat|<table|<row|<cell|a>|<cell|\<assign\>
    \ >|<cell|x<rsub|0>*x<rsub|1><rsup|2>+2*x<rsub|1>+3>>|<row|<cell|b>|<cell|\<assign\>>|<cell|x<rsub|1><rsup|2>+123*x<rsub|2>>>|<row|<cell|a>|<cell|=>|<cell|x<rsub|0>*x<rsub|1><rsup|4>+123*x<rsub|0>*x<rsub|1><rsup|2>*x<rsub|2>+2*x<rsub|1><rsup|3>+3*x<rsub|1><rsup|2>+246*x<rsub|1>*x<rsub|2>+369*x<rsub|2>>>>>
  </eqnarray*>

  Iterators are available to scan a multivariate polynomial, as it is
  illustrate now:

  <\code>
    \ VECTOR::print(cout,a)\<less\>\<less\>endl;

    \ for(mpol_t::iterator it =a.begin(); it !=a.end();it++)
    cout\<less\>\<less\> it-\<gtr\>coeff()\<less\>\<less\>" ";

    \ cout\<less\>\<less\> endl;

    \ for(mpol_t::iterator it =a.begin(); it !=a.end();it++)
    it-\<gtr\>coeff()*=it-\<gtr\>coeff();

    \ cout\<less\>\<less\> a \<less\>\<less\>endl;
  </code>

  yields\ 

  <\equation*>
    [x<rsub|0>*x<rsub|1><rsup|4>,123*x<rsub|0>*x<rsub|1><rsup|2>*x<rsub|2>,2*x<rsub|1><rsup|3>,3*x<rsub|1><rsup|2>,246*x<rsub|1>*x<rsub|2>,369*x<rsub|2>]
  </equation*>

  (that is the list of monomials printed as a vector of monomials)

  <\equation*>
    1 \ 123 \ \ 2 \ 3 \ 246 \ \ 369
  </equation*>

  (that is the list of coefficients of the monomials)

  <\eqnarray*>
    <tformat|<table|<row|<cell|a>|<cell|=
    >|<cell|x<rsub|0>*x<rsub|1><rsup|4>+15129*x<rsub|0>*x<rsub|1><rsup|2>*x<rsub|2>+4*x<rsub|1><rsup|3>+9*x<rsub|1><rsup|2>+60516*x<rsub|1>*x<rsub|2>+136161*x<rsub|2>>>>>
  </eqnarray*>

  (that is the polynomial with the coefficients squared).

  File: <verbatim|step3-3.cc>

  <\exercise>
    Build a function which reduces a polynomial by another according to their
    leading terms. Test it with <with|mode|math|a=2
    *x<rsub|0><rsup|3>*x<rsub|1>-34*x<rsub|0<rsup|>><rsup|2>+23> and
    <with|mode|math|b=3*x<rsub|0><rsup|2>-1>. Compare with different types of
    coefficients <samp|QQ>, <samp|double>, <samp|Zp>, ... Make your first
    implementation more robust <hlink|<with|mode|math|\<boxdot\>>|step3-exo3.cpp>
  </exercise>

  <\exercise>
    Define a function <samp|shifteval> which given a multivariate polynomial
    <with|mode|math|f>(s,t) (with <with|mode|math|x<rsub|0>=s,x<rsub|1>=t>
    and a value <with|mode|math|v<rsub|0>> of type <samp|double>, compute the
    mutlivariate polynomial <with|mode|math|f(u,v<rsub|0>)> (with
    <with|mode|math|x<rsub|2>=u>) <hlink|<with|mode|math|\<boxdot\>>|step3-exo4.cpp>
  </exercise>

  <subsection|Multivariate dense polynomials>

  In some cases such as when using Bernstein basis, it is interesting to
  store all the coefficients of the polynomial, without storing the
  exponents, which are implicitely deduced from the representation. Only an
  array of coefficients and partial degrees in the variable are sufficient to
  recover the polynomial. This is what the class <samp|MPolDse> provides,
  once the following include directive is used:

  <verbatim| #include \<less\>synaps/mpol/MPolDse.h\<gtr\>>

  We need a new interface (not just a new container) since the concept of
  exponents is not attached to this class. Here are some examples, using the
  Bernstein basis:

  <\code>
    \ #include \<less\>synaps/arithm/gmp.h\<gtr\>

    \ typedef MPolDse\<less\>QQ, MPOLDSE::bernstein\<less\>QQ\<gtr\> \<gtr\>
    mpolb_t;
  </code>

  The following instructions\ 

  <\code>
    \ mpolb_t A("x1^2*x0+2*x1+3"), B("x0+x1+1"), C=A*B;\ 

    \ VECTOR::print(cout,A.rep())\<less\>\<less\>endl;

    \ VECTOR::print(cout,B.rep())\<less\>\<less\>endl;

    \ VECTOR::print(cout,C.rep())\<less\>\<less\>endl;
  </code>

  yields:

  <\eqnarray*>
    <tformat|<table|<row|<cell|>|<cell|>|<cell|[3,3,4,4,5,6]>>|<row|<cell|>|<cell|>|<cell|[1,2,2,3]>>|<row|<cell|C
    >|<cell|\<assign\>>|<cell|x<rsub|0><rsup|2>*x<rsub|1><rsup|2>+x<rsub|0>**x<rsub|1><rsup|3>+x<rsub|0>**x<rsub|1><rsup|2>+2*x<rsub|0>*x<rsub|1>+2*x<rsub|1><rsup|2>+3*x<rsub|0>+5*x<rsub|1>+3>>|<row|<cell|>|<cell|>|<cell|[3,<frac|14|3>,7,10,<frac|9|2>,<frac|13|2>,<frac|28|3>,<frac|27|2>,6,<frac|25|3>,12,18]>>>>
  </eqnarray*>

  <strong|Here also, the coefficient type of <samp|MPolDse> should be a field
  type, when we use the Bernstein representation>.

  The default construction\ 

  <\code>
    \ typedef MPolDse\<less\>double\<gtr\> mpold_t;
  </code>

  is using the monomial basis.

  File: <verbatim|step3-4.cc>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Polynomials>
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|Univariate polynomials
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|Multivariate distributed polynomials
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|Multivariate dense polynomials
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>
    </associate>
  </collection>
</auxiliary>