<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Structured matrices>\ 

  Structured matrices are parameterised by the internal representation:

  <\quote-env>
    <\code>
      template\<less\>class R\<gtr\> struct MatrStr\<less\>R\<gtr\>;
    </code>
  </quote-env>

  Structured matrices are two-dimensional arrays which can be represented by
  <with|mode|math|\<cal-O\>(n)> values, where <with|mode|math|n> is the size
  of the matrix. This class provides classical arithmetic operations such as
  addition, substraction, multiplication by scalars, vectors, matrices . . .
  The product of two structured matrices is usually not a structured matrices
  (at least of the same type) and therefore is not implemented. The product
  of a structured matrices by a vector can usually be performed in almost
  linear time, ie. in <with|mode|math|\<cal-O\>(n log(n) )> (or
  <with|mode|math|\<cal-O\>(n log(n) log<rsup|2>(n)> according to the field),
  using FFT.

  <subsection|Examples of containers>

  <subsubsection|Toeplitz container>

  A matrix <with|mode|math|T=(t<rsub|i,j>)<rsub|1\<leqslant\>i,j\<leqslant\>n>>
  is a Toeplitz matrix if for all <with|mode|math|i,j>, the entry
  <with|mode|math|t<rsub|i,j>> depends only on <with|mode|math|i-j>, that is
  if <with|mode|math|t<rsub|i,j>= t<rsub|i+1,j+1>> for all pairs of
  <with|mode|math|(i, j)> and <with|mode|math|(i+1,j+1)> for which the
  entries <with|mode|math|t<rsub|i,j>> and <with|mode|math|t<rsub|i+1, j+1>>
  are defined. Associated with a Toeplitz matrix, we have the Toeplitz
  operator which is a projection composed with the multiplication operator by
  a univariate polynomial in <with|mode|math|K[x]> (see [BP94]). An
  interesting point of this definition is that the product of a
  <with|mode|math|n \<times\>n> Toeplitz matrix by a vector is a subvector of
  the coefficient vector of the product of two polynomials of K[x]. This
  product can be done within <with|mode|math|\<cal-O\>(n log(n))> (or
  <with|mode|math|\<cal-O\>(n log(n) log<rsup|2>(n)> according to the field)
  operations using FFT.

  An implementation of such data-structure is available as

  <\quote-env>
    <\code>
      template\<less\>class C\<gtr\> linalg::toeplitz\<less\>C\<gtr\>;
    </code>
  </quote-env>

  See: <verbatim|synaps/linalg/toeplitz.h>

  \;

  <subsubsection|Hankel container>

  A matrix <with|mode|math|H = (h<rsub|i,j>)> is a Hankel matrix if its
  entries <with|mode|math|h<rsub|i,j>> depends only on <with|mode|math|i +
  j>, that is, if <with|mode|math|h<rsub|i+1,j+1>= h<rsub|i,j>> for all pairs
  <with|mode|math|(i, j)> for which the entries are defined. To this kind of
  matrix, we can associate a Hankel operator which is defined as the
  projection of the multiplication by a fixed Laurent polynomial (a Laurent
  polynomial is a polynomial of <with|mode|math|K[x, x<rsup|-1>]>, that is a
  polynomial in both the variables <with|mode|math|x> and
  <with|mode|math|x<rsup|-1>>). So we can compute the product of a
  <with|mode|math|n \<times\>n> Hankel matrix by a vector as a subvector of
  the product of a fixed polynomial <with|mode|math|h(x)> by a polynomial in
  <with|mode|math|x<rsup|-1>>. This can also be done within
  <with|mode|math|O(n log(n))> ops (or <with|mode|math|O(n log(n)
  log<rsup|2>(n))> according to the field), using FFT. See [4].

  An implementation of such data-structure is available as

  <\quote-env>
    <\code>
      template\<less\>class C\<gtr\> linalg::hankel\<less\>C\<gtr\>;
    </code>
  </quote-env>

  See: <verbatim|synaps/linalg/hankel.h>

  <subsection|Example>

  <\code>
    #include \<less\>synaps/linalg/MatrStr.H\<gtr\>

    #include \<less\>synaps/linalg/hankel.H\<gtr\>

    typedef MatrStr\<less\>linalg::hankel\<less\>double\<gtr\> \<gtr\>
    mstr_t;

    \;

    int main(int argc, char** argv)\ 

    {

    \ \ double v[]={1,2,1,1,3,4,2,2,5,5,5,5,-2, 1,2,3,1, -2.8,-2.4,1,.2,5.8};

    \ \ mstr_t A(4,4,v);

    \ \ std::cout \<less\>\<less\> A \<less\>\<less\> std::endl;
  </code>

  <\with|prog-language|mycas|prog-session|default>
    <\textput>
      <\equation*>
        A\<assign\><matrix|<tformat|<table|<row|<cell|1>|<cell|2>|<cell|1>|<cell|1>>|<row|<cell|2
        >|<cell|1>|<cell|1>|<cell|3>>|<row|<cell|1>|<cell|1>|<cell|3>|<cell|4>>|<row|<cell|1>|<cell|3>|<cell|4>|<cell|5>>>>>
      </equation*>
    </textput>
  </with>

  <\code>
    }
  </code>

  <subsection|Implementation>

  See: <verbatim|synaps/linalg/MatrStr.h>
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.1.1|?>>
    <associate|auto-4|<tuple|1.1.2|?>>
    <associate|auto-5|<tuple|1.2|?>>
    <associate|auto-6|<tuple|1.3|?>>
    <associate|toc-1|<tuple|1|1>>
    <associate|toc-10|<tuple|<uninit>|?>>
    <associate|toc-2|<tuple|1.1|1>>
    <associate|toc-3|<tuple|1.1.1|?>>
    <associate|toc-4|<tuple|1.1.2|?>>
    <associate|toc-5|<tuple|1.2|?>>
    <associate|toc-6|<tuple|1.3|?>>
    <associate|toc-7|<tuple|1.2.2|?>>
    <associate|toc-8|<tuple|1.3|?>>
    <associate|toc-9|<tuple|1.4|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Structured
      matrices> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Examples of containers
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|3fn>|1.1.1<space|2spc>Toeplitz container
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|3fn>|1.1.2<space|2spc>Hankel container
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>
    </associate>
  </collection>
</auxiliary>