
#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS


namespace lapack {

template<class C>
rep2dcplx<C> & rep2dcplx<C>::operator=(const rep2dcplx<C> & v)
{
  this->linalg::rep1d<C>::operator=(v);
  m.nbrow_=v.nbrow(); m.nbcol_=v.nbcol();lda_ = v.lda_;
  return *this;
} 
//------------------------------------------------------------------------
/**
  Solve the system $A\,x = b$ by a Least Square method.
  Put the result in @b@.
  Use the {\sc lapack} routine @zgelsx@.
**/
void solve(LS mthd,
		   const rep2dcplx<double> & A, 
		   linalg::rep1d<complex<double> > & b)
{      
  integer M = A.nbrow(), N=A.nbcol(), NRHS=1;
  integer lda = M, ldb = max(1,max(M,N));

  integer *info = new integer,*RANK= new integer;
  integer *JPVT = new integer[N];
  doublereal RCOND = 0.00001;
  integer n = min(M,N)+ max(N,2*min(M,N)+NRHS);
  doublereal *WORK = new doublereal[2*n],
             *RWORK= new doublereal[2*N];
  doublereal *Bt = new doublereal[2*ldb];
  
  for(int i=0;i < ldb; i++){
      Bt[2*i]=b[i].real();Bt[2*i+1]=b[i].imag();
  }
  F77NAME(zgelsx)(&M,&N,&NRHS, A.tab_, &lda, Bt, &ldb, 
		  JPVT, &RCOND, RANK, WORK, RWORK, info);
  for(int i=0;i < N; i++){
      b[i] = complex<double>(Bt[2*i],Bt[2*i+1]);
  }
  // cout << "Rank: " <<*RANK <<" with RCOND= " << RCOND << endl;
  // cout << "Info: " <<*info << endl;
}
//------------------------------------------------------------------------
/**
  Solve the system $A\,x = b$ by a Least Square method.
  Put the result in @x@.
  Use the {\sc lapack} routine @zgelsx@.
**/
void solve(const LS & mthd,
	   linalg::rep1d<complex<double> > & x, 
	   const rep2dcplx<double> & A, 
	   const linalg::rep1d<complex<double> > & b)
{      
  x = b;
  solve(mthd,A,x);
}//----------------------------------------------------------------------//
/**
 Compute the Singular values of  the matrix @A@, calling the {\sc lapack} 
 subroutine @dgesvd@. The result, put in @x@, is the vector of singular values 
 in decreasing order.  The matrix @A@ is copied.
**/   

void svd(linalg::rep1d<double> & x, const rep2dcplx<double> & A)
{
//if(IsMatrixEmpty(A)) return;

  integer M = A.nbrow(), N = A.nbcol(), LDA = A.lda_;

  double *Ut = new double[M*N*2];
  double *Vt = new double[M*N*2];
  
  integer* LDU = new integer(M);
  integer* LDV = new integer(N);
  integer* INFO= new integer;
  integer* LWORK = new integer(max(3*min(M, N), (5*min(M, N)-4)));
  double*  WORK = new double[2*(*LWORK)+1];

  //if(!work)
  //     Error("%s %s SVD_Lapack_Decomposition(Matrix &, Type *, Matrix &, Matrix &, intege
  // r *) \n",ERR_SPACE, ERR_FUNCTION);

  F77NAME(zgesvd)("N", "N", &M, &N, A.tab_, &LDA, x.tab_, 
  		  Ut, LDU, Vt, LDV, WORK, LWORK, INFO);

  for(unsigned i=0;i<x.size();i++) cout <<x[i]<<" ";cout <<endl;
  //  delete [] work; 
  //  return info;
  //  cout <<"====="<<endl;
  delete [] Ut;
  delete [] Vt;
  
}
//----------------------------------------------------------------------
}// namespace lapack_cpx

__END_NAMESPACE_SYNAPS

