#ifndef SYNAPS_MATRDSE_LET_H
#define SYNAPS_MATRDSE_LET_H

#include <synaps/init.h>
#include <synaps/base/Seq.h>

__BEGIN_NAMESPACE_SYNAPS

template<class R> struct MatrStr;
namespace let {

  template<class C, class R> inline
  void assign(MatrDse<C,R> & r, R * M)
  {
    using namespace MATRIX;
    using let::assign;
    assign(r.rep(), M);
  }
  
  //----------------------------------------------------------------------
  template<class C, class R> inline
  void assign(MatrDse<C,R> & r, MatrDse<C,R>* M)
  {
    using namespace MATRIX;
    using let::assign;
    assign(r.rep(),&M->rep());
  }
  
  //----------------------------------------------------------------------
  template<class C, class R,class T> inline
  void assign(MatrDse<C,R> & r, T* M)
  {
    using namespace MATRIX;
    r.rep().resize(M->nbrow(),M->nbcol());
    for(typename R::size_type i=0; i<M->nbrow();++i) 
      for(typename R::size_type j=0; j<M->nbcol();++j) 
	r(i,j)=(*M)(i,j);
  }
  
  //----------------------------------------------------------------------
  template<class C, class R,class S, class T> inline
  void assign(MatrDse<C,R> & r, const MatrDse<S,T> & M)
  {
    using let::assign;
    
    r.rep().resize(M.nbrow(),M.nbcol());
    for(typename R::size_type i=0; i<M.nbrow();++i) 
      for(typename R::size_type j=0; j<M.nbcol();++j) 
	assign(r(i,j),M(i,j));
  }

  
  //--------------------------------------------------------------------
  template<class C, class R,class S> inline
  void assign(MatrDse<C,R> & r, const MatrStr<S> & M)
  {
    using namespace MATRIX;
    using let::assign;
    
    r.rep().resize(M.nbrow(),M.nbcol());
    for(typename R::size_type i=0; i<M.nbrow();++i) 
      for(typename R::size_type j=0; j<M.nbcol();++j) 
	assign(r(i,j),(M)(i,j));
  }
  
  //----------------------------------------------------------------------
  template<class C,class V,class R>
  void assign(V & v, const OP<'s',MatrDse<C,R>,int> & M)
  {
    using namespace MATRIX;
    v.rep().resize(M.op1.nbrow());
    for(typename R::size_type i=0; i<M.op1.nbrow();++i) v[i]=M.op1(i,M.op2);
  }
  //----------------------------------------------------------------------
  template<class C,class V,class R>
  void assign(V & v, const OP<'s',int,MatrDse<C,R> > & M)
  {
    using namespace MATRIX;
    v.rep().resize(M.op2.nbcol());
    for(typename R::size_type j=0; j<M.op2.nbcol();++j) v[j]=M.op2(M.op1,j);
  }
  //--------------------------------------------------------------------
  template<class T, class C, class R>
  void assign(Seq<VectDse<T> >& s, const MatrDse<C,R>& M)
  {
    s.resize(0);
    VectDse<T> tmp(M.nbcol());
    using let::assign;
    for(unsigned i=0;i<M.nbrow();i++) 
      {
	assign(tmp,row(M,i));
	s.push_back(tmp);
      }
  }

  //--------------------------------------------------------------------
  template<class C, class R, class T, class U>
  void assign(MatrDse<C,R>& M, Seq<VectDse<T,U> >& S)
  {
    PRT_DEBUG("assign "<<S);
    if(S.size() && S[0].size())
      {
	unsigned nbr=S[0].size(), nbc=S.size();
	M.resize(nbr,nbc);
	for(unsigned i=0;i<S.size();i++)
	  {
	    assert(S[i].size()==nbr);
	    for(unsigned j=0;j<nbr;j++) 
		assign(M(i,j),S[i][j]);
	  }
      }
  }


  //--------------------------------------------------------------------
  template<class ROW, class C, class R>
  void insert(Seq<ROW> & s, const MatrDse<C,R>& M, const ByRow& mth)
  {
    using let::assign;
    ROW rw; rw.resize(M.nbcol());
    for(unsigned i=0; i<M.nbrow(); i++)
      {
	assign(rw,row(M,i));
	s.push_back(rw);
      }
  }
  
  
  //--------------------------------------------------------------------
  template<class COL, class C, class R>
  void insert(Seq<COL> & s, const MatrDse<C,R>& M, const ByCol& mth)
  {
    using let::assign;
    COL col; col.resize(M.nbrow());
    for(unsigned i=0; i<M.nbcol(); i++)
      {
	assign(col,Col(M,i));
	s.push_back(col);
      }
  }

  //--------------------------------------------------------------------
  /** Assign a sequence @s@ from a matrix @M@, by row @mth=ByRow()@
      or by column @mth=ByCol()@.
  **/
  template<class ROW, class C, class R,class HOW >
  void assign(Seq<ROW> & s, const MatrDse<C,R>& M, const HOW& mth)
  {
    Seq<ROW> r;
    insert(r,M,mth);
    s=r;
  }

  template<class ROW, class C, class R, class HOW>
    Seq<ROW> convert(const MatrDse<C,R>& M, const type::As<Seq<ROW> >& mth,
		     const HOW& how);
  //--------------------------------------------------------------------
  /// Conversion of a matrix @M@ to a sequence of row vectors.
  template<class ROW, class C, class R> inline 
  Seq<ROW> convert(const MatrDse<C,R>& M, const type::As< Seq<ROW> >& mth)
  {
    return convert(M,mth,ByRow());
  }

  //--------------------------------------------------------------------
  /// Conversion of a matrix @M@ to a sequence of row or column vectors.
  template<class ROW, class C, class R, class HOW>
    Seq<ROW> convert(const MatrDse<C,R>& M, const type::As<Seq<ROW> >& mth,
		   const HOW& how)
  {
    Seq<ROW> result;
    insert(result,M,how);
    return result;
  }
};

__END_NAMESPACE_SYNAPS

#endif
