/*********************************************************************
 *
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
$Id: rep1d.h,v 1.1 2005/07/11 08:33:35 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_LINALG_ARRAY1D_H
#define SYNAPS_LINALG_ARRAY1D_H
//---------------------------------------------------------------------
#include <fstream>
#include <sstream>
#include <iterator>
#include <cassert>
#include <synaps/init.h>
//#include <base/MinMax.h>


__BEGIN_NAMESPACE_SYNAPS

//---------------------------------------------------------------------
///Domain gathering containers and specialisations for dense linear algebra.
namespace linalg {  

/** Unidimensional array of coefficients with a size. **/
template <class C> struct rep1d 
{
public:
  typedef C            value_type;
  typedef unsigned int size_type;
  typedef C*           iterator;
  typedef iterator     const_iterator;
  typedef std::reverse_iterator<const_iterator>  const_reverse_iterator;
  typedef std::reverse_iterator<iterator>        reverse_iterator;

  size_type size_;
  C * tab_;

  rep1d() : size_(0), tab_(0) {}
  rep1d(const rep1d<C> & v);
  rep1d(size_type i);
  rep1d(size_type i, C* t);
  rep1d(C* b, C* e);
  rep1d(size_type, const char * nm);

  //  void reserve(size_type i);
  void resize(const size_type & n);
  rep1d<C> & operator=(const rep1d<C> & v);
  //  rep1d<C> & operator=(rep1d<C>*  v);

  size_type size() const {return size_;}

  iterator        begin()       {return iterator(tab_); }
  const_iterator  begin() const {return const_iterator(tab_); }
  iterator        end()         {return iterator(tab_+size_); }
  const_iterator  end()   const {return const_iterator(tab_+size_); }

  reverse_iterator       rbegin()       {return reverse_iterator(tab_+size_); }
  const_reverse_iterator rbegin() const {return reverse_iterator(tab_+size_); }
  reverse_iterator       rend()         {return reverse_iterator(tab_); }
  const_reverse_iterator rend()   const {return reverse_iterator(tab_); }

  inline       C & operator[] (size_type i)       {return tab_[i];}
  inline const C & operator[] (size_type i) const {return tab_[i];}
  // No test {if(i<size_) return tab_[i]; else return C(0);}

  ~rep1d () { delete[] tab_;}

};
//----------------------------------------------------------------------
template<class C> void assign(rep1d<C> & r, const rep1d<C>* t);
template<class C> typename rep1d<C>::iterator 
    create(typename rep1d<C>::iterator s, const rep1d<C> & a, unsigned int i);
//----------------------------------------------------------------------
} //CLOSE linalg


__END_NAMESPACE_SYNAPS

//======================================================================
#ifndef SEPARATE_COMPILATION
	#include <synaps/linalg/array1d.C>
#endif

#endif // SYNAPS_LINALG_ARRAY1D_H

