/*********************************************************************
*   This file is part of the source code of the SYNAPS kernel.
*   Author(s): Ph. Trebuchet, GALAAD, INRIA
*   $Id: sparse.h,v 1.1 2005/07/11 08:33:36 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_LINALG_SPARSE
#define SYNAPS_LINALG_SPARSE
/*********************************************************************/
//here external includes
#include<cstddef>
#include<cstdlib>
#include<cstring>
#include<cstdio>
#include<cmath>
#include<new>
#include<iostream>
#include<vector>
//------------------------------------------------------------------------
#include <synaps/init.h>
#include "synaps/linalg/MethodName.h"

//#ifdef SYNAPS_WITH_GMP
//	#include "synaps/arithm/QQ.h"
//	#include "synaps/arithm/RR.h"
//#endif //SYNAPS_WITH_GMP


__BEGIN_NAMESPACE_SYNAPS

//=====================================================================
/// Domain for sparse matrix representation.
namespace sparse {

  ///  Containers for sparse matrices.
  template <typename T>
  struct rep2d
  {
    typedef T value_type;
    typedef int size_type;
    typedef T coeff_t;
    int nrow;
    int ncol;
    int nnz;
    int *rowind;
    int *colptr;
    void *nzval;

    rep2d()
    {
      nrow=0;
      ncol=0;
      nnz=0;
      rowind=NULL;
      colptr=NULL;
      nzval=NULL;
    }
    rep2d(const int nrow, const int ncol);
    rep2d(const int nrow, const int ncol, const int nz);
    rep2d(const int nrow, const int ncol, const int nz,
	  T* coeffs, const int *idx);

    ~rep2d();
    int nbrow() {return nrow;};
    int nbcol() {return ncol;};
    int nbrow() const {return nrow;};
    int nbcol() const {return ncol;};
    T & operator() (const int & i,const int & j,const T &d) ;
    T & operator() (const int &i,const int &j);
    T  operator() (const int &i,const int &j)const ;
    inline void destroystore();
  };

  //----------------------------------------------------------------------

  //  template<class C> void solve(LU,linalg::rep1d<C> & x,
  //			       const rep2d<C> & A, const linalg::rep1d<C> & b);
  //template<class C> void solve(LU,rep2d<C> & X, const rep2d<C> & A);

  //  template<class C> integer decomp(LU,rep2d<C> & A);
  //template<class C> integer decomp(LU,rep2d<C> & A, rep2d<C> & U);
  //template<class C> integer decomp(LU,rep2d<C> & A, integer *pivot);

  // template<class C> typename rep2d<C>::size_type rank(const rep2d<C> & M);
  //template<class C> typename rep2d<C>::size_type rank(const rep2d<C> & M,
  //				     typename rep2d<C>::size_type* permut_col,
  //				     typename rep2d<C>::size_type* permut_row);
  //----------------------------------------------------------------------

} //namespace sparse

__END_NAMESPACE_SYNAPS

//======================================================================
#ifndef SEPARATE_COMPILATION
	#include "synaps/linalg/sparse.C"
#endif

#endif // SYNAPS_LINALG_SPARSE
