#define USING_NAMESPACE_SYNAPS 1

#include <cassert>
#include <synaps/linalg.h>
#include <synaps/bignum.h>

#ifdef SYNAPS_WITH_GMP

using std::cout;
using std::cin;
using std::endl;

typedef ZZ C;

//typedef double  C;
typedef VectDse<C> VECT;
typedef MatrDse<C> MAT;
#endif //SYNAPS_WITH_GMP

int main(int argc, char** argv) 
{
#ifdef SYNAPS_WITH_GMP
  using namespace std;
  C v[]={C("112321321"),C("223432243"),C("32342342342"),C("4546456456"),
         5,6,7,8,9,-1,2};
  
  MAT A(3,3,v,ByRow()), B(A);
  assert(A.nbcol() == 3);
  assert(A.nbrow() == 3);
  assert(A.nbcol() == B.nbcol());
  assert(A.nbrow() == B.nbrow());
  B.transpose();
  assert(A.nbcol() == B.nbcol());
  assert(A.nbrow() == B.nbrow());
  for (int i = 0; i < (int)A.nbrow(); i++)
    for (int j = 0; j < (int)A.nbcol(); j++)
      assert(B(i,j) == A(j,i));

  // A is the matrix of multiplication by some element.
  cout <<"A   = "<<A<<endl;
  cmaple <<"A  := "<<A<<endl;
  cout <<"B   = "<<B<<endl;
  //  B.addRow(1,2,2);  cout <<"B   = "<<B<<endl;
  MAT C;
  C= A+B; cout <<"A+B  = " << C <<endl;
  for (int i = 0; i < (int)A.nbrow(); i++)
    for (int j = 0; j < (int)A.nbcol(); j++)
      assert(C(i,j) == A(i,j) + B(i,j));
  C= A-B; cout <<"A-B  = " << C <<endl;
  for (int i = 0; i < (int)A.nbrow(); i++)
    for (int j = 0; j < (int)A.nbcol(); j++)
      assert(C(i,j) == A(i,j) - B(i,j));
  C= A*B; cout <<"A*B  = " << C <<endl;
  C= (A*B)*A; cout <<"A*B*A  = " << C <<endl;  
  C= A*2; cout <<"A*2  = "<<C<<endl;  
  for (int i = 0; i < (int)A.nbrow(); i++)
    for (int j = 0; j < (int)A.nbcol(); j++)
      assert(C(i,j) == A(i,j) * 2);
  VECT V(v+1,v+5);
  cout <<"V  = "<<V<<endl;
  
  VECT W(A*V);
  cout <<"W(A*V)   = "<<A*V<<endl;
  cout <<"A*V      = "<<W<<endl;

  W = Eval(col(A,1));
  cout <<"Col(A,1) = "<<W<<endl;
  assert(W[0] == A(0,1));
  assert(W[1] == A(1,1));
  assert(W[2] == A(2,1));

  W = Eval(row(A,2));
  cout <<"Row(A,2) = "<<W<<endl;
  assert(W[0] == A(2,0));
  assert(W[1] == A(2,1));
  assert(W[2] == A(2,2));

  B = A(Range(1,A.nbrow()-1),Range(1,A.nbcol()-1));
  cout <<"A(1..2,1..2) = "<<B <<endl;
  assert(B.nbcol() == A.nbcol()-1);
  assert(B.nbrow() == A.nbrow()-1);
  for (int i = 0; i < (int)B.nbrow(); i++)
    for (int j = 0; j < (int)B.nbcol(); j++)
      assert(B(i,j) == A(i+1,j+1));

  B = A(Range(2,A.nbrow()-1),Range(1,A.nbcol()-1));
  cout <<"A(2..2,1..2) = "<<B <<endl;
  assert(B.nbcol() == A.nbcol()-1);
  assert(B.nbrow() == A.nbrow()-2);
  for (int i = 0; i < (int)B.nbrow(); i++)
    for (int j = 0; j < (int)B.nbcol(); j++)
      assert(B(i,j) == A(i+2,j+1));

  //  B = Row(A,Range(1,2));  cout <<B <<endl;  B = Col(A,Range(1,2));  cout <<B <<endl;

  //  B = USchur(A,2);
  //cout <<"USchur(A,2) ="<<B<<endl;
  //  cout <<"Rank(A)  = "<<Rank(A)<<endl;
  B = Triang<LU>(A);
  cout <<"LU(A) ="<<B<<endl;
  cout <<"Bareiss(A) ="<<Triang<Bareiss>(A)<<endl;
  ZZ dt = Det(A);
  cout <<"Det      = "<<dt<<endl;
  assert(dt == ZZ("1167201983347656490617"));
  cout <<"A  = "<<A <<endl;

  return 0;
#else 
  std::cout <<"  Gmp is not connected"<<std::endl;
  return(77);
#endif //SYNAPS_WITH_GMP
};

