<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Multivariate polynomials> <index|Multivariate polynomial>\ 

  The multivariate polynomials

  <\quote-env>
    <\code>
      template\<less\>class C,\ 

      \ \ \ \ \ \ \ \ \ class O=Dlex,\ 

      \ \ \ \ \ \ \ \ \ class R=std::list\<less\>Monom\<less\>C,dynamicexp\<less\>'x'\<gtr\>
      \<gtr\> \<gtr\> \<gtr\> \ 

      struct MPol\<less\>C,O,R\<gtr\>;
    </code>
  </quote-env>

  are parameterised by

  <\itemize>
    <item>the type of coefficients <verbatim|C>,

    <item>the order used on the monomials <verbatim|O>. The default is the
    degree lexicographic ordering <verbatim|Dlex>,

    <item>the internal representation of the polynomials <verbatim|R>. The
    default is the list of monomials, with exponents of type
    <verbatim|dynamicexp\<less\>'x'\<gtr\>>.
  </itemize>

  <subsection|Containers for multivariate polynomials>

  The containers <verbatim|R> for this class could be
  <with|font-family|tt|list>, <with|font-family|tt|vector>, ... or any
  container with the member function <verbatim|insert>.

  <subsection|Example>

  File: <verbatim|MPol.cc>

  <subsection|Variable environments>

  The default names for the variables are <verbatim|x0, x1, x2, ...> But one
  can use specific variable environments, in order to precise which variables
  we want to consider. The class <verbatim|Variables> which implements such
  dictionnary between the internal variables <verbatim|xi> and the one we
  want, can be used either when constructing a polynomial or when printing
  it. For instance,

  <\code>
    \ \ \ \ Variables V("u v");

    \ \ \ \ MPol\<less\>double\<gtr\> p("u^2+v*u-v+2*u-1", V);
  </code>

  \ defines a dictionnary <verbatim|V> where the variable <verbatim|u> is
  <verbatim|x0>, <verbatim|v> is <verbatim|x1> so that the polynomial
  <verbatim|p> which is constructed from the string <verbatim|"u^2+..."> is
  in fact:

  <\code>
    \ \ \ \ x0^2+x0*x1+2*x0-x1-1
  </code>

  The name of the variables can be recovered from their indices
  <verbatim|V[0], V[1]>. If \ new variable names are used when constructing a
  polynomial, they are added to the dictionnary <verbatim|V>, in the order
  they are parsed:

  <\code>
    \ \ \ \ MPol\<less\>double\<gtr\> q("u^3-w+t", V);
  </code>

  Now <verbatim|V[2]> is <verbatim|w>, and <verbatim|V[3]> the variable
  <verbatim|t>. Notice, that here if we use an index
  <with|mode|math|i\<geqslant\>4>, we get for <verbatim|V[i]>, the variable
  <verbatim|xi>. In order to print the polynomial, with the appropriate
  variable names, instead of the operator <verbatim|\<less\>\<less\>>, one
  can use the following functions: \ 

  <\code>
    \ \ \ \ Variables W("a b c d");

    \ \ \ \ print(std::cout,p,W)\<less\>\<less\> std::endl;
  </code>

  In this example, it yields:

  <\code>
    \ \ \ \ a^2+a*b+2*a-b-1
  </code>

  Example: <verbatim|Variables.cc>

  <subsection|Sorting monomials>

  Different ordering on monomials are available:

  <\itemize>
    <item><verbatim|Lex><inactive|> which corresponds to the lexicographic
    ordering, with <with|mode|math|x<rsub|0>\<gtr\> x<rsub|1>\<gtr\>
    \<ldots\>.>

    <item><verbatim|DegLex> which corresponds to the degree and lexicographic
    ordering.
  </itemize>

  They provide the <verbatim|operator(m1,m2)> and <verbatim|less(m1,m2)>
  which compare two monomials <verbatim|m1>, <verbatim|m2>, using the
  functions <verbatim|m[i]> and <verbatim|m.size()>.

  <subsection|Implementation>

  <\itemize>
    <item>The class is defined in <verbatim|synaps/mpol/MPol.h> .

    <item>The module of generic functions is <verbatim|MPOLDST> in the file
    <verbatim|synaps/mpol/MPOLDST.m> .
  </itemize>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1|?>>
    <associate|auto-3|<tuple|1.1|?>>
    <associate|auto-4|<tuple|1.2|?>>
    <associate|auto-5|<tuple|1.3|?>>
    <associate|auto-6|<tuple|1.4|?>>
    <associate|auto-7|<tuple|1.5|?>>
    <associate|auto-8|<tuple|1.6|?>>
    <associate|idx-1|<tuple|1|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-3|<tuple|1.2|?>>
    <associate|toc-4|<tuple|1.3|?>>
    <associate|toc-5|<tuple|1.4|?>>
    <associate|toc-6|<tuple|1.5|?>>
    <associate|toc-7|<tuple|1.6|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|idx>
      <tuple|<tuple|Multivariate polynomial>|<pageref|auto-2>>
    </associate>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Multivariate
      polynomials> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Containers for multivariate
      polynomials <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Variable environments
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>Sorting monomials
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|1.5fn>|1.5<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>
    </associate>
  </collection>
</auxiliary>