<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Multivariate resultants>

  The resultant is a tool to projection onto a coefficient space. We consider
  a system of <with|mode|math|n + 1> generic polynomials:

  <\equation*>
    <left|{><tabular|<tformat|<table|<row|<cell|f<rsub|0>(\<b-x\>,\<b-c\>)=<big|sum><rsub|j=0><rsup|k<rsub|0>>c<rsub|0,j>
    \ \<psi\><rsub|0,j>(\<b-x\>)>>|<row|<cell|\<vdots\>>>|<row|<cell|f<rsub|n>(\<b-x\>,\<b-c\>)=<big|sum><rsub|j=0><rsup|k<rsub|n>>
    c<rsub|n,j > \<psi\><rsub|n,j>(\<b-x\>)>>>>>
  </equation*>

  where

  <\itemize>
    <item><with|mode|math|\<b-c\>=(c<rsub|i,j>)> are the parameters,

    <item><with|mode|math|\<b-x\>> is a point of the variety
    <with|mode|math|X> of dimension,

    <item><with|mode|math|\<psi\><rsub|i,j>(\<b-x\>)> are polynomial
    functions on <with|mode|math|X>.
  </itemize>

  The problem consists to find conditions on <with|mode|math|\<b-c\>>, such
  that this system has a root in <with|mode|math|X>. In the case, we are
  considering, there exists a polynomial \ <with|mode|math|res<rsub|X>(\<b-f\>)>
  in the parameters <strong|c>, such that\ 

  <\equation*>
    res<rsub|X>(\<b-f\>)=0\ 
  </equation*>

  iff the previous system has a solution in <with|mode|math|X>. This
  polynomial is called the <em|resultant> of <with|mode|math|\<b-f\>> on
  <with|mode|math|X>. Different constructions of resultants are presented
  hereafter.

  <subsection|Tools for constructing matrices from polynomials>

  Several tools are available to construct matrices of coefficients of lists
  of polynomials:

  <\quote-env>
    <\code>
      template\<less\>class R,class L,class LM\<gtr\>\ 

      \ \ \ \ \ R matrix_of(const L& l, char t='N');

      template\<less\>class R,class L,class LM\<gtr\>\ 

      \ \ \ \ \ R matrix_of(const L& l, const LM & lm, char t='N');
    </code>
  </quote-env>

  The use of such functions is done by specifying the output of matrices:

  <\code>
    MAT m = matrix_of\<less\>MAT\<gtr\>(l);

    MAT m = matrix_of\<less\>MAT\<gtr\>(l,'T');
  </code>

  where <verbatim|l> is a list of multivariate polynomials, <verbatim|MAT>
  the type of output matrix that we want to compute, and <verbatim|'T'>
  specifies if the transposed matrix is computed.

  See <verbatim|synaps/mpol/matrixof.h>

  \;

  <subsection|Projective resultant>

  Here <with|mode|math|X=\<bbb-P\><rsup|n> > and <with|mode|math|f<rsub|i>>
  is a generic polynomial of degree <with|mode|math|d<rsub|i>>,
  <with|mode|math|i=0,\<ldots\>,n>. The construction used is the one
  described by F.S. Macaulay.

  <\code>
    MAT m \ = macaulay\<less\>MAT\<gtr\>(l);

    MAT mt = macaulay\<less\>MAT\<gtr\>(l,'T');
  </code>

  where <verbatim|l> is a list of <with|mode|math|n+1> multivariate
  polynomials in <with|mode|math|n> variables, <verbatim|MAT> the type of
  output matrix that we want to compute, and <verbatim|'T'> specifies if the
  transposed matrix is computed or not.

  See <verbatim| synaps/mpol/macaulay.h>

  <subsection|Toric resultant>

  Here <with|mode|math|X=\<cal-T\><rsup|n> > is the toric variety associated
  to the Minkowsky sum of polytopes <with|mode|math|A<rsub|i>\<subset\>\<bbb-Z\><rsup|n>>
  and <with|mode|math|f<rsub|i>> is a generic polynomial of support in
  <with|mode|math|A<rsub|i>>, <with|mode|math|i=0,\<ldots\>,n>. The
  construction used is the one described in <cite|CaEm93>.

  See: <verbatim|synaps/mpol/Toric.h>

  <subsection|Bezoutian>

  The bezoutian yields a multiple of any resultants <cite|ElMo99>. It is
  defined from <with|mode|math|n+1> (affine) polynomials
  <with|mode|math|f<rsub|0>, f<rsub|1>,\<ldots\>,f<rsub|n>> in the variables
  <with|mode|math|x<rsub|1>,\<ldots\>,x<rsub|n>>.

  See: <verbatim|synaps/mpol/bezoutian.h>

  <subsection|Example>

  <\code>
    #include \<less\>synaps/mpol.h\<gtr\>

    #include \<less\>synaps/mpol/macaulay.h\<gtr\>

    #include \<less\>synaps/mpol/bezoutian.h\<gtr\>

    #include \<less\>synaps/mpol/Toric.h\<gtr\>

    typedef \ MatrDse\<less\>double\<gtr\> matr_t;

    typedef \ MPol\<less\>double\<gtr\> \ mpol_t;

    using std::cout; using std::endl;

    int main ()

    {

    \ \ mpol_t p0("t+x+2*y"), p1("x^2+y^2-1"), p2("y^2+x*y-1");

    \ \ vector\<less\>mpol_t\<gtr\> l; l.push_back(p0); l.push_back(p1);
    l.push_back(p2);

    \;

    \ \ cout\<less\>\<less\> macaulay\<less\>matr_t\<gtr\>(l)\<less\>\<less\>endl;
  </code>

  [...]

  <\code>
    \ \ cout\<less\>\<less\> m_bezout\<less\>matr_t\<gtr\>(l)\<less\>\<less\>endl;
  </code>

  [...]

  <\code>
    \ \ cout\<less\>\<less\> toric\<less\>matr_t\<gtr\>(l)\<less\>\<less\>endl;
  </code>

  [...]

  <\code>
    }
  </code>

  \;
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
    <associate|auto-6|<tuple|1.5|?>>
    <associate|toc-1|<tuple|1|1>>
    <associate|toc-2|<tuple|1.1|1>>
    <associate|toc-3|<tuple|1.2|1>>
    <associate|toc-4|<tuple|1.3|1>>
    <associate|toc-5|<tuple|1.4|1>>
    <associate|toc-6|<tuple|1.5|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|bib>
      CaEm93

      ElMo99
    </associate>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Multivariate
      resultants> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Tools for constructing
      matrices from polynomials <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Projective resultant
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Toric resultant
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>Bezoutian
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.5<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>
    </associate>
  </collection>
</auxiliary>