/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
$Id: Lex.h,v 1.1 2005/07/11 11:17:56 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_MPOL_LEX_H
#define SYNAPS_MPOL_LEX_H

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS

/**
Class specifying the lexicographic order on the monomials of type {\tt M}.
**/
struct Lex
{
  template <class M>
  static bool less (const M &, const M &);
  template <class M>
  bool operator() (const M & m1, const M & m2) const {return less(m1,m2);}
};

template <class M>
inline bool Lex::less(const M & m1, const M & m2) 
{
   int n1 = m1.nvars(), n2 =m2.nvars();
   int i = 0;

   if(std::max(n1,n2)<0)
     return false;
   else
     {
       while(m1[i] == m2[i] && i< n1+1&& i<n2+1) i++;
       if(i==n2+1) 
	 return false;
       else if(i==n1+1)
	 return true;
       else
	 return (m1[i]< m2[i]);
     }
}

__END_NAMESPACE_SYNAPS
#endif // SYNAPS_MPOL_LEX_H

