/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*      Author(s): B. Mourrain, GALAAD, INRIA                         *
**********************************************************************
$Id: MPol_with_texp.h,v 1.1 2005/07/11 11:17:56 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_MPOL_MPOL_WITH_TEXP_H
#define SYNAPS_MPOL_MPOL_WITH_TEXP_H

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS


/// Sum of polynomials 
template <class C, class O, class R> 
inline VAL<OP<'+',MPol<C,O,R>,MPol<C,O,R> > >
  operator+(const MPol<C,O,R> & a, const MPol<C,O,R> & b)
{
  return VAL<OP<'+',MPol<C,O,R>,MPol<C,O,R> > >
    (OP<'+',MPol<C,O,R>,MPol<C,O,R> >(a,b));
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign(MPol<C,O,R> & p, const OP<'+',MPol<C,O,R>, MPol<C,O,R> >& M) 
{
  if(&p == &M.op1 && &p == &M.op2)
    MPOL::add<O,R>(p.rep(),MPol<C,O,R>(M.op2).rep());
  else if(&p == &M.op1)
    MPOL::add<O,R>(p.rep(),M.op2.rep());
  else if(&p == &M.op2)
    MPOL::add<O,R>(p.rep(),M.op1.rep());
  else 
    MPOL::add<O,R>(p.rep(),M.op1.rep(),M.op2.rep());
}
//----------------------------------------------------------------------
// operator - 
template <class C, class O, class R> 
inline VAL<OP<'-',MPol<C,O,R>,MPol<C,O,R> > >
  operator-(const MPol<C,O,R> & a, const MPol<C,O,R> & b)
{
  return VAL<OP<'-',MPol<C,O,R>,MPol<C,O,R> > >
    (OP<'-',MPol<C,O,R>,MPol<C,O,R> >(a,b));
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign( MPol<C,O,R> & p, const OP<'-',MPol<C,O,R>, MPol<C,O,R> >& M) 
{
  MPOL::minus<O,R>(p.rep(),M.op1.rep(),M.op2.rep());
}
//----------------------------------------------------------------------
// Unary operator - 
template <class C, class O, class R> 
inline VAL<OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::coeff_t> >
  operator-(const MPol<C,O,R> & a)
{
  return VAL<OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::coeff_t> >
    (OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::coeff_t>
     (a,(typename MPol<C,O,R>::coeff_t)-1));
}
//----------------------------------------------------------------------
// Product of polynomials 
template <class C, class O, class R>  inline 
VAL<OP<'*',MPol<C,O,R>,MPol<C,O,R> > >
  operator*(const MPol<C,O,R> & a, const MPol<C,O,R> & b)
{
  return VAL<OP<'*',MPol<C,O,R>,MPol<C,O,R> > >
   (OP<'*',MPol<C,O,R>,MPol<C,O,R> >(a,b));
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign( MPol<C,O,R> & p, 
	     const OP<'*',MPol<C,O,R>, MPol<C,O,R> >& M) 
{
  if(M.op1 !=0 && M.op2 !=0)
    MPOL::mult(p.rep(),M.op1.rep(),M.op2.rep(),O());
  else
    p= MPol<C,O,R>(0);
}
//----------------------------------------------------------------------
// Product of a polynomial by a scalar
template <class C, class O, class R> inline 
VAL<OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::coeff_t> >
  operator*(const MPol<C,O,R> & a, const typename MPol<C,O,R>::coeff_t & c)
{
  typedef typename MPol<C,O,R>::coeff_t C;
  return VAL<OP<'.',MPol<C,O,R>,C> >(OP<'.',MPol<C,O,R>,C>(a,c));
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign( MPol<C,O,R> & p, 
	     const OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::coeff_t> & M) 
{ 
  if(M.op1 !=0)
    MPOL::mul_ext(p.rep(),M.op1.rep(),M.op2);
 else
    p= MPol<C,O,R>(0);
}
//----------------------------------------------------------------------
// Product of a polynomial by a monomial
template <class C, class O, class R> inline
VAL<OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t> >
operator*(const MPol<C,O,R> & P, 
		     const typename MPol<C,O,R>::monom_t & m) 
{
  return VAL<OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t> >
    (OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t>(P,m));
}
//----------------------------------------------------------------------
// Product of a polynomial by a monomial
template <class C, class O, class R> inline
VAL<OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t> >
operator*(const typename MPol<C,O,R>::monom_t & m,const MPol<C,O,R> & P) 
{
  return VAL<OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t> >
    (OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t>(P,m));
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign( MPol<C,O,R> & p, 
	     const OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t> & M) 
{
  if(M.op1 !=0)
    MPOL::mul_ext(p.rep(),M.op1.rep(),M.op2);
  else
    p= MPol<C,O,R>(0);
}
//----------------------------------------------------------------------
template <class C, class O, class R> inline
VAL<OP<'/',MPol<C,O,R>,MPol<C,O,R> > >
operator/(const MPol<C,O,R> & A, const MPol<C,O,R> & B) 
{
  return VAL<OP<'/',MPol<C,O,R>,MPol<C,O,R> > >
    (OP<'/',MPol<C,O,R>,MPol<C,O,R> >(A,B));
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign( MPol<C,O,R> & p, 
	     const OP<'/',MPol<C,O,R>,MPol<C,O,R> > & M) 
{
  p=M.op1; p/=M.op2;
}
//----------------------------------------------------------------------
template <class C, class O, class R> inline
VAL<OP<'^',MPol<C,O,R>, unsigned int> >
operator^(const MPol<C,O,R> & A, unsigned int n) 
{
  return VAL<OP<'^',MPol<C,O,R>,unsigned int > >
    (OP<'^',MPol<C,O,R>,unsigned int >(A,n));
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign( MPol<C,O,R> & p, 
	     const OP<'^',MPol<C,O,R>,unsigned int > & M) 
{
  if(M.op2==0) p = MPol<C,O,R>(1);
  else {
    p=1;
    unsigned N = M.op2;
    while(N>0) {p*=M.op1; N--;}
  }
}
//----------------------------------------------------------------------
template<class C, class O, class R> inline
void assign( MPol<C,O,R> & p, 
	     const OP<'.',MPol<C,O,R>,typename MPol<C,O,R>::monom_t> & M) 
{
  if(M.op1 !=0)
    MPOL::mul_ext(p.rep(),M.op1.rep(),M.op2);
  else
    p= MPol<C,O,R>(0);
}
//----------------------------------------------------------------------

__END_NAMESPACE_SYNAPS


#endif // SYNAPS_MPOL_MPOL_WITH_TEXP_H

