/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
$Id: SQVlex.h,v 1.1 2005/07/11 11:17:56 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_MPOL_SQVLEX_H
#define SYNAPS_MPOL_SQVLEX_H

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS


/** Class defining the following order on the monomials $m_{1}$ and $m_{2}$;
 $m_{1} < m_{2}$ if either the maximal of the degrees of each variable is
 strictly greater in $m_{1}$ than in $m_{2}$, or they are equal and $m_{1}$
 is greater for the inverse lexicopgraphic ordering refined by the degree.
**/

struct SQVlex {
  template <class M>
  static bool less (const M &, const M &);
  template <class M>
  bool operator() (const M &, const M &) {return less(m1,m2);}
};


template <class M>
bool SQVlex::less(const M & m1, const M & m2) {

   int n1 = m1.nvars(), n2 =m2.nvars();
   int n = max(n1,n2);
   //cout <<m1<<" "<<m2<<" "<<n1<<" "<<n2 <<endl;
   if(n1 <0 && n2>0) return false;
   if(n2 <0 && n1>0) return true;
   if(n1 <0 && n2<0) return false;

   int d1=0,d2=0, k, c1, c2;
   for(int i =0 ; i<= n1; i++) {c1 = max(d1,(k=m1[i]));d1+=k;}
   for(int i =0 ; i<= n2; i++) {c2 = max(d2,(k=m2[i]));d2+=k;}

   if      (c1 < c2) return false;
   else if (c1 > c2) return true;
   else if (d1 < d2) return false;
   else if (d1 > d2) return true;
   else {
     int i=n;
     while( m1[i] == m2[i] && i>=0) i--;
     if(i <0)
       return false;
     else
     return (m1[i]> m2[i]);
   }
}

__END_NAMESPACE_SYNAPS

#endif // SYNAPS_MPOL_SQVLEX_H

