#ifndef SYNAPS_MPOLDSE_BERNSTEIN_C
#define SYNAPS_MPOLDSE_BERNSTEIN_C

#include "MPOLDSE.h"

__BEGIN_NAMESPACE_SYNAPS

namespace mpoldse 
{
  template<class C> inline
  void binoms( bernstein<C>& mpl ) { MPOLDSE::binoms( mpl ); };
  
  template < class C > inline void
  scale (bernstein < C > &mpl)
  {
   scale (mpl.begin (), mpl.env, binomials < C >::default_);
  };
  
  template < class C > inline void
  uscale (bernstein < C > &mpl)
  {
  uscale (mpl.begin (), mpl.env, binomials < C >::default_);
  };
  
  template < class C > inline void
  convertm2b (bernstein < C > &mpl)
  {
   convertm2b (mpl.begin (), mpl.env, binomials < C >::default_);
  };
  
  
  template < class C > void
  elevate (bernstein < C > &r, const eenv & elev)
  {
    bernstein < C > tmp (elev);
    MPOLDSE::binoms (tmp);
    scale (r);
    MPOLDSE::conv(r, tmp);
    uscale (r);
  };
  /*
  template< class C > void 
  elevate( bernstein< C >& r, const eenv & delta, const bernstein<C>& a )
  {

  };
  */  
  template < class C > void
  add (bernstein < C > &mpl, const C & c)
  {
    vct::scadd (mpl.begin (), c, mpl.esz (), 1);
  };
  
  template < class C > void
  add (bernstein < C > &r, const bernstein < C > &a, const C & c)
  {
    if (r.env != a.env)
      add (r = a, c);
    else
      vct::scadd (r.begin (), a.begin (), c, r.esz ());
  };
  
  template<class C> void
  rewrite( bernstein<C>& r, const eenv& newe )
  {
    if ( newe == r.env ) return;
    eenv elenv(eenv::elevation (newe, r.env));
    elevate (r, elenv) ;
  };
  
  template < class C > void
  add (bernstein < C > &r, const bernstein < C > &a)
  {
    eenv cm(eenv::common(r.env,a.env));
    if ( r.env != cm ) { rewrite(r,cm); };
    if ( a.env == cm ) 
      {
	vct::padd (r.begin (), a.begin (), r.esz (), 1, 1); 
      }
    else 
      {
	bernstein<C> tmp(a);
	rewrite(tmp,cm);
	vct::padd (r.begin (), tmp.begin (), r.esz (), 1, 1);
      };
  };


template < class C > void
add (bernstein < C > &r, const bernstein < C > &a, const bernstein < C > &b)
{
  add (r = a, b);
};


template < class C > void
sub (bernstein < C > &r, const bernstein < C > &a)
{
  eenv cm(eenv::common(r.env,a.env));
  if ( r.env != cm ) { rewrite(r,cm); };
  if ( a.env == cm ) 
    {
      vct::psub (r.begin (), a.begin (), r.esz (), 1, 1); 
    }
  else 
    {
      bernstein<C> tmp(a);
      rewrite(tmp,cm);
      vct::psub (r.begin (), tmp.begin (), r.esz (), 1, 1);
    };
};


template < class C > void
sub (bernstein < C > &r, const bernstein < C > &a, const bernstein < C > &b)
{
  sub (r = a, b);
};


template < class C > void
sub (bernstein < C > &mpl, const C & c)
{
  vct::scsub (mpl.begin (), c, mpl.esz (), 1);
};

template < class C > void
sub (bernstein < C > &mpl, const C & c, const bernstein < C > &a)
{
  if (mpl.env != a.env)
    bernstein < C > (a.env).swap (mpl);
  vct::scsub (c, mpl.begin (), c, a.begin (), a.esz ());
};

template < class C > void
mul (bernstein < C > &r, const bernstein < C > &a, const bernstein < C > &b)
{
  bernstein < C > a_ (a);
  scale (a_);
  bernstein < C > b_ (b);
  scale (b_);
  using namespace MPOLDSE;
  conv (r, a_, b_);
  uscale (r);
};

template < class C > void
mul (bernstein < C > &r, const bernstein < C > &a)
{
  bernstein < C > a_;
  r.swap (a_);
  scale (a_);
  bernstein < C > b_ (a);
  scale (b_);
  MPOLDSE::conv (r, a_, b_);
  uscale (r);
};

template < class C > bernstein < C >::bernstein (const eenv & e): base_type(e) {};

template < class C > bernstein < C >::bernstein (const eenv & e, const C & c): 
  base_type(e,c){};

template < class C > bernstein < C >::bernstein (int nvr, const int *szs, const int *vrs): base_type(nvr,szs,vrs)
{
};

template < class C > bernstein < C >::bernstein (const C & x) : base_type(x)
{}

template < class Result, class Coeff, class Parameters > inline 
void eval (Result & result, const bernstein < Coeff > & controls , const Parameters & parameters )
{
  levalb (result, controls.begin (), controls.env,parameters);
};


template < class Result, class Coeff, class Parameters > inline 
void subs0 (Result *result, const bernstein < Coeff > & controls , const Parameters & parameters )
{ 
  subs0b( result, controls.begin (), controls.env, parameters ); 
};

template < class C > void
casteljau (bernstein < C > &a, bernstein < C > &b, const C & t, int v)
{
  if (a.env != b.env)
    realloc (a, b.env);
  casteljau (a.begin (), b.begin (), t, v);
};

template < class C > void
casteljau (bernstein < C > &left, bernstein < C > &right,
	   const bernstein < C > &source, const C & t, int v)
{
  casteljau (left, right = source, t, v);
};


template < class C > void
casteljau (bernstein < C > &a, bernstein < C > &b, int v)
{
  if (a.env != b.env)
    realloc (a, b.env);
  casteljau (a.begin (), b.begin (), v);
};

template < class C > void
casteljau (bernstein < C > &left, bernstein < C > &right,
	   const bernstein < C > &source, int v)
{
  casteljau (left, right = source, v);
};



template < class C >
  std::ostream & operator<< (std::ostream & o, const bernstein < C > &mpl)
{
  o << "bernstein object:\n";
  o << "\t(addresses) o: " << &mpl << " e: " << mpl.env.data <<  "d: " << mpl.
    begin () << std::endl;
  o << "\t(eenv)" << mpl.env << std::endl;
  o << "\t(data)";
  VECTOR::print (o, mpl.data);
  o << std::endl;
  o << ":object bernstein\n";
  return o;
};

template<class C> void
assign( monomial<C>& monoms, const bernstein<C>& controls )
{
  monoms = controls;
  MPOLDSE::convertb2m(monoms);
};
template < class C, class O, class R > void
convert (MPol < C, O, R > &mpol, const bernstein < C > &mpl)
{
  monomial<C> tmp;
  assign(tmp,mpl);
  convert(mpol,tmp);
};

template < class C > void
cfdump (std::ostream & o, const bernstein < C > &mpl)
{
  for (const C * a = mpl.begin (); a != mpl.end (); o << " " << *a++);
};

template < class C > void
mprint (std::ostream & o, const bernstein < C > &mpl)
{
  MPol < C > mp (0);
  convert (mp, mpl);
  o << mp;

};

template<class C > void 
print( std::ostream& o , const bernstein<C>& mpl ) { 
  
  MPol<C> mp(0);
  convert(mp,mpl);
  o << mp;
};

template < class C > bool bernstein < C >::operator== (const bernstein & mpl) const 
{
  return base_type::operator==(mpl); 
};

template<class C> inline
void diff( bernstein<C>& res, const bernstein<C>& src, int v )
{
  clear( res, eenv::diff(src.env,v) );
  bdiff( res.begin(), src.env, src.begin(), v );
};

template<class C>
bernstein<C>& bernstein<C>::operator=( const C& x ) 
{
  this->~bernstein<C>();
  new (this) bernstein<C>(x);
  return *this;
};

template<class C> inline
void split( bernstein<C>& l, bernstein<C>& r, int v, const C& t   )
{
  if ( l.env != r.env ) { realloc(l,r.env); };
  bsplit( l.begin(), r.begin(), r.env, t, r.env.localv(v) );
};


template<class C> inline
void split( bernstein<C>& l, bernstein<C>& r, int v  )
{
  if ( l.env != r.env ) { realloc(l,r.env); };
  bsplit2( l.begin(), r.begin(), r.env, r.env.localv(v) );
};

template<class C>
void restrict( bernstein<C>& rst, int v, const C& a, const C& b ) { 
  brestrict( rst.begin(), rst.env, rst.env.localv(v), a, b ); 
};

inline eenv face( const eenv& e, int lv ) 
{
  const int * eszs = e.szs();
  const int * evrs = e.vrs();
  const int   envr = e.nvr();
  int szs[ envr ];
  int vrs[ envr ];
  int c = 0;
  for ( int v = 0; v < envr; v ++ ) if ( v != lv ) { szs[c] = eszs[v]; vrs[c++] = evrs[v]; };//evrs[v]; };
  return eenv(c,szs,vrs);
};


template<class C> inline
void lface ( bernstein<C>& f, const bernstein<C>& src, int v )
{

  lface( f.begin(), src.env, src.begin(), v );
};


template<class C> inline
void rface ( bernstein<C>& f, const bernstein<C>& src, int v )
{


  rface( f.begin(), src.env, src.begin(), v );
};


template<class C> inline
void face ( bernstein<C>& f, const bernstein<C>& src, int v, int n )
{
  assert(n==0||n==1);
  eenv fenv(face(src.env,src.env.localv(v)));
  if ( f.env != fenv ) realloc( f, fenv );
  if ( n ) rface( f, src, src.env.localv(v) );
  else     lface( f, src, src.env.localv(v) );
};

}

__END_NAMESPACE_SYNAPS

#endif /* SYNAPS_MPOLDSE_BERNSTEIN_C */
