#ifndef SYNAPS_SBDSLV_EENV_C
#define SYNAPS_SBDSLV_EENV_C
#include <synaps/mpol/mpoldse/vctops.h>
__BEGIN_NAMESPACE_SYNAPS 
namespace mpoldse
{

  inline void eenv::swap (eenv & e)
  {
    std::swap (data, e.data);
  };
  inline int eenv::nvr () const
  {
    return data[1];
  };
  inline const int *eenv::szs () const
  {
    return data + 2;
  };
  inline int *eenv::szs ()
  {
    return cdup ().data + 2;
  };
  inline int eenv::sz (int v) const
  {
    return szs ()[v];
  };
  inline int *eenv::vrs ()
  {
    return szs () + nvr ();
  };
  inline const int *eenv::vrs () const
  {
    return szs () + nvr ();
  };
  inline int *eenv::str ()
  {
    return vrs () + nvr () + 1;
  };
  inline const int *eenv::str () const
  {
    return vrs () + nvr () + 1;
  };
  inline int eenv::st (int v) const
  {
    return str ()[v];
  };
  inline int eenv::sz () const
  {
    return str ()[-1];
  };
  inline eenv::vr_t * eenv::gvr ()
  {
    return (eenv::vr_t *) (str () + nvr ());
  };
  inline const eenv::vr_t * eenv::gvr () const
  {
    return (const eenv::vr_t *) (str () + nvr ());
  };
  inline int eenv::vr (int v) const
  {
    return vrs ()[v];
  };
  inline bool eenv::operator< (const eenv & b) const
  {
    return sz () < b.sz ();
  };
  inline int eenv::msz (int nvr_)
  {
    return 1 + 1 + nvr_ + nvr_ + nvr_ + 1 + 2 * nvr_;
  };
  inline int eenv::msz () const
  {
    return eenv::msz (nvr ());
  };
  inline int *eenv::alloc (int nvr_)
  {
    return data = new int[msz (nvr_)];
  };
  inline void eenv::construct (int nvr_, int const *szs_, int const *vrs_,
			       int *mem)
  {

    if (mem == 0)
      data = alloc (nvr_);
    else
      data = mem;
    data[0] = 1;
    data[1] = nvr_;
    if (szs_)
      {
	std::copy (szs_, szs_ + nvr_, szs ());
	cstr ();
	if (vrs_)
	  {
	    std::copy (vrs_, vrs_ + nvr_, vrs ());
	    cvrs ();
	  }
	else
	  {
	    for (int v = 0; v < nvr (); v++)
	      {
		vrs ()[v] = v;
		gvr ()[v].r = v;
		gvr ()[v].v = v;
	      };
	  };
      };
  };

  inline bool eenv::equal (const eenv & a, const eenv & b)
  {
    if (a.data == b.data)
      return true;
    if (a.nvr () != b.nvr ())
      return false;
    for (int i = 0; i < 1 + 2 * a.nvr (); i++)
      if (a.data[i] != b.data[i])
	return false;
    return true;
  };

  inline bool eenv::equiv (const eenv & a, const eenv & b)
  {
    const int *aszs = a.szs ();
    const int *bszs = b.szs ();
    const eenv::vr_t * agvr = a.gvr ();
    const eenv::vr_t * bgvr = b.gvr ();
    if (a.nvr () != b.nvr ())
      return false;
    for (int i = 0; i < a.nvr (); i++)
      if (agvr[i].v != bgvr[i].v || aszs[agvr[i].r] != bszs[bgvr[i].r])
	return false;
    return true;
  };

  inline void eenv::dealloc ()
  {
    if (data)
      {
	if (--data[0])
	  return;
	delete[]data;
      };
  };

  inline eenv & eenv::copy (eenv & e, const eenv & b)
  {
    e.dealloc ();
    e.data = b.data;
    if (e.data)
      e.data[0]++;
    return e;
  };

  inline eenv & eenv::cdup ()
  {
    assert(data);
    if (data[0] == 1)
      return *this;
    data[0]--;
    int *sv = data;
    int _sz = msz();
    data = alloc (nvr ());
    std::copy (sv, sv + _sz, data);
    return *this;
  };

  inline void eenv::cstr ()
  {
    int i;
    int *_st = str ();
    int *_sz = szs ();
    for (i = nvr () - 2, _st[nvr () - 1] = 1; i >= -1;
	 _st[i] = _sz[i + 1] * _st[i + 1], i--);
  };

  inline void eenv::cvrs ()
  {
    eenv::vr_t * _gvr = gvr ();
    const int *_vr = vrs ();
    for (int i = 0; i < nvr (); _gvr[i].r = i, _gvr[i].v = _vr[i], i++);
    std::sort (gvr (), gvr () + nvr ());
  };

  inline bool eenv::hasvar (int &lv, int gv) const
  {
    int g, d, m;

    if ((gv < gvr ()[0].v) || (gv > gvr ()[(d = nvr ()) - 1].v))
      return false;

    if (gv == gvr ()[g = 0].v)
      {
	lv = gvr ()[0].r;
	return true;
      };

    do
      {
	m = (g + d) / 2;
	if (gvr ()[m].v == gv)
	  {
	    lv = gvr ()[m].r;
	    return true;
	  };
	if (gvr ()[m].v < gv)
	  g = m;
	else
	  d = m;
      }
    while (d - g > 1);

    return false;

  };

  int eenv::mxvr() const { return gvr()[nvr()-1].v; };
  int eenv::mnvr() const { return gvr()[0].v; };
  inline eenv::~eenv ()
  {
    dealloc ();
  };



  inline eenv::eenv (int nvr_, int const *szs_, int const *vrs_, int *mem)
  {
    construct (nvr_, szs_, vrs_, mem);
  };

  inline eenv::eenv (const eenv & e)
  {
    data = e.data;
    if (data)
      data[0]++;
  };

  inline eenv & eenv::operator= (const eenv & e)
  {
    return copy (*this, e);
  };


  inline eenv eenv::mrgvrs (const eenv & a, const eenv & b)
  {

    unsigned c = 0;
    int va, vb;
    int prv = (int) (-1);
    int anvr = a.nvr ();
    int bnvr = b.nvr ();
    const eenv::vr_t * agvr = a.gvr ();
    const eenv::vr_t * bgvr = b.gvr ();

    int tmp[anvr + bnvr];

    c = va = vb = 0;

    do
      {
	if (agvr[va].v <= bgvr[vb].v)
	  {
	    if (agvr[va].v != prv)
	      tmp[c++] = prv = agvr[va].v;
	    va++;
	    continue;
	  };
	if (bgvr[vb].v < agvr[va].v)
	  {
	    if (bgvr[vb].v != prv)
	      tmp[c++] = prv = bgvr[vb].v;
	    vb++;
	    continue;
	  };
      }
    while (va < anvr && vb < bnvr);

    if (va < anvr)
      {
	for (; va < anvr; va++)
	  if (agvr[va].v != prv)
	    tmp[c++] = agvr[va].v;
      };
    if (vb < bnvr)
      {
	for (; vb < bnvr; vb++)
	  if (bgvr[vb].v != prv)
	    tmp[c++] = bgvr[vb].v;
      };

    eenv oenv (c);

    int *ovrs = oenv.vrs ();
    eenv::vr_t * ogvr = oenv.gvr ();

    for (int i = 0; i < oenv.nvr (); i++)
      {
	ovrs[i] = tmp[i];
	ogvr[i].v = tmp[i];
	ogvr[i].r = i;
      };


    return oenv;
  };

  inline eenv eenv::common (const eenv & a, const eenv & b)
  {
    int v, lva, lvb;

    eenv oenv (mrgvrs (a, b));

    const int *ovr = oenv.vrs ();
    const int *aszs = a.szs ();
    const int *bszs = b.szs ();
    int *oszs = oenv.szs ();

    for (v = 0; v < oenv.nvr (); v++)
      {
	if (a.hasvar (lva, ovr[v]))
	  {
	    if (b.hasvar (lvb, ovr[v]))
	      oszs[v] = std::max (aszs[lva], bszs[lvb]);
	    else
	      oszs[v] = aszs[lva];
	  }
	else
	  {
	    if (b.hasvar (lvb, ovr[v]))
	      oszs[v] = bszs[lvb];
	    else
	      std::cout << "erreur dans common\n";
	  };
      };

    oenv.cstr ();
    return oenv;
  };

  inline eenv eenv::mul (const eenv & a, const eenv & b)
  {

    int v, lva, lvb;
    eenv oenv (mrgvrs (a, b));
    //  std::cout << " mrgvrs = " << oenv << std::endl;
    int *oszs = oenv.szs ();
    const int *ovrs = oenv.vrs ();
    const int *aszs = a.szs ();
    const int *bszs = b.szs ();

    for (v = 0; v < oenv.nvr (); v++)
      {
	oszs[v] = 0;
	if (a.hasvar (lva, ovrs[v]))
	  oszs[v] += aszs[lva] - 1;
	if (b.hasvar (lvb, ovrs[v]))
	  oszs[v] += bszs[lvb] - 1;
	oszs[v]++;
      };
    oenv.cstr ();
    return oenv;
  };

  inline eenv eenv::diff (const eenv & b, int lv)
  {
    //    const int * bszs = b.szs();
    eenv oenv (b);
    int *oszs = oenv.szs ();
    oszs[lv] = std::max (oszs[lv] - 1, 1);
    oenv.cstr ();
    return oenv;
  };

  inline bool eenv::vmap (int *vmap, const eenv & o, const eenv & i)
  {
    for (int v = 0; v < i.nvr (); v++)
      {
	int lv;
	if (!o.hasvar (lv, i.vr (v)))
	  {
	    return false;
	  };
	vmap[v] = lv;
      };
    return true;
  };

  inline eenv eenv::rewrite( const eenv& o, const eenv& i )
  {
    int vmap_[ o.nvr() ];
    if ( ! vmap( vmap_, o, i ) ) { std::cerr << "vmap error(rewrite)\n"; exit(1); };
    eenv tmp(o);
    for ( int v = 0; v < o.nvr(); v ++ ) tmp.szs()[v] = 1;
    for ( int v = 0; v < i.nvr(); v ++ ) tmp.szs()[vmap_[v]] = i.szs()[v];
    tmp.cstr();
    return tmp;
  };
  
  bool eenv::oaddress (const eenv & oenv, unsigned *osupp,
		       const eenv & ienv, unsigned *isupp, unsigned nsp)
  {
    //  std::cout << oenv << std::endl;
    //  std::cout << ienv << std::endl;
    int v;
    unsigned c;
    int vmap_[ienv.nvr ()];
    unsigned addi, addo;
    if (!vmap (vmap_, oenv, ienv))
      return false;
    if (isupp)
      for (c = 0; c < nsp; osupp[c] = addo, c++)
	for (addi = isupp[c], addo = 0, v = ienv.nvr () - 1; addi;
	     addi /= ienv.sz (v), v--)
	  addo += (addi % ienv.sz (v)) * oenv.st (vmap_[v]);
    else
      {
	const int *istr = ienv.str ();
	const int *ostr = oenv.str ();
	osupp[0] = 0;
	for (v = 0; v < ienv.nvr (); v++)
	  for (int j = 0; j < istr[-1]; j += istr[v - 1])
	    for (int i = j + istr[v]; i < j + istr[v - 1]; i += istr[v])
	      osupp[i] = osupp[i - istr[v]] + ostr[vmap_[v]];

	/*
	   nsp = ienv.sz();
	   for ( c = 0; c < nsp; osupp[c] = addo, c ++ )
	   for ( addi = c, addo = 0, v = ienv.nvr()-1; addi; addi /= ienv.sz(v), v -- )
	   addo += (addi%ienv.sz(v))*oenv.st(vmap_[v]);
	 */

      };
    return true;
  };



  std::ostream & operator<< (std::ostream & out, const eenv::vr_t & gv)
  {
    out << "( r = " << gv.r << ", v = " << gv.v << ")";
    return out;
  };

  std::ostream & operator<< (std::ostream & out, const eenv & env)
  {
    out << "(" << env.data[0] << ")";
    out << "[ " << env.nvr () << ", ";
    std::cout << "(v)\n";
    vct::print (env.vrs (), env.nvr (), 1, out);
    out << ", ";
    vct::print (env.szs (), env.nvr (), 1, out);
    out << ", ";
    out << "[ " << env.sz () << ", ";
    vct::print (env.str (), env.nvr (), 1, out);
    out << "] ";
    vct::print (env.gvr (), env.nvr (), 1, out);
    return out;
  };

  void eenv::vswap (int *perm)
  {
    for (int i = 0; i < this->nvr (); i++)
      {
	std::swap( vrs()[i], vrs()[perm[i]]);
	std::swap (gvr ()[i].r, gvr ()[perm[i]].r);
	std::swap (szs ()[i], szs ()[perm[i]]);
      };
    cstr ();
  };

  bool eenv::subset (const eenv & a, const eenv & b)
  {
    const int *avrs = a.vrs ();
    const int *aszs = a.szs ();
    const int *bszs = b.szs ();
    int lv;
    for (int i = 0; i < a.nvr (); i++)
      if ((!b.hasvar (lv, avrs[i])) || aszs[i] > bszs[lv])
	return false;
    return true;
  };

  inline bool eenv::operator== (const eenv & e) const
  {
    return eenv::equal (*this, e);
  };

  inline bool eenv::operator!= (const eenv & e) const
  {
    return !eenv::equal (*this, e);
  };

  inline eenv eenv::elevation (const eenv & o, const eenv & i)
  {
    int nszs[o.nvr ()];
    const int *oszs = o.szs ();
    const int *iszs = i.szs ();
    int lv;
    for (int v = 0; v < o.nvr (); v++)
      if (i.hasvar (lv, v))
	nszs[v] = oszs[v] - iszs[lv] + 1;
      else
	nszs[v] = oszs[v];
    return eenv (o.nvr (), nszs, o.vrs ());
  };


// MPol Connection 

  struct vd
  {
    int n, d;
      vd ():n (-1), d (0)
    {
    };
  };

  inline std::ostream & operator<< (std::ostream & o, const vd & vd_)
  {
    o << "vd(" << vd_.n << ", " << vd_.d << ")";
    return o;
  };

  template < class MONOM > void mescan (int &last, vd * vdeg, const MONOM & m)
  {

    for (unsigned i = 0; i < m.size (); i++)
      {
	if (m[i] != 0)
	  {
	    if (vdeg[i].n != -1)
	      vdeg[i].d = std::max ((int) m[i], vdeg[i].d);
	    else
	      {
		if (last != -1)
		  {
		    vdeg[i].n = vdeg[last].n;
		    vdeg[i].d = m[i];
		    vdeg[last].n = i;
		    last = i;
		  }
		else
		  {
		    last = i;
		    vdeg[i].n = i;
		    vdeg[i].d = m[i];
		  };
	      };
	  };
      };
  }
  
  
  struct oulala { 
    int initial; int name; 
    bool operator<( const oulala& b ) const { return name < b.name; }; 
  };


  template < class C, class O, class R >
  inline eenv::eenv (const MPol < C, O, R > &mpol)
  {
    int nvr = MPOLDST::nbvar (mpol);
    vd lenv[nvr];
    int last = -1;
    typename MPol < C, O, R >::const_iterator it;
    for (it = mpol.begin (); it != mpol.end (); mescan (last, lenv, *it++));
    if ( last == -1 ) { nvr = 1; int szs = 1; new (this) eenv(1,&szs,0); return; };
    
    int c = last;
    nvr = 0;
    do
      {
	c = lenv[c].n;
	nvr++;
      }
    while (c != last);
    int vrs[nvr];
    int szs[nvr];
    c = last;

    int k = 0;
    do
      {
	vrs[k] = c;
	szs[k] = lenv[c].d + 1;
	c = lenv[c].n;
	lenv[vrs[k]].n = -1;
	lenv[vrs[k]].d = 0;
	k++;
      }
    while (c != last);
    //    #ifdef CRADO
    //    oulala tmp[ nvr ];
    /*    for ( int i = 0; i < nvr; i ++ )
    { tmp[i].initial = i; tmp[i].name = vrs[i]; };
    
    std::sort(tmp,tmp+nvr);

    for ( int v = 0; v < nvr; v ++ ) 
      {
	std::swap(szs[tmp[v].initial], szs[v]);
	std::swap(vrs[tmp[v].initial], vrs[v]);
	};
    */
    //    #endif
    new (this) eenv (nvr, szs, vrs);
  };
  
}

__END_NAMESPACE_SYNAPS
#endif
