#ifndef SYNAPS_MPOLDSE_MONOMIAL_H
#define SYNAPS_MPOLDSE_MONOMIAL_H

#include <synaps/init.h>
#include <vector>
#include <synaps/base/type.h>
#include <synaps/mpol/MPol.h>
#include <synaps/mpol/mpoldse/eenv.h>
#include <synaps/mpol/mpoldse/binomials.h>

__BEGIN_NAMESPACE_SYNAPS

namespace mpoldse
{
  template<class C>
  struct monomial 
  {
    typedef C coeff_t;
    typedef std::vector<C>  vector_type;
    typedef C * iterator;
    typedef const C * const_iterator;
    typedef typename vector_type::size_type size_type;
    typedef coeff_t value_type;
    static binomials<C> binoms;
    eenv           env;
    vector_type    data;
    monomial( const C& x = 0 );
    template<class X, class O, class R> 
    monomial( const MPol<X,O,R>& mpol );
    monomial( const eenv & e);
    monomial( const eenv& e, const C& c );
    monomial( int nvr, const int * szs, const int * vrs = 0 );
    bool operator==( const C& c ) const;
    C * begin() { return &(data[0]); };
    C * end  () { return &(data[0])+esz(); };
    const C * begin() const { return &(data[0]); };
    const C * end()   const { return &(data[0])+esz(); };
    int  esz() const { return env.sz(); };
    int  nvr() const { return env.nvr(); };
    int * szs() { return env.szs(); };
    int * vrs() { return env.vrs(); };
    int * str() { return env.str(); };
    const int * szs() const { return env.szs(); };
    const int * vrs() const { return env.vrs(); };
    const int * str() const { return env.str(); };
    int size() const { return esz(); };
    const C & operator[]( int i ) const { return data[i]; };
    C & operator[]( int i ) { return data[i]; };
    bool hasvar( int& lv, int gv ) const;
    bool operator==( const monomial& mpl ) const ;
    void swap( monomial<C>& a ) { std::swap(env,a.env); data.swap(a.data); };
  };
  
};


__END_NAMESPACE_SYNAPS

#include "monomial.C"

#endif /* SYNAPS_MPOLDSE_MONOMIAL_H */
