/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*      Author(s): B. Mourrain, GALAAD, INRIA                         *
**********************************************************************/
#ifndef synaps_mpol_subs_h_
#define synaps_mpol_subs_h_
//--------------------------------------------------------------------
#include <synaps/mpol/MPol.h>
#include <synaps/mpol/binomial.h>
#include <synaps/mpol/MPOLDST.m>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
/// Substitute variable of index @i@ by @q@ in polynomial @p@.
template <class X, class C, class O, class R>
MPol<C,O,R> subs(int i, const X & p, const MPol<C,O,R>& q)
{
  return MPOLDST::subs(i,p,q); 
}
//--------------------------------------------------------------------
/// Substitute variable @v@ by @q@ in polynomial @p@.
template <class X, class C, class O, class R>
MPol<C,O,R> subs(char* v, const X & p, const MPol<C,O,R>& q, Variables& vars = Variables::default_ )
{
  typedef typename MPol<C,O,R>::monom_t monom_t;
  int i = vars[v];
  return MPOLDST::subs(i,p,q); 
}
//----------------------------------------------------------------------
template <class C, class O, class R> inline
void swap(MPol<C,O,R> & P, MPol<C,O,R> & Q)
{
  swap(P.rep(),Q.rep());
}
//----------------------------------------------------------------------
/**
    Function which returns the polynomial after making a rotation by t
    in the x0,x1 plane.
    t represents, in the case of rotation, tan(teta/2), with teta is
    the rotation angle.
**/
template<class C,class O,class R> inline
MPol<C,O,R> Rotate(const MPol<C,O,R> &  P,C t)
{
  typedef typename MPol<C,O,R>::const_iterator  const_iterator;
  typedef typename MPol<C,O,R>::coeff_t coeff_t;
  MPol<C,O,R> P4=0;

  coeff_t cos_teta = (1-pow<coeff_t>(t,2))/(1+pow<coeff_t>(t,2));
  coeff_t sin_teta = 2*t/(1+pow<coeff_t>(t,2));

  for(const_iterator i = P.begin(); i != P.end(); ++i)
    {
      int puiss1=(*i)[0];
      int puiss2=(*i)[1];

      coeff_t coeff = i->coeff();
      MPol<C,O,R> P1=0,P2=0;

      for(int j=0;j<=puiss1;j++)
        {
          int puiss_j = puiss1-j;
          coeff_t coeffB=binomial<coeff_t>(puiss1,j);
          MPol<C,O,R> pi1 = Monom<coeff_t,dynamicexp<unsigned> >(coeffB,0,j);
	  MPol<C,O,R> pi2 = Monom<coeff_t,dynamicexp<unsigned> >(1,1,puiss_j);
	  coeff_t k=pow<coeff_t>(cos_teta,j);
          pi1*=k;
          k=pow<coeff_t>(-sin_teta,puiss_j);
          pi2*=k;
	  if ( Degree(pi1*pi2) > 0)
	    P1 += (pi1*pi2);
        }


      for(int j=0;j<=puiss2;j++)
        {
	  int puiss_j = puiss2-j;
          coeff_t coeffB=binomial<coeff_t>(puiss2,j);
          MPol<C,O,R> pi1 = Monom<coeff_t,dynamicexp<unsigned> >(coeffB,0,j);
	  MPol<C,O,R> pi2 = Monom<coeff_t,dynamicexp<unsigned> >(1,1,puiss_j);
	  coeff_t k=pow<coeff_t>(sin_teta,j);
          pi1*=k;
          k=pow<coeff_t>(cos_teta,puiss_j);
          pi2*=k;
	  if ( Degree(pi1*pi2) > 0)
	    P2 += (pi1*pi2);
        }

      MPol<C,O,R> P3("0;");
      if (coeff != 0)
	P3+=coeff;
      if (puiss1)
	P3*=P1;
      if (puiss2)
	P3*=P2;

      if (P3 != MPol<C,O,R>("0;"))
	P4 += P3;
    }
  return P4;
}
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
#endif // synaps_mpol_subs_h_

