#include <synaps/init.h>
#include <synaps/mpol.h>
#include <synaps/mpol/xml.h>
#include <synaps/base/Seq.h>
#include <synaps/arithm/gmp.h>
#include <synaps/mpol/xml.h>
#include <synaps/msolve/sbdslv.h>
#include <fstream>

typedef MPol<double> mpol_t;
void usage (char* cmd)
{
  std::cout<<"usage:\n"<<cmd<<" file [-bZa] ..."<<std::endl;
  std::cout<<"\n  The file should contain the m equations in n variables (m >= n)"<<std::endl;
  std::cout<<"  in the following format:\n  <polynomial>...</polynomial>\n  <polynomial>...</polynomial>\n  ...\n "<<std::endl;
  std::cout<<"(example: <polynomial>x^2+2*y^2-2</polynomial><polynomial>x*y-1</polynomial>)"<<std::endl;
}
int main (int argc, char **argv)
{
  using std::cout;
  using std::endl;
  typedef std::string   str_t;

  if(argc<2)
    {
      usage(argv[0]);
      return 0;
    }
  
  std::fstream in(argv[1],std::ios::in);
  if(!in.good())
    {
      if(str_t(argv[1])!=str_t("-h"))
	{
	  std::cerr<<argv[0]<<": file not found"<<std::endl;
	  return 1;
	}
      usage(argv[0]);
      return 0;
    }
  int N=100;
  bool verbatim=false;
  bool withdomain=true;
  VectDse<double> V;
  double eps=1e-3;
  int nv;
  for(unsigned c=2;c<argc;c++)
    {
      if(str_t(argv[c])==str_t("-nv"))
	{
	  c++; nv=atoi(argv[c]);
	}
      else if(str_t(argv[c])==str_t("-box"))
	{
	  c++; V=VectDse<double>(nv*2,argv[c]);
	  std::cout<<"domain: "<<V<<std::endl;
	  withdomain=true;
	}
      else if(str_t(argv[c])==str_t("-eps"))
	{
	  c++; eps=atof(argv[c]);
	  std::cout<<"epsilon: "<<eps<<std::endl;
	}
      else if(str_t(argv[c])==str_t("-v"))
	{
	  verbatim=true;
	}
    }

  mpol_t p;
  std::list<mpol_t> I; 
  unsigned u=0;
  while(xml::read(in,p))
    {
      std::cout<<"pol["<<u<<"]: deg "<<degree(p)<<" sz "<<p.size() <<std::endl;
      u++;
      if(verbatim)
	std::cout<<"  <polynomial>"<<p<<"</polynomial>"<<endl;
      I.push_back(p);
    }
  std::cout <<"Solution(s):"<<std::endl;
  if(withdomain)
    cout << solve(I, SBDSLV<double,SBDSLV_RDL>(eps),V)<<endl;
  else
    cout << solve(I, SBDSLV<double,SBDSLV_RDL>(eps))<<endl;
}

