
#include <synaps/init.h>


__BEGIN_NAMESPACE_SYNAPS


//void convert(Z<MON_PREMIER,int> &toto,char * chaine)
//{
//  toto=Z<MON_PREMIER,int>(chaine);
//}
template<typename T>
void convert(Z<MON_PREMIER,T> &toto,char * chaine)
{
  toto=Z<MON_PREMIER,T>(chaine);
}

template<typename T>
int
dpivotL(
        const int  jcol,     /* in */
        const Z<MON_PREMIER,T> u,      /* in - diagonal pivoting threshold */
        int        *usepr,   /* re-use the pivot sequence given by perm_r/iperm_r */
        int        *perm_r,  /* may be modified */
        int        *iperm_r, /* in - inverse of perm_r */
        int        *iperm_c, /* in - used to find diagonal of Pc*A*Pc' */
        int        *pivrow,  /* out */
        GlobalLU_t<Z<MON_PREMIER,T> > *Glu      /* modified - global LU data structures */
       )
{
/*
 * Purpose
 * =======
 *   Performs the numerical pivoting on the current column of L,
 *   and the CDIV operation.
 *
 *   Pivot policy:
 *   (1) Compute thresh = u * max_(i>=j) abs(A_ij);
 *   (2) IF user specifies pivot row k and abs(A_kj) >= thresh THEN
 *           pivot row = k;
 *       ELSE IF abs(A_jj) >= thresh THEN
 *           pivot row = j;
 *       ELSE
 *           pivot row = m;
 * 
 *   Note: If you absolutely want to use a given pivot order, then set u=0.0.
 *
 *   Return value: 0      success;
 *                 i > 0  U(i,i) is exactly zero.
 *
 */
    int          fsupc;	    /* first column in the supernode */
    int          nsupc;	    /* no of columns in the supernode */
    int          nsupr;     /* no of rows in the supernode */
    int          lptr;	    /* points to the starting subscript of the supernode */
    int          pivptr, old_pivptr, diag, diagind;
    Z<MON_PREMIER,T>       pivmax, rtemp, thresh;
    Z<MON_PREMIER,T>       temp;
    Z<MON_PREMIER,T>       *lu_sup_ptr; 
    Z<MON_PREMIER,T>       *lu_col_ptr;
    int          *lsub_ptr;
    int          isub, icol, k, itemp;
    int          *lsub, *xlsub;
    Z<MON_PREMIER,T>       *lusup;
    int          *xlusup;
    extern SuperLUStat_t SuperLUStat;
    flops_t  *ops = SuperLUStat.ops;

    /* Initialize pointers */
    lsub       = Glu->lsub;
    xlsub      = Glu->xlsub;
    lusup      = Glu->lusup;
    xlusup     = Glu->xlusup;
    fsupc      = (Glu->xsup)[(Glu->supno)[jcol]];
    nsupc      = jcol - fsupc;	        /* excluding jcol; nsupc >= 0 */
    lptr       = xlsub[fsupc];
    nsupr      = xlsub[fsupc+1] - lptr;
    lu_sup_ptr = &lusup[xlusup[fsupc]];	/* start of the current supernode */
    lu_col_ptr = &lusup[xlusup[jcol]];	/* start of jcol in the supernode */
    lsub_ptr   = &lsub[lptr];	/* start of row indices of the supernode */

#ifdef DEBUG
if ( jcol == MIN_COL ) {
    printf("Before cdiv: col %d\n", jcol);
    for (k = nsupc; k < nsupr; k++) 
	printf("  lu[%d] %f\n", lsub_ptr[k], lu_col_ptr[k]);
}
#endif
    
    /* Determine the largest abs numerical value for partial pivoting;
       Also search for user-let pivot, and diagonal element. */
    if ( *usepr ) *pivrow = iperm_r[jcol];
    diagind = iperm_c[jcol];
    pivmax =(Z<MON_PREMIER,T>)0;
    pivptr = nsupc;
    diag = EMPTY;
    old_pivptr = nsupc;
    for (isub = nsupc; isub < nsupr; ++isub) {
	rtemp = lu_col_ptr[isub];
	
	if ( rtemp != 0 ) {
	    pivmax = rtemp;
	    pivptr = isub;
	    break;
	}
	if ( *usepr && lsub_ptr[isub] == *pivrow ) old_pivptr = isub;
	if ( lsub_ptr[isub] == diagind ) diag = isub;
    }
    //cout<<"comparaison dans dpivvotL "<<pivmax<<endl;
    /* Test for singularity */
    if ( pivmax ==0) {
	*pivrow = lsub_ptr[nsupc];
	//cout<<"degenere et pivrow "<<*pivrow<<endl;
	perm_r[*pivrow] = jcol;
	*usepr = 0;
	return (jcol+1);
    }

    thresh = u * pivmax;
    
    /* Choose appropriate pivotal element by our policy. */
    if ( *usepr ) {
      //fabs!!!!  
      rtemp = lu_col_ptr[old_pivptr];
      //fabs!!!!!!!!!!!!!!!!! faire gaffe en generic 
      if ( rtemp != (Z<MON_PREMIER,T>)0 )
	    pivptr = old_pivptr;
	else
	    *usepr = 0;
    }
    if ( *usepr == 0 ) {
	/* Use diagonal pivot? */
	if ( diag >= 0 ) { /* diagonal exists */
	    rtemp = lu_col_ptr[diag];
	    if ( rtemp != (Z<MON_PREMIER,T>)0  ) pivptr = diag;
        }
	*pivrow = lsub_ptr[pivptr];
    }
    
    /* Record pivot row */
    //    cout<<"piv row "<<*pivrow<<" jcol "<<jcol<<endl;
    perm_r[*pivrow] = jcol;
    
    /* Interchange row subscripts */
    if ( pivptr != nsupc ) {
      itemp = lsub_ptr[pivptr];
      lsub_ptr[pivptr] = lsub_ptr[nsupc];
      lsub_ptr[nsupc] = itemp;

	/* Interchange numerical values as well, for the whole snode, such 
	 * that L is indexed the same way as A.
 	 */
	for (icol = 0; icol <= nsupc; icol++) {
	    itemp = pivptr + icol * nsupr;
	    temp = lu_sup_ptr[itemp];
	    lu_sup_ptr[itemp] = lu_sup_ptr[nsupc + icol*nsupr];
	    lu_sup_ptr[nsupc + icol*nsupr] = temp;
	}
    } /* if */

    /* cdiv operation */
    ops[FACT] += nsupr - nsupc;
    temp = (Z<MON_PREMIER,T>)1 / lu_col_ptr[nsupc];
    for (k = nsupc+1; k < nsupr; k++) 
	lu_col_ptr[k] *= temp;

    return 0;
}
//int Iszero(const Z<MON_PREMIER,int> & c)
//{return c==(int)0;}

template<typename T>
int Iszero(const Z<MON_PREMIER,T> & c)
{return c==(T)0;}


__END_NAMESPACE_SYNAPS



