/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
    2000/03/07 Add solve(.Compagnon) 
$Id: EigenHidden.H,v 1.1 2005/07/11 10:03:50 jppavone Exp $
**********************************************************************/
#ifndef SYNAPS_SOLVE_EIGEN_HIDDEN_H
#define SYNAPS_SOLVE_EIGEN_HIDDEN_H

#include <synaps/init.h>
#include <synaps/linalg/MATRIXPOL.h>
#include <synaps/linalg/Eigen.h>


__BEGIN_NAMESPACE_SYNAPS

//----------------------------------------------------------------------
struct Compagnon {};
//--------------------------------------------------------------------
/** 
    Compute the real solutions of $M(x).v=0$ by transforming this problem
    into an eigenvector problem $(A-x B)\, w =0$. It is assumed
    that  the pencil has only simple real eigenvectors. The case of 
    multiple roots is not handled for the moment. This case can be 
    detected by checking if all the elements of the first line are $1$
    or not. 
    \begin{xxx}
    E = solve<MatC>(Mx, Compagnon());
    \end{xxx}
    where @MatC@ is a type of matrix with complex coefficients.
    \index{solve, Compagnon}
**/
template<class Mat, class MatPol>
Mat solve(const MatPol & MP, Compagnon method) 
{ 
  typedef typename Mat::size_type size_type;
  typedef typename Mat::value_type value_type;
  size_type m=MP.nbcol(),d = MATRIXPOL::degree(MP);
  size_type N= std::max((size_type)0,d*m);
  Mat A = Mat(N,N,(value_type) 0);

  for(size_type i=0; i<N-m; i++) A(i,i+m)=-1;
  for(size_type k=0; k<d; k++)
    for(size_type i=0; i<m; i++) 
      for(size_type j=0; j<m; j++) 
 	A((d-1)*m+i,k*m+j)=MP(i,j)[k];

  Mat B = Mat(N,N,(value_type) 0);
  for(size_type i=0; i<N-m; i++) B(i,i)=-1;

  for(size_type i=0; i<m; i++) 
    for(size_type j=0; j<m; j++) {
      B((d-1)*m+i,(d-1)*m+j)= -MP(i,j)[d];
    }

  Mat E = Eigenvct(A, B, Real());
  for(size_type j=0; j<E.nbcol(); j++)
    if(E(0,j)!=0)
      for(size_type i=E.nbrow(); i>0; i--) E(i-1,j)/=E(0,j);

  return E;
}

__END_NAMESPACE_SYNAPS

#endif // SYNAPS_SOLVE_EIGEN_HIDDEN_H

