/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: space.h,v 1.1 2005/07/11 10:03:56 jppavone Exp $
 ********************************************************************/
#ifndef SYNAPS_SOLVE_SBDSLV_SBDRLSPC_HPP
#define SYNAPS_SOLVE_SBDSLV_SBDRLSPC_HPP
//--------------------------------------------------------------------
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace sbdslv
{
  namespace sbdrl
  {
    template< class solver >
    struct space_rule 
    {
      static bool process( solver& slv, pstack_t& prjs )
      {
      /* arrondi au plus proche autorisé */
	numerics::rounding<creal_t> rnd(numerics::rnd_nr());
      
	unsigned mxp,  mxv = -1;
      creal_t *pmn, *pmx, mxK = 0.0;
      
      for ( unsigned v = 0; v < slv->nvars(); v ++ ) /* pour chaque variable */
	{
	  creal_t K;
	  for ( unsigned p = 0; p < prj->nproj(v); p ++ ) /* pour toutes ses projections */
	    {
	      pmn = prjs[v].mn(p), pmx = prjs[v].mx(p), psz = prjs[v].sz(p);
	      /* on calcule l'écart moyen entre les bornes hautes et basses */
	      for ( k = 0, K = 0.0; k < psz; K += pmx[k]-pmn[k], k ++ ), K /= psz;
	      assert(K>=0.0); /* si ce n'est pas le cas il y a un ENORME bug quelque part dans le solver */
	      /* on selectionne la paire (variable,projection) avec le plus grand écart */
	      if ( K > mxK ) mxK = K, mxv = v, mxp = p;
	    };
	};
      
      /* pile de projections vide, impossible de decider    */
      if ( mxv == -1 )         parametric_rule::process<solver>(slv,prjs);
      /* precision spatiale atteinte, on accepte le domaine */
      if ( Kmx < slv->m_seps ) slv->accept(), return true;

      /* calcul de $D_p(v)$ et de sa primitive */
      pmn = prjs[v].mn(p), pmx = prjs[v].mx(p), psz = prjs[v].sz(p);
      creal_t D[ psz ];
      for ( unsigned i = 0; i < psz; D[i] = pmx[i]-pmn[i], i ++ );
      creal_t prm[ psz + 1 ]; 
      prm[0] = 0.0, prm[1] = tmp[0]/psz;
      /* prm(t) = int(D(v),v=0..t) */
      for ( unsigned i = 2; i < psz+1; prm[i] = tmp[i-1]/psz+prm[i-1], i ++ );
      
      /* calcul de la position de la subdivision                        */
      /* prm(t) := int(D(v),v=0..t)-int(D(v),v=0..1)/2 = prm(t)-mxK/2.0 */
      mxK /= 2.0, for ( unsigned i = 0; i < psz+1; prm[i] -= mxK, i++ ); 
      /* pseudo-hull devrait être remplacer par rockwood, ou mieux la methode devrait être paramétrée */
      /* par un "solveur de premiere racine"                                                          */
      mxK  = sbdslv::lphull(prm,sz,1,Type::false_t());
      /* si la subdivision semble ne rien apporter, on fait appel à la règle paramétrique */
      if ( mxK < 0.01 || mxK > 0.99 ) return parametric_rule::process<solver>(slv,prjs);
      /* sinon on subdivise le domaine comme prévu                                        */
      return slv->split(mxv,mxK);
      };
    };
  };
};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
