/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: nstack.hpp,v 1.1 2005/07/11 10:03:56 jppavone Exp $
 ********************************************************************/
#ifndef SYNAPS_SOLVE_MEMORY_NSTACK_HPP
#define SYNAPS_SOLVE_MEMORY_NSTACK_HPP
//--------------------------------------------------------------------
#include <iostream>
#include <stdlib.h>
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace memory
{
  /* constructeur T() necessaire */
  template< typename T > 
  struct nstack 
  {
    typedef unsigned int sz_t;
    sz_t  m_n, m_sz;
    T  *  m_frees, * m_top;
    inline void * _alloc_( sz_t sz ) { return malloc( sz ); };
    inline T * alloc() {  
      if ( m_frees ) { T * tmp = m_frees; m_frees = prev(m_frees); prev(tmp) = 0; return tmp; };
      T* tmp = (T*)_alloc_( sizeof(T)*m_sz );  
      for ( sz_t i = 0; i < m_n; i ++ ) new((void*)(tmp+i)) T; /* initialisation */
      prev(tmp) = 0; 
      return tmp;
    };
    inline T*& prev( T * chnk ) { return *(T**)(chnk+m_n); };
  public:
    inline void pop() {  T * tmp = prev(m_top); prev(m_top) = m_frees; m_frees = m_top; m_top = tmp; };
    inline T * top()  {  return m_top; };
    inline void push() { T * tmp = alloc(); prev(tmp) = m_top; m_top = tmp; };

    nstack( unsigned n = 1 )
    {
      if ( sizeof(T) > sizeof(T*) ) m_sz = n+1; else m_sz = n+2;
      m_n = n;
      m_frees = 0;
      m_top   = 0;
    };
    
    inline unsigned deep() 
    {
      unsigned rslt = 0;
      T * tmp = top();
      if ( ! tmp ) return 0;
      while ( prev(tmp) ) { rslt ++; tmp = prev(tmp); };
      return rslt;
    };
    
    inline void popall() { while ( m_top ) pop(); };

    ~nstack()
    {
      popall();
      while ( m_frees ) { 
	T * tmp = m_frees; m_frees = prev(m_frees); 
	for ( sz_t i = 0; i < m_n; i ++ ) (tmp+i)->~T();
	free( tmp );
      };
    };
  };
};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
