/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: pstack.h,v 1.1 2005/07/11 10:03:57 jppavone Exp $
 ********************************************************************/
#ifndef SYNAPS_SOLVE_SBDSLV_PSTACK_HPP
#define SYNAPS_SOLVE_SBDSLV_PSTACK_HPP
//--------------------------------------------------------------------
#include <synaps/init.h>
#include "vstack.h"
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace sbdslv
{
  template< class creal_t >
  struct pstack
  {
    typedef unsigned sz_t;
    typedef vstack<creal_t> vstack_t;
    unsigned   m_nvr;
    vstack_t * m_prj;
    inline
    void reset(){ for ( sz_t i = 0; i < m_nvr; m_prj[i].reset(), i ++ );  };
    inline
    void push( sz_t v, creal_t *& pmn, creal_t*& pmx, sz_t sz ) { m_prj[v].push(pmn,pmx,sz);};
    inline
    void add( sz_t v, creal_t * const pmn, creal_t * const pmx, sz_t sz ) { m_prj[v].add(pmn,pmx,sz); };
    inline
    void read( sz_t v, sz_t i, creal_t*& pmn, creal_t *& pmx, unsigned& sz ) { m_prj[v].read(i,pmn,pmx,sz); };
    inline
    sz_t nbp ( sz_t v ) const { return m_prj[v].nbp(); };
    vstack_t& projections( sz_t v ) { return m_prj[v]; };
    bool thickness( const creal_t& eps  ) { 
      for ( unsigned i = 0; i < m_nvr; i ++ ) 
	if ( ! m_prj[i].thickness( eps ) ) return false;
      return true;
    };

    pstack( sz_t nvr ) { m_nvr = nvr; m_prj = new vstack_t[nvr]; };
    ~pstack() { delete[] m_prj; };
  };
};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
