#ifndef SYNAPS_SISL_BSPLINECURVE_H
#define SYNAPS_SISL_BSPLINECURVE_H

#include <synaps/shape/wrappers.h>
#include <synaps/shape/curves/SISLCurveBase.h>


__BEGIN_NAMESPACE_SYNAPS

struct SISLBSplineCurve;
template<> struct interfaceof<SISLBSplineCurve> { typedef IBSplineCurve T; };

struct SISLBSplineCurve : SISLCurveBase, wrapper<SISLBSplineCurve,IBSplineCurve>
{
  typedef  IBSplineCurve base_t;
  typedef       fxv<double,3>         cp;           
  typedef       double                cp_value_type; 
  typedef       cp            *       cp_iterator;
  typedef const cp            * const_cp_iterator;
  typedef       double                kn_value_type;
  typedef       kn_value_type *       kn_iterator;
  typedef const kn_value_type * const_kn_iterator;
  
  // iterator access to knots values
  inline       kn_iterator kn_begin()       { return       (kn_iterator)(sisl_data()->et);         };
  inline const_kn_iterator kn_begin() const { return (const_kn_iterator)(sisl_data()->et);         };  
  inline       kn_iterator kn_end  ()       { return       (kn_iterator)(sisl_data()->et) + nbk(); };
  inline const_kn_iterator kn_end  () const { return (const_kn_iterator)(sisl_data()->et) + nbk(); };
  
  // iterator access to control points
  inline       cp_iterator cp_begin()       { return       (cp_iterator)(sisl_data()->ecoef);      };
  inline const_cp_iterator cp_begin() const { return (const_cp_iterator)(sisl_data()->ecoef);      };
  inline       cp_iterator cp_end  ()       { return       (cp_iterator)(sisl_data()->ecoef)+nbp();};
  inline const_cp_iterator cp_end  () const { return (const_cp_iterator)(sisl_data()->ecoef)+nbp();};
  
  // random access to control points
  inline const cp & operator[](int i) const { return cp_begin()[i]; };
  inline       cp & operator[](int i)       { return cp_begin()[i]; };
  
  inline const kn_value_type & knot( int i ) const { return kn_begin()[i]; };
  inline kn_value_type & knot( int i ) { return kn_begin()[i]; };
};

inline void setDefinition( SISLBSplineCurve & c, int nbpt, int order, const double * knots, 
			   const fxv<double,3> * controls )
{
  if ( c.sisl_data_ ) freeCurve(c.sisl_data_);
  ((SISLBSplineCurve*)&c)->sisl_data_ = newCurve(nbpt,order,(double*)knots,(double*)controls,1,3,1);
};

inline void readGo ( SISLBSplineCurve & c, std::istream & in )
{
  in >> c;
  assert(c.dim()==3 && c.type() == 1);
};

template<class Point>
inline void eval( Point& p, const SISLBSplineCurve& c, const typename Point::value_type& t )
{
  eval( p, (const SISLCurveBase&)c,t);
};

template<class C>
inline void range( C& tmin, C& tmax, const SISLBSplineCurve & c )
{
  domain(tmin,tmax,(const SISLCurveBase&)c);
};

inline double  length(  const IBSplineCurve& c, const SISLBSplineCurve& a, const double & eps  )
{
  std::cout << "sisl length\n";
  return length((const SISLCurveBase&)a, eps );
};

__END_NAMESPACE_SYNAPS

#endif
