#ifndef SYNAPS_MPOL_IMPLICITSURFACE_H
#define SYNAPS_MPOL_IMPLICITSURFACE_H
#include <synaps/shape/wrappers.h>
#include <synaps/mpol/MPol.h>
#include <synaps/arithm/Interval.h>
#include <sstream>

__BEGIN_NAMESPACE_SYNAPS

template<class C> struct MPolImplicitSurface;

template<class C> struct interfaceof< MPolImplicitSurface<C> > { typedef IAlgebraicSurface T; };


template<class C> 
struct MPolImplicitSurface : wrapper< MPolImplicitSurface<C>, IAlgebraicSurface >
{
  typedef IAlgebraicSurface      base_t;
  typedef MPolImplicitSurface<C> self_t;
  typedef self_t                 rep_t;
  typedef C                      coeff_t;
  typedef MPol<C>                polynomial_t;
  Interval<C> m_dmn[3];
  polynomial_t data;
  void setDomain(const Interval<C>& x, const Interval<C>& y, const Interval<C>& z)
  {
    m_dmn[0]=x;m_dmn[1]=y; m_dmn[2]=z;
  }
  MPolImplicitSurface() {}
  MPolImplicitSurface(const char* eq, char* var) : data(eq,var) 
  {
    this->setDomain(Interval<C>(0,1),Interval<C>(0,1),Interval<C>(0,1));
  }

};

template<class C, class S> inline
void setDomain( MPolImplicitSurface<C> & s, S dmns[6] )
{
  using let::assign;
  assign(s.m_dmn[0].lower(),dmns[0]);
  assign(s.m_dmn[0].upper(),dmns[1]);
  assign(s.m_dmn[1].lower(),dmns[2]);
  assign(s.m_dmn[1].upper(),dmns[3]);
  assign(s.m_dmn[2].lower(),dmns[4]);
  assign(s.m_dmn[2].upper(),dmns[5]);
};

template<class C> inline
void setEquation( MPolImplicitSurface<C> & s, const char * eq )
{ s.data = MPol<C>(eq,"x y z"); };

template<class C, class S,class  Point3> inline
void eval( const MPolImplicitSurface<C> & s, 
	   S & value, const Point3& p )
{
  using let::assign;
  assign(value,s.data(p));
};

template<class C> inline
std::string getEquation( const MPolImplicitSurface<C> & s )
{
  std::stringstream tmp;
  MPOLDST::print(tmp,s.data,Variables("x y z"));
  return tmp.str();
};

/**
   The \c operator \c << .\n
   This operator allows us to display a rational curve.
**/
template<class C>
std::ostream & operator<< (std::ostream & os, 
			   const MPolImplicitSurface<C> & s)
{
  os<<"ImplicitSurface("<<s.data;
  for (unsigned i=0; i<3;i++) os<<","<<s.m_dmn[i];
  os <<")";
  return os;
}

__END_NAMESPACE_SYNAPS

#endif
