#ifndef SYNAPS_SHAPE_MPOLDSE_RATIONAL_SURFACE_H
#define SYNAPS_SHAPE_MPOLDSE_RATIONAL_SURFACE_H

#include <synaps/shape/wrappers.h>
#include <synaps/mpol/MPolDse.h>

__BEGIN_NAMESPACE_SYNAPS


template<class C, class R> struct RationalSurface;
template<class C, class R> struct interfaceof< RationalSurface<C,R> > 
{ 
  typedef IRationalSurface T; 
};
template<class C, class R = MPolDse<C> >
struct RationalSurface : wrapper< RationalSurface<C,R> ,IRationalSurface >
{
  typedef IRationalSurface     base_t;
  typedef RationalSurface<C,R> self_t;
  typedef self_t               rep_t;
  typedef C                    coeff_t;
  typedef R                    polynomial_t;

  C umin_,umax_,vmin_,vmax_;
  polynomial_t m_coords[4];
  polynomial_t& operator[]( int i ) { return m_coords[i]; };
  const polynomial_t & operator[]( int i ) const { return m_coords[i]; };
  RationalSurface() 
  {
    umin_ = 0; umax_ = 1;
    vmin_ = 0; vmax_ = 1;
  };
  RationalSurface(const char* w, const char * x, const char * y, const char* z,
		  const char * varnames )
    : umin_(0),umax_(1),vmin_(0),vmax_(1)
  {
    this->setEquations(w,x,y,z,varnames);
  }
};


template<class C, class T, class R> inline 
bool range(  C & umin,  C & umax, C & vmin, C & vmax, const RationalSurface<T,R> & s )
{
  using let::assign;
  assign(umin,s.umin_);
  assign(umax,s.umax_);
  assign(vmin,s.vmin_);
  assign(vmax,s.vmax_);
};

template<class C,class T,class R> inline
bool setRange(  RationalSurface<T,R> & s, const C & umin, const C & umax, const C & vmin, const C & vmax  )
{
  using let::assign;
  assign(s.umin_,umin);
  assign(s.umax_,umax);
  assign(s.vmin_,vmin);
  assign(s.vmax_,vmax);
};


template<class C, class R, class Point, class Prm >
void eval( Point& value,  const RationalSurface<C,R>& srf, const Prm& u, const Prm& v  )
{
  Prm tmp[] = { u, v };
  
  typename Point::value_type w;
  eval(w,srf[0],tmp);
  
  for ( int i = 1; i <= 3;  i++  )
    {
      eval( value[i-1], srf[i], tmp );
      value[i-1] /= w;
    };
};

template<class C,class R> inline
void setEquations( 
		  RationalSurface<C,R> & c, 
		  const char * w, const char * x, const char * y, const char * z,
		  const char * varnames )
{

  using let::assign;
  Variables vars(varnames);
  assign(c.m_coords[0],MPol<C>(w,vars));
  assign(c.m_coords[1],MPol<C>(x,vars));
  assign(c.m_coords[2],MPol<C>(y,vars));
  assign(c.m_coords[3],MPol<C>(z,vars));
  MPolDseSystem(c.m_coords,c.m_coords+4);
  //  for ( int i = 0; i < 4; i ++ ) std::cout << c.m_coords[i] << std::endl;
  /*
  for ( int i = 0; i < 4; i ++ )
    {
      std::cout << std::endl;
      for ( int j = 0; j < c.m_coords[i].rep().nvr(); j ++ )
      std::cout << c.m_coords[i].rep().szs()[j] << " ";
    std::cout << std::endl;
    };
  */
};

/**
   The \c operator \c << .\n
   This operator allows us to display a surface.
**/
template<class C,class R>
std::ostream & operator<< (std::ostream & os, 
			   const RationalSurface<C,R> & s)
{
  os<<"RationalSurface("<<s[0];
  for (unsigned i=1; i<4;++i)os<<","<<s[i];
  os <<", ["<<s.umin_<<","<<s.umax_<<"]x["<<s.vmin_<<","<<s.vmax_<<"]";
  os <<")";
  return os;
}

		     
__END_NAMESPACE_SYNAPS

#include <synaps/shape/wrappers/WRationalSurface.h>

#endif
