/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef synaps_shape_surface_axel_h
#define synaps_shape_surface_axel_h
//===================================================================
#include <synaps/base/io/axel.h>
#include <synaps/shape/base/fxv.h>
#include <synaps/shape/interfaces/IGLGood.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//===================================================================
template<class C,class R> struct RationalSurface;

namespace axel 
{

  template<class C> inline
  ostream& operator<<(ostream& os, const RationalSurface<C>& s)
  {
    os<<"<surface type=\"rational\" color=\""
      <<os.color.r<<" "<<os.color.g<<" "<<os.color.b<<"\">\n";
    os <<"  <domain>"<<s.umin_<<" "<<s.umax_<<" "<<s.vmin_<<" "<<s.vmax_<<"</domain>\n";
    for(unsigned i=1;i<4;i++)
      os <<"  <polynomial>"<<s[i]<<"</polynomial>\n";
    os <<"  <polynomial>"<<s[0]<<"</polynomial>\n";
    os<<"</surface>\n";
    return os;
  }

  ostream& operator<<(ostream& os, const IGLGood& s)
  {
    typedef fxv<double,3> point;
    point * P = (point*)(s.vertices);
    os <<"<curve type=\"mesh\">\n<vect>\n";
    os <<"VECT\n";
    os <<s.nbi/2 << " " << s.nbv <<" "<<s.nbi/2<<"\n";
    for(unsigned i =0;i<s.nbi/2;i++)
      os<<"2 "; os<<"\n";
    for(unsigned i =0;i<s.nbi/2;i++)
      os<<"1 "; os<<"\n";

    for(unsigned i=0;i<s.nbi ;i+=2)
      { 
	os << P[i][0] << " " << P[i][1] << " " << P[i][2]<< " ";
	os << P[i+1][0] << " " << P[i+1][1] << " " << P[i+1][2];
	os << "\n";
      }
    for(unsigned i =0;i<s.nbi/2;i++)
      os<<os.color.r/255.<<" "<<os.color.g/255.<<" "<<os.color.b/255.<<" "<<"1\n";
    os <<"</vect>\n</curve>\n";
    return os;
  }
}
//===================================================================
__END_NAMESPACE_SYNAPS
#endif //

