#ifndef SYNAPS_SHAPE_WIMPLICITSURFACE_H
#define SYNAPS_SHAPE_WIMPLICITSURFACE_H

#include <synaps/shape/wrappers/WShape.h>
#include <synaps/shape/interfaces/IImplicitSurface.h>
# include <synaps/arithm/gmp.h>
# include <synaps/topology/mesh.h>

__BEGIN_NAMESPACE_SYNAPS

namespace  implicit_surface {} ;

template<class C, class R> inline
void setDomain( const wrapper<R,IImplicitSurface>& s, const C coords[6] )
{ 
  using namespace implicit_surface;
  setDomain(s.rep(),coords);
};

template<class C, class R> inline 
void getDomain( const wrapper<R,IImplicitSurface> & s, C coords[6]  )
{
  using namespace implicit_surface;
  getDomain(s.rep(),coords);
};

template<class C, class R> inline
void eval( const wrapper<R,IImplicitSurface>& s, C * img, const C* prm, int n ) 
{  
  using namespace implicit_surface;
  eval( s.rep(), *img, ((const fxv<C,3> *)prm), n ); 
};

template<class C, class R> inline
void eval( const wrapper<R,IImplicitSurface> & s, C & img, const fxv<C,3> &  p )
{
  using namespace implicit_surface;
  eval(s.rep(),img,p);
};

namespace implicit_surface 
{
  template<class C, class Point3, class Surface> inline
  void eval( const Surface & s, C * values, const Point3 * points, int n ) 
  { for ( C * sv = values; sv != values + n; sv ++, points++ ) eval(s,*sv,points); };
};

template<class R>
struct wrapper<R,IImplicitSurface> :  wrapper<R,IImplicitSurface::base_t> 
{
  int grid[3];
  wrapper<R,IImplicitSurface>()              { grid[0] = 50; grid[1] = 50; grid[2] = 50; };
  void  setDomain( const double coords[6] )  { ::setDomain(coords,this->rep()); };
  void  getDomain( double coords[6] )       const  { ::getDomain(coords,this->rep()); };
  void  eval( double * r, const double * p, int n ) const { ::eval(r,p,n,this->rep()); };
};

namespace AlgebraicSurface
{
  template<class Surface>
  IGLGood * mesh( const Surface  & s )
  {

    std::string eq = s.getEquation();
    std::cout << " eq =  " << eq << std::endl;
    scene<MPol<double>, linalg::rep3d<double> > scn;
   
    MPol<double> p( eq.c_str() , Variables("x y z")) ; // glucks ?!
    poly = p ;
    mesh(scn, p, 
	 s.m_dmn[0].lower(),
	 s.m_dmn[0].upper(),
	 s.m_dmn[1].lower(),
	 s.m_dmn[1].upper(),
	 s.m_dmn[2].lower(),
	 s.m_dmn[2].upper(), 
	 pow(0.5, 5), pow(0.5, 7) );
    
    
    int nVertices = scn.listVertices.size() ;
    int nFaces = scn.listTriangles.size() ;
    
    IGLGood * result = new IGLGood( IGLGood::E_TRIANGLE, nVertices, nFaces * 3, true );
    
    fxv<double,3> * vertices = (fxv<double,3>*)(result->vertices);
    fxv<double,3> * normals  = (fxv<double,3>*)(result->normals);
    
    for ( int i = 0 ; i < nVertices ; i++ ) {
      vertices[i][0] = scn.x+scn.u*scn.listVertices[i]->point[0];
      vertices[i][1] = scn.y+scn.v*scn.listVertices[i]->point[1];
      vertices[i][2] = scn.z+scn.w*scn.listVertices[i]->point[2];
    }
    
    MPol<double> dx = diff(p, "x") ;
    MPol<double> dy = diff(p, "y") ;
    MPol<double> dz = diff(p, "z") ;
    
    for ( int i = 0 ; i < nVertices ; i++ ) {
      double dxv = dx(vertices[i]) ;
      double dyv = dy(vertices[i]) ;
      double dzv = dz(vertices[i]) ; 
      double div = sqrt(dxv*dxv + dyv*dyv + dzv*dzv) ;
      normals[i][0] = dxv/div;
      normals[i][1] = dyv/div;
      normals[i][2] = dzv/div;
    }
    
    for ( int i = 0; i < nFaces; i ++ )
      {
	result->indexes[3*i]   = scn.listTriangles[i].a->index;
	result->indexes[3*i+1] = scn.listTriangles[i].b->index;
	result->indexes[3*i+2] = scn.listTriangles[i].c->index;
      };
    
    return result;
    
    /*
      if  (!scn.SingCells.empty() ) {
      int size = (int) scn.SingCells.size();
      
      for ( int i= 0 ; i < size-1 ; i = i + 2  ) {
      double x0 = scn.x+scn.u*scn.SingCells[i][0] ;
      double y0 = scn.y+scn.v*scn.SingCells[i][1] ;
      double z0 = scn.z+scn.w*scn.SingCells[i][2] ;
      
      double x1 = scn.x+scn.u*scn.SingCells[i+1][0] ;
      double y1 = scn.y+scn.v*scn.SingCells[i+1][1] ;
      double z1 = scn.z+scn.w*scn.SingCells[i+1][2] ;
      
      double x = (x0+x1)/2 ;
      double y = (y0+y1)/2 ;
      double z = (z0+z1)/2 ;
      double radius = z1-z ;
      
      QSingularity * sing = new QSingularity(x, y, z, radius, s) ;
      s->push_singularity(sing) ;
      }
      }
    */
  };
};

template<class R>
IGLGood *  mesh( const wrapper<R,IAlgebraicSurface> & s )
{
  using namespace AlgebraicSurface;
  return mesh(s.rep());
};

template<class R>
void setEquation( wrapper<R,IAlgebraicSurface>& s, const char * eq )
{
  using namespace AlgebraicSurface;
  setEquation(s.rep(),eq);
};

template<class R>
std::string  getEquation( const wrapper<R,IAlgebraicSurface> & s )
{
  using namespace AlgebraicSurface;
  return getEquation(s.rep());
};
template<class R>
struct wrapper<R,IAlgebraicSurface> : wrapper<R,IAlgebraicSurface::base_t>
{
  void setEquation( const char * eq ) { ::setEquation(this->rep(),eq); };
  std::string getEquation() const { return ::getEquation(this->rep()); };
};

__END_NAMESPACE_SYNAPS

#endif
