#ifndef SYNAPS_SHAPE_WPARAMETRICCURVE_H
#define SYNAPS_SHAPE_WPARAMETRICCURVE_H

#include <synaps/shape/interfaces/IShape.h>
#include <synaps/shape/interfaces/IParametricCurve.h>
#include <synaps/shape/functions/FParametric.h>
#include <synaps/shape/wrappers/wrapper.h>
#include <synaps/shape/base/fxv.h>

__BEGIN_NAMESPACE_SYNAPS


namespace ParametricCurve
{
  template<class Scalar, class R> inline
    void range( Scalar & tmin, Scalar & tmax, const R & c ) ;
  
  template<class Point, class R> inline
    void evald( Point& dp, const R & c, const typename Point::value_type & t );
  
  template<class Point, class R> inline
    void eval( Point & p, Point & dp, const wrapper<R,IParametricCurve > & c,  const typename Point::value_type & t );
  
  
  template<class R, class Point> inline
    void eval( Point * smp, const R& c, const typename Point::value_type * t, int n );
  
  template<class Scalar> 
    void tsample( Scalar * t, const Scalar&  a_, const Scalar & b, int n );

  
  template<class R, class Scalar> inline
    void tsample( Scalar * t, const R & c, int n );
  
  template<class Point, class Scalar, class R> inline
    void sample( Point * p, Scalar * s, const R & c, int n );
  
};
  
namespace ParametricCurve { };

template<class Scalar, class R> inline
void range( Scalar & tmin, Scalar & tmax, const wrapper<R,IParametricCurve> & c )
{
  using namespace ParametricCurve;
  range(tmin,tmax,c.rep());
};

template<class Point,class R> 
void eval( Point & p, const wrapper<R,IParametricCurve> & c, const typename Point::value_type & t )
{
  using namespace ParametricCurve;
  eval(p,c.rep(),t);
};


template<class Point, class R> inline
void evald( Point& dp, const wrapper<R,IParametricCurve> & c, const typename Point::value_type & t )
{
  using namespace ParametricCurve;
  evald(dp,c.rep(),t);
};

template<class Point, class R> inline
void eval( Point & p, Point & dp, const wrapper<R,IParametricCurve > & c,  const typename Point::value_type & t )
{
  using namespace ParametricCurve;
  eval(p,dp,c.rep(),t);
};

template<class R, class Point> inline
void eval( Point * smp, const wrapper<R,IParametricCurve>& c, const typename Point::value_type * t, int n )
{ 
  using namespace ParametricCurve; 
  eval(smp,c.rep(),t,n); 
};

template<class R, class Scalar> inline
void tsample( Scalar * t, const wrapper<R,IParametricCurve> & c, int n )
{
  using namespace ParametricCurve;
  tsample(t, c.rep(), n );
};

template<class Point, class Scalar, class R> inline
void sample( Point * p, Scalar * s, const wrapper<R,IParametricCurve> & c, int n )
{
  using namespace ParametricCurve;
  sample(p,s,c.rep(),n);
};

template<class C,class R>
IParametricCurve * subcurve( const wrapper<R,IParametricCurve> & c, const C& tmin, const C& tmax )
{
  using namespace ParametricCurve;
  return subcurve(c.rep(),tmin,tmax);
};



template<class R, class C> inline
void split( IParametricCurve *& left,
	    IParametricCurve *& right,
	    const wrapper<R,IParametricCurve> & c,
	    const C & t )
{
  using namespace ParametricCurve;
  split(left,right,c.rep(),t);
};

template<class R, class C> inline
C length( const wrapper<R,IParametricCurve> & c, const C & eps )
{
  using namespace ParametricCurve;
  return length(c.rep(),eps);
};

template<class R> inline
IGLGood * mesh( const wrapper<R,IParametricCurve>& c )
{
  return mesh(&c);  
};


template<class R>
struct wrapper<R,IParametricCurve> :  wrapper<R,IParametricCurve::base_t>
{
  void range( double & tmin, double & tmax ) const  
  { 
    ::range(tmin,tmax,this->rep()); 
  };

  void eval ( double * smp, double t ) const 
  { 
    ::eval(*((fxv<double,3>*)smp),this->rep(),t); 
  };
  
  void eval ( double * smp, const double * tvals, int n ) const
  { 
    ::eval( (fxv<double,3>*)smp,this->rep(),tvals,n); 
  };

  void sample( double * smp, double * tvals, int n ) const
  { 
    ::sample( (fxv<double,3>*)smp,tvals,this->rep(),n);  
  };
  
  void split( IParametricCurve *& a, IParametricCurve *& b, double t ) const
  { 
    ::split(a,b,this->rep(),t);
  }; 
  
  double length( double eps ) const 
  {
    return ::length(this->rep(),eps); 
  };
};


namespace ParametricCurve
{
  template<class Scalar, class R> inline
  void range( Scalar & tmin, Scalar & tmax, const R & c ) 
  { 
    tmin = Scalar(0); 
    tmax = Scalar(1); 
  };
  
  template<class Point, class R> inline
  void evald( Point& dp, const R & c, const typename Point::value_type & t )
  {
    typedef typename Point::value_type C;
    Point p;
    C a,b;
    ::range(a,b,c);
    C h = (b-a)/C(1000);
    a = t - h;
    b = t + h;
    ::eval(p,c,a);
    ::eval(dp,c,b);
    for ( int i = 0; i < Point::_dimension_ ; i ++ )
      dp[i] = (dp[i]- p[i])/(C(2)*h);
  };
  
  template<class Point, class R> inline
  void eval( Point & p, Point & dp, const wrapper<R,IParametricCurve > & c,  const typename Point::value_type & t )
  {
    typedef typename Point::value_type C;
    C a,b;
    ::range(a,b,c);
    C h = (b-a)/C(1000);
    a = t - h;
    b = t + h;
    ::eval(p,c,a);
    ::eval(dp,c,b);
    for ( int i = 0; i < Point::_dimension_ ; i ++ )
      dp[i] = (dp[i]- p[i])/(C(2)*h);
    ::eval(p,c,t);
  };
  
  
  template<class R, class Point> inline
  void eval( Point * smp, const R& c, const typename Point::value_type * t, int n )
  { 
    for ( Point * o = smp; o != smp + n; o++, t++  )
      ::eval(*o,c,*t);
  };
  
  template<class Scalar> 
  void tsample( Scalar * t, const Scalar&  a_, const Scalar & b, int n )
  {
    Scalar a = a_;
    Scalar h = (b-a)/Scalar(n-1);
    for ( Scalar * p = t; p != t + n; *p++ = a, a += h );
  };
  
  template<class R, class Scalar> inline
  void tsample( Scalar * t, const R & c, int n )
  {
    Scalar a,b;
    ::range(a,b,c);
    tsample(t,a,b,n);
  };
  
  template<class Point, class Scalar, class R> inline
  void sample( Point * p, Scalar * s, const R & c, int n )
  {
    ::tsample(s,c,n);
    ::eval(p,c,s,n);
  };
  
};



template<class C, class R> struct SubParametricCurve;
template<class C, class R> struct interfaceof< SubParametricCurve<C,R> > {
  typedef IParametricCurve T; 
};

template<class C, class R>
struct SubParametricCurve : wrapper<SubParametricCurve<C,R>,IParametricCurve>
{
  typedef IParametricCurve base_t;
  double   a,b;
  IShape * orig_;
  SubParametricCurve(){};
  SubParametricCurve( double a_, double b_, const R & c ) 
  {
    a = a_;
    b = b_;
    orig_ = (IShape*)(&c);
    orig_->addref();
  };
  ~SubParametricCurve() { orig_->release(); }
};

template<class R, class C>
IParametricCurve* subcurve( const SubParametricCurve<C,R>& c, const C& tmin, const C& tmax )
{
  return subcurve(*((const R*)(&(c.orig_))),tmin,tmax);
};

template<class Scalar, class R, class C> inline
void range( Scalar & tmin, Scalar & tmax, const SubParametricCurve<C,R>& c )
{
  tmin = c.a;
  tmax = c.b;
};

template<class Point,class C, class R> inline
void eval( Point & p, const SubParametricCurve<C,R> & c, const typename Point::value_type& t  )
{
  ::eval(p,*((const R*)c.orig_),t);
};

namespace ParametricCurve
{
  template<class C,class R> inline
  IParametricCurve * subcurve( const R & c, const C& tmin, const C& tmax ) { 
    return new SubParametricCurve<C,R>(tmin,tmax,c.rep()); 
  };
  
  template<class R, class C> inline
  void split( IParametricCurve *& left,
	      IParametricCurve *& right,
	      const R & c,
	      const C & t )
  {
    C tmin,tmax;
    ::range(tmin,tmax,c);
    assert(t>=tmin && t<=tmax);
    left  = ::subcurve(c,tmin,t);
    right = ::subcurve(c,t,tmax);
  };
  
  
  template<class F, class C>
  C simpson_rule(const F & f, const C&  a, const C&  b, int n )
  {
    C h = (b - a)/(2 * n);
    C sum_even, sum_odd;
    sum_even = 0;
    sum_odd = 0;
    for(int i = 1; i <= n; i++ ) sum_odd = sum_odd  + f(a + (2 * i - 1) * h);
    for(int j = 1; j <  n; j++ ) sum_even = sum_even + f(a + 2 * j * h);
    return ((f(a) + f(b) + 4 * sum_odd + 2 * sum_even) * h / 3);
  };
  
  
  template<class F, class C>
  C simpson_int( const F & f, const C & a, const C & b, const C & eps  )
  {
    using std::abs;
    int n = 16;
    C prev =  simpson_rule(f,a, b, 8);
    C curr =  simpson_rule(f,a, b, n);
    while( abs(curr - prev) > eps)
      {
	n *= 2;
	prev = curr;
	curr = simpson_rule(f,a, b, n);
      };
    return curr;
  };

  template<class R, class C>
  struct parametric_curve_length_function
  {
    const R & rep;
    C operator()( const C & x ) const 
    { 
      fxv<C,3> p;
      ::evald(p,rep,x);
      C s = p*p;
      return sqrt(s);
    };
    parametric_curve_length_function( const R & r ) : rep(r){};
  };
  
  template<class R, class C> inline
  C length( const R & c, const C & eps )
  {
    C a,b;
    ::range(a,b,c);
    parametric_curve_length_function<R,C> f(c);
    return simpson_int( f, a, b, eps );
};
};


#include <synaps/shape/wrappers/WShape.h>

__END_NAMESPACE_SYNAPS

#endif
