#ifndef SYNAPS_SHAPE_WPARAMETRICSURFACE_H
#define SYNAPS_SHAPE_WPARAMETRICSURFACE_H

#include <synaps/shape/wrappers/WShape.h>
#include <synaps/shape/wrappers/WParametricCurve.h>
#include <synaps/shape/interfaces/IParametricSurface.h>

__BEGIN_NAMESPACE_SYNAPS
namespace ParametricSurface {};


namespace ParametricSurface {
  template<class C, class R> inline
  void range( C & umin, C & umax, C & vmin, C & vmax, const R & r   );
  template<class Point, class R> inline
  bool sample( Point * smp, 
	       typename Point::value_type * u, 
	       typename Point::value_type * v, 
	       const R & s,
	       int m, 
	       int n );
  template<class Point, class R >
  bool eval( Point * smp, 
	     const R & s, 
	     const typename Point::value_type * u, 
	     const typename Point::value_type * v, 
	     int m, 
	     int n );
  
  template<class Point, class R> inline
  void eval( Point * samples,   const R & s, const typename Point::value_type * uv, int n );

  
  template<class Point3,class R> inline
  void eval( Point3 & p, const R & s, const typename Point3::value_type & u, const typename Point3::value_type & v );
    

};

template<class C, class R> inline
void range( C & umin, C & umax, C & vmin, C & vmax, const wrapper<R,IParametricSurface> & r   ) 
{ 
  using namespace ParametricSurface;
  range(umin,umax,vmin,vmax,r.rep());
};

template<class Point, class R>  inline
bool sample( Point * smp, 
	     typename Point::value_type * u, 
	     typename Point::value_type * v, 
	     const wrapper<R,IParametricSurface> & s,
	     int m, 
	     int n )

{
  using namespace ParametricSurface;
  return sample(smp,u,v,s.rep(),m,n);
};

template<class Point, class R> inline
void eval( Point * samples, const wrapper<R,IParametricSurface> & srf, const typename Point::value_type * uv, int n )
{ 
  using namespace ParametricSurface;
  eval(samples,srf.rep(),uv,n);
};


template<class Point3,class R> inline
void eval( Point3 & p, const wrapper<R,IParametricSurface> & s, const typename Point3::value_type & u, const typename Point3::value_type & v )
{
  using namespace ParametricSurface;
  eval(p,s.rep(),u,v);
};
  

template<class Point, class R > inline
bool eval( Point * smp, const wrapper<R,IParametricSurface> & s, 
	   const typename Point::value_type * u, const typename Point::value_type * v, 
	   int m, 
	   int n )
{
  using namespace ParametricSurface;
  return eval(smp,s.rep(),u,v,m,n);
};

//template<class R> 
//void isovalues( std::vector<IParametricCurve*> & isos, const mesh

template<class R>
IGLGood * mesh( const wrapper<R,IParametricSurface> & s )
{
  //  std::cout << " ParametricSurface\n";
  return mesh(&s,s.gridm,s.gridn);
};


template<int n>
struct ISORep : wrapper< IParametricCurve, ISORep<n> >
{
  double x;
  IParametricSurface * surf;
  ISORep(IParametricSurface * s );
};

template<int n>
inline void range( double & tmin, double & tmax, const ISORep<n> & iso )
{
  double a;
  if ( n == 0 ) 
    iso->surf->range(a,a,tmin,tmax);
  else 
    iso->surf->range(tmin,tmax,a,a);
};

template<int n>
inline void eval( fxv<double,3> & p, const ISORep<n>& iso, double t )
{
  if ( n == 0 ) iso->surf->eval( p, iso->x, t );
  else          iso->surf->eval( p, t, iso->x );
};




#define IParametricSurfaceWrap \
  void range( double & umin, double & umax, double & vmin, double & vmax )  const \
  { ::range(umin,umax,vmin,vmax,this->rep()); }; \
  void eval( double * p, const double * uv, int n ) const \
  { ::eval(((fxv<double,3>*)p), this->rep(),uv,n); }; \
  void eval( double * p, double u, double v ) const \
  { ::eval(*((fxv<double,3>*)p),this->rep(),u,v); }; \
  bool eval( double * smp, const double * uvals, const double * vvals, int m, int n ) const  \
  { return ::eval((fxv<double,3>*)smp,this->rep(),uvals,vvals,m,n); }; \
  bool sample( double * smp, double * uvals, double * vvals, int m, int n )  const \
  { return ::sample((fxv<double,3>*)smp,uvals,vvals,this->rep(),m,n); } \

template<class R>
struct wrapper<R,IParametricSurface> :  wrapper<R,IParametricSurface::base_t> 
{
  int gridm, gridn;
  wrapper<R,IParametricSurface>() { gridm = 50; gridn = 50; };
  void  setGrid( int m, int n ) 
  {
    gridm = m;
    gridn = n;
    if ( this->mesh_ ) delete this->mesh_;
    this->mesh_ = 0;
  };
  
  template<class C> 
  void range( C & umin, C & umax, C & vmin, C & vmax ) const 
  { ::range(umin,umax,vmin,vmax,this->rep()); };
  template<class Point>
  void eval( Point & p, const typename Point::value_type * uv, int n ) const
  { ::eval(p, this->rep(),uv,n); };
  template<class Point>
  void eval( Point & p, const typename Point::value_type & u, const typename Point::value_type & v ) const 
  { ::eval(p,this->rep(),u,v); };
  template<class Point>
  bool eval( Point * smp, const typename Point::value_type * uvals, const typename Point::value_type * vvals, int m, int n ) const 
  { return ::eval(smp,this->rep(),uvals,vvals,m,n); };
  template<class Point>
  bool sample( Point * smp, typename Point::value_type * uvals, typename Point::value_type * vvals, int m, int n ) const 
  { return ::sample(smp,uvals,vvals,m,n); };
  IParametricSurfaceWrap;
};


/*

struct GaussMap : wrapper<GaussMap,IParametricSurface>
{
  IParametricSurface * surf_;
};

inline void eval( fxv<double,3>& p, const GaussMap& s, const double & u, const double & v )
{
  double umin,umax,vmin,vmax;
  s.range(umin,umax,vmin,vmax);
  s.surf_->eval(p,s,u,v);
};

*/



namespace ParametricSurface {
  template<class Point, class R >
  bool eval( Point * smp, 
	     const R & s, 
	     const typename Point::value_type * u, 
	     const typename Point::value_type * v, 
	     int m, 
	     int n )
    
  {
    typedef typename Point::value_type C;
    for ( const C * p = u; p != u + m; p ++ )
      for ( const C * q = v; q != v + n; q ++, smp ++ )
	::eval(*smp,s,*p,*q);
    return true;
  };
  
  template<class C, class R> inline
  void range( C & umin, C & umax, C & vmin, C & vmax, const R & r   ) { umin = vmin = 0, umax = vmax = 1; };
    
  template<class Point, class R> inline
  void eval( Point * samples,   const R & s, const typename Point::value_type * uv, int n )
  { for ( Point * smp = samples; smp != samples + n; ::eval(*smp, s, uv[0], uv[1] ), smp++, uv += 2 ); };
  
  template<class Point3,class R> inline
  void eval( Point3 & p, const R & s, const typename Point3::value_type & u, const typename Point3::value_type & v )
  {
    typename Point3::value_type tmp[] = { u, v };
    s.eval(&p,tmp,1);
  };
    

  
  template<class Point, class R> inline
  bool sample( Point * smp, 
	       typename Point::value_type * u, 
	       typename Point::value_type * v, 
	       const R & s,
	       int m, 
	       int n )
    
    {
    typename Point::value_type umin,umax,vmin,vmax;
    ::range(umin,umax,vmin,vmax,s.rep());
    tsample(u,umin,umax,m);
    tsample(v,vmin,vmax,n);
    ::eval(smp,s,u,v,m,n);
    return true;
  };
}


__END_NAMESPACE_SYNAPS

#endif
