#include <synaps/shape/gaia/dsearch.h>
#include <synaps/base/chrono.h>
#include <synaps/shape/gmv/stream.h>
#include <fstream>
__BEGIN_NAMESPACE_SYNAPS

namespace gaia {
  
  dsearch::dsearch( const IParametricSurface * s , unsigned w, unsigned h, bool cnf  ) : qsegment(s, w, h ) 
  {
    m_cnf = cnf;
    _st = time();
    unsigned i,j;
    conflict_count = 0;
    nbpoints = 0;
    _branches = new std::list< dcurve * > *[this->regions.size()*this->regions.size()];
    memset(_branches,0,sizeof( std::list< dcurve * > * )*
	   this->regions.size()*this->regions.size() );
    
    for ( i = 0; i < this->regions.size() ; i++ )
      for ( j = i+1; j < this->regions.size() ; j++   )
	{
#define VOISINS_AUSSI
#ifndef VOISINS_AUSSI
	  if ( !this->grp->neighbors(i,j) )
#endif
	    {
	      //cout << i << " " << j << endl;
	      search(i,j);
	      find_branches(i,j);
	    };
	};
    
    //      cout << "search: ok\n";
    std::vector< dcurve * > * tmp = new std::vector<dcurve*>();
    
    for ( i = 0; i < this->regions.size(); i++ )
      for ( j = i+1; j < this->regions.size(); j++ )
	{
	  if ( branches(i,j) )
	    {
	      for (  std::list<dcurve*>::iterator it = branches(i,j)->begin(); it != branches(i,j)->end(); it ++ )
		{
		  tmp->push_back( *it );
		};
	      delete branches(i,j);
	    }
	};
    
    result = reduce2( tmp );
    delete tmp;
    nbcurves = 0;
    lefts = rights = 0;
    sizes = 0;
    if ( result )
      {
	nbpoints = 0;
	std::list<dcurve *>::iterator it;
	nbcurves = result->size();
	
	sizes  = new unsigned[nbcurves];
	
	for ( i = 0, it = result->begin(); it != result->end(); it++ , i++ )
	  {
	    sizes[i] = (*it)->left.size();
	    nbpoints+= sizes[i];
	  };
	
	lefts  = new point2*[nbcurves];
	rights = new point2*[nbcurves];
	unsigned c_n = 0;
	unsigned k;
	for (  it = result->begin(); it != result->end(); it ++, c_n++  )
	  {
	    lefts[c_n]  = new point2[sizes[c_n]];
	    rights[c_n] = new point2[sizes[c_n]];
	    list2_t::iterator itcr, itcl;
	    for ( k = 0, itcl = (*it)->left.begin(), itcr = (*it)->right.begin(); 
		  itcl != (*it)->left.end(); 
		itcl++ , itcr++, k++ )
	      {
		
		lefts[c_n][k] = *itcl;
		rights[c_n][k] = *itcr;
	      };
	  };
	
	for (  std::list<dcurve*>::iterator it = result->begin(); it!= result->end(); it++ )
	  {
	    delete *it;
	  };
	delete result;
      };
    delete[] _branches;
    
    std::cout << "d = " << (time()-_st) << std::endl;
    //    std::cout << nbcurves << std::endl;
  
    //    GnuPlotOutput("dsearch.gmv");
 };
  
  dsearch::~dsearch()
  {
    if ( sizes ) delete[] sizes;
    if ( lefts ) 
      {
	for ( unsigned i = 0; i < nbcurves; i++ )
	  delete[] lefts[i];
	delete[] lefts;
	};
    if ( rights ) 
      {
	for ( unsigned i = 0; i < nbcurves; i++ )
	  delete[] rights[i];
	delete[] rights;
      };

  };
  
  void dsearch::GnuPlotOutput( char * name )
  {
    gmv::stream out(name);
    
    std::vector< vector3 > v;
    for ( unsigned i = 0; i < nbcurves; i++ )
      {
	for ( unsigned k = 0; k < sizes[i]; k++ )
	  {
	    v.push_back(vector3());
	    v.back()[0] = lefts[i][k][0];
	    v.back()[1] = lefts[i][k][1];
	    v.back()[2] = 0;
	    v.push_back(vector3());
	    v.back()[0] = rights[i][k][0];
	    v.back()[1] = rights[i][k][1];
	    v.back()[2] = 0;
	  };
      };
    out.points(&(v[0][0]),v.size());
  }

  
}
__END_NAMESPACE_SYNAPS
  
