#ifndef DSEARCH_H
#define DSEARCH_H
#include <list>
#include <vector>
#include <set>
#include <map>
#include "qsegment.h"
#include "fsvector.h"
#include "tri-tri.h"
#include "s223.h"
#include "ordered_pair.h"
#ifndef SHIFT 
#define SHIFT 3
#endif
//#include <self/quadinter.h>

namespace self {

  template<class FunctionType>
  struct dsearch_t : public qsegment_t< FunctionType >
  {
    bool m_cnf;
    typedef qsegment_t< FunctionType >       qseg_t;
    typedef double       number;
    typedef fsvector< number, 4 >      point4;
    typedef fsvector< number, 3 >      point3;
    typedef fsvector< number, 2 >      point2;
    typedef std::list< point2 >  list2_t; 
    typedef std::list< point3 >  list3_t;
    typedef typename list2_t::iterator p2ref_t;
    typedef typename qseg_t::coord_t coord_t;
    typedef typename qseg_t::rid_t  rid_t;
    typedef typename qseg_t::qnode  qnode;
    typedef typename qseg_t::qtree  qtree;

    struct dcurve
    {      
      unsigned size;
      dcurve  *   owner; 
      list2_t     left,right;
      list3_t     inter;
      
      dcurve( const point2& l, const point2& r, const point3& i ) 
      {
	owner = this;
	left.push_back(l);
	right.push_back(r);
	inter.push_back(i);
	size = 1;
      };
	   
      dcurve( const point2& l0, const point2& l1, const point2& r0, const point2& r1, const point3& i0, const point3& i1 )
      {
	owner = this;
	left.push_back(l0);
	left.push_back(l1);
	right.push_back(r0);
	right.push_back(r1);
	inter.push_back(i0);
	inter.push_back(i1);
	size = 2;
      };
    };
    
    struct sdpoint_t
    {
      typedef number value_type;
      dcurve  *  curve;
      p2ref_t idl;
      p2ref_t idr;
      
      const number& operator[]( unsigned i ) const
      {
	if ( i < 2 )
 	  return (*idl)[i];
 	return (*idr)[i-2];
      };
	  
      number& operator[]( unsigned i ) 
      {
	if ( i < 2 )
 	  return (*idl)[i];
 	return (*idr)[i-2];
      };
    };
	  
    struct pdpoint_t
    {
      typedef number value_type;
      dcurve * curve;
      p2ref_t ref;
      pdpoint_t * a;

      value_type& operator[]( unsigned i ) 
      { 
	if ( i < 2 ) return (*ref)[i];
	return (*(a->ref))[i-2];
      };
      const value_type& operator[]( unsigned i ) const 
      { 
	if ( i < 2 ) return (*ref)[i];
	return (*(a->ref))[i-2];
      };
    };
    
    
    inline static void extremums( sdpoint_t* dst, dcurve* src )
    {
      dst[0].curve = src;
      dst[0].idl   = src->left.begin(); 
      dst[0].idr   = src->right.begin();
      dst[1].curve = src;
      dst[1].idl   = --(src->left.end()); 
      dst[1].idr   = --(src->right.end());
    };
    
    
    inline static void extremums( pdpoint_t * dst, dcurve * c )
    {
      dst[0].curve = dst[1].curve = dst[2].curve = dst[3].curve  = c; 
      dst[0].a = &(dst[1]);  dst[1].a = &(dst[0]);
      dst[0].ref   = c->left.begin(); dst[1].ref   = c->right.begin();
      dst[2].a = &(dst[3]);  dst[3].a = &(dst[2]);
      dst[2].ref   = --(c->left.end()); dst[3].ref   = --(c->right.end());
    };

    inline static  pdpoint_t * sibble( pdpoint_t * p ) 
    { 
      return p->a; 
    };
    
    
    inline static dcurve * curve( pdpoint_t * p )
    { 
      dcurve * tmp      = p->curve;
      while( tmp->owner != tmp ) tmp = tmp->owner;
      p->curve->owner   = tmp;
      //      p->curve = tmp;
      return tmp;
    };

    inline static dcurve *  curve( sdpoint_t * p )
    { 
      dcurve *  tmp      = p->curve;
      while( tmp->owner != tmp ) tmp = tmp->owner;
      p->curve->owner   = tmp;
      //      p->curve = tmp;
      return tmp;
    };
    
    
    inline static bool isfirst ( sdpoint_t* p ) { return curve(p)->left.begin() == p->idl;     };
    inline static bool islast  ( sdpoint_t* p ) { return (--(curve(p)->left.end())) == p->idl; };
    
    inline static bool isfirst ( pdpoint_t* p ) { return curve(p)->left.begin() == p->ref || curve(p)->right.begin() == p->ref; };
    inline static bool islast ( pdpoint_t* p ) { return (--(curve(p)->left.end())) == p->ref ||(--(curve(p)->right.end())) == p->ref;};
    
    inline static bool isextrem( pdpoint_t* p  ) { return  isfirst(p) || islast(p); };
    inline static bool isextrem( sdpoint_t*  p ) { return  isfirst(p) || islast(p); };
    
    inline static void append( dcurve *  a, dcurve   * b ) 
    { 
      a->left.splice ( a->left.end(),  b->left  );
      a->right.splice( a->right.end(), b->right );
      a->inter.splice( a->inter.end(), b->inter );
      b->owner = a; 
      a->size += b->size;
    };
    
    inline static void prepend( dcurve * a, dcurve * b ) 
    { 
      a->left.splice( a->left.begin(), b->left );  
      a->right.splice( a->right.begin(), b->right );
      a->inter.splice( a->inter.begin(), b->inter );
      b->owner = a; 
      a->size += b->size;
    };
    
    inline static void reverse( dcurve * c ) 
    { c->left.reverse(); c->right.reverse(); c->inter.reverse(); };

    // utilisation de size (a faire)
    template<class DPoint>
    inline static  void _link(  DPoint* a, DPoint* b ) 
    {
      if ( islast(a)   ) 
	{
	  if( islast(b) )
	    reverse(curve(b));
	  // last a + first b
	  append(curve(a),curve(b));
	}
      else
	{
	  if ( isfirst(b) )
	    reverse(curve(b));
	  // last b + first a
	  prepend(curve(a),curve(b));
	};
    };
    
    inline static void link( sdpoint_t* a, sdpoint_t* b)
    { _link(a,b); };
    
    inline static void swap( dcurve * c )
    { c->left.swap( c->right ); };
    
    inline static bool isright( pdpoint_t* p )
    { return p->ref == curve(p)->right.begin() || p->ref == --(curve(p)->right.end());};
    
    inline  static void link( pdpoint_t* a, pdpoint_t* b)
    {
      if ( isright(a) ) swap(curve(a));
      if ( isright(b) ) swap(curve(b));
      _link(a,b);
    };
    
    inline static  bool   empty( dcurve * c ) { return c->owner != c; };
    
    inline static  void print( sdpoint_t * p )
    {
      //      patterns::print(cout,*p,4);
    };
    
    template<class DPoint>
    struct assoc_t
    {
      typedef typename DPoint::value_type number;
      
      std::pair<DPoint*,DPoint*>         pp;
      number                        d;
      
      // comparing association on pointers
      struct pair_cmp
      { bool operator()( const assoc_t& a1, const assoc_t& a2 ) const { return a1.pp < a2.pp;  }; };
      // comparing association on distances
      struct dist_cmp
      { bool operator()( const assoc_t& a1, const assoc_t& a2 ) const { return a1.d > a2.d;    }; };
      
      assoc_t(){};
      assoc_t( DPoint * a, DPoint * b, const double& _d ):
	pp(a,b), d(_d){};
    };

        

    typedef std::priority_queue< assoc_t<pdpoint_t>, 
				 std::vector< assoc_t<pdpoint_t> >, 
				 typename assoc_t< pdpoint_t>::dist_cmp > pheap_t;    
    typedef std::priority_queue< assoc_t<sdpoint_t>, 
				 std::vector< assoc_t<sdpoint_t> >, 
				 typename assoc_t< sdpoint_t>::dist_cmp > sheap_t;

    typedef std::set< assoc_t< pdpoint_t > , typename assoc_t<pdpoint_t>::pair_cmp > pset_t;
    
    typedef assoc_t<sdpoint_t> sdassc_t;
    typedef assoc_t<pdpoint_t> pdassc_t;
    

    template <class A, class B>
    inline static void container_add( A& h, const B& a )
    {
      h.push(a);
    };
    // don't work with cl !    
//     inline static void container_add( sheap_t& h, const sdassc_t& a ) 
//     { h.push(a);   }; 

//     inline static void container_add( pheap_t& h, const pdassc_t& a ) 
//     { h.push(a);   }; 

    inline static void container_add( pset_t& s, const pdassc_t& a )
    { s.insert(a); };
    
    
    inline static number distance( sdpoint_t * pa, sdpoint_t * pb )
    {
#define Mmax(a,b) (((a)<(b))?(b):(a))
      
      number acc = 0.0;
      for ( unsigned i = 0; i < 4; i++ )
	acc += ((*pa)[i]-(*pb)[i])*((*pa)[i]-(*pb)[i]);
      return acc;
    };
    
    inline static number distance( pdpoint_t * pa, pdpoint_t * pb )
    {
      
      number acc = 0.0;
      for ( unsigned i = 0; i < 4; i++ )
	acc += ((*pa)[i]-(*pb)[i])*((*pa)[i]-(*pb)[i]);
//Mmax(acc,abs((*pa)[i]-(*pb)[i]));
      return acc;
#undef Mmax

    };
    
    template<class DPoint>
    struct dim_cmp   
    { 
      unsigned dim; 
      dim_cmp(unsigned d = 0): dim(d) {}; 
      dim_cmp next() const { return dim_cmp((dim+1)%4); };
      bool operator()(  DPoint*const& a, DPoint *const& b ) const { return (*a)[dim] < (*b)[dim]; };
      bool operator()( const DPoint& a, const DPoint&  b ) const { return    a[dim] < b[dim];    };
    };
    

    struct sdknode
    {
      sdpoint_t   * data; 
      sdknode * l, * r; 
      sdknode( sdpoint_t * d, sdknode * _l, sdknode * _r ) : data(d), l(_l), r(_r){}; 
      ~sdknode() { if ( l ) delete l; if ( r ) delete r; };
    };
    
    struct pdknode
    {
      pdpoint_t   * data; 
      pdknode * l, * r; 
      pdknode( pdpoint_t * d, pdknode * _l, pdknode * _r ) : data(d), l(_l), r(_r){}; 
      ~pdknode() { if ( l ) delete l; if ( r ) delete r; };
    };
      
    inline static sdknode * make( sdpoint_t * first, sdpoint_t * last, const dim_cmp<sdpoint_t>& f ) 
    {     
      if ( last-first >= 1 ) {
	if ( last-first > 1 ) { 
	  unsigned m = (last-first)>>1;
	  std::nth_element(first, first+m,last,f);
	  return new sdknode( first+m, 
			    make( first, first + m, f.next()), 
			    make( first+m+1, last,  f.next()) ); }
	else { return new sdknode(first,0,0); };};
      return 0; 
    };
    
    inline static pdknode * make( pdpoint_t ** first, pdpoint_t ** last, const dim_cmp<pdpoint_t>& f ) 
    {     
      if ( last-first >= 1 ) {
	if ( last-first > 1 ) { 
	  unsigned m = (last-first)>>1;
	  std::nth_element(first, first+m,last,f);
	  return new pdknode( *(first+m), 
				       make( first, first + m, f.next()), 
				       make( first+m+1, last,  f.next()) ); }
	else { return new pdknode(*first,0,0); };};
      return 0; 
    };
    
    template<class Container>
    inline static void search( Container& c, sdknode * curr, sdpoint_t * const  query, unsigned dim, const number& eps  ) 
    {
      if (! curr ) return;
      
      bool left  = (*query)[dim]-eps<(*(curr->data))[dim];   
      bool right = (*query)[dim]+eps>(*(curr->data))[dim];
	
      if ( (left && right) && (query<curr->data) && (curve(query) != curve(curr->data)) )
	  {
	    number d = distance(curr->data,query);
	    if ( (d<eps) ) 
	      container_add(c,assoc_t<sdpoint_t>(query,curr->data,d));
	  };
      if ( left  ){  search( c, curr->l, query, (dim+1)%4, eps ); };
      if ( right ){  search( c, curr->r, query, (dim+1)%4, eps ); };
    };
    
    template<class Container>
    inline static void search( Container& c, pdknode * curr, pdpoint_t * const  query, unsigned dim, const number& eps  ) 
    {
      if (! curr ) return;
      
      bool left  = (*query)[dim]-eps<(*(curr->data))[dim];   
      bool right = (*query)[dim]+eps>(*(curr->data))[dim];
	if ( (left && right) && (query<curr->data) && (curve(query) != curve(curr->data)) )
	  {
	    number d = distance(curr->data,query);
	    if ( (d<eps) ) 
	      container_add(c,assoc_t<pdpoint_t>(query,curr->data,d));
	  };
	if ( left  ){  search( c, curr->l, query, (dim+1)%4, eps ); };
	if ( right ){  search( c, curr->r, query, (dim+1)%4, eps ); };
    };
    
    
    template<class Container>
    inline static void search( Container& c, sdpoint_t * first, sdpoint_t * last, const number& epsilonn ) 
    {
      sdknode* root;
      root = make(first,last,dim_cmp<sdpoint_t>());
      for ( sdpoint_t * p = first; p < last; p++ )
	search(c,root,p,0,epsilonn);
      delete root;
    };

    
    template<class Container>
    inline static void search( Container& c, pdpoint_t * first, pdpoint_t * last, const number& epsilonn ) 
    {
      pdknode* root;
      pdpoint_t ** tmp = new pdpoint_t*[last-first];
      int i;
      for ( i = 0; i < last-first; i++ )
	{
	  tmp[i] = first + i;
	};

      root = make(tmp,tmp + (last-first),dim_cmp<pdpoint_t>());
      delete[] tmp;
      for ( pdpoint_t * p = first; p < last; p++ )
	search(c,root,p,0,epsilonn);
      delete root;
    };

    
    inline static void build_pheap( pheap_t& h, pdpoint_t * first, pdpoint_t * last, const number& prec )
    {
      search(h,first,last,prec);
    };

    inline static void build_pset( pset_t& s, pdpoint_t * first, pdpoint_t * last, const number& prec )
    {
      search(s,first,last,prec);
    };
    
    inline static void build_sheap( sheap_t& h, sdpoint_t * first, sdpoint_t * last, const number& prec )
    {
      search(h,first,last,prec);
    };
    
    inline static void solve_sheap( sheap_t& h )
    {
      // dcurve * dummy_curve = new dcurve( point2(), point2(), point2(), point2(),
      //				 point3(), point3() );
      while( !h.empty() )
	{
	  sdassc_t a = h.top();
	  //	  cout << a.d << endl;
	  if (( isextrem(a.pp.first) && isextrem(a.pp.second) ) && 
	      ( curve(a.pp.first) != curve(a.pp.second) )) 
	    link(a.pp.first,a.pp.second);
	  h.pop();
	};
    };
    
    
    inline static void print( pdpoint_t * p )
    {
      //patterns::print(cout,*p,4);
    };
    
    inline static void print( std::pair< pdpoint_t*, pdpoint_t*>& a )
    {
      print(a.first);
      std::cout << " " ;
      print(a.second);
    };

    inline static void solve_pheap( pheap_t& h )
    {
      
      while( !h.empty() )
	{
	  pdassc_t a = h.top();
	  
	  if (( isextrem(a.pp.first) && isextrem(a.pp.second) ) && 
	      ( curve(a.pp.first) != curve(a.pp.second) )) 
	    {
	      link(a.pp.first,a.pp.second);
	    };
	  h.pop();
	};
    };
    
    typedef std::pair<pdpoint_t*,pdpoint_t*> ppair_t;
    typedef std::list< ppair_t > links_t;
    typedef std::map< dcurve*, links_t > curves_links_t;
    typedef ordered_pair<dcurve*> cpair_t;
    
    inline static cpair_t curves( const pdassc_t& ass )
    { return cpair_t(curve(ass.pp.first),curve(ass.pp.second)); };
    
    inline static pdassc_t sibbles( const pdassc_t& ass )
    {
       pdpoint_t * a = sibble(ass.pp.first);
       pdpoint_t * b = sibble(ass.pp.second);
       if ( a < b )
 	return pdassc_t(a,b,0);
       else 
 	return pdassc_t(b,a,0);
    };
    
    inline static void satisfy_links( typename curves_links_t::iterator it )
    {
      std::list< ppair_t > * l = &(it->second);
      std::vector< ppair_t > starts;
      std::vector< ppair_t > ends;
      
       starts.reserve(10);
       ends.reserve(10);
   
       for ( typename std::list< ppair_t >::iterator p = l->begin(); 
	     p != l->end(); p ++ )
	 {
	   if ( isfirst(p->first) ) starts.push_back( *p );
	   else if ( islast(p->first) ) ends.push_back( *p );
	 };
      
       if ( starts.size() == 1 )
	 {
	   if ( isextrem( starts[0].first ) && isextrem( starts[0].second ) )
   	    if ( curve(starts[0].first) != curve(starts[0].second))
   	      link( starts[0].first, starts[0].second );
	 };
      
       if ( ends.size() == 1 )
	 {
	   
	   if ( isextrem( ends[0].first ) && isextrem( ends[0].second ) )
	     if ( curve(ends[0].first) != curve(ends[0].second) )
	       link( ends[0].first, ends[0].second );
	 };
    };
    
    typedef typename pset_t::iterator pset_iterator;
    
    inline static void solve_pset( curves_links_t& links, pset_t& s )
    {
      
      while ( !s.empty() )
	{
	  pdassc_t ass(sibbles(*(s.begin())));
	  pset_iterator it; 
	  it = s.find( ass  );
	  if( it  != s.end() )
	    {
	      s.erase(s.find(ass));
	      cpair_t tmp (curves(*(s.begin())));
	      ppair_t p = (s.begin())->pp;
	      if ( curve(p.first) != tmp.first ) 
		{
		  links[tmp.second].push_front(p);
		  std::swap(p.first,p.second);
		  links[tmp.first].push_front(p);
		}
	      else  		
		{
		  links[tmp.first].push_front(p);
		  std::swap(p.first,p.second);
		  links[tmp.second].push_front(p);
		};
	    };
	  s.erase(s.begin());
	};
    };
    
    inline static std::list<dcurve*> * reduce2( std::vector< dcurve * > * conflicts )
    {
      if ( conflicts->size() == 0 ) return 0;
      unsigned   i;
      unsigned   endsize  = 4*conflicts->size();
      pdpoint_t * ends = new pdpoint_t[endsize];
      unsigned c = 0;
      for ( i = 0; i < conflicts->size(); i++ )
	{
	  extremums( ends + c, (*conflicts)[i] );
	  c+= 4;
	};
      
      pset_t s;
      build_pset(s,ends, ends + endsize, 0.01 );
      
      curves_links_t links;
      solve_pset(links,s);

      
      for ( typename curves_links_t::iterator it = links.begin(); 
	    it != links.end(); it++ )
	satisfy_links( it );
      
      delete[] ends;
      
      std::list< dcurve* > * result = new std::list< dcurve * >();
      
      for ( i = 0; i < conflicts->size() ; i++ )
	{
	  if ( empty((*conflicts)[i]) ) delete (*conflicts)[i];
	  else result->push_front( (*conflicts)[i] );
	};

      return result;
    };
    
    inline static std::list<dcurve*> * reduce( std::vector< dcurve * > * conflicts )
    {
      unsigned i;
      if ( conflicts->size() == 0 ) return 0;

      unsigned   endsize  = 2*conflicts->size();
      sdpoint_t * ends = new sdpoint_t[endsize];

      for ( i = 0; i < endsize; i+= 2 )
	extremums( ends + i, (*conflicts)[i/2] );
      
      sheap_t h;
      build_sheap(h,ends, ends + endsize, 0.04 );
      solve_sheap(h);
      delete[] ends;
      std::list< dcurve* > * result = new std::list< dcurve * >();
      for ( i = 0; i < conflicts->size() ; i++ )
	{
	  if ( empty((*conflicts)[i]) ) delete (*conflicts)[i];
	  else result->push_front( (*conflicts)[i] );
	};
      return result;
    };
    
    
    //------------------------------------------------------------------------------
    std::vector< dcurve * >  * curr_config;           // represent the current region/region conflict resultings segments
    std::list  < dcurve * >  ** _branches;            // represent the reduced set of curves for each region/region conflict
    
    // acces in read/write to the (i,j) region/region conflict
    std::list  < dcurve * >  *& branches( unsigned i, unsigned j ) 
    { return _branches[i*this->regions.size()+j]; };


    /*
    void vector_sub( point3& r, const point3& a, const point3& b )
    {
      for ( unsigned i = 0; i < d; i++ )
	r[i] = a[i]-b[i];
    };

    number_t vector_dotprod( const point3& a, const point3& b )
    {
      number_t tmp = 0;
      for ( unsigned i = 0; i < d; i++ )
	tmp += a[i]*b[i];
      return tmp;
    };
    
    void vector_crossprod( point3& r, const point3& a, const point3& b )
    {
      r[2] = a[0]*b[1]-a[1]*b[0];
      r[0] = a[1]*b[2]-a[2]*b[1];
      r[1] = a[2]*b[0]-a[0]*b[2];
    };
    number_t vector_nmax( const point3& a, const point3& b)
    {
      number_t m = std::abs(a[0]-b[0]);
      for ( unsigned i = 1; i < d; i++ )
	{
	  number_t c = std::abs(a[i]-b[i]);
	  if ( c > m ) m = c;
	};
      return m;
    };

    struct FirstOrderException{};

    void reachpt( number_t& du, number_t& dv, const point_t& delta, const point_t& Tu, const point_t& Tv )
    {
      number_t v0,v1;
      number_t m00,m11,m01;
      v0  = vector_dotprod( delta, Tu );
      v1  = vector_dotprod( delta, Tv );
      m00 = vector_dotprod( Tu, Tu );
      m11 = vector_dotprod( Tv, Tv );
      m01 = vector_dotprod( Tu, Tv );
      number_t det = m00*m11-m01*m01;
      if ( det < 1e-20 )  throw FirstOrderException(); 
      du = (m11*v0-m01*v1)/det;
      dv = (m00*v1-m01*v0)/det;
    };
    
    bool hosaka_iter( number_t& du, number_t& dv, number_t& ds, number_t& dt, 
		      point_t& pA, point_t& pB, 
		      const number_t& u, const number_t& v, const number_t& s, const number_t& t )
    {
      
      point_t tAu,tAv,tBu,tBv;

      m_smp->DiffUEval(tAu,u,v); m_smp->DiffVEval(tAv,u,v);
      m_smp->DiffUEval(tBu,s,t); m_smp->DiffVEval(tBv,s,t);
      
      point_t nA,nB,nD;
      
      vector_crossprod( nA, tAu, tAv );
      vector_crossprod( nB, tBu, tBv );      
      vector_crossprod( nD, nA, nB   );
      
      number_t dA,dB,dD;
      
      dA = vector_dotprod( nA, pA );
      dB = vector_dotprod( nB, pB );
      dD = vector_dotprod( nD, pB );
      

      number_t d2nD = vector_dotprod( nD, nD ) ;
      number_t d2nA = vector_dotprod( nA, nA ) ;
      number_t d2nB = vector_dotprod( nB, nB ) ;

      if ( d2nD < 1e-6*d2nA*d2nB ) throw FirstOrderException();
      
      // plane3
      
      point_t c0,c1,x;

      vector_crossprod( c0, nB, nD );
      vector_crossprod( c1, nD, nA );

      x[0] = c0[0]*dA+c1[0]*dB+nD[0]*dD;
      x[1] = c0[1]*dA+c1[1]*dB+nD[1]*dD;
      x[2] = c0[2]*dA+c1[2]*dB+nD[2]*dD;
      
      number_t sc = 1.0/d2nD;
      
      x[0] *= sc;
      x[1] *= sc;
      x[2] *= sc;
      

      point_t Adelta,Bdelta;
      
      vector_sub(Adelta,x,pA);
      vector_sub(Bdelta,x,pB);
      
      reachpt( du, dv, Adelta, tAu, tAv );
      reachpt( ds, dt, Bdelta, tBu, tBv );
      
    };
    

    bool hosaka( point2& l, point2& r )
    {
      point3 pA,pB;
      number u,v,s,t;
      
      const number Sdu(m_smp->du()/3.0);
      const number Sdv(m_smp->dv()/3.0);
      
      u = l[0]*this->m_du+this->Fct->umin();
      v = l[1]*m_dv+this->Fct->vmin();
      
      s = r[0]*this->m_du+this->Fct->umin();
      t = r[1]*m_dv+this->Fct->vmin(); 
      
      this->Fct->Eval(pA,u,v);
      this->Fct->Eval(pB,s,t);
      
      number d = vector_nmax(pA,pB);

         #define exiting()\
	  {\
	   if ( std::abs(du) > Sdu ) return false;\
	   if ( std::abs(ds) > Sdu ) return false;\
	   if ( std::abs(dv) > Sdv ) return false;\
	   if ( std::abs(dt) > Sdv ) return false;\
           l[0] = u;\
           l[1] = v;\
	   r[0] = s;\
           r[1] = t;\
          }


      for( int it = 0; it < 8; it++ )
	{
	  number_t du,dv,ds,dt;
	  try  { hosaka_iter(du,dv,ds,dt,pA,pB,u,v,s,t); }
	  catch( ... ) 
	    {
	      l[0] = u; l[1] = v;
	      r[0] = s; r[1] = t;
	      return false;
	    }

	  exiting();

	  u += du;
	  v += dv;
	  s += ds;
	  t += dt;
	  
	  this->Fct->Eval(pA,u,v);
	  this->Fct->Eval(pB,s,t);
	  
	  number_t err = vector_nmax(pA,pB);

	  if (  err  <  d    )  d = err; else return false;
	  if (  err  <  1e-7 ) 
	  {
	    using namespace std;
	    if ( abs(u-s) < 1e-8 && abs(s-t) < 1e-8 ) return false;
	    return true;
	  };
	};
      return false;
    };
    */
    // add a pair of segment to the segment soup
    void push (  const point2& l0,  const point2& l1, 
		 const point2& r0,  const point2& r1,
		 const point3& i0,  const point3& i1 )
    { 

      curr_config->push_back( new dcurve( l0,l1,r0, r1, i0, i1 ) ); 
    };
    
#define __up__   0
#define __down__ 1
    /* return the up triangle of quad q */
#define __up__triangle__(q)         q[0],q[3],q[2]
    /* return the down triangle of quad q */
#define __down__triangle__(q)       q[0],q[1],q[2]
    /* return the parametric plane of up triangle of q */
#define __up__param_triangle__(q)   q[3],q[2]-q[3],q[0]-q[3]
    /* return the parametric plane of down triangle of q */
#define __down__param_triangle__(q) q[1],q[0]-q[1],q[2]-q[1]
#define __convert_order__(i,point) { point[(i+1)%2] = 1.0-point[(i+1)%2]; }

    /* check for the intersection beetween {UP,DOWN}/{UP,DOWN} triangles */
#define __triangle_triangle_case__(trig0,trig1)                           \
    {                                                                     \
      if ( geom::intersectp_triangles3_isegment                           \
	   ( coplanar, seg[0], seg[1],                                    \
	     trig0##triangle__(a),                                        \
	     trig1##triangle__(b), point3::value_type(1e-12) ))           \
	{                                                                 \
	  if ( !coplanar )                                                \
	    {                                                             \
	      spc2prms(/* points in parameter space */                    \
		     seg0[0], seg0[1],                                    \
		     /* corresponding points in R3 */                     \
		      seg[0],  seg[1],                                    \
		     /* parametric plane = base + 2 vectors  */           \
		     trig0##param_triangle__(a)                           \
		     );                                                   \
	      spc2prms( seg1[0], seg1[1],                                 \
			seg[0], seg[1],                                   \
			trig1##param_triangle__(b) );                     \
	      /* retrieve the correct offset coordinates in quad */       \
	    __convert_order__(trig0,seg0[0]);                             \
	    __convert_order__(trig0,seg0[1]);                             \
	    __convert_order__(trig1,seg1[0]);                             \
	    __convert_order__(trig1,seg1[1]);                             \
	    }                                                             \
	else                                                              \
	  {\
	    seg0[0][0] = 1.0/3.0; seg0[0][1] = 1.0/3.0;                   \
	    seg0[1][0] = 1.0/3.0; seg0[1][0] = 1.0/3.0;                   \
	    seg1[0][0] = 1.0/3.0; seg1[0][1] = 1.0/3.0;                   \
	    seg1[1][0] = 1.0/3.0; seg1[1][1] = 1.0/3.0;                   \
            __convert_order__(trig0,seg0[0]);                             \
	    __convert_order__(trig0,seg0[1]);                             \
	    __convert_order__(trig1,seg1[0]);                             \
	    __convert_order__(trig1,seg1[1]);                             \
	  };                                                              \
	    {                                                             \
              point2 pf,ps;                                               \
	      pf[0] = f->umin;                                            \
	      pf[1] = f->vmin;                                            \
	      ps[0] = s->umin;                                            \
	      ps[1] = s->vmin;                                            \
	      seg0[0]+=pf;                                                \
	      seg0[1]+=pf;                                                \
	      seg1[0]+=ps;                                                \
	      seg1[1]+=ps;                                                \
	      push( seg0[0], seg0[1], seg1[0], seg1[1], seg[0],seg[1] );  \
	    }                                                             \
	};                                                                \
    }                                                                     \
    
    std::vector< point3 > cnfdata3;
    std::vector< point2 > cnfdata2;
    
    void cnfpush( qtree f, qtree s )
    {
      point3 a[4],b[4]; //      point3 a[4]; point3 b[4];
      fill_points(a,f);
      fill_points(b,s);
      cnfdata3.push_back( ((number)(1.0/4.0))*(a[0]+a[1]+a[2]+a[3]) );
    };
    
    // push an quad / quad conflict
    // intersection is computed if any and added to the current conflict set of segments
    void push( qtree f, qtree s )
    {
      /*      if ( f->umin == 0 || f->umax == h-1 ) return;
	      if ( f->vmin == 0 || f->vmin == w-1 ) return;
	      if ( s->umin == 0 || s->umax == h-1 ) return;
	      if ( s->vmin == 0 || s->vmin == w-1 ) return;
      */
      if ( abs(f->umin-s->umin)+abs(f->vmin-s->vmin) < SHIFT ) return;
      conflict_count ++;
      if ( m_cnf ) 
	{
	  cnfpush(f,s);
	};
      AABB<number,3> bfs;
      point3 a[4],b[4]; //      point3 a[4]; point3 b[4];
      fill_points(a,f);
      fill_points(b,s);
      Union(bfs,f->box,s->box);
      bool ancien = true;
      number scale = 1.0/Lmax(bfs);
      for ( unsigned i = 0; i < 4; i++ )
	{
	  for ( unsigned d = 0; d < 3; d++ )
	    {
	      a[i][d] -= min(bfs[d]);
	      b[i][d] -= min(bfs[d]);
	      a[i][d] *= scale;
	      b[i][d] *= scale;
	    };
	};
      {
	point3  seg[2];
	point2 seg0[2], seg1[2];
	bool   coplanar;
	__triangle_triangle_case__(__up__,__up__);
	__triangle_triangle_case__(__up__,__down__);
	__triangle_triangle_case__(__down__,__down__);
	__triangle_triangle_case__(__down__,__up__);
      }
      
    };

    // use of the qsegment_t bounding box hierarchy to retrieve unit quad that intersects
#define push_s push
#define push_f push
    
    // search specialisation s is known to be a leaf
    void search_f( rid_t const i, rid_t const j, qtree f, qtree const s )
    {
      if (Intersectp(f->box,s->box))
	{
	  split(i,f);
	  if ( leaf(f) ){ push_f(f,s); return; };
	  search_f(i,j,f->l,s);
	  search_f(i,j,f->r,s);
	};
    };
    
    // same thing here f is a leaf
    void search_s( rid_t    const i, rid_t    const j,  qtree const f, qtree s )
    {
      if ( Intersectp( f->box, s->box ) )
	{
	  split(j,s);
	  if ( leaf(s) ) { push_s(f,s); return; };
	  search_s(i,j,f,s->l);
	  search_s(i,j,f,s->r);
	};
    };
    
    // search generic case: both f and s are root node
    void search( const rid_t i, const rid_t j, qtree f, qtree s )
    {
      // if there bounding boxes intersect
      if ( Intersectp(f->box,s->box) )
	{
	  // compute if needed the next level in the hierarchy
	  split (i,f), split(j,s);
	  // if f and s are root node
	  if ( !(f->leaf()) && !(s->leaf()) )
	    {
	      // check all the possible intersection between  sons
	      search(i,j,f->l,s->l), search(i,j,f->l,s->r);
	      search(i,j,f->r,s->l), search(i,j,f->r,s->r);
	      return;
	    };
	  
	  // s is leaf (not f) switch to specialized case
	  if ( !leaf(f) ) 
	    {
	      search_f(i,j,f->l,s);
	      search_f(i,j,f->r,s);
	      return;
	    };
	  // same thing with f
	  if ( !leaf(s) ) 
	    {
	      search_s(i,j,f,s->l);
	      search_s(i,j,f,s->r);
	      return;
	    };
	  // two unit quad had been reached push the conflict
	  // the function push( qtree, qtree ) compute the intersection on the fly if any
	  push(f,s);
	};
    };

    // launch the search for region i vs region j
    void search( rid_t i, rid_t j )
    {
      curr_config = new std::vector< dcurve* >();
      search( i, j, this->trees[i], this->trees[j] );
    };
    

//     void print( dcurve * c )
//     {
//       for ( list2_t::iterator it = c->left.begin(); it != c->left.end(); it ++ )
// 	{
// 	  for ( unsigned i = 0; i < 4; i++ )
// 	    cout << (*it)[i] << " ";
// 	  cout << endl;
// 	};
//     };

    // try to reduce the number of double curves
    void find_branches( rid_t id0, rid_t id1 )
    {
      branches( id0, id1 ) = reduce( curr_config );
      delete curr_config;
    };
    
    // stats structure
    struct 
    {
      double search_time;
      double recons_time;
    } stats;
  
    std::list< dcurve * >  *result;

    point2 ** lefts;
    point2 ** rights;

    unsigned * sizes;
    unsigned nbcurves;
    unsigned nbpoints;
    unsigned conflict_count;
    
    double _st;
    dsearch_t( FunctionType * f , unsigned w, unsigned h, bool cnf = false  ) 
      : qseg_t(f, w, h ) 
    {
      m_cnf = cnf;
      _st = time();
      unsigned i,j;
      conflict_count = 0;
      nbpoints = 0;
      _branches = new std::list< dcurve * > *[this->regions.size()*this->regions.size()];
      memset(_branches,0,sizeof( std::list< dcurve * > * )*
	     this->regions.size()*this->regions.size() );
      
      for ( i = 0; i < this->regions.size() ; i++ )
	for ( j = i+1; j < this->regions.size() ; j++   )
	  {
#ifndef VOISINS_AUSSI
	    if ( !this->grp->neighbors(i,j) )
#endif
	      {
		//cout << i << " " << j << endl;
		search(i,j);
		find_branches(i,j);
	      };
	  };
      
      //      cout << "search: ok\n";
      std::vector< dcurve * > * tmp = new std::vector<dcurve*>();

      for ( i = 0; i < this->regions.size(); i++ )
	for ( j = i+1; j < this->regions.size(); j++ )
	  {
	    if ( branches(i,j) )
	      {
		for ( typename std::list<dcurve*>::iterator it = branches(i,j)->begin(); it != branches(i,j)->end(); it ++ )
		  {
		    tmp->push_back( *it );
		  };
		delete branches(i,j);
	      }
	  };
      
      result = reduce2( tmp );
      delete tmp;
      nbcurves = 0;
      lefts = rights = 0;
      sizes = 0;
      if ( result )
	{
	  nbpoints = 0;
	  typename std::list<dcurve *>::iterator it;
	  nbcurves = result->size();

	  sizes  = new unsigned[nbcurves];
	  
	  for ( i = 0, it = result->begin(); it != result->end(); it++ , i++ )
	    {
	      sizes[i] = (*it)->left.size();
	      nbpoints+= sizes[i];
	    };
	  
	  lefts  = new point2*[nbcurves];
	  rights = new point2*[nbcurves];
	  unsigned c_n = 0;
	  unsigned k;
	  for (  it = result->begin(); it != result->end(); it ++, c_n++  )
	    {
	      lefts[c_n]  = new point2[sizes[c_n]];
	      rights[c_n] = new point2[sizes[c_n]];
	      typename list2_t::iterator itcr, itcl;
	      for ( k = 0, itcl = (*it)->left.begin(), itcr = (*it)->right.begin(); 
		    itcl != (*it)->left.end(); 
		    itcl++ , itcr++, k++ )
		{

		  lefts[c_n][k] = *itcl;
		  rights[c_n][k] = *itcr;
		};
	    };
	  
	  for ( typename std::list<dcurve*>::iterator it = result->begin(); it!= result->end(); it++ )
	    {
	      delete *it;
	    };
	  delete result;
	};
      delete[] _branches;
      std::cout << " recherche = " << (time()-_st) << std::endl;
    };

    void GnuPlotOutput( char * name )
    {
      std::ofstream out(name);
      for ( unsigned i = 0; i < nbcurves; i++ )
	{
	  for ( unsigned k = 0; k < sizes[i]; k++ )
	    {
	      out << lefts [i][k][0] << " " << lefts[i][k][1] << std::endl;
	      out << rights[i][k][0] << " " << rights[i][k][1] << std::endl;
	    };
	};
    };
    void scale()
    {
      number um = this->Fct->umin();
      number vm = this->Fct->vmin();
      for ( unsigned i = 0; i < nbcurves; i++ )
	{
	  for ( unsigned k = 0; k < sizes[i]; k++ )
	    {
	      lefts[i][k][0] = this->m_du*lefts [i][k][0] + um;
	      lefts[i][k][1] = this->m_dv*lefts [i][k][1] + vm;
	      rights[i][k][0] = this->m_du*rights[i][k][0] + um;
	      rights[i][k][1] = this->m_dv*rights[i][k][1] + vm;
	    };
	};
    };
    ~dsearch_t()
    {
      if ( sizes ) delete[] sizes;
      if ( lefts ) 
	{
	  for ( unsigned i = 0; i < nbcurves; i++ )
	    delete[] lefts[i];
	  delete[] lefts;
	};
      if ( rights ) 
	{
	  for ( unsigned i = 0; i < nbcurves; i++ )
	    delete[] rights[i];
	  delete[] rights;
	};
    };

  };
};

#endif
