#include <synaps/shape/gaia/lsegment.h>

__BEGIN_NAMESPACE_SYNAPS

namespace gaia
{
  void lsegment::pushm(vcode_t code, coord_t j)
  {
    marks[_size].code = code;
    marks[_size].j    = j;
    marks[_size].head =-1;
    marks[_size].next = 0;
    _size++;
  }
  
  void lsegment::addneighbors( mark_t * m0, mark_t * m1 )
  {
    if ( coherent_code( m0->code, m1->code )  )
      {
	grp->add_link(m0->head,m1->head);
	grp->add_link(m1->head,m0->head);
      };
  };
  
  void lsegment::pushr( coord_t i, mark_t * p )
  {
    p->head =  regions.size();
    regions.push_back( region_t() );
    region_t * r =&(regions.back());
    r->_umin = i;
    r->_umax = 1;
    r->data = (void*)p;
    r->_code = p->code;
    grp->add_node();
  };
  
  void lsegment::lines_changes()
  {
    int i,j;
    lines = (unsigned*)(malloc(sizeof(unsigned)*nrows()));
    unsigned max_marks = nrows()*ncols();
    while ( !(marks = (mark_t*)(malloc(sizeof(mark_t)*max_marks))) ) max_marks >>= 1;
    vector3 * ptr  = base();
    for ( i = 0; i < this->nrows()-1; i++ )
      {
	lines[i] = nmarks();
	vcode_t code = vcode(*ptr,*(ptr+ncols()),*(ptr+1));
	pushm(code,0);
	for (  j = 0; j < ncols()-1; j++, ptr++ )
	  {
	    vcode_t tmp = vcode(*ptr,*(ptr+ncols()),*(ptr+1));
	    if ( tmp != code )
	      { code = tmp; pushm(code,j); };
	  };
	pushm(code,ncols()-1);
	ptr++;
      };
    lines[nrows()-1] = nmarks();
    marks = (mark_t*)realloc(marks,sizeof(mark_t)*nmarks());
  };

  
  void lsegment::find_regions()
  {
    mark_t * p;
    for ( p = begin(0); p != end(0); p++ )
      pushr(0,p);
    for ( p = begin(0)+1; p != end(0); p++ )
      addneighbors(p,p-1);
    
    mark_t * neighbors[ncols()];
    unsigned n_neigh;
    coord_t i;
#define INTERVALTEST ( std::abs(std::min(down->b(),up->b())-std::max(down->a(),up->a())) > 1 )
    for ( i = 0; i < nrows()-2; i++ )
      {
	mark_t * startup = begin(i);
	for ( mark_t * down = begin(i+1); down != end(i+1); down++ )
	  {
	    while ( startup->b() <= down->a() ) startup++;
	    mark_t * up = startup;
	    n_neigh = 0;
	    while( down->b() > up->a() )
	      if ( (up->next == 0) && ( up->code == down->code ) && INTERVALTEST)
		{ 
		  regions[up->head]._umax++;
		  up->next = down;
		  down->head = up->head, up++; 
		  break; 
		}
	      else 
		neighbors[n_neigh++] = up, up++;
	    
	      if ( down->head == -1 ) pushr( i+1, down );
	    
	      for ( unsigned k = 0; k < n_neigh; k ++ ) addneighbors(down,neighbors[k]);
	      while( down->b() > up->a() ) addneighbors(down,up), up++;
	  };
      };
  };

  void lsegment::promote( region_t& r )
  {
    mark_t     * mpointer;
    bounds_t * bpointer; 
    mpointer = (mark_t*)(r.data);
    r.data   = (void*)(malloc(sizeof(bounds_t)*(r._umax)));
    bpointer = (bounds_t*)(r.data);
    
    for ( unsigned i = 0; i < r._umax; i++ )
      {
	bpointer[i][0] = mpointer->a();
	bpointer[i][1] = mpointer->b();
	mpointer = mpointer->next;
      };
    
    r._umax += r._umin;
    r.data  = (void*)(((bounds_t*)(r.data))-r._umin);
  };
  
  void lsegment::convert_regions()
    {
      for ( unsigned i = 0; i < regions.size(); i++ )
	promote( regions[i] );
    };
  
  double _st;
  lsegment::lsegment( const IParametricSurface * s , unsigned _w, unsigned _h ) : sample(s,_w,_h)
  {
    _st = time();
    _size = 0;
    lines_changes  ();
    grp = new graph_t();
    find_regions   ();
    convert_regions();
    delete[] marks;
    delete[] lines;
    std::cout << "l = " << (time()-_st) << std::endl;
  };
  
  lsegment::~lsegment()
  {
    delete grp;
    for ( unsigned i = 0; i < regions.size(); i++ )
      rfree(regions[i]);
  };
};
	
__END_NAMESPACE_SYNAPS
