/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1161

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1161(SISLObject *po1,double *cmax,double aepsge,SISLIntdat **pintdat,int *jstat)
#else
void s1161(po1,cmax,aepsge,pintdat,jstat)
     SISLObject *po1;
     double *cmax;
     double aepsge;
     SISLIntdat **pintdat;
     int    *jstat;
#endif
                                     
{
  int    klevel=0;     
  int    knum=0;       
  int    kpar;         
  int    ki;               
  int    kedge;        
  int idim  = 1;       
  int kstat = 0;       
  int kpos  = 0;       
  double tpar;         
  SISLEdge   *qedge[2];        
  SISLObject *qdum = SISL_NULL;     
  SISLObject *qob  = SISL_NULL;     
  SISLIntdat *qintdat = SISL_NULL;  
  
  qedge[0] = SISL_NULL;
  qedge[1] = SISL_NULL;
  
  if (po1->iobj == SISLPOINT) 
    {
      
      
      
      if (po1->p1->idim != idim ) goto err106;
      
      
      tpar = po1->p1->ecoef[0] - *cmax;
      
      if (fabs(tpar) <= aepsge)
	
        
	*jstat = 1;         
      
      else if (tpar > (double)0.0)
	{
	  
	  
	  *jstat = 2;         
	  *cmax   = po1->p1->ecoef[0];
	}
      
      else 
	
	*jstat = 0;         
      
      
      if ( *jstat > 0 )
	{
	  SISLIntpt *qt;
	  
	  
	  qt = newIntpt(0,cmax,DZERO);
	  if (qt == SISL_NULL) goto err101;
	  
	  
	  s6idnpt(pintdat,&qt,1,&kstat);
	  if (kstat < 0) goto error;
	}
      
    }
  
  
  else if (po1->iobj > SISLPOINT)
    {
      
      
      
      *jstat = 0;
      
      s1190(po1,cmax,aepsge,&kstat);
      if (kstat == 1) goto out;
      
      
      
      if ((qdum = newObject(SISLPOINT)) == SISL_NULL) goto err101;
      
      
      
      kedge  = 2 * po1->iobj;
      kpar   = kedge/2;
      
      
      if ((qedge[0] = newEdge(kedge)) == SISL_NULL) goto err101;
      
      
      for (ki=0; ki<kedge; ki++)
	{
	  
	  
	  kpar   = ((ki == kedge/2) ? kedge/2-1:kpar-1);
	  
	  
	  if ((qob = newObject(po1->iobj - 1)) == SISL_NULL) goto err101;	
	  
	  
	  if (po1->iobj == SISLCURVE)
	    
	    
	    s1438(po1->c1,ki,&(qob->p1),&tpar,&kstat);
	  
	  else if (po1->iobj == SISLSURFACE)
	    
	    
	    s1435(po1->s1,ki,&(qob->c1),&tpar,&kstat);
	  
	  else
	    
	    goto err121;
	  
	  if (kstat < 0) goto error;
	  
	  
	  s1161(qob,cmax,aepsge,&qintdat,&kstat);
	  if (kstat < 0) goto error;
	  
	  if (kstat == 2)
	    {
	      
	      
	      if (*pintdat != SISL_NULL)
		{
		  freeIntdat(*pintdat);
		  *pintdat = SISL_NULL;
		}
	      
	      if (qedge[0] != SISL_NULL)
		{
		  
		  freeEdge(qedge[0]);
		  if ((qedge[0] = newEdge(kedge)) == SISL_NULL) goto err101;	      
		}
	      
	    }  
	  
	  
	  if (kstat)
	    {
	      
	      
	      *jstat = max(*jstat,kstat);         
	      
	      
	      
	      
	      s6idput(pintdat,qintdat,kpar,tpar,&kstat);
	      if (kstat < 0) goto error;
	      
	      
	      s6idedg(po1,qdum,1,kpar+1,tpar,*pintdat,
		      &(qedge[0]->prpt[ki]),&(qedge[0]->ipoint),&kstat);
	      if (kstat < 0) goto error;
	    }
	  
	  if (qintdat != SISL_NULL) freeIntdat(qintdat);
	  qintdat = SISL_NULL;
	  freeObject(qob);
	}	  
      
      
      
       
      
      
      
      if (po1->o1 == SISL_NULL) po1->o1 = po1;
      
      
      s1162(po1,cmax,aepsge,pintdat,qedge,klevel,knum,&kstat);
      if (kstat < 0)  goto error;
      *jstat = max(*jstat,kstat);
      
      
      s6idlis(po1,po1,pintdat,&kstat);
      if (kstat < 0)  goto error;
    }
  
  else 
    
    goto err121;
  
  
  goto out; 
  
  
  
  
  
  
 err101: *jstat = -101;
  s6err("s1161",*jstat,kpos);
  goto out;
  
  
 err106: *jstat = -106;
  s6err("s1161",*jstat,kpos);
  goto out;
  
  
 err121: *jstat = -121;
  s6err("s1161",*jstat,kpos);
  goto out;
  
  
  error : *jstat = kstat;
  s6err("s1161",*jstat,kpos);
  goto out;
  
 out:
  
  if (qedge[0] != SISL_NULL) freeEdge(qedge[0]);
  
  
  if (qdum != SISL_NULL) freeObject(qdum);
  
}


