/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1224

#include "sislP.h"
 


#if defined(SISLNEEDPROTOTYPES)
void 
   s1224(double et1[], double et2[], int ik1, int ik2, int in1, int in2, 
	 int ibase1, int ibase2, double par[], int ider, 
	 double ebder[], int *jstat)
#else
void s1224(et1, et2, ik1, ik2, in1, in2, ibase1, ibase2, par, ider, 
	   ebder, jstat)
     double et1[];
     double et2[];
     int    ik1;
     int    ik2;
     int    in1;
     int    in2;
     int    ibase1;
     int    ibase2;
     double par[];
     int    ider;
     double ebder[];
     int    *jstat;
#endif
                                     
{
   int kstat=0;          
   int kpos=0;           
   int ki, kj, kl, kn;   
   int knumb1;           
   int knumb2;           
   double sdum1[100];    
   double sdum2[100];    
   double *sder1 = SISL_NULL; 
   double *sder2 = SISL_NULL; 
   

   
   
   knumb1 = ik1*(ider + 1);
   knumb2 = ik2*(ider + 1);
   
   if (knumb1 > 100)
   {
      if ((sder1 = newarray(knumb1, double)) == SISL_NULL)
	 goto err101;
   }
   else
      sder1 = &sdum1[0];
   
   if (knumb2 > 100)
   {
      if ((sder2 = newarray(knumb2, double)) == SISL_NULL)
	 goto err101;
   }
   else
      sder2 = &sdum2[0];
   
   
   
   s1222(et1, ik1, in1, ibase1, par[0], ider, sder1, &kstat);
   if (kstat < 0) goto error;

   
   
   s1222(et2, ik2, in2, ibase2, par[1], ider, sder2, &kstat);
   if (kstat < 0) goto error;
   
   
   
   for (ki = 0, kl = 0; ki < (ider + 1); ki++)
   {
      for (kj = ki, kn = 0; kj > -1; kj--, kn++, kl++)
	 ebder[kl] = sder1[kj]*sder2[kn];
   }
   
   
   
   goto out;
   
   
   
   err101: 
      *jstat = -101;
   s6err("s1224",*jstat,kpos);
   goto out;
   
   
   
   error:  
      *jstat = kstat;
   s6err("s1224",*jstat,kpos);
   goto out;
   
   out: 
      if (knumb1 > 100 && sder1 != SISL_NULL) freearray(sder1);
      if (knumb2 > 100 && sder2 != SISL_NULL) freearray(sder2);
      
      return;
}

