/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1232

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1232(double et1[],int in,int ik,
	   double afak1,double afak2,double et2[],int *jstat)
#else
void s1232(et1,in,ik,afak1,afak2,et2,jstat)
     double et1[];
     int    in;
     int    ik;
     double afak1;
     double afak2;
     double et2[];
     int    *jstat;
#endif
                   
{
  int kpos = 0;   
  int ki;         
  double tleng;   
  double tstart;  
  double tend;    
  
  
  
  if (ik < 1) goto err110;
  if (in < ik) goto err111;
  
  
  
  tleng = et1[in] - et1[ik-1];
  if (tleng <= (double)0.0) goto err112;  
  
  
  
  memcopy(et2,et1,in+ik,double);
  
  if (afak1 > (double)0.0)
    {
      
      
      
      tstart = et1[ik-1] - tleng*afak1;
      for (ki=0; ki<ik; ki++) et2[ki] = tstart;
    }
  
  if (afak2 > (double)0.0)
    {
      
      
      
      tend = et1[in] + tleng*afak2;
      for (ki=in; ki<in+ik; ki++) et2[ki] = tend;
    }
  
  
  
  *jstat = 0;
  goto out;
  
  
  
 err110: *jstat = -110;
  s6err("s1232",*jstat,kpos);
  goto out;
  
  
  
 err111: *jstat = -111;
  s6err("s1232",*jstat,kpos);
  goto out;
  
  
  
 err112: *jstat = -112;
  s6err("s1232",*jstat,kpos);
  goto out;
  
 out: return;
}

