/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1343

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1343(SISLCurve *pc,double eeps[],int ilend,int irend,double aepsco,
	   int itmax, SISLCurve **rc,int *jstat)
#else
void s1343(pc,eeps,ilend,irend,aepsco,itmax,rc,jstat)
     SISLCurve  *pc;
     double eeps[];
     int    ilend;
     int    irend;
     double aepsco;
     int    itmax;
     SISLCurve  **rc;
     int    *jstat;
#endif
{
  int i,j;                   
  int index = 0;
  int idim = pc->idim;       
  int stat = 0;              
  int kpos = 0;
  int km;
  int leftknot = 0;
  double *error1 = SISL_NULL;
  double *error2 = SISL_NULL;
  double *epar = SISL_NULL;
  double *derive = SISL_NULL;
  double *kp = SISL_NULL;
  double *kder = SISL_NULL;
  SISLCurve *ocurve = SISL_NULL;
  SISLCurve *qc_kreg = SISL_NULL;    
  
  
  

  if (!pc) goto err150;
  s1707(pc, &stat);
  if (stat<0) goto error;
  
  
 
  if (pc->cuopen == SISL_CRV_PERIODIC)
  {
     make_cv_kreg(pc, &qc_kreg, &stat);
     if (stat < 0) goto error;
     
     
     
     ilend = MAX(ilend, 2);
     irend = MAX(irend, 2);
  }
  else 
     qc_kreg = pc;
  
  

  if (qc_kreg->in < qc_kreg->ik || qc_kreg->ik < 1 || idim < 1) goto err103;
  if (qc_kreg->ik < 5)
    { 
      

      s1750(qc_kreg, 4, &ocurve, &stat);
      if (stat < 0) goto error;

      

      if( (error2 = newarray( idim, DOUBLE )) == SISL_NULL) goto err101;
      s1340( ocurve, eeps, ilend, irend, aepsco, itmax, rc, 
	    error2, &stat);
      if (stat < 0) goto error;

      

      goto out;
    }

  

  if( (error1 = newarray(idim, DOUBLE)) == SISL_NULL) goto err101;

  for (i=0; i<idim; i++)
    error1[i] = 0.5*eeps[i];

  

  s1355( qc_kreg, error1, &epar, &km, &stat );
  if (stat < 0) goto error;

  

  derive = newarray( idim * 2, DOUBLE );
  kp     = newarray( idim * km, DOUBLE );
  kder   = newarray( idim * km, DOUBLE );
  if (derive == SISL_NULL || kp == SISL_NULL || kder == SISL_NULL) goto err101;

  for(i=0; i<km; i++)
    {
      if(epar[i] == epar[i+1])
	s1227( qc_kreg, 1, epar[i], &leftknot, derive, &stat );
      else
	s1221( qc_kreg, 1, epar[i], &leftknot, derive, &stat );

      for(j=0; j<idim; j++, index++)
	{
	  kp[index] = derive[j];
	  kder[index] = derive[j+idim];
	}
    }
  if (stat < 0) goto error;

  

  s1379( kp, kder, epar, km, idim, &ocurve, &stat );
  if (stat < 0) goto error;

  

  if( (error2 = newarray( idim, DOUBLE )) == SISL_NULL) goto err101;
  s1340( ocurve, error1, ilend, irend, aepsco, itmax, rc, 
	error2, &stat);
  if (stat < 0) goto error;

  
  
  if (pc->cuopen == SISL_CRV_CLOSED || 
      pc->cuopen == SISL_CRV_PERIODIC)
     (*rc)->cuopen = SISL_CRV_CLOSED;
  
  
  
  *jstat = 0;
  goto out;
  
  

 err101: 
   *jstat = -101;
   s6err("s1343",*jstat,kpos);
   goto out;
  
  

  err103: 
    *jstat = -103; 
    s6err("s1343",*jstat,kpos);
    goto out;
  
  

  err150: 
    *jstat = -150;
    s6err("s1343",*jstat,kpos);
    goto out;
  
  

  error: 
    *jstat = stat;
    s6err("s1343",*jstat,kpos);
    goto out;

  

  out:
    

    if( error1 != SISL_NULL) freearray(error1);
    if( error2 != SISL_NULL) freearray(error2);
    if( epar   != SISL_NULL) freearray(epar);
    if( derive != SISL_NULL) freearray(derive);
    if( kp     != SISL_NULL) freearray(kp);
    if( kder   != SISL_NULL) freearray(kder);

    

    if( ocurve != SISL_NULL) freeCurve(ocurve);
    if (qc_kreg != SISL_NULL && qc_kreg != pc) freeCurve(qc_kreg);

    return;
}

