/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1354

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1354(SISLCurve *oldcurve,SISLCurve *rankcurve,rank_info *ranking,
	   double eps[],double epsco[],int startfix,int endfix,int mini,
	   int maxi,SISLCurve **newcurve,double maxerr[],int *stat)
#else
void s1354(oldcurve, rankcurve, ranking, eps, epsco,
	   startfix, endfix, mini, maxi, newcurve, maxerr, stat)
     SISLCurve	*oldcurve;
     SISLCurve	*rankcurve;
     rank_info	*ranking;
     double	eps[];
     double	epsco[];
     int	startfix;
     int	endfix;
     int	mini;
     int	maxi;
     SISLCurve	**newcurve;
     double	maxerr[];
     int	*stat;
#endif
{
  int k = oldcurve->ik;           
  int dim = oldcurve->idim;       
  int mprio = rankcurve->in;
  int *prio = ranking->prio;
  int *group = ranking->groups;
  int antrem = ranking->antrem;
  int antgr = ranking->antgr;
  char *del_array;                
  char big, bigco;                
  int lstat=0;			  
  int pos=0;			  
  int nlim = MAX(k, startfix+endfix); 

				
  int i, start, stop, indx, count, r, p, hn;
  SISLCurve *hcurve = SISL_NULL;
  double h;
  double *local_err = SISL_NULL, *l2_err = SISL_NULL, *ltau = SISL_NULL;

  

  del_array = newarray(mprio-k, char);
  if (del_array == SISL_NULL) goto err101;

  

  *newcurve = newCurve(mprio, k, rankcurve->et, rankcurve->ecoef, 1, dim, 1);
  if (newcurve == SISL_NULL) goto err101;

  

  while (mini+1 < maxi)
  {

    

    for (i=0; i<mprio-k; i++) del_array[i] = 0;

    

    start = 0;
    stop = group[0];
    count = 0;

    

    while (stop <= antrem)
    {
      for (i=start; i<stop; i++) del_array[prio[i]] = 1;

      count++;

      if (count < antgr)
      {
	start = stop;
	stop = group[count];
      }
      else
      {

	

	stop = stop + 1;
	start = stop + 1;
      }
    }

    

    r = stop - start;
    p = antrem - start;

    if (p > 0)
    {
      h = (double) (r+1) / (double) p;
      for (i=0; i<p; i++)
      {
	indx = start - 1 + (int) floor( h*(i+0.5)+0.5 );
	del_array[prio[indx]] = 1;
      }
    }

    

    hn = mprio - antrem ;

    

    if (ltau != SISL_NULL) freearray(ltau);
    ltau = newarray(hn+k, double);
    if (ltau == SISL_NULL) goto err101;

    

    for (i=0; i<k; i++)
    {
      ltau[i] = rankcurve->et[i];
      ltau[i+hn] = rankcurve->et[i+mprio];
    }

    

    for (indx=k, i=0; i<mprio-k; i++)
      if (!del_array[i]) ltau[indx++] = rankcurve->et[i+k];

    

    if (local_err != SISL_NULL) freearray(local_err);
    if (l2_err != SISL_NULL) freearray(l2_err);

    sh1365(oldcurve, ltau, k, hn, startfix, endfix,
	   &hcurve, &local_err, &l2_err, &lstat);
    if (lstat < 0) goto err;

    
    big = 0;
    bigco = 0;

    for (i=0; i<dim; i++)
    {
      big = big || (local_err[i] > eps[i]);
      bigco = bigco || (local_err[i] > epsco[i]);
    }

    

    big = big || (bigco && hn < nlim);

    if (big)
    {

      

      if (hcurve != SISL_NULL) freeCurve(hcurve);
      hcurve = SISL_NULL;
      maxi = antrem;
    }
    else
    {

      

      mini = antrem;
      if (*newcurve != SISL_NULL) freeCurve(*newcurve);
      *newcurve = hcurve;
      hcurve = SISL_NULL;
      for (i=0; i<dim; i++)  maxerr[i] = local_err[i];
    }

    

    antrem = mini + (maxi-mini)/2;
  }

  *stat = 0;

  goto out;



  

err101:
  *stat = -101;
  goto out;

  

err:
  *stat = lstat;
  s6err("s1354", *stat, pos);

  

out:
  if (hcurve != SISL_NULL) freeCurve(hcurve);
  if (del_array != SISL_NULL) freearray(del_array);
  if (local_err != SISL_NULL) freearray(local_err);
  if (l2_err != SISL_NULL) freearray(l2_err);
  if (ltau != SISL_NULL) freearray(ltau);

  return;
}

