/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1359

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1359(double egeo[],double aepsge,int idim,int inbinf,
	   int ipar,double epar[],SISLCurve **rcurve,int *jstat)
#else
void s1359(egeo,aepsge,idim,inbinf,ipar,epar,rcurve,jstat)
     double egeo[];
     double aepsge;
     int    idim;
     int    inbinf;
     int    ipar;
     double epar[];
     SISLCurve  **rcurve;
     int    *jstat;
#endif
{
  int kn;             
  int kk = 4;         
  int knt;            
  int kvert;          
  int kpos =1;        
  int kstat;          
  int ki,kj;          
  int kv1,kv2,kv3;    
  int kincre;         
  int kcycpos;        
  double *sprevp;     
  double *sprevt;     
  double *sprevc;     
  double *sprevr;     
  double snprevt[3];  
  double *scurp;      
  double *scurt;      
  double *scurc;      
  double *scurr;      
  double sncurt[3];   
  double tcos;        
  double tl1,tl2;     
  double tangle;      
  double tdist;       
  double tpar;        
  double *st = SISL_NULL;  
  double *scoef=SISL_NULL; 
  double tmpval=aepsge;
  double max_dist;
  
  
  if (idim != 2 && idim != 3) goto err105;

  

  if (inbinf < 2) goto err181;
  
  
  if (idim==2)
    kincre = 7;
  else
    kincre = 10;
  
  
  
  
  if (ipar==0)
    {
      double tmin,tmax,*sp;
      
      for (kj=0 ; kj<idim ; kj++)
        {
	  tmin = *(egeo+kj);
	  tmax = tmin;
	  for (ki=0,sp=egeo+kj ; ki < inbinf ; ki++,sp+=kincre)
            {
	      tmin = MIN(tmin,*sp);
	      tmax = MAX(tmax,*sp);
            }
	  tmpval = MAX(tmpval,(tmax-tmin));
        }
    }
  
  kn = 3*(inbinf-1) + 1;
  scoef = newarray(idim*kn,DOUBLE);
  if (scoef == SISL_NULL) goto err101;
  
  st = newarray(kk+kn,DOUBLE);                                                  
  if (st == SISL_NULL) goto err101;
  
  
  if (ipar==0)
    {
      epar[0] = DZERO;
    }
  
  st[0] = epar[0];
  st[1] = epar[0];
  st[2] = epar[0];
  st[3] = epar[0];
  
  
  memcopy(scoef,egeo,idim,DOUBLE);
  
  
  
  
  sprevp = egeo;
  sprevt = sprevp + idim;
  sprevc = sprevt + idim;
  sprevr = sprevc + idim;
  
  
  
  (void)s6norm(sprevt,idim,snprevt,&kstat);
  
  for (ki=1,knt=4,kvert=idim;ki<inbinf;ki++)
    {
      
      
      
      
      
      scurp = sprevp + kincre;
      scurt = sprevt + kincre;
      scurc = sprevc + kincre;
      scurr = sprevr + kincre;
      
      
      
      (void)s6norm(scurt,idim,sncurt,&kstat);
      
      
      
      tcos = s6scpr(snprevt,sncurt,idim);
      
      
      
      if (tcos >= DZERO)             
	tcos = MIN((double)1.0,tcos);
      else
	tcos = MAX((double)-1.0,tcos);
      
      tangle = fabs(acos(tcos));
      
      if (tangle < ANGULAR_TOLERANCE) tangle = DZERO;
      
      tdist = s6dist(sprevp,scurp,idim);
      
      
      
      
      if (DEQUAL(tangle,DZERO) || *sprevr < DZERO)
        {
	  
	  tl1 = tdist/(double)3.0;
        }
      else
        {
	  
	  tl1 = s1325(*sprevr,tangle);
        }
      
      
      
      if (DEQUAL(tangle,DZERO) || *scurr < DZERO)
        {
	  
	  tl2 = tdist/(double)3.0;
        }
      else
        {
	  
	  tl2 = s1325(*scurr,tangle);
        }
      
      
      
      

      if (tangle < 0.1)		max_dist = (double)0.35*tdist;
      else if (tangle < 0.35)	max_dist = (double)0.40*tdist;
      else if (tangle < 0.75)	max_dist = (double)0.50*tdist;
      else 			max_dist = (double)0.70*tdist;

      if ( tl1 > max_dist) tl1 = max_dist;
      if ( tl2 > max_dist) tl2 = max_dist;
      
                                                   
      
      
      if (ipar==0)
        {
	  
	  
	  if (DNEQUAL(*sprevr,(double)-1.0) && 
	      DNEQUAL(*scurr,(double)-1.0))
            {
	      tpar = (double)0.5*tangle*(*sprevr+*scurr);
            }
	  else if (DNEQUAL(*sprevr,(double)-1.0) && 
		   DEQUAL(*scurr,(double)-1.0))
            {
	      tpar = (double)0.5*(*sprevr*tangle + tdist);
            }
	  else if (DEQUAL(*sprevr,(double)-1.0) && 
		   DNEQUAL(*scurr,(double)-1.0))
            {
	      tpar = (double)0.5*(tdist + tangle*(*scurr));
            }
	  else
            {
	      tpar =  tdist;
            }
	  
	  tpar = MAX(tpar,tdist);
	  tpar = MAX(tpar,aepsge);
	  
	  
	  
          
	  
          if (tangle <= PIHALF)
	     tpar = MIN(tpar, ((double)1.1*tdist));
          else
	     tpar = MIN(tpar,tmpval);
	  
          
	  
	  if (DEQUAL((epar[ki-1]+tpar),epar[ki-1]))
            {
	      tpar = fabs(epar[ki-1])*(double)0.1;
            }
	  
	  if (DEQUAL(tpar,DZERO))
            {
	      tpar = (double)1.0;
            }
	  
	  epar[ki] = epar[ki-1] + tpar;
	  
        }
      
      
      st[knt]   = epar[ki];
      st[knt+1] = epar[ki];
      st[knt+2] = epar[ki];
      
      
      
      for (kj=0,kv1=kvert,kv2=kv1+idim,kv3=kv2+idim ; kj<idim ;
	   kj++,kv1++,kv2++,kv3++)
        {
	  scoef[kv1] = sprevp[kj] + tl1*sprevt[kj];
	  scoef[kv2] = scurp[kj]  - tl2*scurt[kj];
	  scoef[kv3] = scurp[kj];
        }
      
      
      sprevp = scurp;
      sprevt = scurt;
      sprevc = scurc;
      sprevr = scurr;
      for (kj=0;kj<idim;kj++) snprevt[kj] = sncurt[kj];
      
       
      
      if (DNEQUAL(epar[ki-1],epar[ki]))
        {
	  kvert+=(3*idim);
	  knt+=3;
        }
    }
  
  
  
  kn = kvert/idim;
  
  
  
  st[kn+kk-1] = st[kn+kk-2];
  
  

  for (ki=0, kcycpos=1 ; ki<idim ; ki++)
    if (egeo[ki] != egeo[(inbinf-1)*kincre+ki]) kcycpos=0;
  
  if (kcycpos ==1)
    {
      st[0] -= (st[kn] - st[kn-1]);
      st[kn+kk-1] += (st[kk] - st[kk-1]);
    }
  
  
  
  
  kpos = 1;
  *rcurve = SISL_NULL;
  *rcurve = newCurve(kn,kk,st,scoef,1,idim,1);
  if (*rcurve == SISL_NULL) goto err101;
  
  
  if (kcycpos)
    {
       test_cyclic_knots(st,kn,kk,&kstat);
       if (kstat<0) goto error;
      if (kstat == 2) (*rcurve)->cuopen = SISL_CRV_PERIODIC;
    }
  *jstat = 0;
  goto out;
  
  
  
 err101: *jstat = -101;
  s6err("s1359",*jstat,kpos);
  goto out;

  
  
 error: *jstat = kstat;
  s6err("s1359",*jstat,kpos);
  goto out;
  
  
  
 err105: *jstat = -105;
  s6err("s1359",*jstat,kpos);
  goto out;

  

  err181: *jstat = -181;
   s6err("s1359",*jstat,kpos);
   goto out;
  
  
 out:
  
  
  if (st != SISL_NULL)    freearray(st);
  if (scoef != SISL_NULL) freearray(scoef);
  
  
  return;
}

