/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1366

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1366(SISLSurf *ps, double aoffset, double aepsge, double amax, int idim,
	   double *eknot13, int in13, int ik13,
	   double *eknot24, int in24, int ik24,
	   SISLSurf **rs, int *jstat)
#else
void s1366(ps,aoffset,aepsge,amax,idim,eknot13,in13,ik13,
	   eknot24,in24,ik24,rs,jstat)
     SISLSurf   *ps;
     double aoffset;
     double aepsge;
     double amax;
     int    idim;
     double *eknot13;
     int    in13;
     int    ik13;
     double *eknot24;
     int    in24;
     int    ik24;
     SISLSurf   **rs;
     int   *jstat;
#endif
{
  int kstat = 0;     
  int kpos = 0;      
  int ki;            
  int kk1;           
  int kk2;           

  int kder = 1;      
  int klfs=0;       
  int klft=0;       
  int knbpnt=0;      
  int km1=1;         
  int km2=1;         
  int kain=0;        
  int kmaxis=512;    
  int kmaxik1=512;   
  int kmaxik2=512;   
  int kpar=3;        
  int lend[4];       
  int kopt=3;        
  int ktmax=10;      
  double *spar1;     
  double *spar2;     
  double seps[3];    
  double sedgeps[12];
  double tepsco;     
  double smxerr[3];  
  double *spnt=SISL_NULL; 
  double *stng1=SISL_NULL;
  double *stng2=SISL_NULL;
  double *scrss=SISL_NULL; 
  double spar[2];    
  double sder[27];   

  
  tepsco = (double)0.000001;

  for (ki=0; ki<3; ki++) seps[ki] = aepsge;

  for (ki=0; ki<4; ki++) lend[ki] = 1;

  for (ki=0; ki<12; ki++) sedgeps[ki] = aepsge;

  

  spnt = newarray(idim*kmaxis,DOUBLE);
  if (spnt == SISL_NULL) goto err101;

  stng1 = newarray(idim*kmaxis,DOUBLE);
  if (stng1 == SISL_NULL) goto err101;

  stng2 = newarray(idim*kmaxis,DOUBLE);
  if (stng2 == SISL_NULL) goto err101;

  scrss = newarray(idim*kmaxis,DOUBLE);
  if (scrss == SISL_NULL) goto err101;

  

  spar1 = newarray(kmaxik1,DOUBLE);
  if (spar1 == SISL_NULL) goto err101;

  

  spar2 = newarray(kmaxik2,DOUBLE);
  if (spar2 == SISL_NULL) goto err101;

  
  spar[0] = eknot13[ik13-1];
  spar[1] = eknot24[ik24-1];

  
  s1367(ps,aoffset,aepsge,idim,spar,kder,&klfs,&klft,sder,&kstat);
  if (kstat<0) goto error;

  
  for (ki=0; ki<idim; ki++)
  {
    spnt[kain] = sder[ki];
    stng1[kain] = sder[ki+idim];
    stng2[kain] = sder[ki+2*idim];
    scrss[kain] = sder[ki+3*idim];
    kain += 1;
  }

  
  spar1[km1-1] = spar[0];
  spar2[km2-1] = spar[1];

  knbpnt += 1;

  for (kk1=ik13; kk1<=in13; kk1++)
  {

    
    if (DNEQUAL(spar[0],eknot13[kk1]))
    {

      
      spar[0] = eknot13[kk1];
      s1367(ps,aoffset,aepsge,idim,spar,kder,&klfs,&klft,sder,&kstat);
      if (kstat<0) goto error;

      
      knbpnt += 1;
      km1 += 1;

      
      if (knbpnt>=kmaxis)
      {
	kmaxis += 512;
	spnt  = increasearray(spnt,idim*kmaxis,DOUBLE);
	if ( spnt == SISL_NULL )  goto err101;
	stng1 = increasearray(stng1,idim*kmaxis,DOUBLE);
	if ( stng1 == SISL_NULL )  goto err101;
	stng2 = increasearray(stng2,idim*kmaxis,DOUBLE);
	if ( stng2 == SISL_NULL )  goto err101;
	scrss = increasearray(scrss,idim*kmaxis,DOUBLE);
	if ( scrss == SISL_NULL )  goto err101;
      }

      
      if (km1>=kmaxik1)
      {
	kmaxik1 = km1 + 512;	
	spar1 = increasearray(spar1,kmaxik1,DOUBLE);
	if ( spar1 == SISL_NULL )  goto err101;
      }

      for (ki=0; ki<idim; ki++)
      {
	spnt[kain] = sder[ki];
	stng1[kain] = sder[ki+idim];
	stng2[kain] = sder[ki+2*idim];
	scrss[kain] = sder[ki+3*idim];
	kain += 1;
      }

      
      spar1[km1-1] = spar[0];
    }
  }


  for (kk2=ik24; kk2<=in24; kk2++)
  {

    
    if (DNEQUAL(spar[1],eknot24[kk2]))
    {

      
      spar[0] = eknot13[ik13-1];
      klfs = 0;

      
      spar[1] = eknot24[kk2];
      s1367(ps,aoffset,aepsge,idim,spar,kder,&klfs,&klft,sder,&kstat);
      if (kstat<0) goto error;

      
      knbpnt += 1;
      km2 += 1;

      
      if (knbpnt>=kmaxis)
      {
	kmaxis += 512;
	spnt  = increasearray(spnt,idim*kmaxis,DOUBLE);
	if ( spnt == SISL_NULL )  goto err101;
	stng1 = increasearray(stng1,idim*kmaxis,DOUBLE);
	if ( stng1 == SISL_NULL )  goto err101;
	stng2 = increasearray(stng2,idim*kmaxis,DOUBLE);
	if ( stng2 == SISL_NULL )  goto err101;
	scrss = increasearray(scrss,idim*kmaxis,DOUBLE);
	if ( scrss == SISL_NULL )  goto err101;
      }

      
      if (km2>=kmaxik2)
      {
	kmaxik2 = km2 + 512;  
	spar2 = increasearray(spar2,kmaxik2,DOUBLE);
	if ( spar1 == SISL_NULL )  goto err101;
      }

      for (ki=0; ki<idim; ki++)
      {
	spnt[kain] = sder[ki];
	stng1[kain] = sder[ki+idim];
	stng2[kain] = sder[ki+2*idim];
	scrss[kain] = sder[ki+3*idim];
	kain += 1;
      }

      
      spar2[km2-1] = spar[1];

      for (kk1=ik13; kk1<=in13; kk1++)
      {

	
	if (DNEQUAL(spar[0],eknot13[kk1]))
	{
	  spar[0] = eknot13[kk1];

	  
	  s1367(ps,aoffset,aepsge,idim,spar,kder,
		&klfs,&klft,sder,&kstat);
	  if (kstat<0) goto error;

	  
	  knbpnt += 1;

	  
	  if (knbpnt>=kmaxis)
	  {
	    kmaxis += 512;
	    spnt  = increasearray(spnt,idim*kmaxis,DOUBLE);
	    if ( spnt == SISL_NULL )  goto err101;
	    stng1 = increasearray(stng1,idim*kmaxis,DOUBLE);
	    if ( stng1 == SISL_NULL )  goto err101;
	    stng2 = increasearray(stng2,idim*kmaxis,DOUBLE);
	    if ( stng2 == SISL_NULL )  goto err101;
	    scrss = increasearray(scrss,idim*kmaxis,DOUBLE);
	    if ( scrss == SISL_NULL )  goto err101;
	  }

	  for (ki=0; ki<idim; ki++)
	  {
	    spnt[kain] = sder[ki];
	    stng1[kain] = sder[ki+idim];
	    stng2[kain] = sder[ki+2*idim];
	    scrss[kain] = sder[ki+3*idim];
	    kain += 1;
	  }
	}
      }
    }
  }

  

  s1347(spnt,stng1,stng2,scrss,km1,km2,idim,kpar,spar1,spar2,seps,lend,
	sedgeps,tepsco,kopt,ktmax,rs,smxerr,&kstat);
  if (kstat<0) goto error;

  

  *jstat = 0;
  goto out;

  

err101:
  *jstat = -101;
  s6err("s1366",*jstat,kpos);
  goto out;

  

error:
  *jstat = kstat;
  s6err("s1366",*jstat,kpos);
  goto out;

out:
  if (spnt != SISL_NULL) freearray(spnt);
  if (stng1 != SISL_NULL) freearray(stng1);
  if (stng2 != SISL_NULL) freearray(stng2);
  if (scrss != SISL_NULL) freearray(scrss);
  if (spar1 != SISL_NULL) freearray(spar1);
  if (spar2 != SISL_NULL) freearray(spar2);

  return;
}

