/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1366

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1367(SISLSurf *ps,double aoffset,double aepsge,int idim,double epar[],
	   int ider,int *ilfs,int *ilft,double eder[],int *jstat)
#else
void s1367(ps,aoffset,aepsge,idim,epar,ider,ilfs,ilft,eder,jstat)
     SISLSurf   *ps;
     double aoffset;
     double aepsge;
     int    idim;
     double epar[];
     int    ider;
     int    *ilfs;
     int    *ilft;
     double eder[];
     int    *jstat;
#endif
{
  int kstat = 0;     
  int kpos = 0;      
  int kdim;          
  int ki;            
  
  int kder = ider+1; 
  int knmb;          
  double snorm[3];   
  double utang[3];   
  double vtang[3];   
  double sdum[27];   
  double *sder=SISL_NULL; 
  
  if (idim !=3) goto err105;
  
  
  
  kdim = ps->idim;
  knmb = kdim*kder*kder;
  if (knmb>27)
    sder = newarray(knmb,DOUBLE);
  else
    sder = &sdum[0];
 

  

  for (ki=0;ki<knmb;ki++) eder[ki] = DZERO;


  if (DEQUAL(aoffset,DZERO))
    {
      
      
      s1424(ps,kder,kder,epar,ilfs,ilft,sder,&kstat);
      if (kstat<0) goto error;
      
      
      
      for (ki=0;ki<idim;ki++)
        {
	  eder[ki] = sder[ki];
	  eder[ki+kdim] = sder[ki+kdim];
	  eder[ki+2*kdim] = sder[ki+3*kdim];
	  eder[ki+3*kdim] = sder[ki+4*kdim];
        }
    }
  else
    {
      
      double tlen1,tlen2,tnorm,tang=(double)0.0;
      
      
      
      s1424(ps,kder,kder,epar,ilfs,ilft,sder,&kstat);
      if (kstat<0) goto error;
      
      
      
      
      
      s6crss(sder+kdim,sder+3*kdim,snorm);
      
      
      
      tlen1 = s6length(sder+kdim,kdim,&kstat);
      tlen2 = s6length(sder+3*kdim,kdim,&kstat);
      tnorm = s6length(snorm,kdim,&kstat);
      
      
      
      if (tlen1 != DZERO && tlen2 != DZERO && tnorm != DZERO)
        tang = tnorm/(tlen1*tlen2);

      
      if (tang == (double)0.0) *jstat = 2;
      else if (tang <= ANGULAR_TOLERANCE) *jstat = 1;   
      else *jstat = 0;
      

      

      if (tnorm < 0.00000001)
      {
	  

	  s6degnorm(ps,2,epar,sder,utang,vtang,snorm,&kstat);
	  if(kstat < 0) goto error;
	  if(kstat == 0)
	  {
              tnorm = s6length(snorm,kdim,&kstat);
	      *jstat = 3;
              for (ki=0;ki<idim;ki++)
		       eder[ki]=sder[ki]+aoffset*snorm[ki]/tnorm;
	  }
	  else
	  {
              for (ki=0;ki<idim;ki++) eder[ki]=sder[ki];
	  }
	  goto out;
      }

      
      
      
      
      for (ki=0;ki<idim;ki++) eder[ki]=sder[ki]+aoffset*snorm[ki]/tnorm;
      
      

      if (ider > 0)
	{
	  
	  double *spnt;      
	  double tder1,tder2;
	  double snoru1[3];  
	  double snoru2[3];  
	  double snoru[3];   
	  double snorv1[3];  
	  double snorv2[3];  
	  double snorv[3];   
	  double snoruv1[3]; 
	  double snoruv2[3]; 
	  double snoruv3[3]; 
	  double snoruv[3];  
	  double tnun;       
	  double tnvn;       
	  double tunvn;      
	  double tnuvn;      
	  double snorus[3];  
	  double snorvs[3];  
	  double snoruvs[3]; 
	  
	  
	  
	  tder1 = s6length(sder+kdim,kdim,&kstat);
	  tder2 = s6length(sder+3*kdim,kdim,&kstat);
	  
	  
	  
	  for (ki=0;ki<idim;ki++)
            {
	      spnt = sder+kdim+ki;
	      *spnt /= tder1;
	      
	      spnt = sder+2*kdim+ki;
	      *spnt /= (tder1+tder1);
	      
	      spnt = sder+3*kdim+ki;
	      *spnt /= tder2;
	      
	      spnt = sder+4*kdim+ki;
	      *spnt /= (tder1+tder2);
	      
	      spnt = sder+5*kdim+ki;
	      *spnt /= (tder1+tder1+tder2);
	      
	      spnt = sder+6*kdim+ki;
	      *spnt /= (tder2+tder2);
	      
	      spnt = sder+7*kdim+ki;
	      *spnt /= (tder1+tder2+tder2);
	      
	      spnt = sder+8*kdim+ki;
	      *spnt /= (tder1+tder1+tder2+tder2);
            }
	  
	  
	  
	  
	  
	  s6crss(sder+2*kdim,sder+3*kdim,snoru1);
	  s6crss(sder+kdim,sder+4*kdim,snoru2);
	  
	  for (ki=0;ki<idim;ki++) snoru[ki]=snoru1[ki]+snoru2[ki];
	  
	  
	  
	  
	  
	  s6crss(sder+4*kdim,sder+3*kdim,snorv1);
	  s6crss(sder+kdim,sder+6*kdim,snorv2);
	  
	  for (ki=0;ki<idim;ki++) snorv[ki]=snorv1[ki]+snorv2[ki];
	  
	  
	  
	  
	  
	  s6crss(sder+5*kdim,sder+3*kdim,snoruv1);
	  s6crss(sder+2*kdim,sder+6*kdim,snoruv2);
	  s6crss(sder+kdim,sder+7*kdim,snoruv3);
	  
	  for (ki=0;ki<idim;ki++) 
	    snoruv[ki]=snoruv1[ki]+(double)2.0*snoruv2[ki]+snoruv3[ki];
	  
	  
	  
	  
	  
	  
	  tnun = s6scpr(snorm,snoru,idim);
	  
	  for (ki=0;ki<idim;ki++) 
	    snorus[ki]=
	      snoru[ki]/tnorm-(tnun/(tnorm*tnorm*tnorm))*snorm[ki];
	  
	  
	  
	  
	  
	  tnvn = s6scpr(snorm,snorv,idim);
	  
	  for (ki=0;ki<idim;ki++) 
	    snorvs[ki]=
	      snorv[ki]/tnorm-(tnvn/(tnorm*tnorm*tnorm))*snorm[ki];
	  
	  
	  
	  
	  
	  tunvn = s6scpr(snoru,snorv,idim);
	  tnuvn = s6scpr(snorm,snoruv,idim);
	  
	  for (ki=0;ki<idim;ki++) 
	    snoruvs[ki]=
	      snoruv[ki]/tnorm-(tnvn/(tnorm*tnorm*tnorm))*snoru[ki]-
		(tnun/(tnorm*tnorm*tnorm))*snorv[ki]-
		  (tunvn/(tnorm*tnorm*tnorm))*snorm[ki]-
		    (tnuvn/(tnorm*tnorm*tnorm))*snorm[ki]+
		      (((double)3.0*tnun*tnvn)/(tnorm*tnorm*tnorm*tnorm*tnorm))*
			snorm[ki];
	  
	  
	  
	  
	  
	  for (ki=0;ki<idim;ki++) 
            eder[ki+kdim]=sder[ki+kdim]+aoffset*snorus[ki];
	  
	  
	  
	  
	  
	  for (ki=0;ki<idim;ki++) 
            eder[ki+2*kdim]=sder[ki+3*kdim]+aoffset*snorvs[ki];
	  
	  
	  
	  
	  
	  for (ki=0;ki<idim;ki++) 
            eder[ki+3*kdim]=sder[ki+4*kdim]+aoffset*snoruvs[ki];
	  
	  
	  
	  
	  
	  for (ki=0;ki<idim;ki++)
            {
	      spnt = eder+kdim+ki;
	      *spnt *= tder1;
	      
	      spnt = eder+2*kdim+ki;
	      *spnt *= tder2;
	      
	      spnt = eder+3*kdim+ki;
	      *spnt *= (tder1+tder2);
            }
	  
	}
    }
  
  
  
  *jstat = 0;
  goto out;
  
  
  
  err105: *jstat = -105;
  s6err("s1367",*jstat,kpos);
  goto out;
  
  
  
  error : *jstat = kstat;
  s6err("s1367",*jstat,kpos);
  goto out;
  
  out: return;
}

