/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1384

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1384(SISLCurve *pcurve,SISLSurf *psurf,int idim,int iside,double ax,
	   int *ileftc,int *ilefts1,int *ilefts2,double eder[],
	   double ederu[],double ederv[],double edern[],int *jstat)
#else
void s1384(pcurve,psurf,idim,iside,ax,ileftc,ilefts1,ilefts2,
           eder,ederu,ederv,edern,jstat)
     SISLCurve  *pcurve;
     SISLSurf   *psurf;
     int    idim;
     int    iside;
     double ax;
     int    *ileftc;
     int    *ilefts1;
     int    *ilefts2;
     double eder[];
     double ederu[];
     double ederv[];
     double edern[];
     int    *jstat;
#endif
                                     
{
  int kstat=0;        
  int kpos=0;         
  int kder=3;         
  int ki;             
  int knum1,knum2;    
  int kdimc;          
  int kdims;          
  double sdumc[8];    
  double sdums[30];   
  double snorm[3];    
  double *sders=SISL_NULL; 
  double *sderc=SISL_NULL; 
  double tduds,tdvds; 
  double tddudss,tddvdss; 
  double *sdpdu,*sdpdv;
  double *sddpduu;    
  double *sddpduv;    
  double *sddpdvv;    
  double *ps;         
  double tlsn;        
  
  kdimc = pcurve -> idim;
  kdims = psurf  -> idim;
  
  if (kdimc !=2 && kdims != idim) goto err105;
  
  knum1 = kdimc*(kder+1);
  knum2 = kdims*(kder+1)*(kder+2)/2;
  
  if (knum1>8)
  {
    if((sderc = newarray(knum1,DOUBLE)) == SISL_NULL) goto err101;
  }
  else  
    sderc = sdumc;
  
  if (knum2>30)
  {
    if((sders = newarray(knum2,DOUBLE)) == SISL_NULL) goto err101;
  }
  else   
    sders = sdums;
  
   
  
  
  if (iside != -1)
    {
      
      s1221(pcurve,kder,ax,ileftc,sderc,&kstat);
      if (kstat<0) goto error;
    }
  else
    {
      
      s1227(pcurve,kder,ax,ileftc,sderc,&kstat);
      if (kstat<0) goto error;
    }
  
   
  
  s1421(psurf,kder,sderc,ilefts1,ilefts2,sders,snorm,&kstat);            
  if (kstat<0) goto error;
  
  
  
  tduds  = sderc[2];
  tdvds  = sderc[3];
  tddudss = sderc[4];
  tddvdss = sderc[5];
  sdpdu   = sders + idim;
  sdpdv   = sdpdu + idim;
  sddpduu = sdpdv + idim;
  sddpduv = sddpduu + idim;
  sddpdvv = sddpduv + idim;
  
  
  
  
  
  for (ki=0;ki<idim;ki++) eder[ki] = sders[ki];
  
  
  
  ps = eder + idim;
  for (ki=0;ki<idim;ki++)
    *(ps++) = sdpdu[ki]*tduds + sdpdv[ki]*tdvds;
  
  
  
  for (ki=0;ki<idim;ki++)
    *(ps++) = sddpduu[ki]*tduds*tduds + sdpdu[ki]*tddudss +
      sddpdvv[ki]*tdvds*tdvds + sdpdv[ki]*tddvdss +
	(double)2.0*sddpduv[ki]*tduds*tdvds;
  
  
  
  ps = ederu;
  for (ki=0;ki<idim;ki++)
    *(ps++) = sdpdu[ki];
  
  for (ki=0;ki<idim;ki++)
    *(ps++) = sddpduu[ki]*tduds + sddpduv[ki]*tdvds;
  
  
  
  
  ps = ederv;
  for (ki=0;ki<idim;ki++)
    *(ps++) = sdpdv[ki];
  
  for (ki=0;ki<idim;ki++)
    *(ps++) = sddpduv[ki]*tduds + sddpdvv[ki]*tdvds;
  
  
  
  
  
  
  s6crss(ederu,ederv,snorm);
  
  tlsn = s6length(snorm,3,&kstat);
  
  if (kstat<0) goto error;
  
  if (DEQUAL(tlsn,DZERO)) tlsn = (double)1.0;
  
  s6crss(eder+idim,snorm,edern);
  
  ps = edern;
  
  for (ki=0;ki<idim;ki++,ps++)
    *(ps) /= tlsn;
  
  
  
  
  
  *jstat = 0;
  goto out;
  
  

  err101: *jstat = -101;
    s6err("s1384",*jstat,kpos);
    goto out;
  
  

  err105: *jstat = -105;
    s6err("s1384",*jstat,kpos);
    goto out;
  
  
  
  error:  *jstat = kstat;
    s6err("s1384",*jstat,kpos);
    goto out;
  
  out:
  
    if (knum1>8)
      { 
        if (sderc != SISL_NULL) freearray(sderc);
      }
    if (knum2>30) 
      {
        if (sders != SISL_NULL) freearray(sders);
      }
    return;
}

