/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1507

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1507(SISLCurve **curves, int nc, int periodic,
      SISLCurve ***newcurves, int *jstat)
#else
void s1507(curves, nc, periodic, newcurves, jstat)
     SISLCurve **curves;
     int nc;
     int periodic;
     SISLCurve ***newcurves;
     int *jstat;
#endif
{
  int kpos = 0;			

  int i,j;                      

  int k,k1,k2;                  
  int n;                        
  int dim;                      
  int inext;                    
  int nheal;                    

  SISLCurve **curves2;          

  double startpar;              
  double endpar;                
  double len;                   
  double average;               
  double c1,c2,c3;              
  double knotdiff1,knotdiff2;   
  double lambda,denom;          


  *jstat = 0;

  if( (curves2=newarray(nc,SISLCurve*)) == SISL_NULL) goto err101;
  memzero(curves2, nc, SISLCurve*);

  if (nc <= 0) goto err102;
  if (nc == 1) goto out; 

  dim = curves[0]->idim;

  
  for(i=0; i<nc; i++)
  {
    
    if(curves[i]->idim != dim) goto err102;

    
    if(curves[i]->ikind == 2 || curves[i]->ikind == 4) goto err102;

    
    n = curves[i]->in;
    if(n < 4) goto err102;
  }

  
  for(i=0; i<nc; i++)
  {
    
    curves2[i] = copyCurve(curves[i]);

    
    n = curves2[i]->in;
    k  = curves2[i]->ik;
    startpar = curves2[i]->et[k-1];
    endpar = curves2[i]->et[n];
    len = endpar - startpar;

    for(j=0; j<n+k; j++)
    {
      curves2[i]->et[j]  = (curves2[i]->et[j] - startpar) / len;
    }
  }

  

  nheal = (periodic == 0 ? nc - 1 : nc);

  

  for(i=0; i<nheal; i++)
  {
    inext = (i < nc-1 ? i+1 : 0);

    n = curves2[i]->in;
    for(k=0; k<dim; k++)
    {
      average = (curves2[i]->ecoef[dim*(n-1)+k] +
                 curves2[inext]->ecoef[k]) * 0.5;
      curves2[i]->ecoef[dim*(n-1)+k] = average;
      curves2[inext]->ecoef[k] = average;
    }
  }


  
  

  for(i=0; i<nheal; i++)
  {
    inext = (i < nc-1 ? i+1 : 0);

    n = curves2[i]->in;
    k1 = curves2[i]->ik;
    k2 = curves2[inext]->ik;

    knotdiff1 = (curves2[i]->et[n+k1-2] - curves2[i]->et[n-1])
                    / (double)(k1-1);
    knotdiff2 = (curves2[inext]->et[k2] - curves2[inext]->et[1])
                    / (double)(k2-1);
    lambda = knotdiff2 / (knotdiff1 + knotdiff2);
    denom = 2.0 * (1.0 - lambda + lambda*lambda);

    for(k=0; k<dim; k++)
    {
      c1 = curves2[i]->ecoef[dim*(n-2)+k];
      c2 = curves2[inext]->ecoef[k];
            
      c3 = curves2[inext]->ecoef[dim+k];

      curves2[i]->ecoef[dim*(n-2)+k]
        = ( (1.0 + lambda*lambda) * c1 + (1.0 - lambda) * c2
           - lambda * (1.0 - lambda) * c3 ) / denom;

      curves2[inext]->ecoef[dim+k]
        = ( - lambda * (1.0 - lambda) * c1 + lambda * c2
           + (1.0 + (1.0 - lambda)*(1.0 - lambda)) * c3 ) / denom;

      curves2[i]->ecoef[dim*(n-1)+k]
        = (1.0 - lambda) * curves2[i]->ecoef[dim*(n-2)+k]
          + lambda * curves2[inext]->ecoef[dim+k];

      curves2[inext]->ecoef[k]
        = (1.0 - lambda) * curves2[i]->ecoef[dim*(n-2)+k]
          + lambda * curves2[inext]->ecoef[dim+k];
    }
  }


  *newcurves = curves2;

  goto out;

  

err101:
  *jstat = -101;
  s6err ("s1507", *jstat, kpos);
  goto out;

  

err102:
  *jstat = -102;
  goto out;

out:

  return;
}

