/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1604

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1604(double epoint[],int inbpnt,double astpar,int iopen,int idim,int ik,
	   SISLCurve **rc,int *jstat)
#else
void s1604(epoint,inbpnt,astpar,iopen,idim,ik,rc,jstat)
	   double epoint[];
	   int    inbpnt;
	   double astpar;
	   int    iopen;
	   int    idim;
	   int    ik;
	   SISLCurve  **rc;
	   int    *jstat;
#endif
{
  int kstat;          
  int kn;             
  int kk;             
  int kpos=0;         
  int ki;             

  double *spara=SISL_NULL; 
  double *scoef=SISL_NULL; 
  double *sknot=SISL_NULL; 
  double tdist;       
  double tlastpar;    
  SISLCurve *qc=SISL_NULL;
  SISLCurve *qc2 = SISL_NULL;


  


  if (iopen==SISL_CRV_CLOSED) iopen = SISL_CRV_PERIODIC;



  kk = ik;
  if  (inbpnt < kk)
    kk = inbpnt;

  if  (kk < 2) goto err109;

  if  (iopen != SISL_CRV_OPEN && iopen != SISL_CRV_CLOSED
       && iopen != SISL_CRV_PERIODIC) goto err113;

  

  spara = newarray(inbpnt+1,DOUBLE);
  if (spara == SISL_NULL) goto err101;

  

  spara[0] = astpar;
  tlastpar = astpar;

  for (ki=1; ki<inbpnt; ki++)
    {
      tdist = s6dist(&epoint[ki*idim-idim],&epoint[ki*idim],idim);
      tlastpar = tlastpar + tdist;
      spara[ki] = tlastpar;
    }


  

  tdist = s6dist(epoint,&epoint[(inbpnt-1)*idim],idim);
  tlastpar = tlastpar + tdist;
  spara[inbpnt] = tlastpar;


  

  s1902(spara,inbpnt,kk,iopen,&sknot,&kstat);
  if (kstat < 0 || sknot == SISL_NULL) goto error;

  

  scoef = newarray((inbpnt+kk-1)*idim,DOUBLE);
  if (scoef == SISL_NULL) goto err101;

  

  memcopy (scoef,epoint,inbpnt*idim,DOUBLE);
  kn = inbpnt;

  

  if (!(iopen == SISL_CRV_OPEN))
    {
      memcopy(&scoef[inbpnt*idim],epoint,(kk-1)*idim,DOUBLE);
      kn = kn + kk - 1;
    }

  
  
  qc = newCurve(kn,kk,sknot,scoef,1,idim,2); 
  if (!qc) goto err101;

  qc->cuopen = iopen;

  if (iopen == SISL_CRV_CLOSED)
    {
      

      s1713 (qc, sknot[kk - 1], sknot[kn], &qc2, &kstat);
      if (kstat < 0)
	goto error;

      if (qc != SISL_NULL)
	freeCurve (qc);
      qc = qc2;
    }

  

  if (kk < ik)
    {
				
      s1750(qc,ik,rc,&kstat);
				

				
				
				
      if (kstat< 0) goto error;
    }
				
  else
    {
      *rc = qc;
      qc = SISL_NULL;
    }
				
				
				
				

  *jstat = 0;
  goto out;

  

 err101:
  *jstat = -101;
  s6err("s1604",*jstat,kpos);
  goto out;

  

 err109:
  *jstat = -109;
  s6err("s1604",*jstat,kpos);
  goto out;

  

 err113:
  *jstat = -113;
  s6err("s1604",*jstat,kpos);
  goto out;


  

 error:
  *jstat = kstat;
  s6err("s1604",*jstat,kpos);
  goto out;

 out:
  if (spara  != SISL_NULL) freearray(spara);
  
  if (qc != SISL_NULL) freeCurve(qc);
  return;
}

