/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1612

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1612(SISLCurve *pc,double aepsge,double **gpoint,int *jnbpnt,int *jleng,int *jstat)
#else
void s1612(pc,aepsge,gpoint,jnbpnt,jleng,jstat)
     SISLCurve  *pc;
     double aepsge;
     double **gpoint;
     int    *jnbpnt;
     int    *jleng;
     int    *jstat;
#endif
{
  int kstat;          
  int kn;             
  int kk;             
  int kdim;           
  int ki;             
  int kleft;          
  int knbreak=0;      
  int knbpnt;         
  int kleng;          
  int kvlast;         
  int kpos=0;         
  
  double *spoint=SISL_NULL;
  double tdist;       
  double tpar;        
  double *sbreak = SISL_NULL;  
  double *st;         
  
  
  SISLCurve *qcnew1=SISL_NULL; 
  SISLCurve *qcnew2=SISL_NULL; 
  
  
  
  
  if (aepsge <= (double)0.0) goto err120;
  
  
  
  spoint = *gpoint;
  knbpnt = *jnbpnt;
  kleng  = *jleng;
  
  
  
  kn    = pc -> in;
  kk    = pc -> ik;
  kdim  = pc -> idim;          
  st    = pc -> et;
  
  
  
  s1235(st,kn,kk,&knbreak,&sbreak,&kstat);
  if (kstat < 0) goto error;
  
  
  
  if (knbreak > 2) 
    {
      for (ki=1; ki<knbreak-1; ki++)
	{
	  tpar = sbreak[ki];
	  
	  s1710(pc,tpar,&qcnew1,&qcnew2,&kstat);
	  if(kstat < 0) goto error;
	  
	  
	  
	  if (qcnew1)
	  {
	     s1612(qcnew1,aepsge,&spoint,&knbpnt,&kleng,&kstat);
	     if (kstat < 0) goto error;
	  }
	  
	  
	  
	  if (qcnew2)
	  {
	     s1612(qcnew2,aepsge,&spoint,&knbpnt,&kleng,&kstat);
	     if (kstat < 0) goto error;
	  }
	}
    }
  else      
    
    {
      
      
      s1840(pc,&tdist,&kstat);
      if (kstat < 0) goto error;
      
      if (tdist < aepsge)
	{
	  
	  
	  kvlast = (kn-1) * kdim;
	  
	  knbpnt += 1;
	  
	  
	  
	  if (kleng < (knbpnt+1)*kdim )
	    {
	      kleng  += 100*kdim;
	      spoint = increasearray(spoint,kleng,DOUBLE);
	      if (!spoint) goto err101;
	    }
	  
	  
	  
	  memcopy (&spoint[(knbpnt-1)*kdim],&pc->ecoef[kvlast],kdim,DOUBLE);
	}
      else
	{
	  
	  
	  tpar = (st[0] + st[kn+kk-1]) / (double)2.0; 
	  
	  if (kn > kk) 
	    {    
	      
	      
	      kleft = 0;
	      s1219(st,kk,kn,&kleft,tpar,&kstat);
	      if (kstat < 0) goto error;
	      
	      
	      
	      if (fabs(tpar-st[kleft]) < fabs(st[kleft+1]-tpar)) 
		{
		  tpar = st[kleft];
		}
	      else
		{
		  tpar = st[kleft+1];
		}
	    }
	  
	  s1710(pc,tpar,&qcnew1,&qcnew2,&kstat);
	  if(kstat < 0) goto error;
	  
	  
	  
	  if (qcnew1)
	  {
	     s1612(qcnew1,aepsge,&spoint,&knbpnt,&kleng,&kstat);
	     if (kstat < 0) goto error;
	  }
	  
	  
	  
	  if (qcnew2)
	  {
	     s1612(qcnew2,aepsge,&spoint,&knbpnt,&kleng,&kstat);
	     if (kstat < 0) goto error;
	  }
	}
    }
  
  *gpoint = spoint;
  *jnbpnt = knbpnt;
  *jleng  = kleng;
  *jstat = 0;
  goto out;
  
  
  
 err101: 
  *jstat = -101;
  s6err("s1612",*jstat,kpos);
  goto out;
  
  
  
 err120: 
  *jstat = -120;
  s6err("s1612",*jstat,kpos);
  goto out;
  
  
    
  
 error:  
  *jstat = kstat;
  s6err("s1612",*jstat,kpos); 
  goto out;
  
  
 out:
  
  
  
  if (sbreak) freearray(sbreak);
  if (qcnew1) freeCurve(qcnew1);
  if (qcnew2) freeCurve(qcnew2);
  
  return;
}       
   
          

