/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1613BEZ

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1613bez(SISLCurve *pc,int idiv,double aepsge,double **gpar,int *jnpar,
	      int *jstat)
#else
void s1613bez(pc,idiv,aepsge,gpar,jnpar,jstat)
     SISLCurve  *pc;
     int idiv;
     double aepsge;
     double **gpar;
     int    *jnpar;
     int    *jstat;
#endif
{
   static int klevel = 0;  
   int kstat = 0;        
   int ki;               
   int kord = pc->ik;    
   int kdim = pc->idim;  
   int knpar;            
   int kbez = 3;         
   int kdiv = 10;        
   int kparout;          
   int knunion;          
   int knparts = idiv;   
   double tstart = *(pc->et+kord-1);   
   double tend = *(pc->et+pc->in);     
   double tint = (tend - tstart)/(double)(idiv+1);  
   double tpar;             
   double *sparout = SISL_NULL;  
   double *spar = SISL_NULL;     
   double *sparunion = SISL_NULL;  
   double *spar2 = SISL_NULL;    
   double *sta,*stb,*s1;    
   SISLCurve *qc = SISL_NULL;    
   SISLCurve *qbez = SISL_NULL;  

   
   
   klevel++;
   if (klevel > 200)
   {
      *jstat = 2;
      goto out;
   }
   
   
   
   if (pc->ik != pc->in) goto err111;

   
      
   if ((spar = newarray(idiv,DOUBLE)) == SISL_NULL) goto err101;
     
   for (ki=0, tpar=tstart+tint; ki<idiv; ki++, tpar+=tint)
      spar[ki] = tpar;
   
   
   
   newknots(pc->et,pc->in,pc->ik,spar,idiv,REL_PAR_RES,&spar2,&knpar,&kstat);
   if (kstat < 0) goto error;
		  
   
		  
   s1018(pc,spar2,knpar,&qc,&kstat);
   if (kstat < 0) goto error;
		  
   
		  
   for (sta=qc->ecoef, ki=0; ki<=idiv; sta=stb+kdim, ki++)
   {
      for (stb=sta+(kord-1)*kdim, s1=sta+kdim; s1<stb; s1+=kdim)
      {
	 if (s6dline(sta,stb,s1,kdim,&kstat) > aepsge) break;
	 if (kstat < 0) goto error;
      }
      if (kstat < 0) goto error;
		     
      if (s1 < stb)
      {
	 
	 
	 if ((qbez = newCurve(kord,kord,qc->et+ki*kord,sta,kbez,kdim,0))
	     == SISL_NULL) goto err101;
	 
	 
	 
	 s1613bez(qbez,kdiv,aepsge,&sparout,&kparout,&kstat);
	 if (kstat < 0) goto error;
	 
	 if (kstat == 2)
	 {
	    
	    
	    if (spar != SISL_NULL) freearray(spar);
	    if (qbez != SISL_NULL) freeCurve(qbez);
	    
	    *jstat = kstat;
	    goto out;
	 }
	 
	 
	 
	 s6takeunion(spar,knparts,sparout,kparout,&sparunion,&knunion,&kstat);
	 if (kstat < 0) goto error;
	 
	 freearray(spar);
	 spar = sparunion;
	 sparunion = SISL_NULL;
	 knparts = knunion;
	 
	 if (sparout != SISL_NULL) freearray(sparout);
	 sparout = SISL_NULL;
	 
	 freeCurve(qbez);
	 qbez = SISL_NULL;
      }
   }
   
   
   
   *gpar = spar;
   *jnpar = knparts;
   
   *jstat = 0;
   goto out;
   
   
   
   err101 :
      *jstat = -101;
   goto out;
   
   
   
   err111 :
      *jstat = -111;
   goto out;
      
   
   
   error :
      *jstat = kstat;
   goto out;
   
   out:
      
      
      klevel--;
   
      
      
      if (spar2 != SISL_NULL) freearray(spar2);
      if (qc != SISL_NULL) freeCurve(qc);
		      
      return;
}
     

