/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1616

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1616(double epoint[], int inbpnt, int idim, int eptyp[],
	   double econic[], int *jstat)
#else
void s1616(epoint, inbpnt, idim, eptyp, econic, jstat)
     double epoint[];
     int    inbpnt;
     int    idim;
     int    eptyp[];
     double econic[];
     int    *jstat;
#endif
{
  int ki, kj, kk;
  int kdim;
  int kn;
  double tmax = (double) HUGE;
  int kperm = 0;
  int ktyp;
  int kdir;
  double *smatrix = SISL_NULL;
  double *save = SISL_NULL;
  double *sright = SISL_NULL;
  double tx, ty, tdx, tdy;
  int *npiv = SISL_NULL;
  double solu[6];
  double tdiff;
  double tdum;
  int kpos = 0;
  int kstat = 0;

  *jstat = 0;


  

  smatrix = newarray (inbpnt * inbpnt, DOUBLE);
  if (smatrix == SISL_NULL)
    goto err101;

  sright = newarray (inbpnt, DOUBLE);
  if (sright == SISL_NULL)
    goto err101;

  save = newarray (inbpnt * inbpnt, DOUBLE);
  if (save == SISL_NULL)
    goto err101;

  npiv = newarray (inbpnt, INT);
  if (npiv == SISL_NULL)
    goto error;

  kn = inbpnt + 1;
  kdim = inbpnt * inbpnt;


  

  if (inbpnt == 3)
    {

      

      for (ki = 0; ki < inbpnt; ki++)
	{
	  kk = idim * ki;
	  ktyp = eptyp[ki];

	  if (ktyp < 3)
	    {
	      tx = epoint[kk];
	      ty = epoint[kk + 1];

	      smatrix[inbpnt*ki] = ((double) 2.0) * tx;
	      smatrix[inbpnt*ki + 1] = ((double) 2.0) * ty;
	      smatrix[inbpnt*ki + 2] = (double) 1.0;
	      sright[ki] = -(tx * tx + ty * ty);
	    }
	  else if (ktyp > 2)
	    {

	      

	      kdir = 1;
	      if (ktyp == 4)
		kdir = -1;

	      tx = epoint[kk + idim * kdir];
	      ty = epoint[kk + idim * kdir + 1];
	      tdx = epoint[kk];
	      tdy = epoint[kk + 1];

	      smatrix[inbpnt*ki] = ((double) 2.0) * tdx;
	      smatrix[inbpnt*ki + 1] = ((double) 2.0) * tdy;
	      smatrix[inbpnt*ki + 2] = (double) 0.0;
	      sright[ki] = -tdx * tx * ((double) 2.0) - tdy * ty * ((double) 2.0);
	    }
	}
    }
  else if (inbpnt == 4)
    {
      

      for (ki = 0; ki < inbpnt; ki++)
	{
	  kk = idim * ki;
	  ktyp = eptyp[ki];
	  if (ktyp < 3)
	    {
	      tx = epoint[kk];
	      ty = epoint[kk + 1];

	      smatrix[inbpnt*ki] = tx * tx;
	      smatrix[inbpnt*ki + 1] = ty * ty;
	      smatrix[inbpnt*ki + 2] = ((double) 2.0) * tx;
	      smatrix[inbpnt*ki + 3] = ((double) 2.0) * ty;
	      sright[ki] = -(double) 1.0;
	    }
	  else if (ktyp > 2)
	    {

	      

	      kdir = 1;
	      if (ktyp == 4)
		kdir = -1;

	      tx = epoint[kk + idim * kdir];
	      ty = epoint[kk + idim * kdir + 1];
	      tdx = epoint[kk];
	      tdy = epoint[kk + 1];

	      smatrix[inbpnt*ki] = ((double) 2.0) * tdx * tx;
	      smatrix[inbpnt*ki + 1] = ((double) 2.0) * tdy * ty;
	      smatrix[inbpnt*ki + 2] = ((double) 2.0) * tdx;
	      smatrix[inbpnt*ki + 3] = ((double) 2.0) * tdy;
	      sright[ki] = (double) 0.0;
	    }
	}
    }
  else if (inbpnt == 5)
    {

      

      for (ki = 0; ki < inbpnt; ki++)
	{
	  kk = idim * ki;
	  ktyp = eptyp[ki];
	  if (ktyp < 3)
	    {
	      tx = epoint[kk];
	      ty = epoint[kk + 1];

	      smatrix[inbpnt*ki] = tx * tx;
	      smatrix[inbpnt*ki + 1] = ((double) 2.0) * tx * ty;
	      smatrix[inbpnt*ki + 2] = ty * ty;
	      smatrix[inbpnt*ki + 3] = ((double) 2.0) * tx;
	      smatrix[inbpnt*ki + 4] = ((double) 2.0) * ty;
	      sright[ki] = -(double) 1.0;
	    }
	  else if (ktyp > 2)
	    {

	      

	      kdir = 1;
	      if (ktyp == 4)
		kdir = -1;

	      tx = epoint[kk + idim * kdir];
	      ty = epoint[kk + idim * kdir + 1];
	      tdx = epoint[kk];
	      tdy = epoint[kk + 1];

	      smatrix[inbpnt*ki] = ((double) 2.0) * tdx * tx;
	      smatrix[inbpnt*ki + 1] = ((double) 2.0) * tdy * tx + ((double) 2.0) * tdx * ty;
	      smatrix[inbpnt*ki + 2] = ((double) 2.0) * tdy * ty;
	      smatrix[inbpnt*ki + 3] = ((double) 2.0) * tdx;
	      smatrix[inbpnt*ki + 4] = ((double) 2.0) * tdy;
	      sright[ki] = (double) 0.0;
	    }
	}
    }

  

  for (ki = 0; ki < kn; ki++)
    {
      

      for (kj = 0; kj < kdim; kj++)
	save[kj] = smatrix[kj];


      

      for (kj = 0; kj < inbpnt; kj++)
	econic[kj] = sright[kj];

      s6lufacp (smatrix, npiv, inbpnt, &kstat);

      if (kstat >= 0 && kstat != 1)
        s6lusolp (smatrix, econic, npiv, inbpnt, &kstat);
      kstat = 0;

      

      for (kj = 0; kj < kdim; kj++)
	smatrix[kj] = save[kj];

      s1618 (smatrix, sright, econic, inbpnt, &tdiff);

      if (tdiff < tmax)
	{
	  

	  tmax = tdiff;
	  kperm = ki;

	  for (kj = 0; kj < inbpnt; kj++)
	    solu[kj] = econic[kj];
	  
          if (inbpnt == 3) break;
	}
	
      if (ki < (kn - 1))
	{
	  

	  for (kj = 0; kj < inbpnt; kj++)
	    {
	      kk = inbpnt * kj;
	      tdum = -sright[kj];
	      sright[kj] = -smatrix[kk + ki];
	      smatrix[kk + ki] = tdum;
	    }
	}
    }

  

  if (tmax > 0.0001)
    {
      *jstat = 105;
      econic[0] = 0.;
      econic[1] = 0.;
      econic[2] = 0.;
      econic[3] = (epoint[1] - epoint[2*inbpnt - 1])/2.;
      econic[4] = (epoint[2*inbpnt - 2] - epoint[0])/2.;
      econic[5] = epoint[0]*epoint[2*inbpnt - 1] - 
	 epoint[1]*epoint[2*inbpnt - 2];
      goto out;
    }

  

  for (kj = 0; kj < inbpnt; kj++)
    econic[kj] = solu[kj];

  econic[inbpnt] = (double) 1.0;


  

  if (kperm != 0)
    {
      for (ki = 1; ki <= kperm; ki++)
	{
	  kj = kperm - ki;
	  tdum = econic[kj];
	  econic[kj] = econic[inbpnt];
	  econic[inbpnt] = tdum;
	}
    }

  

  if (inbpnt == 3)
    {
      

      for (ki = 0; ki < inbpnt; ki++)
	econic[ki + inbpnt] = econic[ki];

      econic[0] = (double) 1.0;
      econic[1] = (double) 0.0;
      econic[2] = (double) 1.0;
    }
  else if (inbpnt == 4)
    {
      

      for (ki = 5; ki > 1; ki--)
	econic[ki] = econic[ki - 1];

      econic[1] = (double) 0.0;
    }

  goto out;

  

err101:
  *jstat = -101;
  s6err ("s1616", *jstat, kpos);
  goto out;

  

error:
  *jstat = kstat;
  s6err ("s1616", *jstat, kpos);
  goto out;

out:

  

  if (smatrix != SISL_NULL)
    freearray (smatrix);
  if (save != SISL_NULL)
    freearray (save);
  if (sright != SISL_NULL)
    freearray (sright);
  if (npiv != SISL_NULL)
    freearray (npiv);

  return;
}

