/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1619

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1619(double epoint[], int inbpnt, int idim, int eptyp[],
	   double econic[], int ityp, double etang[],
	   double *ashape, int *jstat)
#else
void s1619(epoint, inbpnt, idim, eptyp, econic, ityp,etang, ashape, jstat)
     double epoint[];
     int    inbpnt;
     int    idim;
     int    eptyp[];
     double econic[];
     int    ityp;
     double etang[];
     double *ashape;
     int    *jstat;
#endif
{
  double ta11, ta12, ta13, ta22, ta23, ta33;
  double tx, ty, tx1, ty1, tx2, ty2, axt, ayt;
  double tv1, tv2, tvx, tvy, tnx, tny, tix, tiy;
  double ta, tb, tc, te1, te2, te3;
  double tys1, tys2, txs1, txs2;
  double as, ts1, ts2;
  double tlong, tlength;
  double tdum;
  int ktyp;

  int ki, kk;
  int kki = 0;

  int krem = 0;
  int ksam = 0;

  *jstat = 0;


  

  kk = idim * (inbpnt - 1);

  ta11 = econic[0];
  ta12 = econic[1];
  ta13 = econic[3];		
  ta22 = econic[2];		
  ta23 = econic[4];
  ta33 = econic[5];

  tx = (epoint[0] + epoint[kk]) / ((double) 2.0);
  ty = (epoint[1] + epoint[kk + 1]) / ((double) 2.0);


  

  tx1 = -ta22 * epoint[1] - ta12 * epoint[0] - ta23;
  ty1 = ta11 * epoint[0] + ta12 * epoint[1] + ta13;
  tlong = sqrt (tx1 * tx1 + ty1 * ty1);
  tx1 = tx1 / tlong;
  ty1 = ty1 / tlong;

  tx2 = -ta22 * epoint[kk + 1] - ta12 * epoint[kk] - ta23;
  ty2 = ta11 * epoint[kk] + ta12 * epoint[kk + 1] + ta13;
  tlong = sqrt (tx2 * tx2 + ty2 * ty2);
  tx2 = tx2 / tlong;
  ty2 = ty2 / tlong;


  

  tdum = ty1 * tx2 - ty2 * tx1;


  

  krem = 0;

  if (fabs (tdum) <= REL_PAR_RES)
    {
      

      krem = 1;
      tv1 = tx1;
      tv2 = ty1;
    }
  else
    {
      axt = (epoint[0] * ty1 * tx2 - epoint[kk] * ty2 * tx1 +
	     tx1 * tx2 * (epoint[kk + 1] - epoint[1])) / tdum;
      ayt = (ty1 * ty2 * (epoint[0] - epoint[kk]) -
	     epoint[1] * tx1 * ty2 + epoint[kk + 1] * tx2 * ty1) / tdum;


      

      tv1 = tx - axt;
      tv2 = ty - ayt;
      tlong = sqrt (tv1 * tv1 + tv2 * tv2);
      tv1 = tv1 / tlong;
      tv2 = tv2 / tlong;
    }

  ta = -tv2;
  tb = tv1;
  tc = -tx * ta - ty * tb;

  if (fabs (ta) >= fabs (tb))
    {
      

      te1 = ta11 * tb * tb / (ta * ta) - ((double) 2.0) * ta12 * tb / ta + ta22;
      te2 = ((double) 2.0) * tb * tc * ta11 / (ta * ta) - ((double) 2.0) * ta12 * tc / ta
	- ((double) 2.0) * ta13 * tb / ta + ((double) 2.0) * ta23;
      te3 = ta11 * tc * tc / (ta * ta) - ((double) 2.0) * ta13 * tc / ta + ta33;
      tdum = te2 * te2 - ((double) 4.0) * te1 * te3;


      

      if (tdum < 0.0)
	{
	  

	  *jstat = 1;
	  goto out;
	}

      tdum = sqrt (tdum);
      tys1 = (-te2 - tdum) / (((double) 2.0) * te1);
      tys2 = (-te2 + tdum) / (((double) 2.0) * te1);
      txs1 = -(tb * tys1 + tc) / ta;
      txs2 = -(tb * tys2 + tc) / ta;
    }
  else
    {
      

      te1 = ta11 - ((double) 2.0) * ta12 * ta / tb + ta22 * ta * ta / (tb * tb);
      te2 = ((double) 2.0) * ta * tc * ta22 / (tb * tb) - ((double) 2.0) * ta12 * tc / tb
	- ((double) 2.0) * ta23 * ta / tb + ((double) 2.0) * ta13;
      te3 = ta22 * tc * tc / (tb * tb) - ((double) 2.0) * ta23 * tc / tb + ta33;
      tdum = te2 * te2 - ((double) 4.0) * te1 * te3;


      

      if (tdum < (double) 0.0)
	{
	  

	  *jstat = 1;
	  goto out;
	}

      tdum = sqrt (tdum);
      txs1 = (-te2 - tdum) / (((double) 2.0) * te1);
      txs2 = (-te2 + tdum) / (((double) 2.0) * te1);
      tys1 = -(ta * txs1 + tc) / tb;
      tys2 = -(ta * txs2 + tc) / tb;
    }

  

  if (krem == 1)
    {
      

      axt = txs1;
      ayt = tys1;
    }

  tlength = (tx - axt) * (tx - axt) + (ty - ayt) * (ty - ayt);


  

  if (tlength < (double) 0.0)
    {
      

      *jstat = 1;
      goto out;
    }


  ts1 = ((txs1 - tx) * (axt - tx) + (tys1 - ty) * (ayt - ty)) / tlength;
  ts2 = ((txs2 - tx) * (axt - tx) + (tys2 - ty) * (ayt - ty)) / tlength;

  if (ts1 >= (double) 1.0 && ts2 >= (double) 1.0)
    {
      

      *jstat = 1;
      goto out;
    }

  

  if (ityp >= 3)
    {
      

      as = ts1;

      if (as >= (double) 1.0)
	as = ts2;

      if (as >= (double) 1.0)
	{
	  

	  *jstat = 1;
	  goto out;
	}

      ts1 = as;
      ts2 = as;
    }

  

  for (ki = 1; ki < inbpnt - 1; ki++)
    {
      ktyp = eptyp[ki];
      if (ktyp < 3)
	break;
    }

  

  if (ktyp < 3)
    {
      

      tnx = -epoint[1] + epoint[kk + 1];
      tny = epoint[0] - epoint[kk];


      

      kki = idim * ki;
      tix = epoint[kki] - epoint[0];
      tiy = epoint[kki + 1] - epoint[1];


      

      tvx = axt - epoint[0];
      tvy = ayt - epoint[1];

      ksam = 1;
      if ((tnx * tix + tny * tiy) * (tnx * tvx + tny * tvy) < (double) 0.0)
	ksam = 0;
    }
  else if (ktyp > 3)
    {
      

      ktyp = eptyp[1];

      if (ktyp == 4)
	{
	  

	  tvx = axt - epoint[0];
	  tvy = ayt - epoint[1];
	  ksam = 1;
	  if ((epoint[idim] * tvx + epoint[idim + 1] * tvy) < (double) 0.0)
	    ksam = 0;
	}
      else
	{
	  

	  ktyp = eptyp[inbpnt - 2];

	  if (ktyp != 3)
	    {
	      

	      *jstat = 1;
	      goto out;
	    }

	  

	  tvx = epoint[kk] - axt;
	  tvy = epoint[kk + 1] - ayt;
	  ksam = 1;
	  if ((epoint[kki - idim] * tvx + epoint[kki - idim + 1] * tvy)
	      < (double) 0.0)
	    ksam = 0;
	}
    }

  

  if (ksam == 1)
    {
      

      if (krem == 0)
	{

	  

	  as = ts1;
	  if (as < (double) 0.0)
	    as = ts2;
	  if (as < (double) 0.0)
	    {
	      

	      *jstat = 0;
	      goto out;
	    }
	  else
	    {
	      

	      if (krem == 1)
		{
		  

		  axt = txs2;
		  ayt = tys2;
		}
	      else
		{
		  

		  as = ts1;
		  if (as > (double) 0.0)
		    as = ts2;
		  if (as > (double) 0.0)
		    {
		      

		      *jstat = 1;
		      goto out;
		    }
		}
	    }
	}
    }

  

  

  if (krem == 1)
    {
      axt = axt - (epoint[0] + epoint[kk]) / ((double) 2.0);
      ayt = ayt - (epoint[1] + epoint[kk + 1]) / ((double) 2.0);
      as = (double) 0.0;
    }

  *ashape = as;
  etang[0] = axt;
  etang[1] = ayt;
  if (idim == 3)
    etang[2] = (double) 0.0;

  goto out;

out:

  return;
}

