/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1773

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
static
void
s1773_s9corr(double [],double,double,double,double,double,double);
static
void
s1773_s9dir(double *,double *,double *,double [],double [],double [],
		  double, int,int *);
#else
static void s1773_s9corr();
static void s1773_s9dir();
#endif

#if defined(SISLNEEDPROTOTYPES)
void 
s1773(SISLPoint *ppoint,SISLSurf *psurf,double aepsge,
	   double estart[],double eend[],double enext[],double gpos[],int *jstat)
#else
void s1773(ppoint,psurf,aepsge,estart,eend,enext,gpos,jstat)
     SISLPoint  *ppoint;
     SISLSurf   *psurf;
     double aepsge;
     double estart[];
     double eend[];
     double enext[];
     double gpos[];
     int    *jstat;
#endif
                       
{                        
  int kstat = 0;            
  int kpos = 0;             
  int kleft1=0;             
  int kleft2=0;             
  int kder=1;               
  int kdim;                 
  int knbit;                
  int kdir;                 
  int kdeg;                 
  double tdelta[2];         
  double tdist;             
  double td[2],t1[2],tdn[2];
  double tprev;             
  double *sval =SISL_NULL;        
  double *sdiff;            
  double *snorm;            
  double snext[2];          
  double guess[2];          
  
  guess[0] = enext[0];
  guess[1] = enext[1];
  
  
  
  if (ppoint->idim != psurf->idim) goto err106;
  
  kdim = ppoint -> idim;
  
  if (kdim == 1)
    {
      s1173(ppoint,psurf,aepsge,estart,eend,guess,gpos,&kstat);
      if (kstat < 0)
        goto error;
      else
        {
	  if (DNEQUAL(gpos[0],estart[0]) &&
	      DNEQUAL(gpos[0],eend[0]) &&
	      DNEQUAL(gpos[1],estart[1]) &&
	      DNEQUAL(gpos[1],eend[1])) 
	    *jstat = (kstat==1 ? 1:3);
	  else
	    *jstat = 0;
	  goto out;
        }
    }
  
  
  
  tdelta[0] = psurf->et1[psurf->in1] - psurf->et1[psurf->ik1 - 1];
  tdelta[1] = psurf->et2[psurf->in2] - psurf->et2[psurf->ik2 - 1];
  
  
  
  sval = newarray(8*kdim,double);
  if (sval == SISL_NULL) goto err101;
  
  sdiff = sval + 6*kdim;
  snorm = sdiff + kdim;
  
  
  
  tprev = (double)HUGE;
  
  
  
  
  s1421(psurf,kder,guess,&kleft1,&kleft2,sval,snorm,&kstat);
  if (kstat < 0) goto error;
  
  
  
  s1773_s9dir(&tdist,td,td+1,sdiff,ppoint->ecoef,sval,
	      aepsge,kdim,&kdeg);
  
  
  
  t1[0] = td[0];
  t1[1] = td[1];
  s1773_s9corr(t1,guess[0],guess[1],estart[0],eend[0],estart[1],eend[1]);
  
  
  
  for (knbit = 0; knbit < 30; knbit++)
    {
      
      
      snext[0] = guess[0] + t1[0];
      snext[1] = guess[1] + t1[1];
      
      s1421(psurf,kder,snext,&kleft1,&kleft2,sval,snorm,&kstat);
      if (kstat < 0) goto error;
      
      
      
      s1773_s9dir(&tdist,tdn,tdn+1,sdiff,ppoint->ecoef,
	    sval,aepsge,kdim,&kdeg);
      
      
      
      kdir = (s6scpr(td,tdn,2) >= DZERO);     
      
      
      
      if (tdist < tprev/(double)2 || kdir)
	{
	   guess[0] += t1[0];
	   guess[1] += t1[1];
  
	  
  
	  
          td[0] = t1[0] = tdn[0];
          td[1] = t1[1] = tdn[1];
	  
	  
	  
	  s1773_s9corr(t1,guess[0],guess[1],estart[0],eend[0],estart[1],eend[1]);
          tprev = tdist;

	  if ( (fabs(t1[0]/tdelta[0]) <= REL_COMP_RES) &&
	      (fabs(t1[1]/tdelta[1]) <= REL_COMP_RES)) break;
	}
      
      
      
      else
	{
          t1[0] /= (double)2;
          t1[1] /= (double)2;
          
	}
      if (guess[0]==guess[0]+t1[0] &&
	  guess[1]==guess[1]+t1[1]) break;
    }
  
  
  
  if (tdist <= aepsge)
  {
     *jstat = 1;
     
     
  }
  else if(kdeg)
     *jstat = 9;
  else
     *jstat = 2;
  
  gpos[0] = guess[0];
  gpos[1] = guess[1];
  
  
  
  goto out;
  
  
  
 err101: *jstat = -101;
  s6err("s1773",*jstat,kpos);
  goto out;                  
  
  
  
 err106: *jstat = -106;
  s6err("s1773",*jstat,kpos);
  goto out;                  
  
  
  
  error : *jstat = kstat;
  s6err("s1773",*jstat,kpos);
  goto out;                  
  
 out:    if (sval != SISL_NULL) freearray(sval);
}

#if defined(SISLNEEDPROTOTYPES)
static
void
s1773_s9corr(double gd[],double acoef1,double acoef2,
		   double astart1,double aend1,double astart2,double aend2)
#else
static void s1773_s9corr(gd,acoef1,acoef2,astart1,aend1,astart2,aend2)
     double gd[];
     double acoef1;
     double acoef2;
     double astart1;
     double aend1;
     double astart2;
     double aend2;
#endif
                       
{
  if (acoef1 + gd[0] < astart1)  gd[0] = astart1 - acoef1;
  else if (acoef1 + gd[0] > aend1) gd[0] = aend1 - acoef1;
  
  if (acoef2 + gd[1] < astart2)  gd[1] = astart2 - acoef2;
  else if (acoef2 + gd[1] > aend2) gd[1] = aend2 - acoef2;
}

#if defined(SISLNEEDPROTOTYPES)
static
void
s1773_s9dir(double *cdist,double *cdiff1,double *cdiff2,
		  double PS[],double eval1[],double eval2[],
		  double aepsge, int idim,int *jstat)
#else
static void s1773_s9dir(cdist,cdiff1,cdiff2,PS,
			eval1,eval2,aepsge,idim,jstat)
     double *cdist;
     double *cdiff1;
     double *cdiff2;
     double PS[];
     double eval1[];
     double eval2[];
     double aepsge;
     int    idim;
     int    *jstat;
#endif
                       
{                        
  int kstat=0;		          
  register double tdet;		  
  register double t1,t2,t3,t4,t5; 
  register double *S, *Su, *Sv;
  
                                  
  register double ref, ang;       
  register double l1, l2;         
  register double min_ang=10e-11; 
  
  
  
  *jstat = 0;
  *cdiff1 = DZERO;
  *cdiff2 = DZERO;
  
  
  S   = eval2;
  Su  = S   + idim;
  Sv  = Su  + idim;
  

  
  l1 = s6length(Su,idim,&kstat);
  l2 = s6length(Sv,idim,&kstat);
  ang = s6ang(Su,Sv,idim);
  if (min(l1,l2) < aepsge || ang < min_ang) *jstat = 1;

  
  s6diff(eval1,S,idim,PS);
  *cdist = s6length(PS,idim,&kstat);
  
  if (*jstat == 1)
  {
     if (l1 < aepsge)
     {
	if (l2 > aepsge)
	   
	   *cdiff2 = s6scpr(PS,Sv,idim)/l2*l2;
     }
     else if (l2 < aepsge)
	   
	   *cdiff1 = s6scpr(PS,Su,idim)/(l1*l1);
     else 
     {
	
	*cdiff1 = s6scpr(PS,Su,idim)/(l1*l1);
      }
	
  }
  else 
     
  {
     
     t1 =  s6scpr(Su,Su,idim) ; 
     t2 =  s6scpr(Su,Sv,idim) ; 
     t3 =  s6scpr(Sv,Sv,idim) ; 
     t4 =  s6scpr(PS,Su,idim);
     t5 =  s6scpr(PS,Sv,idim);
     
     ref = max(fabs(t1),fabs(t2));
     ref = max(ref,fabs(t3));
     
     
     tdet = t1*t3 - t2*t2;
     
     if (DEQUAL(ref+fabs(tdet),ref))
     {
	*jstat = 1;
     }
     else 
     {
	
	
	*cdiff1 =  (t4*t3-t5*t2)/tdet;
	*cdiff2 =  (t1*t5-t2*t4)/tdet;
     }
  }
}

