/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1839

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1839(SISLSurf *ps1,double epol[],int in,int idim,int *jstat)
#else
void s1839(ps1,epol,in,idim,jstat)
     SISLSurf   *ps1;
     double epol[];
     int    in;
     int    idim;
     int    *jstat;
#endif
{
  int kstat = 0;           
  int kpos = 0;            
  int ki;                  
  int kn1,kn2;             
  int kk1,kk2;             
  int kvec;                
  int klap;                
  double *scoef;           
  register double *s1,*s2,
  *s3,*s4;                 
  double *sdir = SISL_NULL;     
  
  
  
  if (idim != 2 && idim != 3) goto err105;
  if (idim != ps1->idim) goto err106;
  
  
  
  kn1 = ps1 -> in1;
  kn2 = ps1 -> in2;
  kk1 = ps1 -> ik1;
  kk2 = ps1 -> ik2;
  scoef = ps1 -> ecoef;
  
  
  
  if (kk1 > 2 || kk2 > 2) kvec = 10; else kvec = 2;
  
  
  
  sdir = newarray(kvec*idim,double);
  if (sdir == SISL_NULL) goto err101;
  
  
  
  for (s1=sdir,s2=s1+idim,s3=scoef,s4=scoef+idim*(kn1*kn2-1); s1<s2;
       s1++,s3++,s4++)
    *s1 = *s4 - *s3;
  
  
  
  for (s1=sdir+idim,s2=s1+idim,s3=scoef+idim*kn1*(kn2-1),
       s4=scoef+idim*(kn1-1); s1<s2; s1++,s3++,s4++)
    *s1 = *s4 - *s3;
  
  if (kvec > 2)
    {
      
      
      
      for (s1=sdir+2*idim,s2=s1+idim,s3=scoef; s1<s2; s1++,s3++)
	{       
	  *s1 = *(s3+idim) - *s3;
	  *(s1+idim) = *(s3+idim*kn1) - *s3;
	}
      
      
      
      for (s1=sdir+4*idim,s2=s1+idim,s3=scoef+idim*(kn1-1); s1<s2; s1++,s3++)
	{
	  *s1 = *(s3-idim) - *s3;
	  *(s1+idim) = *(s3+idim*kn1) - *s3;
	}
      
      
      
      for (s1=sdir+6*idim,s2=s1+idim,s3=scoef+idim*kn1*(kn2-1);s1<s2;s1++,s3++)
	{
	  *s1 = *(s3+idim) - *s3;
	  *(s1+idim) = *(s3-idim*kn1) - *s3;
	}
      
      
      
      for (s1=sdir+8*idim,s2=s1+idim,s3=scoef+idim*(kn1*kn2-1);s1<s2;s1++,s3++)
	{
	  *s1 = *(s3-idim) - *s3;
	  *(s1+idim) = *(s3-kn1*idim) - *s3;
	}
    }
  
  
  
  klap = 1;
  s1834(scoef,kn1*kn2,epol,in,idim,sdir,sdir+idim,&kstat);
  if (kstat < 0) goto error;
  klap = kstat;
  
  if (klap == 1)
    {
      s1834(scoef,kn1*kn2,epol,in,idim,sdir+idim,sdir,&kstat);
      if (kstat < 0) goto error;
      klap = kstat;
    }
  
  
  
  ki = 2;
  while (ki<kvec && klap == 1)
    {
      s1834(scoef,kn1*kn2,epol,in,idim,sdir+ki*idim,sdir+(ki+1)*idim,&kstat);
      if (kstat < 0) goto error;
      klap = kstat;
      ki += 2;
    }
  
  
  
  *jstat = klap;
  goto out;
  
  
  
 err101: *jstat = -101;
  s6err("s1839",*jstat,kpos);
  goto out;
  
  
  
 err105: *jstat = -105;
  s6err("s1839",*jstat,kpos);
  goto out;
  
  
  
 err106: *jstat = -106;
  s6err("s1839",*jstat,kpos);
  goto out;
  
  
  
  error : *jstat = kstat;
  s6err("s1839",*jstat,kpos);
  goto out;
  
 out: 
  
  
  
  if (sdir != SISL_NULL) freearray(sdir);
  
  return;
}

