/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1902

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1902 (double epar[], int in, int ik, int cuopen, double **eknots, int *jstat)

#else
void
s1902 (epar, in, ik, cuopen, eknots, jstat)
     double epar[];
     int in;
     int ik;
     int cuopen;
     double **eknots;
     int *jstat;

#endif
{
  int ki;			
  int kpar;			
  int kn;			
  int kk2;			
  int kstop;			
  int smult;			
  int emult;			
  int kmult;
  double tprev;			
  double th;			
  double tval1;			
  double tval2;			
  double tparint;		
  double tdum;			
  double dummy1, dummy2;
  double delta;
  int count1;

  

  if (cuopen == SISL_CRV_OPEN)
    {
      

      *eknots = newarray (in +ik, DOUBLE);
      if (*eknots == SISL_NULL)
	goto err101;

      kk2 = ik / 2;
      tval1 = epar[0];
      tval2 = epar[in -1];

      

      for (ki = 0; ki < ik; ki++)
	(*eknots)[ki] = tval1;

      

      for (smult = 1; epar[smult - 1] == epar[smult]; smult++) ;

      

      for (emult = 1, count1 = in -2; epar[count1] == epar[count1 + 1];
	   count1--)
	emult++;


      if (ik % 2 == 0)
	{
	  

	  dummy1 = (double)0.5 * (epar[smult] + tval1);
	  dummy2 = (double)0.5 * (epar[in -emult - 1] +tval2);
	  if (dummy1 == dummy2)
	    {
	      dummy1 = (epar[smult] + tval1 + tval1)/(double)3;
	      dummy2 = (epar[in -emult - 1] + tval2 + tval2)/(double)3;
	    }
	  for (count1 = 0; count1 < smult - kk2; count1++, ki++)
	    (*eknots)[ki] = dummy1;

	  for (kpar = smult + MAX (0, kk2 - smult), kstop = in -emult -
	       MAX (0, kk2 - emult);
	       kpar < kstop; kpar++, ki++)
	    (*eknots)[ki] = epar[kpar];

	  for (count1 = 0; count1 < emult - kk2; count1++, ki++)
	    (*eknots)[ki] = dummy2;
	}
      else
	{
	  

	  if (smult - kk2 > 0)
	    {
	      delta = (epar[smult] - tval1) / (smult - kk2 + 1);
	      dummy1 = tval1 + delta;
	      for (count1 = 0; count1 < smult - kk2;
		   count1++, dummy1 += delta, ki++)
		(*eknots)[ki] = dummy1;
	    }

	  for (kpar = smult + MAX (0, kk2 - smult), kstop = in -emult -
	       MAX (0, kk2 - emult);
	       kpar < kstop; kpar++, ki++)
	    (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]);

	  if (emult - kk2 > 0)
	    {
	      delta = (tval2 - epar[in -emult - 1]) / (emult - kk2 + 1);
	      dummy2 = epar[in -emult - 1] +delta;
	      for (count1 = 0; count1 < emult - kk2; count1++, dummy2 +=
		   delta, ki++)
		(*eknots)[ki] = dummy2;
	    }
	}

      


      for (ki = 0; ki < ik; ki++)
	(*eknots)[in +ki] = tval2;
    }
  else
    {
      

      *eknots = newarray (in +2 * ik, DOUBLE);
      if (*eknots == SISL_NULL)
	goto err101;

      kn = in +1;
      kk2 = ik / 2;
      kstop = in +2 * ik - 1;
      tparint = epar[in] -epar[0];

      if (ik % 2 == 0)
	{
	  

	  for (ki = 0, kpar = in -ik + ik / 2; ki < ik / 2; ki++, kpar++)
	    (*eknots)[ki] = epar[kpar] - tparint;
	  
	  

	  for (kpar = 0; kpar < kn; ki++, kpar++)
	    (*eknots)[ki] = epar[kpar];


	  

	  for (kpar = 1; ki < kstop; ki++, kpar++)
	    {
	      tdum = tparint;

	      

	      if (kpar > in)
		{
		  tdum += tparint;
		  kpar = 1;
		}
	      (*eknots)[ki] = epar[kpar] + tdum;
	    }
	}
      else
	{
	  

	  for (ki = 0, kpar = in -ik + ik / 2; ki < ik / 2 + 1; ki++, kpar++)
	    (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]) -
	     tparint;

	  

	  for (kpar = 0; kpar < in; ki++, kpar++)
	    (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]);

	  

	  for (kpar = 0; ki < kstop; ki++, kpar++)
	    {
	      tdum = tparint;

	      

	      if (kpar >= in)
		{
		  tdum += tparint;
		  kpar = 0;
		}
	      (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]) + tdum;
	    }
	}
    }


  

  if (cuopen == SISL_CRV_OPEN)
    kstop = in +ik;

  for (ki = 1, tprev = (*eknots)[0], kmult = 0; ki < kstop; ki++, tprev = th)
    {
       th = (*eknots)[ki];
      kmult++;
      if (tprev > th)
	goto err112;		
      if (tprev < th)
	kmult = 1;
      if (kmult > ik)
	goto err112;		
    }

  

  *jstat = 0;
  goto out;

  

err101:
  *jstat = -101;
  s6err ("s1902", *jstat, 0);
  goto out;

  

err112:*jstat = -112;
  s6err ("s1902", *jstat, 0);
  goto out;

out:
  return;
}

