/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1903

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1903 (double epar[], int in, int ik, int cuopen, double *eknots[], int *jstat)

#else
void
s1903 (epar, in, ik, cuopen, eknots, jstat)
     double epar[];
     int in;
     int ik;
     int cuopen;
     double *eknots[];
     int *jstat;

#endif
{
  int kpos = 0;
  int ki;			
  int kpar;			
  int kn;			
  int kk2;			
  int kstop;			
  int kmult;			
  double tprev;			
  double curr;			
  double tval1;			
  double tval2;			
  double tparint;		
  double tdum;			

  *jstat = 0;


  

  if (cuopen)
    {
      

      *eknots = newarray (in +ik, DOUBLE);
      if (*eknots == SISL_NULL)
	goto err101;

      kk2 = ik / 2;
      tval1 = epar[0];
      tval2 = epar[in -1];

      

      for (ki = 0; ki < ik; ki++)
	(*eknots)[ki] = tval1;

      if (ik % 2 == 0)
	{
	  

	  for (kpar = kk2, kstop = in -kk2; kpar < kstop; kpar++, ki++)
	    (*eknots)[ki] = epar[kpar];
	}
      else
	{
	  

	  for (kpar = kk2, kstop = in -kk2 - 1; kpar < kstop; kpar++, ki++)
	    (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]);
	}

      

      for (ki = 0; ki < ik; ki++)
	(*eknots)[in +ki] = tval2;
    }
  else
    {
      

      *eknots = newarray (in +2 * ik, DOUBLE);
      if (*eknots == SISL_NULL)
	goto err101;

      kn = in +1;
      kk2 = ik / 2;
      kstop = in +2 * ik - 1;
      tparint = epar[in] -epar[0];

      if (ik % 2 == 0)
	{
	  

	  for (ki = 0, kpar = in -ik + 1; ki < ik - 1; ki++, kpar++)
	    (*eknots)[ki] = epar[kpar] - tparint;

	  

	  for (kpar = 0; kpar < kn; ki++, kpar++)
	    (*eknots)[ki] = epar[kpar];

	  

	  for (kpar = 1; ki < kstop; ki++, kpar++)
	    {
	      tdum = tparint;

	      

	      if (kpar > kn)
		{
		  tdum += tparint;
		  kpar -= in;
		}
	      (*eknots)[ki] = epar[kpar] + tdum;
	    }
	}
      else
	{
	  

	  for (ki = 0, kpar = in -ik + 1; ki < ik - 1; ki++, kpar++)
	    (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]) - tparint;

	  

	  for (kpar = 0; kpar < in; ki++, kpar++)
	    (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]);

	  

	  for (kpar = 0; ki < kstop; ki++, kpar++)
	    {
	      tdum = tparint;

	      

	      if (kpar > kn)
		{
		  tdum += tparint;
		  kpar -= in;
		}
	      (*eknots)[ki] = (double) 0.5 *(epar[kpar] + epar[kpar + 1]) + tdum;
	    }
	}
    }


  

  if (cuopen)
    kstop = in +ik;

  for (ki = 1, tprev = (*eknots)[0], kmult = 0; ki < kstop; ki++, tprev = curr)
    {
       curr = (*eknots)[ki];
      kmult++;
      if (tprev > curr)
	goto err112;		
      if (tprev < curr)
	kmult = 1;
      if (kmult > ik)
	goto err112;		
    }

  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1903", *jstat, kpos);
  goto out;

  

err112:
  *jstat = -112;
  s6err ("s1903", *jstat, kpos);
  goto out;

out:
  return;
}

