/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1917
#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1917 (int inbcrv, double ecoef[], int in2,
       int idim, int eptyp[], double astpar, int iopen,
       double *par[], int *der[], int *inumb, int *jstat)
#else
void
s1917 (inbcrv, ecoef, in2, idim, eptyp, astpar, iopen,
       par, der, inumb, jstat)
     int inbcrv;
     double ecoef[];
     int in2;
     int idim;
     int eptyp[];
     double astpar;
     int iopen;
     double *par[];
     int *der[];
     int *inumb;
     int *jstat;
#endif
{
  int kpos = 0;
  int knext;			
  int ktang;			
  int kcurv;			
  int kerr;			
  int kprev;			
  int knumb;			
  int kpopar;
  int kkkksm;
  int kant;			
  int ktyp = 0;
  int ki, kj, kl, kr;		
  int legal;			
  int dummy1;
  int dummy2;

  double tsum;			
  double tdum;			
  int    kdum;
  double tdiff;

  *jstat = 0;


  

  if (idim < 1)
    goto err102;
  if (inbcrv < 2)
    goto err179;


  

  *par = newarray (inbcrv + 1, DOUBLE);
  if (*par == SISL_NULL)
    goto err101;

  *der = newarray (inbcrv, INT);
  if (*der == SISL_NULL)
    goto err101;


  

  kerr = 0;
  knumb = 0;
  kpopar = 0;
  kkkksm = 0;


  

  knext = TRUE;
  ktang = FALSE;
  kcurv = FALSE;


  

  kant = inbcrv;
  ki = 1;

  while (ki <= kant)
    {
      legal = TRUE;

      ktyp = eptyp[ki - 1];

      if (ktyp == 1 || ktyp == 2)
	{
	  if (ktyp == 2 && ki != 1 && ki != inbcrv)
	    kerr = 1;

	  if (knext == FALSE)
	    {
	      ktang = FALSE;
	      kcurv = FALSE;
	    }
	  knext = FALSE;


	  

	  if (kkkksm != 0)
	    {
	      tsum = (double) 0.0;
	      for (kl = 0; kl < in2; kl++)
		{
		  tdum = (double) 0.0;
		  dummy1 = (ki - 1) * in2 * idim + kl * idim;
		  dummy2 = (kkkksm - 1) * in2 * idim + kl * idim;
		  for (kj = 0; kj < idim; kj++)
		    {
		      tdiff = ecoef[dummy1 + kj] - ecoef[dummy2 + kj];
		      tdum += tdiff * tdiff;
		    }
		  tsum += sqrt (tdum);
		}
	      if (tsum == (double) 0.0)
		legal = FALSE;
	      else
		astpar += tsum / (double)in2;
	    }

	  if (legal)
	    {
	      (*der)[knumb] = 0;
	      for (kj = kpopar; kj <= knumb; kj++)
		(*par)[kj] = astpar;

	      kprev = ki;
	      kkkksm = ki;
	      kpopar = knumb + 1;
	    }
	}
      else if (ktyp == 3 || ktyp == 13)
	{
	  

	  

	  if (knext && ktang)
	    legal = FALSE;
	  else
	    {
	      if (knext == FALSE)
		kcurv = FALSE;
	      ktang = TRUE;
	      knext = TRUE;

	      

	      (*der)[knumb] = 1;
	    }
	}
      else if (ktyp == 4 || ktyp == 14)
	{
	  

	  

	  if (knext || (ktang && !knext))
	    legal = FALSE;
	  else
	    {
	      ktang = TRUE;

	      

	      (*der)[knumb] = 1;
	      (*par)[knumb] = astpar;
	      kpopar = knumb + 1;
	    }
	}
      else if (ktyp == 5)
	{
	  

	  

	  if (knext && kcurv)
	    legal = FALSE;
	  else
	    {
	      if (knext == FALSE)
		ktang = FALSE;
	      kcurv = TRUE;
	      knext = TRUE;

	      

	      (*der)[knumb] = 2;
	    }
	}
      else if (ktyp == 6)
	{
	  

	  

	  if (knext || (kcurv && !knext))
	    kcurv = TRUE;

	  

	  (*der)[knumb] = 2;
	  (*par)[knumb] = astpar;
	  kpopar = knumb + 1;
	}
      else
	legal = FALSE;


      if (legal)
	{
	  

	  

	  ki++;
	  knumb++;
	}
      else
	{
	  kerr = 1;

	  

	  kant--;

	  if (ki <= kant)
	    {
	      memcopy (&eptyp[ki - 1], &eptyp[ki], kant - ki, INT);
	      memcopy (&ecoef[(ki - 1) * in2 * idim], &ecoef[ki * in2 * idim],
		       (kant - ki) * in2 * idim, DOUBLE);
	    }
	}
    }

  

  if (knext)
    {
      

      if (ktang)
	knumb--;
      if (kcurv)
	knumb--;

      ktang = FALSE;
      kcurv = FALSE;
      kerr = 1;
    }

  if (kpopar > knumb)
    {
      

      kerr = 1;
      knumb = kpopar;
    }

  

  if (iopen != SISL_CRV_OPEN)
    {
      for (ki = 1; eptyp[ki] != 1 && eptyp[ki] != 2; ki++)
	if (ki > inbcrv)
	  goto err179;


      

      tsum = (double) 0.0;

      for (kl = 0; kl < in2; kl++)
	{
	  tdum = (double) 0.0;
	  dummy1 = (ki - 1) * idim * in2 + kl * idim;
	  dummy2 = (kprev - 1) * idim * in2 + kl * idim;
	  for (kj = 0; kj < idim; kj++)
	    {
	      tdiff = ecoef[dummy1 + kj] - ecoef[dummy2 + kj];
	      tdum += tdiff * tdiff;
	    }
	  tsum += sqrt (tdum);
	}

      

      if (tsum == (double) 0.0)
	{
	  knumb--;
	  kerr = 1;
	}
      else
	{
	  astpar += tsum / (double)in2;
	  (*par)[knumb] = astpar;
	}
    }

  

  

  for (kl = knumb - 2; kl >= 1 && (*par)[kl] >= (*par)[knumb - 1] &&
       (*der)[kl] != 0; kl--) ;

  if (kl >= 1 && (*par)[kl] >= (*par)[knumb - 1])
    {
      

      (*der)[kl] = (*der)[knumb - 1];
      (*der)[knumb - 1] = 0;
      kdum = eptyp[knumb - 1];
      eptyp[knumb - 1] = eptyp[kl];
      eptyp[kl] = kdum - 1;


      for (kr = kl + 1; kr < knumb - 1; kr++)
	{
	  ktyp = eptyp[kr];
	  if (ktyp == 4 || ktyp == 6)
	    (eptyp[kr])--;
	}

      for (kr = 0; kr < in2; kr++)
	{
	  dummy1 = (knumb - 1) * in2 * idim + kr * idim;
	  dummy2 = kl * in2 * idim + kr * idim;
	  for (kj = 0; kj < idim; kj++)
	    {
	      tdum = ecoef[dummy1 + kj];
	      ecoef[dummy1 + kj] = ecoef[dummy2 + kj];
	      ecoef[dummy2 + kj] = tdum;

	      ktyp = eptyp[kl];
	      if (ktyp >= 10)
		ecoef[dummy2 + kj] = (double) 2.0 *ecoef[dummy1 + kj]
		- ecoef[dummy2 + kj];
	    }
	}
    }

  

  for (kl = knumb - 3; kl >= 1 && (*par)[kl] >= (*par)[knumb - 2] &&
       (*der)[kl] != 1; kl--) ;

  if (kl >= 1 && (*par)[kl] >= (*par)[knumb - 2])
    {
      

      (*der)[kl] = (*der)[knumb - 2];
      (*der)[knumb - 2] = 1;
      kdum = eptyp[knumb - 2];
      eptyp[knumb - 2] = eptyp[kl];
      eptyp[kl] = kdum;

      for (kr = 0; kr < in2; kr++)
	{
	  dummy1 = (knumb - 2) * in2 * idim + kr * idim;
	  dummy2 = kl * in2 * idim + kr * idim;
	  for (kj = 0; kj < idim; kj++)
	    {
	      tdum = ecoef[dummy1 + kj];
	      ecoef[dummy1 + kj] = ecoef[dummy2 + kj];
	      ecoef[dummy2 + kj] = tdum;
	    }
	}
    }

  if (knumb < 2)
    goto err187;

  

  *inumb = knumb;

  if (kerr == 1)
    *jstat = 102;
  else
    goto out;


  

err101:
  *jstat = -101;
  s6err ("s1917", *jstat, kpos);
  goto out;

  

err102:
  *jstat = -102;
  s6err ("s1917", *jstat, kpos);
  goto out;

  

err179:
  *jstat = -179;
  s6err ("s1917", *jstat, kpos);
  goto out;

  

err187:
  *jstat = -187;
  s6err ("s1917", *jstat, kpos);
  goto out;

out:
  return;
}

