/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1938

#include "sislP.h"
#define MAX_SIZE 50


#if defined(SISLNEEDPROTOTYPES)
void
s1938 (SISLSurf * srf, double etr1[], int inr1, double etr2[], int inr2,
       double **surfr, int *jstat)
#else
void
s1938 (srf, etr1, inr1, etr2, inr2, surfr, jstat)
     SISLSurf *srf;
     double etr1[];
     int inr1;
     double etr2[];
     int inr2;
     double **surfr;
     int *jstat;

#endif
{
  int kpos = 0;
  int ki, kj, kl, kr, low;	
  int start;
  int rem;			
  int nu;			
  int ins1;			
  int iordr1;			
  int ins2;			
  int iordr2;			
  double *knt1 = SISL_NULL;		
  double *knt2 = SISL_NULL;		
  double *coef = SISL_NULL;		
  int idim;			
  double sum;			
  double sarray[MAX_SIZE];
  int alloc_needed=FALSE;
  double *alfa = SISL_NULL;		
  double *ktsurf = SISL_NULL;	

  *jstat = 0;


  

  knt1 = srf->et1;
  knt2 = srf->et2;
  ins1 = srf->in1;
  ins2 = srf->in2;
  iordr1 = srf->ik1;
  iordr2 = srf->ik2;
  if (srf->ikind == 2 || srf->ikind == 4)
    {
      idim = srf->idim + 1;
      coef = srf->rcoef;
    }
  else
    {
      idim = srf->idim;
      coef = srf->ecoef;
    }
     
  

  if (iordr1 < 1 || iordr2 < 1)
    goto err115;
  if (ins1 < iordr1 || ins2 < iordr2)
    goto err116;
  if (idim < 1)
    goto err102;


  

  if (MAX(iordr1,iordr2) > MAX_SIZE)
    {
      if ((alfa = newarray(MAX(iordr1,iordr2),DOUBLE)) == SISL_NULL)
	goto err101;
      alloc_needed = TRUE;
    }
  else
    alfa = sarray;
  
  ktsurf = newarray (inr1 * inr2 * idim, DOUBLE);
  if (ktsurf == SISL_NULL)
    goto err101;


  

  *surfr = newarray (inr1 * inr2 * idim, DOUBLE);
  if (*surfr == SISL_NULL)
    goto err101;


  

  kj = iordr1 - 1;

  for (ki = 0; kj < ins1; ki++)
    {
      if (ki >= inr1)
	goto err116;
      if (knt1[kj] > etr1[ki])
	continue;
      if (knt1[kj] < etr1[ki])
	goto err117;
      kj++;
    }

  

  nu = 1;
  for (kj = 0; kj < inr1; kj++)
    {
      

      for (; (((knt1[nu - 1] > etr1[kj]) || (etr1[kj] >= knt1[nu]))
	      && (nu != ins1)); nu++) ;


      

      s1937 (knt1, iordr1, kj + 1, nu, alfa, etr1);


      

      low = nu - iordr1 + 1;
      for (ki = 0; ki < ins2; ki++)
	{
	  rem = idim * kj + idim * inr1 * ki;

	  for (kl = 0; kl < idim; kl++)
	    {
	      sum = (double) 0.0;
	      start = nu - iordr1 + 1;
	      if (start < 1)
		start = 1;

	      for (kr = start; kr <= nu; kr++)
		sum += alfa[kr - low] * coef[ki * ins1 * idim + (kr - 1) * idim + kl];
	      ktsurf[rem + kl] = sum;
	    }
	}
    }

  

  kj = iordr2 - 1;

  for (ki = 0; kj < ins2; ki++)
    {
      if (ki >= inr2)
	goto err116;
      if (knt2[kj] > etr2[ki])
	continue;
      if (knt2[kj] < etr2[ki])
	goto err117;
      kj++;
    }

  

  nu = 1;
  for (ki = 0; ki < inr2; ki++)
    {
      

      for (; (((knt2[nu - 1] > etr2[ki]) || (etr2[ki] >= knt2[nu]))
	      && (nu != ins2)); nu++) ;


      

      s1937 (knt2, iordr2, ki + 1, nu, alfa, etr2);


      

      low = nu - iordr2 + 1;
      for (kj = 0; kj < inr1; kj++)
	for (kl = 0; kl < idim; kl++)
	  {
	    sum = (double) 0.0;
	    start = nu - iordr2 + 1;
	    if (start < 1)
	      start = 1;
	    for (kr = start; kr <= nu; kr++)
	      sum += alfa[kr - low] * ktsurf[kj * idim + (kr - 1) * idim * inr1 + kl];
	    (*surfr)[ki * idim * inr1 + kj * idim + kl] = sum;
	  }
    }


  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1938", *jstat, kpos);
  goto out;

  


err102:
  *jstat = -102;
  s6err ("s1938", *jstat, kpos);
  goto out;

  

err115:
  *jstat = -115;
  s6err ("s1938", *jstat, kpos);
  goto out;

  

err116:
  *jstat = -116;
  s6err ("s1938", *jstat, kpos);
  goto out;

  

err117:
  *jstat = -117;
  s6err ("s1938", *jstat, kpos);
  goto out;

out:
  if (alloc_needed)
    freearray (alfa);
  if (ktsurf != SISL_NULL)
    freearray (ktsurf);

  return;
}
#undef MAX_SIZE

