/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1941

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1941(SISLCurve *pcurve,int icont,int *jstat)
#else
void s1941(pcurve,icont,jstat)
     SISLCurve  *pcurve;
     int        icont;
     int    	*jstat;
#endif
{
  double *scycl=SISL_NULL;                    
  double *smatrix=SISL_NULL;                   
  double *salloc=SISL_NULL;                    
  double *salfa=SISL_NULL;                     
  double *spek=SISL_NULL;                      
  double *scoef=SISL_NULL;                     
  double *sb=SISL_NULL;                        
  double *sfrom,*sto;
  double *sp;                             
  double *st1=SISL_NULL;                       
  double *stx=SISL_NULL;                       

  int    kdim = pcurve->idim;
  int    kk = pcurve->ik;
  int    kn = pcurve->in;

  int    kcont;                           
  int    kmult;                           
  int    ki,kj,kl;
  int    kperlength;
  int    kant;
  int    kleft=0;                         
  int    kpl,kfi,kla;                     
  int    kstat;
  int    *mpiv=SISL_NULL;                      
  int    kpos = 0;
  int    knst1;                           
  int    knstx;                           




  

  if (icont < 0) goto finished;
  kcont = icont;
  if (icont >= kk-2) icont = kk-2;

  
  kmult = kk - kcont - 1;

  

  kant = kk-kmult;


  

  mpiv = new0array(2*kk,INT);
  if (mpiv == SISL_NULL) goto err101;

  salloc = new0array(3*kn+9*kk+4*kk*kk+kdim*kn,DOUBLE);
  if (salloc == SISL_NULL) goto err101;
  scycl = salloc;                  
  smatrix = scycl + kn + kk;  
  salfa = smatrix + 4*kk*kk;     
  scoef = salfa + kk;           
  sb    = scoef + kdim*kn;    
  sp    = sb + 2*kk;              
  st1   = sp + kk;                
  stx   = st1 + kn + 2*kk;       



  

  memcopy(scoef,pcurve->ecoef,kdim*kn,DOUBLE);



  


  

  memcopy(scycl,pcurve->et,kn+kk,DOUBLE);

  

  kperlength = kn - kk + kmult;

  

  for (ki=kk-kmult-1 ; 0<=ki ; ki--)
    {
      scycl[ki] = scycl[kk-1] - (scycl[kn] - scycl[ki+kperlength]);
    }


  

  for (ki=kmult ; ki < kk ; ki++)
    {
      scycl[kn+ki] = scycl[kn] + (scycl[kk+ki-kmult] - scycl[kk-1]);

    }
      

  memcopy(st1,scycl,kant,DOUBLE);
  memcopy(st1+kant,pcurve->et,kn+kk,DOUBLE);
  memcopy(st1+kant+kk+kn,scycl+kn+kk-kant,kant,DOUBLE);
  knst1 = kn + 2*kant;

  memcopy(stx,scycl,kn,DOUBLE);
  memcopy(stx+kn,st1+kant+kn,kk+kant,DOUBLE);
  knstx = kn + kant;

  


  


  


  

  for (ki=kant,spek=smatrix ; ki <kk+kant ; ki++, spek+=kk)
    {
      

      s1219(stx,kk,kn,&kleft,st1[ki],&kstat);
      if (kstat<0) goto error;

      s1701(ki,kleft,kk,knstx,&kpl,&kfi,&kla,st1,stx,sp,salfa,&kstat);
      if(kstat<0) goto error;

      

      memcopy(spek+kfi,salfa+kpl+kfi,kla-kfi+1,DOUBLE);
    }



  

  s6lufacp(smatrix,mpiv,kk,&kstat);
  if (kstat<0) goto error;

  


  for (kl=0 ; kl<kdim ; kl++)
  {
     for (kj=0, sfrom=(pcurve->ecoef)+kl,sto=sb ;
      kj<kk ; kj++,sfrom+=kdim,sto++)
	*sto = *sfrom;
     
     
     
     s6lusolp(smatrix,sb,mpiv,kk,&kstat);
     if (kstat<0) goto error;
     
     
     
     for (kj=0, sto=scoef+kl,sfrom=sb ;
      kj<kk ; kj++,sfrom++,sto+=kdim)
	*sto = *sfrom;
  }


  


  for (ki=0,spek=smatrix ; ki<kk*kk ; ki++,spek++) *spek = DZERO;


  for (ki=kn-kk ,spek=smatrix ; ki <kn ; ki++, spek+=kk)
    {
      s1219(scycl,kk,kn,&kleft,stx[ki],&kstat);
      if (kstat<0) goto error;

      s1701(ki,kleft,kk,kn,&kpl,&kfi,&kla,stx,scycl,sp,salfa,&kstat);
      if(kstat<0) goto error;

      

      memcopy(spek+kfi-(kn-kk),salfa+kpl+kfi,kla-kfi+1,DOUBLE);
    }



  

  s6lufacp(smatrix,mpiv,kk,&kstat);
  if (kstat<0) goto error;

  

  for (kl=0 ; kl<kdim ; kl++)
  {
     for (kj=0, sfrom=scoef+kdim*(kn-kk)+kl,sto=sb ;
      kj<kk ; kj++,sfrom+=kdim,sto++)
	*sto = *sfrom;
     
     
     
     s6lusolp(smatrix,sb,mpiv,kk,&kstat);
     if (kstat<0) goto error;
     
     
     
     for (kj=0, sto=scoef+kdim*(kn-kk)+kl,sfrom=sb ;
      kj<kk ; kj++,sto+=kdim,sfrom++)
	*sto = *sfrom;
  }


  

  memcopy(pcurve->ecoef,scoef,kdim*kn,DOUBLE);
  memcopy(pcurve->et,scycl,kn+kk,DOUBLE);

  
  pcurve->cuopen = SISL_CRV_PERIODIC;


  

 finished:

  *jstat = 0;
  goto out;

  

 err101:
  *jstat = -101;
  s6err("s1941",*jstat,kpos);
  goto out;



  

  error :
    *jstat = kstat;
  s6err("s1941",*jstat,kpos);
  goto out;
 out:

  
  if (salloc != SISL_NULL) freearray(salloc);
  if (mpiv != SISL_NULL) freearray(mpiv);

  return;

}

