/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1991

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
     s1991(SISLCurve *pc,double aepsge,int *jstat)
#else
void s1991(pc,aepsge,jstat)
     SISLCurve  *pc;
     double aepsge;
     int    *jstat;
#endif
{
  int kpos = 0;     
  int kfirst = 1;   
  int kn;           
  int kdim;	    
  int kin;          
  int k1,k2;        
  double *t=SISL_NULL;   
  double tlen;      
  double tang;	    
  double t1,t2;     
  double *scoef;    



  

  if (pc->pdir != SISL_NULL) goto out;


  

  kdim = pc -> idim;
  kn = pc -> in;


  

  if ((pc->pdir = newdir(kdim))==SISL_NULL) goto err101;

  
  pc->pdir->aang = DZERO;
  pc->pdir->igtpi = 0;
  pc->pdir->ecoef[0] = (double) 1.0;

  for (k2 = 1;k2<kdim;k2++)
    pc->pdir->ecoef[k2] = DZERO;


  

  if ((t = newarray(kdim,double)) == SISL_NULL) goto err101;

  

  if ((pc->pdir->esmooth = newarray(kn*kdim,DOUBLE)) == SISL_NULL) goto err101;
  scoef = pc->pdir->esmooth;

  

   
   
   memcopy(scoef, pc->ecoef, kn*kdim, DOUBLE);

  

  for (k2=0,kin=0; kin < kn-1; kin++)
    {

      

      for (tlen=DZERO,k1=0; k1 < kdim; k1++,k2++)
	{
	  t[k1] = scoef[k2+kdim] - scoef[k2];
	  tlen += t[k1]*t[k1];
	}

      tlen = sqrt(tlen);

      if (tlen > aepsge)
	for (k1=0; k1 < kdim; k1++) t[k1] /= tlen;
      else
	{
	  
	  continue;

	  
	  

	}


      

      if (kfirst)
	{

	  

	  for (k1=0; k1 < kdim; k1++)
	    pc->pdir->ecoef[k1]= t[k1];

	  

	  pc->pdir->aang = DZERO;

	  kfirst = 0;   
	}
      else
	{

	  

	  for (tang=DZERO,k1=0;k1<kdim;k1++)
	    tang += pc->pdir->ecoef[k1]*t[k1];

	  if (tang >= DZERO) tang = min((double)1.0,tang);
	  else               tang = max((double)-1.0,tang);

	  tang = acos(tang);

	  if (tang + pc->pdir->aang >= PI)
	    {
	      

	      pc->pdir->igtpi = 1;
	      goto out;
	    }
	  else if (tang > pc->pdir->aang)
	    {
	      

	      

	      t1 = (tang - pc->pdir->aang)/((double)2*tang);
	      t2 = (double)1 - t1;

	      for (tlen=DZERO,k1=0; k1<kdim; k1++)
		{
		  pc->pdir->ecoef[k1] =
		    pc->pdir->ecoef[k1]*t2 + t[k1]*t1;
		  tlen += pc->pdir->ecoef[k1]*
		    pc->pdir->ecoef[k1];
		}
	      tlen = sqrt(tlen);

	      if (tlen > DZERO)
		for (k1=0; k1 < kdim; k1++)
		  pc->pdir->ecoef[k1] /= tlen;
	      else
		{
		  

		  pc->pdir->igtpi = 1;
		  goto out;
		}


	      

	      pc->pdir->aang = (tang + pc->pdir->aang)/
		(double)2;
	    }
	}
    }



  if (pc->pdir->aang >= SIMPLECASE)
    {
      

      pc->pdir->igtpi = 3;
      goto out;
    }


  *jstat = 0;
  goto out;


  

 err101: *jstat = -101;
  s6err("s1991",*jstat,kpos);
  goto out;

 out:    if (t != SISL_NULL) freearray(t);

}

#if 0
#if defined(SISLNEEDPROTOTYPES)
static void
  s1991_s9smooth(double ecoef1[],int in,int idim,double aepsge,
		 double ecoef2[],int *jstat)
#else
static void s1991_s9smooth(ecoef1,in,idim,aepsge,ecoef2,jstat)
   double ecoef1[];
   int    in;
   int    idim;
   double aepsge;
   double ecoef2[];
   int    *jstat;
#endif
{
   int kstat = 0;      
   int kn2 = in/2;     
   int ki;             
   double *sdiff1=SISL_NULL;
   double *sdiff2=SISL_NULL;
   double alfa,dnum;   
   double tdist;       
   double *s1,*s2,*s3; 
   double *st1,*st2;   

   
   if ((sdiff1 = newarray(idim,DOUBLE)) == SISL_NULL) goto err101;
   if ((sdiff2 = newarray(idim,DOUBLE)) == SISL_NULL) goto err101;


   
   memcopy(ecoef2,ecoef1,in*idim,DOUBLE);

   

   for (s1=ecoef2, st1=s1+(kn2-1)*idim; s1<st1; s1=s2)
   {
      for (s2=s1+2*idim, st2=st1+idim; s2<=st2; s2+=idim)
      {
	 if (s6dist(s1,s2,idim) < aepsge) continue;

	 for (s3=s1+idim; s3<s2; s3+=idim)
	 {
	    

	    tdist = s6dline(s1,s2,s3,idim,&kstat);
	    if (kstat < 0) goto error;

	    

	    if (kstat || tdist >= aepsge) break;  
	 }
	 if (s3 < s2) break; 
      }

      

      s2 -= idim;
      
      

      s6diff(s2,s1,idim,sdiff1);
      dnum = s6scpr(sdiff1,sdiff1,idim);


      for (s3=s1+idim; s3<s2; s3+=idim)
      {
	  if (dnum > DZERO)
	     {
		s6diff(s2,s3,idim,sdiff2);
		alfa = s6scpr(sdiff2,sdiff1,idim)/dnum;
		for (ki=0;ki<idim;ki++) s3[ki]=
		   alfa*s1[ki] + ((double)1.0 - alfa)*s2[ki];
	     }
      }

   }

   

   for (s1=ecoef2+(in-1)*idim, st1=s1-(kn2-1)*idim; s1>st1; s1=s2)
   {
      for (s2=s1-2*idim, st2=st1-idim; s2>=st2; s2-=idim)
      {
	 if (s6dist(s1,s2,idim) < aepsge) continue;

	 for (s3=s1-idim; s3>s2; s3-=idim)
	 {
	    

	    tdist = s6dline(s1,s2,s3,idim,&kstat);
	    if (kstat < 0) goto error;

	    

	    if (kstat || tdist >= aepsge) break;  
	 }
	 if (s3 > s2) break; 
      }

      

      s2 += idim;
      
      

      s6diff(s2,s1,idim,sdiff1);
      dnum = s6scpr(sdiff1,sdiff1,idim);

      for (s3=s1-idim; s3>s2; s3-=idim)
      {
	  if (dnum > DZERO)
	     {
		s6diff(s2,s3,idim,sdiff2);
		alfa = s6scpr(sdiff2,sdiff1,idim)/dnum;
		for (ki=0;ki<idim;ki++) s3[ki]=
		   alfa*s1[ki] + ((double)1.0 - alfa)*s2[ki];
	     }
      }
   }

   

   *jstat = 0;
   goto out;

  

   err101: *jstat = -101;
   goto out;

   

   error : *jstat = kstat;
   goto out;

   out :
      if (sdiff1) freearray(sdiff1);
      if (sdiff2) freearray(sdiff2);
      return;
}
#endif 

