/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S2533

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s2533(double *et, int ik, int in, int multinc, int newik, int *newin,
	   double **newet, int *stat)
#else
void s2533(et, ik, in, multinc, newik, newin, newet, stat)
      double *et;
      int ik;
      int in;
      int multinc; 
      int newik;
      int *newin;
      double **newet;
      int *stat;
#endif
{
   int ki, kj, kl;          
   int add_knot;            
   int knot_pos;            
   int *new_mult = SISL_NULL;    
   double *knot_par = SISL_NULL; 


   

   if (et == SISL_NULL || multinc < 0 || newik < (multinc + 2)) 
      goto err150;

   

   if (in > ik)
   {
      
      
      add_knot = (newik - ik) + multinc;
      
      if ((new_mult = newarray(in - ik, INT)) == SISL_NULL) goto err101;
      if ((knot_par = newarray(in - ik, DOUBLE)) == SISL_NULL) goto err101;
   }
   
   
   
   *newin = newik;
   knot_pos = 0;
   for (ki = ik, kl = 0; ki < in; kl++)
   {
      knot_par[kl] = et[ki];
      
      new_mult[kl] = add_knot + 1;
      kj = ki + 1;
      while (DEQUAL(et[kj], et[ki]))
      {
	 new_mult[kl]++;
	 kj++;
      }
      
      if (new_mult[kl] >= newik)
	 goto err150;
      
      *newin += new_mult[kl];
      knot_pos++;
      ki = kj;
   }
   
   
   
   if ((*newet = newarray((*newin + newik), DOUBLE)) == SISL_NULL) goto err101;
   
   
   
   for (kl = 0; kl < newik; kl++)
      (*newet)[kl] = et[ik - 1];
   
   for (ki = 0; ki < knot_pos; ki++)
   {
      for (kj = 0; kj < new_mult[ki]; kj++, kl++)
	 (*newet)[kl] = knot_par[ki];
   }
   
   for (ki = 0; ki < newik; ki++, kl++)
      (*newet)[kl] = et[in];
   
   goto out;
  
  
  
   

   
   
 err101: 
   *stat = -101;
   s6err("s2533", *stat, 0);
   goto out;

   
   
 err150:
   *stat = -150;
   s6err("s2533", *stat, 0);
   goto out;

   

 out:
   if (new_mult != SISL_NULL) freearray(new_mult); 
   if (knot_par != SISL_NULL) freearray(knot_par); 
   
   return;
}

