/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S2541

#include "sislP.h"




#if defined(SISLNEEDPROTOTYPES)
void
   s2541(SISLSurf *surf,
	 void evalp(SISLSurf *surf, int ider,int iside1,int iside2,
		    double parvalue[], int *leftknot1, int *leftknot2,
		    double *result, int *istat),
	 int dim,
	 int export_par_val,
	 int pick_subpart,
	 double boundary[],
	 int n_u,
	 int n_v,
	 double **garr,
	 int *stat)
#else
void
   s2541(surf, evalp, dim, export_par_val, pick_subpart, boundary, n_u, n_v,
	 garr, stat)
      SISLSurf *surf;
      void evalp();
      int dim;
      int export_par_val;
      int pick_subpart;
      double boundary[];
      int n_u;
      int n_v;
      double **garr;
      int *stat;
#endif
{
  int dimpnt;			
  int kder = 0;                 
  int ki, kj;               	
  int klfs = 0;                 
  int klft = 0;            	
  int kside1=0,kside2=0;    	
  int incr;    			
  double duv[2];		
  double UV[2];			
  double *sarr = SISL_NULL;    	
  double *sp = SISL_NULL;      	
  SISLSurf *temp = SISL_NULL;  	
  


  

  *garr  = SISL_NULL;
  *stat = 0;


  
  if ( !surf ) 		goto err150;
  if (dim < 1) 	        goto err102;
  if ( n_u < 1) 	goto err172;
  if ( n_v < 1) 	goto err172;

  

  if (pick_subpart == 1)
  {
     s1001 (surf, boundary[0], boundary[1], boundary[2], boundary[3],
   	 &temp, stat);
     if (*stat < 0) goto error;
  }
  else
  {
     temp = surf;
     boundary[0] = temp->et1[temp->ik1 - 1];
     boundary[1] = temp->et2[temp->ik2 - 1];
     boundary[2] = temp->et1[temp->in1];
     boundary[3] = temp->et2[temp->in2];
  }

  
  incr = (export_par_val? 2 : 0);
  dimpnt = incr+dim;
  if ((sarr = newarray(dimpnt*(n_u+1)*(n_v+1), double)) == SISL_NULL) goto err101;

  

  duv[0] = (boundary[2] - boundary[0])/n_u;
  duv[1] = (boundary[3] - boundary[1])/n_v;

  for (kj=0,UV[1]=boundary[1],sp=sarr;kj<n_v;kj++,UV[1] += duv[1])
  {
     for (ki=0,UV[0]=boundary[0];ki<n_u;ki++,UV[0] += duv[0],sp += dimpnt)
     {
	if (export_par_val)
	{
	   sp[0] = UV[0];
	   sp[1] = UV[1];
	}
	evalp(temp,kder=0,kside1=0,kside2=0,UV,&klfs,&klft,
	      sp+incr, stat);
	if (*stat < 0) goto error;
     }

     

     UV[0] = boundary[2];
     if (export_par_val)
     {
	sp[0] = UV[0];
	sp[1] = UV[1];
     }
     evalp(temp,kder=0,kside1=0,kside2=0,UV,&klfs,&klft,
	   sp+incr, stat);
     if (*stat < 0) goto error;
     sp += dimpnt;
  }

  

  for (ki=0,UV[0]=boundary[0];ki<n_u;ki++,UV[0] += duv[0],sp += dimpnt)
  {
     UV[1] = boundary[3];
     if (export_par_val)
     {
	sp[0] = UV[0];
	sp[1] = UV[1];
     }
     evalp(temp,kder=0,kside1=0,kside2=0,UV,&klfs,&klft,
	   sp+incr, stat);
     if (*stat < 0) goto error;
  }

  

  UV[0] = boundary[2];
  UV[1] = boundary[3];
  if (export_par_val)
  {
     sp[0] = UV[0];
     sp[1] = UV[1];
  }
  evalp(temp,kder=0,kside1=0,kside2=0,sp,&klfs,&klft,
	sp+incr, stat);
  if (*stat < 0) goto error;


  
  *garr  = sarr;
  sarr   = SISL_NULL;
  *stat = 0;

  goto out;



  
  
  


  
err101:
  *stat = -101;
  s6err("s2541", *stat, 0);
  goto out;

   
err102:
   *stat = -102;
   s6err("s2541", *stat, 0);
   goto out;

   
err150:
  *stat = -150;
  s6err("s2541", *stat, 0);
  goto out;

  
err172:
  *stat = -172;
  s6err("s2541", *stat, 0);
  goto out;

  
error:
  s6err("s2541", *stat, 0);
  goto out;

  
  
  

out:
  

  if (pick_subpart == 1 && temp) freeSurf(temp);
  if ( sarr )  		         freearray(sarr);
  return;

}

