/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6CURVRAD

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
      s6curvrad(double epnt1[],double epnt2[],double etang[],int idim,
		double *crad,int *jstat)
#else	 
void s6curvrad(epnt1,epnt2,etang,idim,crad,jstat)
     int idim,*jstat;
     double epnt1[],epnt2[],etang[],*crad;
#endif     
{
  int kstat = 0;        
  double tdist;         
  double tdot;          
  double tlmid;         
  double tcos;          
  double tang;          
  double tdum;          
  double trad;          
  double sdiff[3];      
  
  

  if (idim != 3) goto err104;
  
  

  tdist = s6dist(epnt1,epnt2,idim);
  s6diff(epnt2,epnt1,idim,sdiff);

  tdot = s6scpr(etang,sdiff,idim);
  tlmid = s6length(etang,idim,&kstat);
  
  tcos = (tlmid*tdist != DZERO) ? fabs(tdot/(tlmid*tdist)) : fabs(tdot);
  tcos = MIN((double)1.0,tcos);
  
  tang = 2*acos(tcos);
  tdum = sqrt(2-2*cos(tang));
  trad = (tdum > REL_COMP_RES) ? tdist/tdum : -1.0;
  
  
  
  *crad = trad;
  *jstat = 0;
  goto out;

  

  err104 :
    *jstat = -104;
  goto out;

  out :
    return;
}    

