/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6DPLANE

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
double
  s6dplane(double eq1[],double eq2[],double eq3[],double epoint[],
	   int idim,int *jstat)
#else
double s6dplane(eq1,eq2,eq3,epoint,idim,jstat)
   double eq1[];
   double eq2[];
   double eq3[];
   double epoint[];
   int    idim;
   int    *jstat;
#endif
{
   int kstat = 0;         
   double tdist;          
   double snorm[3];       
   double sdiff1[3];      
   double sdiff2[3];      
   double sdiff3[3];      
   
   
   
   if (idim != 3) goto err104;
   
   
   
   s6diff(eq2,eq1,idim,sdiff1);
   s6diff(eq3,eq1,idim,sdiff2);
   s6diff(epoint,eq1,idim,sdiff3);
   
   
   
   s6crss(sdiff1,sdiff2,snorm);
   (void)s6norm(snorm,idim,snorm,&kstat);
   
   
   
   if (kstat)
      tdist = fabs(s6scpr(sdiff3,snorm,idim));
   else 
      tdist = s6dist(eq1,epoint,idim);   

   
   
   *jstat = 0;
   goto out;
   
   
   
   err104 : *jstat = -104;
   goto out;
   
   out :
      return tdist;
 }

