/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6STRIDER

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s6strider(double eder[],int idim,int ider,double gder[],int *jstat)
#else
void s6strider(eder,idim,ider,gder,jstat)
     double eder[];
     int    idim;
     int    ider;
     double gder[];
     int    *jstat;
#endif
{
  int kpos=0;          
  double w0;           
  int ki;              
  int idu;             
  int idv;             
  int *binom=SISL_NULL;     
  int *binomu=SISL_NULL;    
  int *binomv=SISL_NULL;    
  double *sum1=SISL_NULL;   
  double *sum2=SISL_NULL;   
  double sumdum1[4];   
  double sumdum2[4];   
  int idimp1;          
  int iw;              
  int igder;           
  int i,iu,iv,j,k;     
  int iderp1;          
  int igrow;             
  int iwrow;             
  int iutemp,ivtemp;   
  int tot,temp1;       
  int bidum[10];       
  double temp;         
  
  if (ider<0) goto err178;
  if (idim<1) goto err102;
  
  *jstat = 0;

   
  
  w0 = eder[idim];
  if (DEQUAL(w0,DZERO)) w0 = (double)1.0;

  

  if(ider == 0)
  {
    for(ki=0; ki<idim; ki++)
      gder[ki] = eder[ki] / w0;

    goto out;
  }

  

  idimp1  = idim + 1;
  iderp1 = ider + 1;
  igrow   = iderp1 * idim;
  iwrow   = igrow + iderp1;  

  

  if (ider > 3)
  { 
    binom = newarray((iderp1*(iderp1+1)) >> 1, INT);
    if(binom == SISL_NULL) goto err179;
  }
  else
  { 
    binom = bidum;
  }

  for(j=0,k=0; j<=ider; j++,k+=j)
  {
      
  
      binom[k] = 1;
  
      for(i=k+1; i<k+j; i++)
      {
          binom[i] = binom[i-j-1] + binom[i-j];
      }

      binom[k+j] = 1;
  }

  

  if (idim > 4)
  { 
    sum1 = newarray(idim, DOUBLE);
    if(sum1 == SISL_NULL) goto err179;
    sum2 = newarray(idim, DOUBLE);
    if(sum2 == SISL_NULL) goto err179;
  }
  else
  { 
    sum1=sumdum1;
    sum2=sumdum2;
  }
  
  

  for(idv=0,binomv=binom; idv<=ider; idv++,binomv+=idv)
  {
    for(idu=0,binomu=binom; idu<=ider-idv; idu++,binomu+=idu)
    {
      if(idu == 0 && idv == 0)
      {
          
    
          for(ki=0; ki<idim; ki++)
            gder[ki] = eder[ki] / w0;
      }
      else
      {
          
    
          tot = idu + idv;
          temp1 = ((tot * (tot+1)) >> 1) + idv;
    
          j = temp1 * idim;
          k = j + temp1;

          

          

          for(ki=0; ki<idim; ki++)
            sum2[ki] = (double)0.0;        

          for(iv=0; iv<=idv; iv++)
          {
            for(ki=0; ki<idim; ki++)
               sum1[ki] = (double)0.0;	               
            ivtemp = idv-iv;

            for(iu=0; iu<=idu; iu++)
            {
                tot = iu + iv;
                temp1 = ((tot * (tot+1)) >> 1) + iv;

	        igder = temp1 * idim;
                iutemp = idu-iu;

                tot = iutemp + ivtemp;
	        temp1 = ((tot * (tot+1)) >> 1) + ivtemp;

                iw   = temp1 * idimp1 + idim;

     	      
  
                if(iu<idu || iv<idv)
  	        {
  	       	  
  
  	            if(iu>0 && iu<idu)
  	       	    {
  		      temp = (double)binomu[iu] * eder[iw];
                      for(ki=0; ki<idim; ki++,igder++)
  	       	         sum1[ki] += temp * gder[igder];
  		     }
  		     else
                       for(ki=0; ki<idim; ki++,igder++)
  		         sum1[ki] += eder[iw] * gder[igder];
                }
            }
  
  	    
  
  	    if(iv>0 && iv<idv)
              for(ki=0; ki<idim; ki++)
  	          sum2[ki] += (double)binomv[iv] * sum1[ki];
  	    else
              for(ki=0; ki<idim; ki++)
		 sum2[ki] += sum1[ki];		    
          }
          for(ki=0; ki<idim; ki++,j++,k++)
            gder[j] = (eder[k] - sum2[ki]) / w0;
      }
    }
  }  

  

  if (ider > 3 && binom != SISL_NULL)
     freearray(binom);
  
  if (idim > 4)
  { 
     if(sum1 != SISL_NULL) freearray(sum1);
     if(sum2 != SISL_NULL) freearray(sum2);
  }

  

  goto out;

  

  err102: 
    *jstat = -102;
    s6err("s6strider",*jstat,kpos);
    goto out;

  

  err178: 
    *jstat = -178;
    s6err("s6strider",*jstat,kpos);
    goto out;

  

  err179: 
    *jstat = -179;
    s6err("s6strider",*jstat,kpos);
    goto out;

  out:
    return;
}


