/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define SH1263

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static void sh1263_s9makec0(SISLCurve **,int,int *);
static void sh1263_s9checkcrtan(SISLCurve *,int *); 
#else
static void sh1263_s9makec0();
static void sh1263_s9checkcrtan(); 
#endif

#if defined(SISLNEEDPROTOTYPES)
void
      sh1263(SISLCurve *vcurve[],int iedge,SISLCurve *vboundc[],int *jstat)
#else
void sh1263(vcurve,iedge,vboundc,jstat)
   SISLCurve *vcurve[];
     int iedge;
   SISLCurve *vboundc[];
     int *jstat;
#endif     
{
  int kstat = 0;        
  int kstat1 = 0;       
  int ki;               
  int kcopy = 1;        
  int knmbx = 8*iedge;  

  double tonethird = (double)1.0/(double)3.0;  
  double *scoef = SISL_NULL;     
  double (*s1)[4],(*s2)[4]; 
  
  SISLCurve* (*qc)[3];          
  SISLCurve **qcurve = SISL_NULL;    
  SISLCurve *qpt;               
  
  

  if ((scoef = newarray(knmbx,DOUBLE)) == SISL_NULL) goto err101;
  
  

  if ((qcurve = newarray(3*iedge,SISLCurve*)) == SISL_NULL) goto err101;
  
  for (ki=0; ki<3*iedge; qpt++,ki++)
    {
      qpt=vcurve[ki];

      

      if ((qcurve[ki] = newCurve(qpt->in,qpt->ik,qpt->et,qpt->ecoef,qpt->ikind,
				 qpt->idim,kcopy)) == SISL_NULL) goto err101;
     } 

  
  
  s1349(3*iedge,qcurve,&kstat);
  if (kstat < 0) goto error;
		 
  

  for (ki=0, qc=(SISLCurve*(*)[3])qcurve; ki<iedge; ki++,qc++)
    {
       sh1260(tonethird,*qc,3,&kstat);
      if (kstat < 0) goto error;
    }
  
  

  sh1262(qcurve,iedge,knmbx,scoef,&kstat);
  if (kstat < 0) goto error;

  kstat1 = kstat;
  
  
  
  

  for (ki=0,s1=(double(*)[4])scoef,s2=s1+iedge; ki<iedge; ki++,s1++,s2++)
    {
      

      vboundc[2*ki] = qcurve[3*ki];
      
      

      sh1261(qcurve[3*ki+1],qcurve[3*ki+2],*s1,4,*s2,4,vboundc+2*ki+1,&kstat);
      if (kstat < 0) goto error;
      
      

      sh1263_s9checkcrtan(vboundc[2*ki+1],&kstat);
      if (kstat < 0) goto error;
    }
  
  if (kstat1 == 2)
  {
     
     
     sh1263_s9makec0(vboundc,iedge,&kstat);
     if (kstat < 0) goto error;
  }

  

  *jstat = kstat1;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :
    
    

  if (scoef != SISL_NULL) freearray(scoef);
  if (qcurve != SISL_NULL)
    {
      for (ki=0; ki<3*iedge; ki+=3)
	{
	  if (qcurve[ki+1] != SISL_NULL) freeCurve(qcurve[ki+1]);
	  if (qcurve[ki+2] != SISL_NULL) freeCurve(qcurve[ki+2]);
	}
      freearray(qcurve);
    }
  
  return;
}

  



   
#if defined(SISLNEEDPROTOTYPES)
   static
      void
	    sh1263_s9makec0(SISLCurve *vbound[],int iedge,int *jstat)
#else	       
static void sh1263_s9makec0(vbound,iedge,jstat)
   SISLCurve *vbound[];
   int iedge;
   int *jstat;
#endif     
{
   int kstat = 0;                
   int ki,kj,kk,kl;              
   int kn;                       
   int kn2;                      
   int k1;                       
   int kdim = vbound[0] -> idim; 
   int kleft = 0;                
   double tpar1,tpar2;           
   double tref;                  
   double tmult;                 
   double sdiff[3];              
   double sder[9];               
   SISLCurve *qcross;            
   SISLCurve *qc;                
   
   
   
   for (ki=0; ki<iedge; ki++)
   {
      qcross = vbound[2*ki+1];
      
      
      
      
      kj = (ki > 0) ? ki-1 : iedge-1;
      qc = vbound[2*kj];
      
      
      
      tpar1 = *(qc->et + qc->in);
      tpar2 = *(qcross->et + qcross->ik - 1);
      
      
      
      s1221(qc,1,tpar1,&kleft,sder,&kstat);
      if (kstat < 0) goto error;
      
      s1221(qcross,0,tpar2,&kleft,sder+2*kdim,&kstat);
      if (kstat < 0) goto error;
      
      tref = s6length(sder+kdim,kdim,&kstat);
      if (kstat < 0) goto error;
      
      
      
      for (kk=0; kk<kdim; kk++)
	 if (DNEQUAL(tref+sder[kdim+kk],tref-sder[2*kdim+kk])) break;
      
      if (kk < kdim)
      {
	 
	 
	 
	 
	 for (kl=0; kl<kdim; kl++) sder[kdim+kl] *= -(double)1.0;
	 
	 
	 
	 s6diff(sder+kdim,qcross->ecoef,kdim,sdiff);
	 
	 
	    
	 for (kn=qcross->in, kn2=kn/2, k1=0, tmult=(double)1.0;
	  k1<kn2; k1++, tmult -= (double)1.0/(double)kn2)
	    for (kl=0; kl<kdim; kl++)
	       qcross->ecoef[k1*kdim+kl] += tmult*sdiff[kl];
      }
	    
      
      
      
      kj = (ki < iedge-1) ? ki+1 : 0;
      qc = vbound[2*kj];
      
      
      
      tpar1 = *(qc->et + qc->ik - 1);
      tpar2 = *(qcross->et + qcross->in);
      
      
      
      s1221(qc,1,tpar1,&kleft,sder,&kstat);
      if (kstat < 0) goto error;
      
      s1221(qcross,0,tpar2,&kleft,sder+2*kdim,&kstat);
      if (kstat < 0) goto error;
      
      tref = s6length(sder+kdim,kdim,&kstat);
      if (kstat < 0) goto error;
      
      
      
      for (kk=0; kk<kdim; kk++)
	 if (DNEQUAL(sder[kdim+kk]+tref,sder[2*kdim+kk]+tref)) break;
      
      if (kk < kdim)
      {
	 
	 
	 
	 
	 s6diff(sder+kdim,qcross->ecoef+(qcross->in-1)*kdim,kdim,sdiff);
	 
	 
	    
	 for (kn=qcross->in, kn2=kn/2, k1=kn-1, tmult=(double)1.0;
	  k1>kn2; k1--, tmult -= (double)1.0/(double)kn2)
	    for (kl=0; kl<kdim; kl++)
	       qcross->ecoef[k1*kdim+kl] += tmult*sdiff[kl];
      }
      
   }

  *jstat = 0;
  goto out;
  
  
  
  error : *jstat = kstat;
  goto out;
  
  out:
     return;
}     

   
#if defined(SISLNEEDPROTOTYPES)
   static
      void
	    sh1263_s9checkcrtan(SISLCurve *pcrtan,int *jstat)
#else	       
static void sh1263_s9checkcrtan(pcrtan,jstat)
     int *jstat;
     SISLCurve *pcrtan;
#endif     
{

  *jstat = 0;
  
  return;
}     

   
   






