/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1463

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
typedef void (*fshapeProc)(double [],double [],int,int,int *);
#else
typedef void (*fshapeProc)();
#endif

#if defined(SISLNEEDPROTOTYPES)
void
  sh1463(fshapeProc fshape,
	 SISLCurve *vboundc[],int icurv,double etwist[],
	 double etang[],double eder[],int *jstat)
#else	 
void sh1463(fshape,vboundc,icurv,etwist,etang,eder,jstat)
     fshapeProc    fshape;
     double        etwist[],etang[],eder[];
     SISLCurve     *vboundc[];
     int           icurv,*jstat;
#endif     
{
  int kstat = 0;     
  int kder = 2;      
  int ki;            
  int kdim = 3;      
  int klfs = 0;      
  int klft = 0;      
  double spar[2];    
  double sder[18];   
  double snorm[3];   
  SISLSurf *qsurf = SISL_NULL;  
  SISLCurve *qc[8];        
  SISLCurve *qpt;          

  

  for (ki=0; ki<8; ki++)
    {
      qpt = vboundc[ki];
      
      

      if (qpt->idim != kdim) goto err104;
      
      qc[ki] = newCurve(qpt->in,qpt->ik,qpt->et,qpt->ecoef,qpt->ikind,kdim,1);
      if (qc[ki] == SISL_NULL) goto err101;
    }
    
  

  s1706(qc[4]);
  s1706(qc[5]);  
  s1706(qc[6]);  
  s1706(qc[7]);  

  

  s1401(qc,etwist,&qsurf,&kstat);
  if (kstat < 0) goto error;
   
  

  spar[0] = (double)0.5*(*(qsurf->et1+qsurf->ik1-1) + *(qsurf->et1+qsurf->in1));
  spar[1] = (double)0.5*(*(qsurf->et2+qsurf->ik2-1) + *(qsurf->et2+qsurf->in2));
  
  s1421(qsurf,kder,spar,&klfs,&klft,sder,snorm,&kstat);
  if (kstat < 0) goto error;
  
  

  for (ki=0; ki<kdim; ki++)
    {
      etang[ki] = (double)0.5*sder[2*kdim+ki];
      etang[kdim+ki] = (double)0.5*sder[kdim+ki];
      etang[2*kdim+ki] = -(double)0.5*sder[2*kdim+ki];
      etang[3*kdim+ki] = -(double)0.5*sder[kdim+ki];
    }
  
  

  fshape(sder,etang,kdim,icurv,&kstat);
  if (kstat < 0) goto error;
  
  

  memcopy(eder,sder,kdim,DOUBLE);
  memcopy(eder+kdim,etang+3*kdim,kdim,DOUBLE);
  memcopy(eder+2*kdim,etang,kdim,DOUBLE);
  
  

  for (ki=0; ki<kdim; ki++)
    {
      eder[3*kdim+ki] = (double)0.1*sder[5*kdim+ki];
      eder[4*kdim+ki] = (double)0.1*sder[4*kdim+ki];
      eder[5*kdim+ki] = (double)0.1*sder[3*kdim+ki];
    }
  
  *jstat = 0;
  goto out;
  
  

  err101 :
  *jstat = -101;
  goto out;
 
  

  err104 :
    *jstat = -104;
  goto out;
   
  

 error:
  *jstat = kstat;
  goto out;
  
  out :

    

    for (ki=0; ki<8; ki++)
      if (qc[ki] != SISL_NULL) freeCurve(qc[ki]);
  if (qsurf != SISL_NULL) freeSurf(qsurf);
  
    return;
}

