/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define SH1924

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
      sh1924(double *ea,double *eb,int in,int ik,int idim,
	     int *nstart,int *jstat)
#else
void sh1924(ea,eb,in,ik,idim,nstart,jstat)
   double *ea;
   double *eb;
   int in;
   int ik;
   int idim;
   int *nstart;
   int *jstat;
#endif     
{
   int ki,kj,kr;         
   int ki1,kih,kim;      
   int kjs,kjh; 
   int kik1 = ik-1;      
   double thelp;         
   double *ssum=SISL_NULL;    

   
   
   if ((ssum = new0array(idim,DOUBLE)) == SISL_NULL) goto err101;
   
   
   
   for (ki=0; ki<in; ki++)
     {
	ki1 = ki - 1;
	for (kr=0; kr<idim; kr++) ssum[kr] = (double)0.0;
	
	
	
	for (kjs=nstart[ki], kjh=ki+kjs-ik+1, kj=kjs;
	 kj<kik1; kjh++, kj++)
	  {
	     thelp = ea[ki*ik+kj];
	     for (kr=0; kr<idim; kr++)
	       ssum[kr] += thelp*eb[kjh*idim+kr];
	  }
	
	
	
	if (DEQUAL(ea[ki*ik+kik1],DZERO)) goto err106;
	
	thelp = (double)1.0/ea[ki*ik+kik1];
	for (kr=0; kr<idim; kr++)
	  eb[ki*idim+kr] = (eb[ki*idim+kr] - ssum[kr])*thelp;
     }
   
   
   
   for (ki=in-1, ki1=in-1, kih=0;
    kih<in; ki--, kih++)
     {
	
	
	for (;;ki1--)
	  if (nstart[ki1] < ik-ki1+ki) break;
	
	for (kr=0; kr<idim; kr++) ssum[kr] = (double)0.0;
	
	
	
	for (kjs=ki+1, kim=ik-kjs+ki-1, kj=kjs;
	 kj<=ki1; kim--, kj++)
	  {
	     thelp = ea[kj*ik+kim];
	     for (kr=0; kr<idim; kr++)
	       ssum[kr] += thelp*eb[kj*idim+kr];
	  }
	
	thelp = (double)1.0/ea[ki*ik+ik-1];
	for (kr=0; kr<idim; kr++)
	  eb[ki*idim+kr] = (eb[ki*idim+kr] - ssum[kr])*thelp;
     }
   
   
   
   
   *jstat = 0;
   goto out;
   
   
   
   err101: *jstat = -101;
   goto out;
   
   
   
   err106: *jstat = -106;
   goto out;
   
   out:
      
      
      if (ssum != SISL_NULL) freearray(ssum);
	  
      return;
}
   

