/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SHEVALC

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
shevalc(SISLCurve *pc1,int ider,double ax,double aepsge,int *ileft,
	     double eder[],int *jstat)
#else
void shevalc(pc1,ider,ax,aepsge,ileft,eder,jstat)
     SISLCurve *pc1;
     int ider;
     double ax;
     double aepsge;
     int *ileft;
     double eder[];
     int *jstat;
#endif
{
  int kstat=0;        
  int kdim = pc1->idim;  
  double *scoef=SISL_NULL;    
  double *s1,*s2,*s3,*s4; 
  SISLCurve *qc = SISL_NULL;   

  

  if (kdim == 1)
  {

    

    if ( pc1->ikind == 2 || pc1->ikind == 4 )
      goto err151;

     

     if ((scoef = newarray(pc1->in,DOUBLE)) == SISL_NULL) goto err101;

     for (s1=pc1->ecoef, s2=scoef, s3=s1+pc1->in; s1<s3; s1=s4)
     {
	*s2 = *s1;
	for (s2++, s4=s1+1; s4<s3; s4++, s2++)
	{
	   if (fabs((*s4)-(*s1)) < aepsge) *s2 = *s1;
	   else break;
	}
     }

     

     if ((qc = newCurve(pc1->in,pc1->ik,pc1->et,scoef,pc1->ikind,
			kdim,0)) == SISL_NULL) goto err101;
  }
  else
    qc = pc1;

  



  

  s1221(qc,ider,ax,ileft,eder,&kstat);
  if (kstat < 0) goto error;

  
  *jstat = 0;
  goto out;


  
 err151:
  *jstat = -151;
  goto out;

  
 err101:
  *jstat = -101;
  goto out;

  

 error:
  *jstat = kstat;
  goto out;

out:
   

   if (scoef != SISL_NULL) freearray(scoef);
   if (qc != SISL_NULL && qc != pc1 ) freeCurve(qc);

   return;
}

