/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SPLI_SILH

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
    spli_silh (SISLIntdat ** pintdat,
	    SISLObject * po1,
	    int *jstat)

#else
void
   spli_silh (pintdat,
	    po1,
	    jstat)

     SISLIntdat **pintdat;
     SISLObject *po1;
     int *jstat;

#endif
{
  int kstat;			
  int kpos = 0;			
  int ktype = 0;		
  int klist = 0;		
  int indstart = 0;		
  int indlast = 0;		
  int index1 = 0;		
  int indum = 0;		
  int ki=0, kp=0;		
  int mu_1=0, mu_2=0;		
  int left1=0,left2=0;		
  int ac_no=0;		        
  int knum=0;		        
  int test=0;		        
  SISLIntpt *pfirst=SISL_NULL;	
  SISLIntpt *plast=SISL_NULL;	
  SISLIntpt *plast_prev=SISL_NULL;	
  SISLIntpt *pshadow=SISL_NULL;	
  SISLIntpt *pnext=SISL_NULL;	
  SISLIntpt *prev=SISL_NULL;	 	
  SISLIntpt *pother=SISL_NULL;	
  SISLSurf  *psurf=SISL_NULL;	
  double arr[2];            	
   
       
  *jstat = 0;
  
       
  if ((*pintdat) == SISL_NULL)
    goto out;
  
  if (po1->iobj != SISLSURFACE) goto errinp;
  psurf = po1->s1;
   

  
  for (ki = 0; ki < (*pintdat)->ilist; ki++)
  {
     knum     = (*pintdat)->vlist[ki]->inumb;
     pfirst   = (*pintdat)->vlist[ki]->pfirst;
     plast    = (*pintdat)->vlist[ki]->plast;
     indstart = (*pintdat)->vlist[ki]->ind_first;
     indlast  = (*pintdat)->vlist[ki]->ind_last;
     ktype    = (*pintdat)->vlist[ki]->itype;
     prev     = pfirst;
     pnext    = pfirst->pnext[indstart];
     ac_no    = 1;

     
     
     if (pfirst->iinter == SI_TRIM) continue;
     
     if (pfirst == plast &&
	 knum > 3)
     {
	mu_1 = s6knotmult(psurf->et1,psurf->ik1,
			   psurf->in1,&left1,pfirst->epar[0], &kstat);
	mu_2 = s6knotmult(psurf->et2,psurf->ik2,
			  psurf->in2,&left2,pfirst->epar[1], &kstat);
	
	
	
	
	
	kpos = 10;
	plast_prev = plast->pnext[indlast];
	pshadow    = hp_copyIntpt(plast);
	if (!pshadow) goto err101;
	sh6idnpt(pintdat, &pshadow, test=FALSE, &kstat);
	if (kstat < 0) goto error;
	
	
	sh6insertpt(plast_prev, plast, pshadow, &kstat);
	if (kstat < 0) goto error;
	
	sh6disconnect(plast, pshadow, &kstat);
	if (kstat < 0) goto error;
	sh6getlist(pshadow, plast_prev, &index1, &indum, &kstat);
	kpos = 11;
	if (kstat < 0 || index1 < 0) goto errinconsis;
	
	plast    = (*pintdat)->vlist[ki]->plast = pshadow;
	indlast  = (*pintdat)->vlist[ki]->ind_last = index1;

	
	sh6getlist(pfirst, pnext, &index1, &indum, &kstat);
	kpos = 12;
	if (kstat < 0 || index1 < 0) goto errinconsis;
	indstart = (*pintdat)->vlist[ki]->ind_first = index1;
	
	
     }
     
     if (knum > 2)
     {
	while (pnext != plast)
	{
	   ac_no++;
	   kpos = 1;
	   if (ac_no >= knum) goto errinconsis;
	   sh6getother(pnext,prev,&pother,&kstat);
	   if (kstat < 0) goto error;
	   
	   mu_1 = s6knotmult(psurf->et1,psurf->ik1,
			      psurf->in1,&left1,pnext->epar[0], &kstat);
	   mu_2 = s6knotmult(psurf->et2,psurf->ik2,
			      psurf->in2,&left2,pnext->epar[1], &kstat);
	   
	   if(mu_1 > psurf->ik1 - 3 ||
	      mu_2 > psurf->ik2 - 3)
	   {

	      
	      pshadow = hp_copyIntpt(pnext);
	      if (!pshadow) goto err101;
	      sh6idnpt(pintdat, &pshadow, test=FALSE, &kstat);
	      if (kstat < 0) goto error;
	      
	      sh6insertpt(prev, pnext, pshadow, &kstat);
	      if (kstat < 0) goto error;
	      
	      sh6disconnect(pnext, pshadow, &kstat);
	      if (kstat < 0) goto error;
	      
	      
	      sh6getlist(pshadow, prev, &index1, &indum, &kstat);
	      kpos = 3;
	      if (kstat < 0 || index1 < 0) goto errinconsis;
	      
	      (*pintdat)->vlist[ki]->inumb    = ac_no;
	      (*pintdat)->vlist[ki]->ind_last = index1;
	      (*pintdat)->vlist[ki]->plast    = pshadow;
	      
	      
	      sh6getlist(pnext, pother, &index1, &indum, &kstat);
	      kpos = 4;
	      if (kstat < 0 || index1 < 0) goto errinconsis;
	      
	      klist = (*pintdat)->ilist;
	      kpos = 5;
	      if (klist > (*pintdat)->ilmax) goto errinconsis;
	      if (klist == (*pintdat)->ilmax)
	      {
		 
		 (*pintdat)->ilmax += 20;
		 
		 if (((*pintdat)->vlist =
		      increasearray ((*pintdat)->vlist, (*pintdat)->ilmax,
				     SISLIntlist *)) == SISL_NULL)
		    goto err101;
	      }
	      
	      
	      if(((*pintdat)->vlist[klist] = newIntlist (pnext, plast, ktype))
		 == SISL_NULL) goto err101;
	      (*pintdat)->vlist[klist]->inumb = knum + 1 - ac_no;
	      (*pintdat)->vlist[klist]->ind_first = index1;
	      (*pintdat)->vlist[klist]->ind_last  = indlast;
	      for (kp = 0; kp < 4; kp++)
		 (*pintdat)->vlist[klist]->pretop[kp] =
		    (*pintdat)->vlist[ki]->pretop[kp];
	      
	      (*pintdat)->ilist++;
	      break;
	   }
	   else
	   {
	      prev = pnext;
	      pnext = pother;
	   }
	   
	} 
	
     }
  }
  
  
  for (ki = 0; ki < (*pintdat)->ilist; ki++)
  {
     pfirst   = (*pintdat)->vlist[ki]->pfirst;
     plast    = (*pintdat)->vlist[ki]->plast;
     indstart = (*pintdat)->vlist[ki]->ind_first;
     indlast  = (*pintdat)->vlist[ki]->ind_last;

     
     pnext    = pfirst->pnext[indstart];
     s6diff(pnext->epar,pfirst->epar,2,arr);
     pfirst->iside_1 = (arr[0]>=0 ? 1 : -1);
     pfirst->iside_2 = (arr[1]>=0 ? 1 : -1);
     
     
     pnext    = plast->pnext[indlast];
     s6diff(pnext->epar,plast->epar,2,arr);
     plast->iside_1 = (arr[0]>=0 ? 1 : -1);
     plast->iside_2 = (arr[1]>=0 ? 1 : -1);
     
     
     
     if (pfirst->geo_data_1)
	freearray (pfirst->geo_data_1);
     if (pfirst->geo_data_2)
	freearray (pfirst->geo_data_2);
     pfirst->geo_data_1 = SISL_NULL;
     pfirst->size_1 = 0;
     pfirst->geo_data_2 = SISL_NULL;
     pfirst->size_2 = 0;
     pfirst->evaluated = FALSE;
     
     if (plast->geo_data_1)
	freearray (plast->geo_data_1);
     if (plast->geo_data_2)
	freearray (plast->geo_data_2);
     plast->geo_data_1 = SISL_NULL;
     plast->size_1 = 0;
     plast->geo_data_2 = SISL_NULL;
     plast->size_2 = 0;
     plast->evaluated = FALSE;
  }  
    
     
  
  *jstat = 0;
  goto out;
  
  
  
  err101:
  *jstat = -101;
  s6err ("spli_silh", *jstat, kpos);
  goto out;

  
  errinp:
  *jstat = -200;
  s6err ("spli_silh", *jstat, kpos);
  goto out;

  
  errinconsis:
  *jstat = -200;
  s6err ("spli_silh", *jstat, kpos);
  goto out;

  error:
  *jstat = kstat;
  s6err ("spli_silh", *jstat, kpos);
  goto out;

out:
  ;
}

