#include <synaps/mpol/xml.h>
#include <synaps/topology/mesh.h>
#include <synaps/topology/mesh/axel.h>

void usage (char* cmd)
{
  std::cout<<"usage:\n"<<cmd<<" file [-bZa] ..."<<std::endl;
  std::cout<<"\n  The file should contain the equation of an implicit curve, with (big) integer,"<<std::endl;
  std::cout<<"  in the format <polynomial>...</polynomial> (example: <polynomial>x^2+2*y^2-1</polynomial>)"<<std::endl;
  std::cout<<"options:\n";
  std::cout<<"   -b x0 x1 y0 y1   box where the topology is computed (default -10 10 -10 10)"<<std::endl;
  std::cout<<"   -max n1          maximal number of subdivisions for a singular box"<<std::endl;
  std::cout<<"   -min n2          minimal number of subdivisions for a smooth box"<<std::endl;
}

int main(int argc, char **argv)
{
  typedef MPol<double>      mpol_t;
  typedef std::string   str_t;

  using std::cout;
  using std::endl;
  if(argc<2)
    {
      usage(argv[0]);
      return 0;
    }
  std::ifstream in(argv[1],std::ios::in);
  if(!in.good())
    {
      if(str_t(argv[1])!=str_t("-h"))
	{
	  std::cerr<<argv[0]<<": file "<<argv[1]<<" not found"<<std::endl;
	  return 1;
	}
      usage(argv[0]);
      return 0;
    }
  Seq<mpol_t> lp;
  mpol_t p;
  while(xml::read(in,p,Variables("x y z")))
    {
      std::cout<<"<polynomial degree=\""<<degree(p)<<"\" mdegree=\""
	       <<degree(p,0)<<" "<<degree(p,1)<<" "<<degree(p,2)<<"\""
	       <<"  size=\""<<p.size()<<"\">\n";
      print(std::cout,p,Variables("x y z"));
      std::cout<<"\n</polynomial>"<<std::endl;
      lp.push_back(p);
    }

  double x0=-3.1,x1=3,y0=-3.2,y1=3.1,z0=-3.05,z1=3.15;
  int Nmax=7, Nmin=5;
  int N=1;
  unsigned Z=1;

  for(unsigned c=2;c<argc;c++)
    {
      if(str_t(argv[c])==str_t("-max"))
	{
	  c++; Nmax=atoi(argv[c]);
	}
      else if(str_t(argv[c])==str_t("-min"))
	{
	  c++; Nmin=atoi(argv[c]);
	}
      else if(str_t(argv[c])==str_t("-b"))
	{
	  c++; if(c<argc) x0=atof(argv[c]);
	  c++; if(c<argc) x1=atof(argv[c]);
	  y0=x0; y1=x1;
	  c++; if(c<argc) y0=atof(argv[c]);
	  c++; if(c<argc) y1=atof(argv[c]);
	  z0=y0; z1=y1;
	  c++; if(c<argc) z0=atof(argv[c]);
	  c++; if(c<argc) z1=atof(argv[c]);

	}
      else if(str_t(argv[c])==str_t("-h"))
	{
	  usage(argv[0]);
	  return 0;
	}
    }

  std::cout<<" Box: "
	   <<x0<<" "<<x1<<" "
	   <<y0<<" "<<y1<<" "
	   <<z0<<" "<<z1<<std::endl;
  std::cout<<" Max: "<<Nmax<<std::endl;
  std::cout<<" Min: "<<Nmin<<std::endl;

  double max = pow(0.5, Nmax), min = pow(0.5, Nmin);

  axel::ostream os("tmp.axl");
  os<<Color(1,0,0);
  for(unsigned i=0;i<lp.size();i++)
    {
      scene<MPol<double>, linalg::rep3d<double> > scn;
      poly=lp[i];
      mesh(scn, lp[i], x0, x1, y0, y1, z0, z1, min, max);
      int out=0;
      axel::print(os,scn,os.color.r,os.color.g,os.color.b,1,out);
    }
  os.view();
}

