<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Subdivision methods for implicit curves in 3D>

  In this section, we describe a subdivision method, for computing the
  topology of a curve <with|mode|math|\<cal-C\>>, defined by two implicit
  equations <with|mode|math|f(x,y,z)=0>, <with|mode|math|g(x,y,z)=0>, where
  <with|mode|math|f,g\<in\>\<bbb-R\>[x,y]>. It uses the representation of
  polynomials in the Bernstein basis of a domain
  <with|mode|math|D=[a,b]\<times\>[c,d]\<times\>[e,f]\<subset\>\<bbb-R\><rsup|3>>:

  <\equation*>
    f(x,y,z)=<big|sum><rsub|0\<leqslant\>i\<leqslant\>d<rsub|1>,0\<leqslant\>j\<leqslant\>d<rsub|2>,0\<leqslant\>k\<leqslant\>d<rsub|3>>
    c<rsub|i,j> \ B<rsup|i><rsub|d<rsub|1>>(x;a,b)B<rsup|j><rsub|d<rsub|2>>(y;c,d)B<rsup|k><rsub|d<rsub|3>>(z;e,f)
  </equation*>

  where <with|mode|math|(B<rsup|i><rsub|d>(x;u,v))<rsub|i=0,\<ldots\>,d>> is
  the Bernstein basis in degree <with|mode|math|d>, on the interval
  <with|mode|math|[u,v]>. The subdivision algorithm is also based on a
  criterion for detecting the regularity of the curve
  <with|mode|math|\<cal-C\>>, in the domain <with|mode|math|D>.

  <subsection|Implementation>

  <\code>
    void topology::assign(point_graph\<less\>C\<gtr\> & g,
    MPol\<less\>T\<gtr\>& p1, MPol\<less\>T\<gtr\>& p2,

    \ \ \ \ \ \ \ \ \ TopSbdCurve3d\<less\>C\<gtr\> mth,\ 

    \ \ \ \ \ \ \ \ \ C x0, C x1, C y0, C y1, C z0, C,z1);
  </code>

  <\itemize>
    <item>It computes a graph of points <verbatim|g> of type
    <verbatim|topology::point_graph\<less\>C\<gtr\>>, which is isotopic to
    the curve defined by the polynomial <verbatim|p1>, <verbatim|p2>, in the
    box <with|mode|math|D>. The coordinates of the points in <verbatim|g> are
    of type <verbatim|C>. The implementation is designed for
    <verbatim|C=double>.

    <item>The input coefficients of <verbatim|p> are of type <verbatim|T>.
    During the computation, they are rounded to <verbatim|double> number
    types. If the topology in the box can be certified from this
    approximation, a graph of points connecting the intersection of the curve
    with the boundary of <with|mode|math|B> is computed. Otherwise the box is
    subdivided into <with|mode|math|4> subboxes and the test is applied
    recursively.

    <item>If the curve has singular points, the subdivision will continue
    until the size of the box is smaller than the precision
    <with|mode|math|\<varepsilon\>> specified by the class
    <verbatim|TopSbdCurbe3d\<less\>C\<gtr\>>. The default value is
    <with|mode|math|10<rsup|-3>>. \ \ 
  </itemize>

  See <verbatim|synaps/topology/TopSbdCurve3d.h>

  <subsection|Example>

  <action|<with|mode|math|\<boxdot\>>|TopSbdCrv3d.cc>

  Here is a picture of the corresponding result:

  <\with|par-mode|center>
    <postscript|sbdv_icurve3d-fig1.ps|*2/8|||||>
  </with>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|1>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Subdivision
      methods for implicit curves in 3D> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>
    </associate>
  </collection>
</auxiliary>