<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Sweeping an implicit 2d curve>

  Let \ <with|mode|math|\<cal-C\>> be a planar implicit curve defined by a
  polynomial equation <with|mode|math|f(x,y)=0>, where <with|mode|math|f
  \<in\> \<bbb-Q\>[x,y]>. We are going to describe an algorithm, which
  outputs a graph of points in the plane, \ isotopic to the curve
  <with|mode|math|\<cal-C\>>, and with the following properties:

  <\itemize>
    <item>the points (or vertices of the graph) are on the curve
    <with|mode|math|\<cal-C\>>,

    <item>they are connected by segments, which intersect only at these
    end-points.
  </itemize>

  The method which is described here belongs to the family of sweeping
  algorithms, also related to Morse functions and critical values. It
  proceeds as follows:

  <\itemize>
    <item>First, we choose a direction, and consider a virtual line
    orthogonal to this direction, sweeping the plane;

    <item>We detect at which critical position, the topology of the
    intersection of the line and the curve changes and compute the
    corresponding intersection points;

    <item>We compute the intersection points for sample lines in between
    these critical positions.

    <item>We connect the computed points by segments, in order to get a graph
    isotopic to <with|mode|math|\<cal-C\>>.
  </itemize>

  Let us assume that the sweeping direction that we have choosen is the
  <with|mode|math|x>-axis. Then the topology of
  \ <with|mode|math|L<rsub|x<rsub|0>>\<cap\> \<cal-C\>> (where
  <with|mode|math|L<rsub|x>> is the line orthogonal to the
  <with|mode|math|x>-axis through the point <with|mode|math|(x,0)>) changes,
  when \ <with|mode|math|L<rsub|x<rsub|0>>\<cap\> \<cal-C\>> contains
  multiple points, that is when

  <\equation*>
    f(x,y)=0,\<partial\><rsub|y>f(x,y)=0
  </equation*>

  has a real solution.\ 

  The corresponding <with|mode|math|x>-coordinates of these points satisfies
  <with|mode|math|res<rsub|y>(f,\<partial\><rsub|y>f)(x)=r(x)=0>. We solve
  this equation. Let <with|mode|math|\<alpha\><rsub|1>\<less\>\<cdots\>\<less\>\<alpha\><rsub|s>>
  be the real roots of <with|mode|math|r(x)=0>. For each
  <with|mode|math|\<alpha\><rsub|i>>, we \ compute the corresponding
  <with|mode|math|y> such that <with|mode|math|f(\<alpha\><rsub|i>,y)=0>. At
  this point, we also check that the curve is in <em|generic> position, that
  is, there is at most one multiple solution of
  <with|mode|math|f(\<alpha\><rsub|i>,y)=0>.

  Once these points are computed, we compute intermediate (rational) points
  <with|mode|math|\<mu\><rsub|0>\<less\> \<mu\><rsub|1>\<less\>\<cdots\>\<less\>\<mu\><rsub|s>>,
  such that <with|mode|math|\<mu\><rsub|i-1>\<less\>\<alpha\>\<less\>\<mu\><rsub|i>>
  and the corresponding <with|mode|math|y> such that
  <with|mode|math|f(\<alpha\><rsub|i>,y)=0>.

  All these points are collected and sorted by lexicographic order such that
  <with|mode|math|x\<gtr\>y>. The subset of points, corresponding to two
  consecutive <with|mode|math|x>-coordinates are connected, according to
  their <with|mode|math|y>-coordinates, in the order described below:

  <\with|par-mode|center>
    <postscript|sweep_implicit2d-fig1.eps|*3/8|||||>
  </with>

  <subsection|Implementation>

  <\code>
    void topology::assign(topology::point_graph\<less\>C\<gtr\>& g,\ 

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ const MPol\<less\>U\<gtr\>&
    p,TopSweep2d\<less\>C,SLV\<gtr\> mth);
  </code>

  <\itemize>
    <item>It computes a graph of points <verbatim|g> (of type
    <verbatim|topology::point_graph\<less\>C\<gtr\>>) which is isotopic to
    the curve <with|mode|math|\<cal-C\>>, defined by the polynomial
    <verbatim|p>. The coefficients of the points in the graph are of type
    <verbatim|C>. The polynomial <verbatim|P> is converted to a polynomial
    with rational coefficients, if needed.

    <item>The class <verbatim|TopSweep2d> depends on two parameters:

    <\itemize>
      <item><verbatim|C>, which is the type of coefficients of the points in
      the result,

      <item><verbatim|SLV>, which is the type of univariate solver to be
      used. The default value for <verbatim|SLV> is
      <verbatim|Aberth\<less\>RR\<gtr\>>.
    </itemize>

    This class provides \ following definition and static function:

    <\itemize>
      <item><verbatim|typedef u_solution_t>; the type of the solutions which
      are computed by the solver,\ 

      <item><verbatim|static Seq\<less\>u_solution_t\<gtr\> u_solve( const
      UPOL& p);> which specifies the univariate solver, \ used in the
      algorithm.
    </itemize>
  </itemize>

  <subsection|Example>

  <action|<with|mode|math|\<boxdot\>>|TopSweep2d.cc>

  Here is the graph,which is computed:

  <\with|par-mode|center>
    <postscript|sweep_implicit2d-fig2.ps|*2/8|||||>
  </with>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|1>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Sweeping
      an implicit 2d curve> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>
    </associate>
  </collection>
</auxiliary>