/*******************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): 
 *   $Id: cell2d.h,v 1.1 2005/07/11 14:02:22 mourrain Exp $
 ********************************************************************/
#ifndef synaps_mesh_cell2d_H
#define synaps_mesh_cell2d_H
/********************************************************************/
#include <synaps/init.h>
#include <synaps/linalg/MatrDse.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
/** Structure which stores a 2d box and the Bernstein coefficients of the
    polynomial on the corresponding box.
**/
template<class C, class R = MatrDse<C> >
struct cell2d
{
  typedef typename R::row_type row_type;
  typedef typename R::col_type col_type;
  typedef C coeff_t;
  typedef R matr_t;
  C x0,x1,y0,y1;
  int reg;
  matr_t bz_pol;

  cell2d():x0(0),x1(1),y0(0),y1(1),reg(0),bz_pol(){}
  cell2d(coeff_t a, coeff_t b, coeff_t c, coeff_t d, matr_t M): 
    x0(a),x1(b),y0(c),y1(d),reg(0),bz_pol(M){}
  cell2d(const cell2d<C,R>& c): 
    x0(c.x0),x1(c.x1),y0(c.y0),y1(c.y1),
    reg(c.reg),
    bz_pol(c.bz_pol) {}
  cell2d& operator= (const cell2d<C,R> & c)
  {
    x0=c.x0;
    x1=c.x1;
    y0=c.y0;
    y1=c.y1;
    reg=c.reg;
    bz_pol=c.bz_pol;
    return *this;
  }
  int nbrow() const {return bz_pol.nbrow();}
  int nbcol() const {return bz_pol.nbcol();}

  typename R::coeff_t operator()(int i,int j) const {return bz_pol(i,j);}
};
//--------------------------------------------------------------------
template<class C, class R>
C size(const cell2d<C,R>& c)
{
  return std::max(fabs(c.x1-c.x0),fabs(c.y1-c.y0));
}

__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //synaps_mesh_cell2d_H
