/*********************************************************************
*      This file is part of the source code of SYNAPS library          *
*   Author(s): L. Deschamps                                          *
*              G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
/** \file                                                           **/
#ifndef synaps_topology_point3d_H
#define synaps_topology_point3d_H
/*********************************************************************/
#include <synaps/init.h>

namespace topology 
{

  /*!
    Point class that defines the structure of a point. 
  */
  template < class Typ >
  class point3d
  {
    
  public:
    
    /*! The type of the coefficients. */
    typedef Typ coeff_t;
    
    Typ x;
    Typ y;
    Typ z;

    /*! Default constructor. */
    point3d():x(0),y(0),z(0) {};	

    point3d(int i):x(0),y(0),z(0) {};

    /*! Regular constructor. */
    point3d( const Typ num1, const Typ num2, const Typ num3)
      :x(num1),y(num2),z(num3) {};

    /*! Copy constructor. */
    point3d( const point3d & point3D )
      {
	x = point3D.x;
	y = point3D.y;
	z = point3D.z; 
      }

    /*! Affectation operator. */
    point3d& point3d::operator= (const point3d& point3D)
      {  
	x = point3D.x;
	y = point3D.y;
	z = point3D.z;
	
	return ( *this );
      }

    /*! The \c operator[] */
    Typ   operator[](unsigned i) const 
      {
	if(i==0) return x; 
	else if (i==1) return y; 
	else if(i==2) return z;
	else return 0;
      }

    /*! The \c operator[] */
    Typ&  operator[](unsigned i) 
      {
	assert(i<3);
	if(i==0) return x; 
	else if (i==1) return y; 
	else if(i==2) return z;
      }
    
  };
  
/*! Equality operator. */
template < class C >
bool operator == ( const topology::point3d< C >& p1,
		   const topology::point3d< C >& p2 ) 
{
  return ( p1.x == p2.x ) && ( p1.y == p2.y ) && ( p1.z == p2.z );  
}

/*! Output stream. */
template < class C >
std::ostream & operator<<( std::ostream & os, const topology::point3d< C > & p ) 
{
  os << p.x << " " << p.y << " " << p.z ;
  return os;
}

}

/*********************************************************************/

#endif // synaps_topology_point3d_H
