/*********************************************************************
*      This file is part of the source code of SYNAPS library          *
*   Author(s):                                                       *
*              G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
/** \file                                                           **/
#ifndef synaps_topology_point_set_H
#define synaps_topology_point_set_H

/*********************************************************************/
#include <iostream>
#include <vector>
#include "synaps/init.h"
#include "synaps/topology/point.h"
/*********************************************************************/
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace topology
{
//!  Structure that defines a point_set.
  template <class C, class Point= topology::point<C> >
struct point_set
{
  typedef std::vector<Point>             rep_t;
  typedef Point                          value_type;
  typedef Point                          point_t;
  typedef unsigned                       size_type;
  typedef typename rep_t::iterator       iterator;
  typedef typename rep_t::const_iterator const_iterator;

  /*! The type of the coefficients. */
  typedef typename Point::coeff_t coeff_t;

  rep_t rep_;
  
  point_set():rep_() {}
  
  /*! Copy constructor. */
  point_set( const point_set& S ): rep_(S.rep_){}
  
  /*! Constructor from a point. */
  point_set( const Point & p1): rep_() 
    {
      rep_.push_back(p1);
    }

  /*! Insert a point in the point set. */
  point_set<C,Point>& insert( const Point & p1)
    {
      rep_.push_back(p1); return *this;
    }
  
  iterator       begin()        {return rep_.begin();}
  const_iterator begin() const  {return rep_.begin();}
  iterator       end()          {return rep_.end();}
  const_iterator end()   const  {return rep_.end();}

  unsigned size() const {return rep_.size();} 

  void resize(int n) {return rep_.resize(n);}

  Point & operator[] (unsigned i)       {return rep_[i];}
  Point   operator[] (unsigned i) const {return rep_[i];}

};

  //--------------------------------------------------------------------
  /*!
    Another output stream for the geomview format.
  */
  template <class C, class R>
  std::ostream& 
  print( std::ostream& os, const topology::point_set<C,R>& s ) 
  {
    typedef typename topology::point_set<C,R>::const_iterator const_iterator; 
    const_iterator it=s.begin();
    os <<(*it); ++it;
    for( ; it !=s.end(); it++) os<<","<<*it;
    return os; 
  }

  /*! Output stream. */
  template <class Point>
  std::ostream & 
  operator<<( std::ostream& os, const topology::point_set<Point>& s )
  {
    os << "PointSet("; topology::print(os,s);os << ")";
    return os;
  }
  
  /*!
    The \c operator \c >> .\n
    This operator allows us to read a point_set.
  */
  template<class Point>
  std::istream &
  operator>>(std::istream & is, topology::point_set<Point>& s)
  {
    is >> s.a >> s.b;
    return is;
  }
} // namespace topology

/*********************************************************************/
__END_NAMESPACE_SYNAPS
#endif // synaps_topology_point_set_H
