/*********************************************************************
*      This file is part of the source code of SYNAPS library          *
*   Author(s): L. Deschamps                                          *
*              G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
#ifndef synaps_topology_segment_H
#define synaps_topology_segment_H
/*********************************************************************/
#include <synaps/init.h>
#include <iostream>
#include "synaps/base/shared_object.h"
#include "synaps/topology/point.h"
/*********************************************************************/
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace topology
{
//! Structure that defines a segment.
/*! 
  Structure that defines a segment with two points.
*/
template <class Point>
struct segment 
{
  /*! The type of the coefficients. */
  typedef typename  Point::coeff_t coeff_t;

  /*! The two points which define a segment. */
  Point a,b;
  
  /*! Default constructor. */
  segment():a(),b() {}
  
  /*! Copy constructor. */
  segment( const segment& Segment )
    {
      a = Segment.a;
      b = Segment.b;
    }
  
  /*! Regular constructor. */
  segment( const Point& p1, const Point& p2 )
    {
      a = p1;
      b = p2;
    }
  
};

/*! Output stream. */
template <class Point>
std::ostream & 
operator<<( std::ostream& os, const topology::segment<Point>& s )
{
  os << "Segment[" << s.a << ";" << s.b << "]\n";
  return os;
}

/*!
  The \c operator \c >> .\n
  This operator allows us to read a segment.
*/
template<class Point>
std::istream &
operator>>(std::istream & is, topology::segment<Point>& s)
{
  is >> s.a >> s.b;
  return is;
}

/*!
  Another output stream for the geomview format.
*/
template <class Point>
std::ostream& 
print( std::ostream& os, const topology::segment<Point>& t ) 
{
  assert(t.a.size() == t.b.size());
  if (t.a.size() == 2)
    {
      os << " "   << t.a[0] <<" " <<t.a[1]  << " 0 "
	 << " " << t.b[0] <<" " << t.b[1]  << " 0 \n";
    }
  else
    {
      os << " "   << t.a[0] <<" " << t.a[1] <<" " << t.a[2]
	 << " " << t.b[0] <<" " << t.b[1] <<" " << t.b[2] << "\n";
    }
  return os; 
}

#if 0
/*!
  Another output stream for the geomview format when we use
  the rational number of gmp.
 */
std::ostream&  print( std::ostream& os, const segment<topology::point<QQ> >& t ) 
{
  assert(t.a.size() == t.b.size());
  if (t.a.size() == 2)
    {
      os << " "   << t.a[0].get_d() <<" " <<t.a[1].get_d()  << " 0 "
	 << " " << t.b[0].get_d() <<" " << t.b[1].get_d()  << " 0 \n";
    }
  else
    {
      os << " "   << t.a[0].get_d() <<" " << t.a[1].get_d() <<" " << t.a[2].get_d()
	 << " " << t.b[0].get_d() <<" " << t.b[1].get_d() <<" " << t.b[2].get_d() << "\n";
    }
  return os; 
}

/*!
  Another output stream for the geomview format when we use
  the floating point of gmp.
 */
std::ostream& 
print( std::ostream& os, const segment<topology::point<RR> >& t ) 
{
  assert(t.a.size() == t.b.size());
  if (t.a.size() == 2)
    {
      os << " "   << t.a[0].get_d() <<" " <<t.a[1].get_d()  << " 0 "
	 << " " << t.b[0].get_d() <<" " << t.b[1].get_d()  << " 0 \n";
    }
  else
    {
      os << " "   << t.a[0].get_d() <<" " << t.a[1].get_d() <<" " << t.a[2].get_d()
	 << " " << t.b[0].get_d() <<" " << t.b[1].get_d() <<" " << t.b[2].get_d() << "\n";
    }
  return os; 
}

#endif // synaps_HAVE_LIBGMP

//--------------------------------------------------------------------
} //topology
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/*********************************************************************/
#endif // synaps_topology_segment_H
