/*********************************************************************
*      This file is part of the source code of SYNAPS library          *
*   Author(s): L. Deschamps                                          *
*              G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
#ifndef synaps_topology_vertex_H
#define synaps_topology_vertex_H
/*********************************************************************/
#include <vector>
#include <synaps/init.h>
/*********************************************************************/
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace topology
{
//! The vertex structure for meshing implicit surface.
/*! 
  Vertex class that defines a vertex of a mesh with a point and an 
  index.
*/
template <class Point>
struct vertex 
{ 
  /*! The type of the coefficients. */
  typedef typename Point::coeff_t   coeff_t;

  /*! Coordinates of the vertex. */
  Point point;
    
  /*! Index of the vertex in the list of all the vertex constituting
    the mesh. */
  unsigned index;
    
  /*! Default constructor. */
  vertex() {point = Point();}
  
  /*! Regular constructor. */
  vertex( coeff_t x, coeff_t y, coeff_t z ) 
    {
      point = Point(x,y,z);
    }

private:
  
  /*!
    Copy constructor.
    This is private to prohibit the copy constructor.
  */
  vertex( const vertex & Vertex )
    {std::cerr << " vertex(const vertex & Vertex) \n";}
};

/*! Comparison operator. */
template <class Point>
bool 
operator == ( const vertex<Point> &v1, const vertex<Point> &v2 )
{return ( v1.point == v2.point );}

/*!
  The \c operator \c << .\n
  This operator allows us to display a vertex.
*/
template<class Point>
std::ostream &
operator<<(std::ostream & os, const vertex<Point> & v)
{
  os << "Vertex:" << std::endl;
  os << "->index: " << v.index << std::endl;
  os << "->point: " << v.point << std::endl;
  return os;
}
  

} //topology
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/*********************************************************************/
#endif // synaps_topology_vertex_H
