#include <iostream>
#include <list>
#include "synaps/util/Clock.h>
#include <synaps/inout/xml.H>
#include "synaps/upol/UPolDse.H"

template<class UPOL,class MTH>
void run_solve(int argc, char** argv, const MTH & mth) 
{
  using namespace std;
  double total=0,start,end;

  using std::cout;
  using std::endl;

  int N=0;
  bool verbose=false;
  for(unsigned i=0;i<argc;i++)
    if (string(argv[i]) == string("-h"))
      {
	std::cout<<"Usage: "<<argv[0]<<" file [-h|-v]"<<std::endl;
	return;
      }
    else if (string(argv[i]) == string("-v"))
      verbose=true;
    else if (string(argv[i]) == string("-n"))
      N = atoi(argv[i+1]);

  std::ifstream in(argv[1],std::ios::in);
  if(!in.good())
    {
      std::cout<<"file: "<<argv[1]<<std::endl;
      return;
    }
  std::string s;
  //  xml::read(in,s,"name");
  //  xml::print(std::cout,s,"name")<<std::endl;
  typedef UPOL upol_t;
  std::list<upol_t>  L;
  upol_t p;
  int nbp=0;
  int deg=0;
  while(xml::read(in,p) && (N==0 || nbp < N)) {
    L.push_back(p);
    deg= std::max(deg,Degree(p));
    nbp++;
  }
  int nr=0;
  for(typename std::list<upol_t>::const_iterator it = L.begin(); it != L.end(); ++it)
    {
      if(verbose)
	{ 
	  xml::print(std::cout,*it,"polynomial")<<std::endl;
	  nr+= Solve(*it,mth).size();
	  xml::print(std::cout,Solve(*it,mth),"solution")<<std::endl;
	}
      else
	{
	  start = clock();
	  nr+= Solve(*it,mth).size();
	  end = clock();
	  total += (end - start);
	}    }
  cout << "<nb_pol>" <<  nbp << "</nb_pol>";
  cout << "<degree>" <<  deg << "</degree>";
  cout << "<nb_root>" <<nr<<"</nb_root>";
  if(!verbose)
    {
      cout << "<time>" << (total / CLOCKS_PER_SEC) << "</time> ";
      cout << "<speed>" << int(nr/(total / CLOCKS_PER_SEC)) << "</speed>" << endl;
    }
}

template<class UPOL,class MTH>
void run_rootof(int argc, char** argv, const MTH & mth) 
{
  using namespace std;
  double total=0,start,end;

  using std::cout;
  using std::endl;

  bool verbose=false;
  int iters=0, deg=0, N=0;
  for(unsigned i=0;i<argc;i++)
    if (string(argv[i]) == string("-h"))
      {
	std::cout<<"Usage: "<<argv[0]<<" file [-h|-v]"<<std::endl;
	return;
      }
    else if (string(argv[i]) == string("-v"))
      verbose=true;
    else if (string(argv[i]) == string("-n"))
      N = atoi(argv[i+1]);

  std::ifstream in(argv[1],std::ios::in);
  std::string s;
  //  xml::read(in,s,"name");
  //  xml::print(std::cout,s,"name")<<std::endl;
  typedef UPOL upol_t;
  std::list<upol_t>  L;
  upol_t p;
  int nbp=0;
  while(xml::read(in,p) && (N==0 || nbp < N)) {
    L.push_back(p);
    deg= std::max(deg,Degree(p));    
    nbp++;
  }

  for(typename std::list<upol_t>::const_iterator it = L.begin(); it != L.end(); ++it)
    {
      if(verbose)
	{ 
	  xml::print(std::cout,*it,"polynomial")<<std::endl;
	  xml::print(std::cout,RootOf(*it,mth),"solution")<<std::endl;
	}
      else
	{
	  start = clock();
	  RootOf(*it,mth);
	  end = clock();
	  total += (end - start);
	}

    }
  cout << "<nb_pol>" <<  nbp << "</nb_pol>";
  cout << "<degree>" <<  deg << "</degree>";
  if(!verbose)
    {  
      cout << "<time>" << (total / CLOCKS_PER_SEC) << "</time> ";
    }
}
