<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Dense univariate polynomials>\ 

  <index|univariate polynomial>A dense univariate polynomial
  <with|mode|math|f=f<rsub|0>+f<rsub|1><space|0.25spc>x+\<cdots\>+f<rsub|d><space|0.25spc>x<rsup|d>>
  is represented by an array of coefficients
  <with|mode|math|[f<rsub|0>,\<ldots\>,f<rsub|d>]>. All its coefficients are
  stored. It is assumed here that the coefficient ring has the basic
  arithmetic operations.

  The implementation is parameterised as follows:

  <\quote-env>
    <\code>
      template\<less\>class C, class R=upol::rep\<less\>C\<gtr\> \<gtr\>
      struct UPolDse;
    </code>
  </quote-env>

  where

  <\itemize>
    <item>The type <verbatim|C> is the coefficient type.

    <item>The type <verbatim|R> corresponds to the internal representation.
  </itemize>

  <subsection|Containers>

  <\itemize>
    <item>The default implementation for <verbatim|R> is
    <verbatim|upol::rep\<less\>C\<gtr\>>. The corresponding basis used to
    store the polynomial is the monomial bais <with|mode|math|(x<rsup|i>)>.

    <item>The container <with|font-family|tt|std::vector\<less\>C\<gtr\>> of
    the <with|font-shape|small-caps|stl> can also be used. Here also, we use
    the monomial basis.

    <item>The container <verbatim|bezier::rep1d\<less\>C\<gtr\>> (see
    <verbatim|synaps/upol/bezier/rep1d.h>). In this case, it is assumed the
    polynomial is represented in the Bernstein basis
    <with|mode|math|<left|(><stack|<tformat|<table|<row|<cell|d>>|<row|<cell|i>>>>><right|)>x<rsup|i>(1-x)<rsup|d-i>>,
    if \ it is of degree <with|mode|math|d>. When a polynomial is constructed
    for a string (eg. <verbatim|x^2+x-1>), the conversion from the monomial
    basis to the Bersntein basis is performed. When the polynomial is
    printed, the converse transformation is applied.\ 
  </itemize>

  <subsection|Implementation>

  Generic functions for univariate polynomials are available in the module
  <verbatim|UPOLDAR>.\ 

  See: <verbatim|synaps/upol/UPolDse.h>, <verbatim|synaps/upol/UPOLDAR.m>

  <subsection|Example>

  <action|<with|mode|math|\<boxdot\>>|UPolDse.cc>

  <subsection|Gcd and Lcm>

  <\itemize>
    <item><verbatim|content(p)> computes the gcd of the coefficients of
    <verbatim|p>. If their type is <verbatim|double>, it returns the leading
    coefficient for non-zero polynomials and <with|mode|math|1> otherwise.

    <item><verbatim|gcd(p,q)> computes the gcd of two polynomials. It is
    based on the Sturm-Habicht construction. The content of the returned
    polynomial is <with|mode|math|\<pm\>1>.

    <item><verbatim|lcm(p,q)> computes the squarefree part of a polynomial
    <with|mode|math|p>, that is <with|mode|math|<frac|p(x)*q(x)|gcd(p(x),q(x))>>.
    <verbatim|>
  </itemize>

  See: <verbatim|synaps/upol/gcd.h>

  <subsection|Squarefree Factorisation>

  <\itemize>
    <item><verbatim|square_free(p)> computes the squarefree part of a
    polynomial <with|mode|math|p>, that is
    <with|mode|math|<frac|p(x)|gcd(p(x),p<rprime|'>(x))>>.\ 

    <item><verbatim|square_free_factorisation(p)> computes the product of
    factors of multiplicity <with|mode|math|i> in <with|mode|math|p>. It
    outputs a sequence of polynomials, the <with|mode|math|i<rsup|th>>
    polynomial corresponding to the factors of multiplicity
    <with|mode|math|i>.
  </itemize>

  See: <verbatim|synaps/upol/square_free.h>\ 
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1|?>>
    <associate|auto-3|<tuple|1.1|?>>
    <associate|auto-4|<tuple|1.2|?>>
    <associate|auto-5|<tuple|1.3|?>>
    <associate|auto-6|<tuple|1.4|?>>
    <associate|auto-7|<tuple|1.5|?>>
    <associate|auto-8|<tuple|1.6|?>>
    <associate|auto-9|<tuple|2|?>>
    <associate|idx-1|<tuple|<uninit>|?>>
    <associate|toc-1|<tuple|<uninit>|?>>
    <associate|toc-2|<tuple|<uninit>|?>>
    <associate|toc-3|<tuple|<uninit>|?>>
    <associate|toc-4|<tuple|<uninit>|?>>
    <associate|toc-5|<tuple|<uninit>|?>>
    <associate|toc-6|<tuple|<uninit>|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|idx>
      <tuple|<tuple|univariate polynomial>|<pageref|auto-2>>
    </associate>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Dense
      univariate polynomials> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Containers
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>General Tools for
      Univariate Polynomials <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|1.5fn>|1.5<space|2spc>Gcd and Lcm
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>

      <with|par-left|<quote|1.5fn>|1.6<space|2spc>Squarefree Factorisation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-8>>
    </associate>
  </collection>
</auxiliary>