/*********************************************************************
*      This file is part of the source code of AXEL library          *
*   Author(s): M. Celikbas                                           *
*              G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
#ifndef synaps_upol__bspline_BSControl_1_H
#define synaps_upol__bspline_BSControl_1_H
/*********************************************************************/
#include <synaps/init.h>
#include <synaps/upol/bspline/BSPLINE.h>
/*********************************************************************/
__BEGIN_NAMESPACE_SYNAPS 
//--------------------------------------------------------------------
//! The BSpline curve container
/*!
  Here, we have implemented various functions we need for
  the spline curves.
*/
class BSControl_1
{

public:
  
  /*! The default constructor. */
  BSControl_1() {}
 
  /*! The default destructor. */
  ~BSControl_1() {}

  /*********************************************************************/

  /*!
    \param result.
    \param b the bspline curve which we want to evaluate.
    \param x the node where we want to evaluate the bspline curve.

    see in the implementation module the function 
    \ref curve_value "value" for more details.
  */
  template<class C, class N, class Pt, class R> 
  void value(R& result, const bspline::curve<C,N,Pt>& b, const C& x)
    {
      using namespace BSPLINE;
      value(result, b.points_, b.nodes_, b.degree_, x);
    }
  //--------------------------------------------------------------
  /*!
    \param b the bspline curve which we want to insert a node.
    \param x the node we want to insert in the bspline curve.

    see in the implementation module the function 
    \ref curve_insert "insert" for more details.
  */
  template<class C, class N, class Pt>
  void insert(bspline::curve<C,N,Pt>& b, const C& x)
    {
      using namespace BSPLINE;
      insert(b.points_, b.nodes_, b.degree_, x);
    }
  //--------------------------------------------------------------
  /*!
    \param b the bspline curve.
    
    see in the implementation module the function 
    \ref curve_derive "derive" for more details.
  */
  template<class C, class N, class Pt>
  void derive(bspline::curve<C,N,Pt>& b)
    {
      using namespace BSPLINE;
      derive<C,N,Pt>(b.points_, b.nodes_, b.degree_--);	
    }
  //--------------------------------------------------------------
  /*!
    \param f the file where we store the polyline of control.
    \param b the bspline.
    \param inf.
    \param sup.

    
    see in the implementation module the function 
    \ref curve_draw_polyline "draw_polyline" for more details.
  */
  template<class C, class N, class Pt>
  void draw_polyline(std::ostream& f, const bspline::curve<C,N,Pt>& b,
		     int inf=-1, int sup=-1)
    {
      using namespace BSPLINE;
      BSPLINE::draw_polyline(f, b.points_, inf, sup);
    }
  //--------------------------------------------------------------
  /*!
    \param f the file where we store the bspline curve.
    \param b the bspline curve.
    \param inf.
    \param sup.
    
    
    see in the implementation module the function
    \ref curve_draw_curve "draw_curve" for more details.
  */
  template<class C, class N, class Pt>
  void draw_curve(std::ostream& f, const bspline::curve<C,N,Pt>& b,
		  int inf=-1, int sup=-1)
    {
      using namespace BSPLINE;
      BSPLINE::draw_curve<C,N,Pt>(f, b.points_, b.nodes_, b.degree_,
			inf, sup);
    }
  //--------------------------------------------------------------    
  /*!
    \param bl it represents the left side of the bspline curve after we subdivide.
    \param br it represents the right side of the bspline curve after we subdivide.
    \param b the bspline curve.
    \param x the node where we subdivide the bspline curve.

    see in the implementation module the function 
    \ref curve_subdivide "subdivide" for more details.
  */
  template<class C, class N, class Pt>
  void subdivide(bspline::curve<C,N,Pt>& bl, bspline::curve<C,N,Pt>& br,
		 const bspline::curve<C,N,Pt>& b, const C& x)
    {
      using namespace BSPLINE;
      bl.degree_ = br.degree_ = b.degree_;
      subdivide(bl.points_, bl.nodes_, br.points_, br.nodes_,
		b.points_, b.nodes_, b.degree_, x);
    }
  //-------------------------------------------------------------- 
  /*!
    \param bb the bspline curve after incrementation of the degree.
    \param b the bspline curve.

    see in the implementation module the function
    \ref curve_incr_degree "incr_degree" for more details.
  */
  template<class C, class N, class Pt>
  void incr_degree(bspline::curve<C,N,Pt>& bb, const bspline::curve<C,N,Pt>& b)
    {
      using namespace BSPLINE;
      bb.degree_=b.degree_+1;
      incr_degree<C,N,Pt>(bb.points_, bb.nodes_,
			  b.points_, b.nodes_, b.degree_);
    }
  //--------------------------------------------------------------
  /*!
    \param res.
    \param b the bspline curve.
    \param x a node.

    see in the implementation module the function
    \ref curve_basis_functions "basis_functions" for more details.
  */
  template<class C, class N, class Pt>
  void basis_functions(N& res, const bspline::curve<C,N,Pt>& b, const C& x)
    {
      using namespace BSPLINE;
      basis_functions(res, b.nodes_, b.degree_, x);
    }


};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS 
/*********************************************************************/
#endif //
