<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Univariate Bezier Subdivision Solver>

  We use the representation of a polynomial <with|mode|math|f> in the
  Bernstein basis: <with|mode|math|f(x) = <big|sum><rsub|i=0><rsup|d>
  b<rsub|i> B<rsub|i><rsup|d>(x)>, where <with|mode|math|B<rsub|i><rsup|d>(x)=<matrix|<tformat|<table|<row|<cell|d>>|<row|<cell|i>>>>>x<rsup|i>(1-x)<rsup|d-i>>.The
  coefficients <with|mode|math|\<b-b\> = [b<rsub|i>]<rsub|i=0,\<ldots\>,d>>
  in this basis are called the control coefficients. We denote by
  <with|mode|math|V(\<b-b\>)> the number of sign variation of sequence
  <with|mode|math|\<b-b\>>, not taking into account the <with|mode|math|0> of
  this sequence. Descartes' rule applies on this representation as follows:\ 

  <\proposition>
    The number of sign changes <with|mode|math|V(\<b-b\>)> of
    \ <with|mode|math|\<b-b\>= [b<rsub|i>]<rsub|i=0,\<ldots\>,d>> bounds the
    number of real roots of f on <with|mode|math|[0, 1]> and is equal to it
    modulo 2.\ 

    <\itemize>
      <item>If <with|mode|math|V(\<b-b\>)=0>, the number of real root on [0,
      1] is 0

      <item>If <with|mode|math|V(\<b-b\>)=1>, the number of real root on [0,
      1] is 1.
    </itemize>
  </proposition>

  \ We also use de Casteljau subdivision algorithm to subdivide the
  representation. This yields the following algorithm:\ 

  <\notation>
    <name|Isolation of the roots of <with|mode|math|f> on the interval
    <with|mode|math|[a,b]>.>

    INPUT: A representation <with|mode|math|(\<b-b\>, [a, b])> associate with
    <with|mode|math|f>.\ 

    <\itemize>
      <item>Compute the number of sign changes <with|mode|math|V(\<b-b\>)> of
      the coefficient sequence <with|mode|math|\<b-b\>>.

      <item>If <with|mode|math|V(\<b-b\>)=0>, remove the interval.\ 

      <item>If <with|mode|math|V(\<b-b\>)\<gtr\>1> and the size
      <with|mode|math|\|b-a\|> is greater than
      <with|mode|math|\<varepsilon\>>, subdivide the representation into two
      subrepresentations, corresponding to the two halves of the input
      interval and apply recursively the algorithm to them.

      <item>If <with|mode|math|V(\<b-b\>)\<gtr\>1> and the size
      <with|mode|math|\|b-a\|\<less\>\<varepsilon\>>, output the
      <with|mode|math|\<varepsilon\>>-root <with|mode|math|<frac|a+b|2>> with
      mutliplicity <with|mode|math|V(\<b-b\>)>.
    </itemize>

    OUTPUT: a list of subintervals of <with|mode|math|[a, b]> containing
    exactly one real root of <with|mode|math|f> or intervals of size
    <with|mode|math|\<varepsilon\>> containing a
    <with|mode|math|\<varepsilon\>>-multiple root.
  </notation>

  Several implementations, based on this scheme are available. The methods
  which compute the roots, within a given precision are named
  <verbatim|Slv...>, those which isolate the roots are named
  <verbatim|Isl...>. We describe now the solvers of these families available
  in the library.\ 

  <subsection|Bezier Standard Approximation Solver>

  <\code>
    Seq\<less\>O\<gtr\> solve(const UPOL& p, SlvBzStd\<less\>O,C\<gtr\> mth);
  </code>

  <\itemize>
    <item>It computes all the real roots, within a given precision. The
    parameter <verbatim|O> precises the type of the solutions, and
    <verbatim|C> is the number type used to perform this computation. It is
    not necessarly the type of the coefficients of <verbatim|p>, but it
    should be a field type. The default value of <verbatim|C> is
    <verbatim|double> so that it can be used as
    <verbatim|SlvBzStd\<less\>O\<gtr\>()>.

    <item>The precision of the computation can be given as an argument of the
    constructor: <verbatim|SlvBzStd\<less\>O,C\<gtr\>(1e-10)>. The default
    type for this precision <with|mode|math|\<varepsilon\>> is
    <verbatim|double> and the default value is <verbatim|1e-6>. If one want
    to use another type of precision (say <verbatim|X>), one can use instead
    <verbatim|SlvBzStd\<less\>O,C,X\<gtr\>(xeps)>;

    <item>This solver perform a binary subdivision of an initial interval
    containing the roots. It stops the recursion when the size of the
    subintervals is smaller that the precision
    <with|mode|math|\<varepsilon\>> specificied by the class
    <verbatim|SlvBzStd\<less\>O,C\<gtr\>> and output in the result the middle
    of the interval. The subintervals which are removed have no sign
    variation. The \ intervals which are kept may have a sign variation
    bigger that <with|mode|math|1>. The sequence <verbatim|s> contains the
    approximated roots in increasing order.

    <item>It can be called to isolate the roots in an interval
    <verbatim|[a,b]> (where <verbatim|a> and <verbatim|b> are of type
    <verbatim|O>): \ <verbatim|solve(p,SlvBzStd\<less\>O,C\<gtr\>(),a,b);>

    <item>It can be used to compute quickly numerical approximation of roots
    taking <verbatim|C=double> for instance, or with exact arithmetic (eg.
    rational numbers <verbatim|C=QQ>) to get certified approximation.
  </itemize>

  See: <verbatim|synaps/usolve/bezier/SlvBzStd.h>

  <subsection|Bezier Standard Isolation Solver>

  <\code>
    Seq\<less\>Interval\<less\>O\<gtr\> \<gtr\> solve(const UPOL& p,
    IslBzStd\<less\>O,C\<gtr\> mth);
  </code>

  <\itemize>
    <item>It isolates the roots of the polynomial <verbatim|p>, that is, it
    computes a sequence of intervals containg a unique root, or an interval
    of size <with|mode|math|\<varepsilon\>>, where
    <with|mode|math|\<varepsilon\>> is the precision defined in in the class
    <verbatim|IslBzStd\<less\>O,C\<gtr\>>, if <verbatim|p> has mutiple roots
    or a cluster of roots. This precision
    <with|mode|math|\<varepsilon\>=><verbatim|eps> is specified by
    <verbatim|IslBzStd\<less\>O,C\<gtr\>(eps)>.

    <item>The result is a sequence of intervals with endpoints of type
    <verbatim|O>. The type <verbatim|C> is the number type used during the
    computation. It is not necessarly the type of the coefficients of
    <verbatim|p>, but it should be a field type. The default value of
    <verbatim|C> is <verbatim|double> so that it can be used as
    <verbatim|IslBzStd\<less\>O\<gtr\>()>.

    <item>This solver perform the same subdivision as for the previous
    solver, except that it stops the recursion when the sign variation is
    <with|mode|math|1>. If there is a multiple or a cluster of roots, the
    recursion will be continue until the size of the subinterval is smaller
    that the precision <with|mode|math|\<varepsilon\>> specificied by the
    class <verbatim|IslBzStd\<less\>O,C\<gtr\>>. The subintervals which are
    removed have no sign variation. The intervals kept, which are os size
    <with|mode|math|\<varepsilon\>> may have a sign variation bigger that
    <with|mode|math|1>. The sequence <verbatim|s> contains the subintervals
    sorted in increasing order. These intervals may share some ending points.

    <item>It can be called to isolate the roots in an interval
    <verbatim|[a,b]> (where <verbatim|a> and <verbatim|b> are of type
    <verbatim|O>): \ <verbatim|solve(p,IslBzStd\<less\>O,C\<gtr\>(),a,b);>
  </itemize>

  See: <verbatim|synaps/usolve/bezier/IslBzStd.h>

  <subsection|Bezier Integer Approximation Solver>

  <\code>
    Seq\<less\>O\<gtr\> solve(const UPOL& p, SlvInteger\<less\>O\<gtr\> mth);
  </code>

  <\itemize>
    <item>Same behavior as the solver <verbatim|SlvBzStd>, except that the
    internal coeffcient type used to perform the subdivision is the integer
    type <verbatim|ZZ>. The de Casteljau algorithm is adapted so that the
    representation of the internal polynomial remains with integer
    coefficients.\ 

    <item>The same convention as before are used: <verbatim|O> is the output
    type of the solutions; the precision <with|mode|math|\<varepsilon\>=><verbatim|eps>
    can be specified by <verbatim|SlvBzInteger\<less\>O\<gtr\>(eps)>.\ 

    <item>It can also be called to isolate the roots in an interval
    <verbatim|[a,b]> (where <verbatim|a> and <verbatim|b> are of type
    <verbatim|O>): \ <verbatim|solve(p,SlvBzInteger\<less\>O\<gtr\>(),a,b);>
  </itemize>

  See: <verbatim|synaps/usolve/bezier/IslBzInteger.h>

  <subsection|Bezier Bounding Approximation Solver >

  <\code>
    Seq\<less\>O\<gtr\> solve(const UPOL& p, SlvBzBgd\<less\>O\<gtr\> mth);
  </code>

  <\itemize>
    <item>It computes all the real roots, within a given precision
    <with|mode|math|\<varepsilon\>>. The parameter <verbatim|O> specifies the
    output type, for the approximate solutions.

    <item>The precision <with|mode|math|\<varepsilon\>=><verbatim|eps> of the
    approximation can be given as an argument of the constructor:
    <verbatim|SlvBzBdg\<less\>O\<gtr\>(eps)>. The default type for this
    precision <with|mode|math|\<varepsilon\>> is <verbatim|double> and the
    default value is <verbatim|1e-6>.

    <item>This solver assume that the coefficients are of the exact type. It
    performs a transformation in the Bernstein basis, using this exact
    arithmetic. Then the coefficients are scaled and rounded below and above
    to <verbatim|double>. The subdivision is performed, exploiting the
    coefficient sign variations on this upper and lower enveloppes of the
    input polynomial. If these polynomial enveloppes do not allow us to
    approximate the roots within the precision
    <with|mode|math|\<varepsilon\>>, we obtain a suspect subinterval (of size
    bigger than <with|mode|math|\<varepsilon\>>) on which the solver
    <verbatim|SlvBzInteger> is applied.\ 
  </itemize>

  See: <verbatim|synaps/usolve/bezier/SlvBzBdg.h>

  <subsection|Rockwood Approximation Solver >

  <\code>
    O solve(const UPOL& p, SlvRkw\<less\>O,C\<gtr\> mth);
  </code>

  <\itemize>
    <item>It computes an approximation of the first positive root of the
    polynomial <verbatim|p>, using Rockwood's method. The internal type used
    during the computation is <verbatim|C>, which default value is
    <verbatim|double>.\ 

    <item>The precision <with|mode|math|\<varepsilon\>=><verbatim|eps> of the
    approximation is an argument of the constructor:
    <verbatim|SlvRkw\<less\>O,C\<gtr\>(eps)>. The default type for this
    precision <with|mode|math|\<varepsilon\>> is <verbatim|double> and the
    default value is <verbatim|1e-6>.
  </itemize>

  See: <verbatim|synaps/usolve/bezier/SlvRkw.h>

  <subsection|Example>

  <action|<with|mode|math|\<boxdot\>>|BzSolver.cc>

  \;
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-10|<tuple|3|?>>
    <associate|auto-11|<tuple|3.1|?>>
    <associate|auto-12|<tuple|4|?>>
    <associate|auto-13|<tuple|4.1|?>>
    <associate|auto-14|<tuple|4.2|?>>
    <associate|auto-15|<tuple|4.2|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
    <associate|auto-6|<tuple|1.5|?>>
    <associate|auto-7|<tuple|1.6|?>>
    <associate|auto-8|<tuple|2|?>>
    <associate|auto-9|<tuple|2.1|?>>
    <associate|toc-1|<tuple|1|1>>
    <associate|toc-10|<tuple|1.4.1|?>>
    <associate|toc-11|<tuple|1.5|?>>
    <associate|toc-2|<tuple|1.1|1>>
    <associate|toc-3|<tuple|1.1.1|1>>
    <associate|toc-4|<tuple|1.2|1>>
    <associate|toc-5|<tuple|1|?>>
    <associate|toc-6|<tuple|1.2.1|?>>
    <associate|toc-7|<tuple|1.3|?>>
    <associate|toc-8|<tuple|1.3.1|?>>
    <associate|toc-9|<tuple|1.4|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Univariate
      Bezier Subdivision Solver> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Bezier Standard
      Approximation Solver <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Bezier Standard Isolation
      Solver <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Bezier Integer
      Approximation Solver <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>Bezier Bounding
      Approximation Solver \ <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.5<space|2spc>Rockwood Approximation
      Solver \ <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|1.5fn>|1.6<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>
    </associate>
  </collection>
</auxiliary>