
#ifndef _SYNAPS_SOLVE_CONTFRAC_H_
#define _SYNAPS_SOLVE_CONTFRAC_H_

#include <synaps/init.h>
#include <synaps/arithm/Interval.h>
#include <synaps/arithm/traits.h>
//#include <synaps/arithm/RealOf.h>
#include <synaps/upol/UPolDse.h>
#include <synaps/usolve.h>

#include <synaps/usolve/solver_base.h>
#include <synaps/usolve/Small_degree.h>
#include <synaps/usolve/sturm/common.h>
#include <synaps/usolve/contfrac/solver.h>
#include <synaps/usolve/contfrac/lower_bound.h>


__BEGIN_NAMESPACE_SYNAPS


/// Class
template < class NT , class LowerBound = CauchyBound

>
struct ContFrac : public Small_degree<NT>
{
  typedef LowerBound        bound;
  typedef ContFrac          self_t;
};




template < class NT, class LB > inline
Seq< typename ContFrac<NT,LB>::RO_t > 
solve( const typename ContFrac<NT>::Poly& f, ContFrac<NT,LB>)
{
  typedef ContFrac<NT,LB>           K;
  typedef typename K::RO_t     RO_t;
    
  Seq<RO_t> sol;
  
  ALGEBRAIC::solve_contfrac(f, std::back_inserter(sol.rep()), K());
  std::stable_sort( sol.begin(), sol.end(), ALGEBRAIC::Refine_compare());
    
  return sol;
}

__END_NAMESPACE_SYNAPS

#endif // _SYNAPS_SOLVE_CONTFRAC_H_

