/**********************************************************************
 *   This file is part of the source code of the SYNAPS library.      *
 *   Author(s): B. Mourrain, GALAAD, INRIA                            *
 *   $Id: Dichotomy.h,v 1.1 2005/07/11 10:03:50 jppavone Exp $
 **********************************************************************/
#ifndef SYNAPS_SOLVE_DICHOTOMY_H
#define SYNAPS_SOLVE_DICHOTOMY_H

#include <synaps/init.h>
#include <synaps/arithm/sign.h>


__BEGIN_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
/** Specify the Dichotomy solver. The parameter \c eps is the precision
    wanted on the root.
 **/
struct Dichotomy {
  double eps;
  Dichotomy():eps(1.e-6){}
  Dichotomy(double x):eps(x){}
};
//----------------------------------------------------------------------
/** solve the univariate polynomial \c p of type \c T, by dichotomy, in
    the interval [a,b]. If the sign of \c p at \c a and \c b is not different
    an error is printed.
**/
template<class T,class C>
C RootOf(const T &p, const C & a, const C & b, const Dichotomy & Mth)
{
	register C u0=a, u1=b, um;
	int s0=sign(p(a)), s1=sign(p(b));
	while((u1-u0)>Mth.eps && s0 != s1)
	{
		um = C((u0+u1)/2);
		int sm= sign(p(um));
		if(sm != s0)
		{ s1=sm; u1=um; }
		else if(sm != s1)
		{ s0=sm; u0=um; }
		else if(sm!=0)
		{
			std::cerr <<"Error in sign evaluation"<< std::endl;
			return 100000;//	  exit(1);
		}
	}
	if ((s0 == s1) && (a != b))
		std::cerr <<"No opposite sign at "<<a<<" and "<<b<<std::endl;
	return (u0+u1)/2;
}

__END_NAMESPACE_SYNAPS

/********************************************************************/
#endif // SYNAPS_SOLVE_DICHOTOMY_H
