#ifndef _synaps_solve_SturmMem_H_
#define _synaps_solve_SturmMem_H_

#include <synaps/init.h>
#include <synaps/usolve/Algebraic.h>
#include <synaps/usolve/solver_base.h>
#include <synaps/usolve/sturm/common.h>

__BEGIN_NAMESPACE_SYNAPS

template < typename NT >
struct SturmMem : public solver_base
{
  
  typedef NumberTraits<NT>             NTR;
  
  typedef typename NTR::RT             RT;
  typedef typename NTR::FT             FT;
  typedef typename NTR::XT             XT;
  typedef RR                           RL;
  
  typedef typename NTR::FIT            FIT;
  typedef typename NTR::XIT            XIT;
  typedef typename NTR::XIT            RIT;
  //  typedef typename NTR::RIT            RIT;
  

  typedef UPolDse<RT>    Poly;
  typedef UPolDse<RT>    RT_Poly;
  typedef UPolDse<FT>    FT_Poly;

  typedef UPolDse<XT>    XT_Poly;
  typedef UPolDse<RL>    RL_Poly;

  typedef ALGEBRAIC::root_of<RT, Poly>   RO_t;
  typedef ALGEBRAIC::root_of<RT, Poly>   solution_type;


  typedef SturmMem    self_t;

  //  std::map<Poly,SturmSeq<NT> > mem_;
  SturmSeq<NT> mem_;
};


template < typename UPOL, typename NT >
Seq
< 
  typename SturmMem<NT>::RO_t
> 
solve( const UPOL& F,
       SturmMem<NT>& mth,
       const typename SturmMem<NT>::FT& a, 
       const typename SturmMem<NT>::FT& b
     )
{
  typedef SturmMem<NT>    K;
  typedef typename K::RT    RT;
  typedef typename K::FT    FT;
  typedef typename K::FIT   FIT;
  typedef typename K::RO_t  RO_t;
  typedef typename K::Poly  Poly;
  typedef typename K::RO_t  RO_t;
  typedef typename K::Poly  poly_t;
  typedef typename Poly::coeff_t  CF_T;


  Seq<RO_t> sol;
  poly_t f; let::assign(f,F);

  SturmSeq<CF_T>   sth;
  if(mth.mem_.size()==0)
    {
      mth.mem_ = SturmSeq<CF_T>(f,diff(f),HABICHT());
      sth=mth.mem_;
    }
  else
    sth=mth.mem_;
    
  FIT I(a, b);
  sturm_rootof( sth, I, 1, std::back_inserter(sol.rep()), K());


  return sol;
}

__END_NAMESPACE_SYNAPS

#endif // _synaps_solve_SturmMem_H_

    
