
#ifndef SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMPARISON_1X_H
#define SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMPARISON_1X_H


#include <synaps/usolve/algebraic/Algebraic_config.h>
#include <synaps/arithm/gmp.h>
#include <synaps/upol/UPOLDAR.m>

__ALGEBRAIC_BEGIN_NAMESPACE
// TODO: TODO:
//    Put them in internal namespace !!!

// Forward declaration
template < typename RT, typename P > struct root_of;
//-------------------------------------------------------------------------------

template < typename RO_t >  inline
int
compare_11(const RO_t& a, const RO_t& b)
{
	assert(a.degree() == 1);
	assert(b.degree() == 1);
	assert(singleton(a.interval()) == true);
	assert(singleton(b.interval()) == true);
	assert(a.is_rational() == true);
	assert(b.is_rational() == true);

	// Since they are rationals we can consider
	// only an endpoint of their isolating intervals
	if (a.left() < b.left()) return SMALLER;
	if (a.left() > b.left()) return LARGER;
	return EQUAL;
}
//-----------------------------------------------------------------------------------------

template < typename RO_t >  inline
int
compare_1x(const RO_t& a, const RO_t& b)
{
	typedef typename RO_t::NTR    NTR;
	typedef typename NTR::FT      FT;

	assert(a.degree() == 1);
	assert(b.degree() > 1);
	assert(singleton(a.interval()) == true);
	assert(singleton(b.interval()) == false);
	assert(a.is_rational() == true);
	assert(b.is_rational() == false);


	FT x = a.left();
	if (b.interval() >= x) return SMALLER;
	if (b.interval() <= x) return LARGER;

	int sgn_on_x = sign( UPOLDAR::eval_horner(b.poly(), x) );
	if (sgn_on_x == 0) return EQUAL;
	// TODO: When ZERO change the representation of b

	if (sgn_on_x == b.sign_low()) return SMALLER;
	return LARGER;
}
//-----------------------------------------------------------------------------------------

template < typename RO_t >  inline
int
compare_x1(const RO_t& a, const RO_t& b)
{
	assert(a.degree() > 1);
	assert(b.degree() == 1);
	assert(singleton(a.interval()) == false);
	assert(singleton(b.interval()) == true);
	assert(a.is_rational() == false);
	assert(b.is_rational() == true);

	return opposite(compare_1x(b, a));

}
//-----------------------------------------------------------------------------------------




__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMPARISON_1X_H

