// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)


#ifndef SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_43_H
#define SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_43_H

#include <synaps/usolve/algebraic/comparison/auto/Auto_Results.h>

__ALGEBRAIC_BEGIN_NAMESPACE


template < class POLY, class RO_t >
inline
int sign_at_43(const POLY& f, const RO_t& x)
{
  typedef typename RO_t::NTR    NTR;
  typedef typename NTR::RT      RT;
  typedef typename NTR::FT      FT;

  print(__FUNCTION__);

  POLY P(3, AsSize());

  RT t9 = f[4] * x[1];
  RT t8 = -f[3] * x[3] + f[4] * x[2];
  RT t1 = x[3] * x[3];
  P[2] = -x[3] * t9 + t1 * f[2] + t8 * x[2];
  P[1] = x[2] * t9 + t1 * f[1] + (-f[4] * x[0] - x[1] * f[3]) * x[3];
  P[0] = t1 * f[0] + t8 * x[0];
  //   cout << "Reduced = "; UPOLDAR::print(cout, P); cout << endl;
  if ((UPOLDAR::degree(P) <= 0) && (P[0] == RT(0))) return EQUAL;
  if (UPOLDAR::degree(P) == 0) return sign( P[UPOLDAR::degree(P)] );
  int tmp1 = sign(P[UPOLDAR::degree(P)] );
  make_lcoeff_positive(P);
  //  cout << " f = " << f << " dg(f) = " << UPOLDAR::degree(f) << endl;
  //  cout << " P = " << P << " dg(P) = " << UPOLDAR::degree(P) << endl;
  int tmp2 = sign_at(P, x);
  return tmp1 * tmp2;


#if 0
  int where = 0;
  in_isolating_interval(f, x, where);
  FT I2 = x.right_isolating_point();
  FT I1 = x.left_isolating_point();
  int sgn_x_I2 = ((x.degree()-1-x.which()) % 2 ? NEGATIVE : POSITIVE);
  int sgn_x_I1 = opposite(sgn_x_I2);
  if (sgn_x_I2 == POSITIVE) where ^= 1;
  if (sgn_x_I1 == POSITIVE) where ^= 2;
  cout << sgn_x_I2 << " " << sgn_x_I1 << endl;
  return static_full_sturm_34<POLY>(
	  x[3], x[2], x[1], x[0],
	  f[4], f[3], f[2], f[1], f[0],
	  x.left_isolating_point(),
	  x.right_isolating_point(),
	  where);
#endif
}

__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_43_H

