// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)


#ifndef SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_44_H
#define SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_44_H

#include <synaps/usolve/algebraic/comparison/auto/Auto_Results.h>


__ALGEBRAIC_BEGIN_NAMESPACE


template < typename POLY,
		   typename RT,
		   typename FT >  inline
Comparison_result
static_degJ2_sturm_44(
	const RT& a1, const RT& b1, const RT& c1, const RT& d1, const RT& e1,
	const RT& a2, const RT& b2, const RT& c2, const RT& d2, const RT& e2,
	const FT& a,  const FT& b,
	int& where)
{
	print(__FUNCTION__);
	Comparison_result result;

	RT J3 = d2 * a1 - a2 * d1;
	RT J4 = e2 * a1 - a2 * e1;
	if (J3 == RT(0)) {
	  if (J4 == RT(0)) return Result_deg_2::on_end(where);
	  if (J4 > RT(0)) { where ^= 16; where ^= 32; }
	  return Result_deg_3::on_end(where);
	}
	POLY S2(2, AsSize());
	S2[1] = J3;
	S2[0] = J4;

//   cout << "S2 = "; UPOLDAR::print(cout, S2); cout << " d = " << UPOLDAR::degree(S2) << endl;

    if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
	print("S2 on b", sign(UPOLDAR::eval_horner(S2, b)));
	result = Static_Result::Result_4.on_S2_p2q2(where);
	if (result != UNKNOWN) return result;
	if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
	print("S2 on a", sign(UPOLDAR::eval_horner(S2, a)));
	result = Static_Result::Result_4.on_S2_p1q1(where);
	if (result != UNKNOWN) return result;

	RT J7 = -b2 * e1 + e2 * b1;
	RT t6 = J3 * J4;
	RT t2 = J4 * J4;
	RT t1 = J3 * J3;
	RT J6 = -b2 * d1 + d2 * b1 + J4;
	RT S3 = t1 * J7 * J7 + (-2 * J7 * t6 + t2 * J6) * J6
		    + (RT(2) * (-c2 * e1 + e2 * c1) * t6 - (-c2 * d1 + d2 * c1 + J7) * t2
			- t1 * (-d2 * e1 + e2 * d1)) * (-b2 * c1 + c2 * b1 + J3);
	print("S3", S3);
	S3 = -S3;
	if (S3 ==  RT(0)) return EQUAL;
	if (S3 > RT(0)) where ^= 64;
	result = Static_Result::Result_4.on_S3(where);
	if (result != UNKNOWN) return result;
	return EQUAL;
}



template < typename POLY,
		   typename RT,
		   typename FT >    inline
Comparison_result
static_degJ1_sturm_44(
	const RT& a1, const RT& b1, const RT& c1, const RT& d1, const RT& e1,
	const RT& a2, const RT& b2, const RT& c2, const RT& d2, const RT& e2,
	const FT& a,  const FT& b,
	int& where)
{
  print(__FUNCTION__);
  Comparison_result result;

  RT J2 = c2 * a1 - a2 * c1;
  if (J2 == RT(0)) {
		return static_degJ2_sturm_44<POLY>(
				a1, b1, c1, d1, e1,
				a2, b2, c2, d2, e2,
				a, b,
				where);
  }
  RT J3 = d2 * a1 - a2 * d1;
  RT J4 = e2 * a1 - a2 * e1;
  POLY S2(3, AsSize());
  S2[2] = J2;
  S2[1] = J3;
  S2[0] = J4;

// cout << "S2 = "; UPOLDAR::print(cout, S2); cout << " d = " << UPOLDAR::degree(S2) << endl;

  if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
  print("S2 on b", sign(UPOLDAR::eval_horner(S2, b)));
  result = Static_Result::Result_5.on_S2_p2q2(where);
  if (result != UNKNOWN) return result;
  if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
  print("S2 on a", sign(UPOLDAR::eval_horner(S2, a)));
  result = Static_Result::Result_5.on_S2_p1q1(where);
  if (result != UNKNOWN) return result;

  RT t50 = RT(2) * J2;
  RT J7 = -b2 * e1 + e2 * b1;
  RT t46 = J3 * J4;
  RT t43 = J3 * J3;
  RT t41 = J2 * J2;
  int t39 = ( sign(J2) == POSITIVE ? -1 : 1 );
  RT J5 = -b2 * c1 + c2 * b1 + J3;
  RT J6 = -b2 * d1 + d2 * b1 + J4;
  RT J8 = -c2 * d1 + d2 * c1 + J7;
  RT J9 = e2 * c1 - c2 * e1;
  POLY S3(2, AsSize());
  S3[1] = t39 * (J6 * J3 * t50 - t41 * J8 - J5 * t43);
  S3[0] = t39 * (-t41 * J9 - t46 * J5 + (J6 * J4 + J3 * J7) * J2);

//  cout << "S3 = "; UPOLDAR::print(cout, S3); cout << " d = " << UPOLDAR::degree(S3) << endl;
  if ((UPOLDAR::degree(S3) <= 0) && S3[0] == RT(0))
	return Result_deg_3::on_end(where);

  if (sign(UPOLDAR::eval_horner(S3, b)) >= 0) where ^= 64;
  print("S3 on b", sign(UPOLDAR::eval_horner(S3, b)));
  result = Static_Result::Result_5.on_S3_p2q2(where);
  if (result != UNKNOWN) return result;
  if (sign(UPOLDAR::eval_horner(S3, a)) >= 0) where ^= 128;
  print("S3 on a", sign(UPOLDAR::eval_horner(S3, a)));
  result = Static_Result::Result_5.on_S3_p1q1(where);
  if (result != UNKNOWN) return result;

  RT t42 = J4 * J4;
  RT J10 = -d2 * e1 + e2 * d1;
  RT S4 = t43 * J7 * J7 + (-2 * J7 * t46 + t42 * J6) * J6
	+ (-J8 * J10 + J9 * J9) * t41 + (2 * J9 * t46 - t42 * J8 - t43 * J10) * J5
	+ ((J7 * J8 - J6 * J9) * J4 + (-J7 * J9 + J6 * J10) * J3) * t50;
  S4 = t39 * S4;
  print("S4", S4);
  if (S4 ==  RT(0)) return EQUAL;
  if (S4 > RT(0)) where ^= 256;
  result = Static_Result::Result_5.on_S4(where);
  if (result != UNKNOWN) return result;
  return EQUAL;
}


template < typename POLY,
		   typename RT,
		   typename FT >  inline
Comparison_result
static_full_sturm_44(
	const RT& a1, const RT& b1, const RT& c1, const RT& d1, const RT& e1,
	const RT& a2, const RT& b2, const RT& c2, const RT& d2, const RT& e2,
	const FT& a,  const FT& b,
	int& where)
{
	print(__FUNCTION__);
	Comparison_result result;

	RT	J1 = b2 * a1 - a2 * b1;
//	print("J1", J1);
	if (J1 == RT(0)) {
		return static_degJ1_sturm_44<POLY>(
			a1, b1, c1, d1, e1,
			a2, b2, c2, d2, e2,
			a, b,
			where);
	}
	RT	J2 = c2 * a1 - a2 * c1;
	RT	J3 = d2 * a1 - a2 * d1;
	RT	J4 = e2 * a1 - a2 * e1;
	POLY S2(4, AsSize());
	S2[3] = J1;
	S2[2] = J2;
	S2[1] = J3;
	S2[0] = J4;

//  cout << "S2 = "; UPOLDAR::print(cout, S2); cout << " d = " << UPOLDAR::degree(S2) << endl;
	if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
	print("S2 on b", sign(UPOLDAR::eval_horner(S2, b)));
	result = Static_Result::Result_6.on_S2_p2q2(where);
	if (result != UNKNOWN) return result;
	if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
	print("S2 on a", sign(UPOLDAR::eval_horner(S2, a)));
	result = Static_Result::Result_6.on_S2_p1q1(where);
	if (result != UNKNOWN) return result;

	RT	t19 = J2 * J2;
	RT	J5 = -b2 * c1 + c2 * b1 + J3;
	RT	t27 = J1 * J5 - t19;
	RT	J6 = -b2 * d1 + d2 * b1 + J4;
	RT	t30 = J4 * J2;
	RT	J7 = e2 * b1 - b2 * e1;
	RT	t26 = -J1 * J7 + t30;
	RT	t33 = J3 * J2;
	POLY S3(3, AsSize());
	S3[2] = t27;
	S3[1] = J1 * J6 - t33;
	S3[0] = -t26;

//  cout << "S3 = "; UPOLDAR::print(cout, S3); cout << " d = " << UPOLDAR::degree(S3) << endl;
    if ((UPOLDAR::degree(S3) <= 0) && S3[0] == RT(0))
		return Result_deg_3::on_end(where);

	if (sign(UPOLDAR::eval_horner(S3, b)) >= 0) where ^= 64;
	print("S3 on b", sign(UPOLDAR::eval_horner(S3, b)));
	result = Static_Result::Result_6.on_S3_p2q2(where);
	if (result != UNKNOWN) return result;
	if (sign(UPOLDAR::eval_horner(S3, a)) >= 0) where ^= 128;
	print("S3 on a", sign(UPOLDAR::eval_horner(S3, a)));
	result = Static_Result::Result_6.on_S3_p1q1(where);
	if (result != UNKNOWN) return result;

	RT t29 = J2 * J7;
	RT J9 = -c2 * e1 + e2 * c1;
	RT t28 = J6 * J9;
	RT t25 = t29 - J5 * J4;
	RT t20 = J3 * J3;
	RT t17 = J6 * J6;
	RT J8 = -c2 * d1 + d2 * c1 + J7;
	POLY S4(2, AsSize());
	S4[1] = -J5 * t20 - J1 * t17 + 2 * J6 * t33 + J8 * t27;
	S4[0] = t26 * J6 + t27 * J9 + t25 * J3;

//  cout << "S4 = "; UPOLDAR::print(cout, S4); cout << " d = " << UPOLDAR::degree(S4) << endl;
    if ((UPOLDAR::degree(S4) <= 0) && S4[0] == RT(0))
		return Result_deg_4::on_end(where);

	if (sign(UPOLDAR::eval_horner(S4, b)) >= 0) where ^= 256;
	print("S4 on b", sign(UPOLDAR::eval_horner(S4, b)));
	result = Static_Result::Result_6.on_S4_p2q2(where);
	if (result != UNKNOWN) return result;
	if (sign(UPOLDAR::eval_horner(S4, a)) >= 0) where ^= 512;
	print("S4 on a", sign(UPOLDAR::eval_horner(S4, a)));
	result = Static_Result::Result_6.on_S4_p1q1(where);
	if (result != UNKNOWN) return result;

	RT t16 = J7 * J7;
	RT t18 = J9 * J9;
	RT t21 = J4 * J4;
	RT J10 = -d2 * e1 + e2 * d1;
	RT S5 = t19 * t18 - 2 * t28 * t30 + t17 * t21 + (t16 - J5 * J10) * t20
		+ (-J5 * t18 + 2 * J7 * t28 - t17 * J10) * J1
		+ 2 * ((J2 * J10 - J4 * J7) * J6 - t25 * J9) * J3
		+ (-t19 * J10 + 2 * t29 * J4 - J1 * t16 + (J1 * J10 - t21) * J5) * J8;
	print("S5", S5);
	if (S5 ==  RT(0)) return EQUAL;
	if (S5 > RT(0)) where ^= 1024;
	result = Static_Result::Result_6.on_S5(where);
	if (result != UNKNOWN) return result;
	return EQUAL;
}


template < typename POLY,
           typename RO_t >  inline
int
sign_at_44(const POLY& f, const RO_t& x)
{
  typedef typename RO_t::NTR    NTR;
  typedef typename NTR::RT      RT;
  typedef typename NTR::FT      FT;

  print(__FUNCTION__);
  int where = 0;
  in_isolating_interval(f, x, where);
  return static_full_sturm_44<POLY>(
	  x[4], x[3], x[2], x[1], x[0],
	  f[4], f[3], f[2], f[1], f[0],
	  x.left(),
	  x.right(),
	  where);
}

#if 0
template < typename POLY, typename RO_t >
inline
int NTR::sign_at_43(const POLY& f, const RO_t& x)
{
  typedef typename RO_t::RT RT;
	typedef typename RO_t::FT FT;
  print(__FUNCTION__);
  int where = 0;
  in_isolating_interval(f, x, where);

  return static_full_sturm_44<POLY>(
	  RT(0), x[3], x[2], x[1], x[0],
	  f[4], f[3], f[2], f[1], f[0],
	  x.left_isolating_point(),
	  x.right_isolating_point(),
	  where);
}
#endif

template < typename POLY,
           typename RO_t >  inline
int
sign_at_42(const POLY& f, const RO_t& x)
{
  typedef typename RO_t::NTR    NTR;
  typedef typename NTR::RT      RT;
  typedef typename NTR::FT      FT;

  print(__FUNCTION__);
  int where = 0;
  in_isolating_interval(f, x, where);

  return static_full_sturm_44<POLY>(
	  RT(0), RT(0), x[2], x[1], x[0],
	  f[4], f[3], f[2], f[1], f[0],
	  x.left(),
	  x.right(),
	  where);
}


template < typename POLY,
           typename RO_t >  inline
int
sign_at_24(const POLY& f, const RO_t& x)
{
  typedef typename RO_t::NTR    NTR;
  typedef typename NTR::RT      RT;
  typedef typename NTR::FT      FT;

  print(__FUNCTION__);
  int where = 0;
  in_isolating_interval(f, x, where);

  return static_full_sturm_44<POLY>(
	  x[4], x[3], x[2], x[1], x[0],
	  RT(0), RT(0), f[2], f[1], f[0],
	  x.left(),
	  x.right(),
	  where);
}


__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_44_H

