/*******************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *              B. Mourrain, GALAAD, INRIA
 *   $Id: Bdg.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bezier_IslBzBdgSturm_H
#define synaps_solve_bezier_IslBzBdgSturm_H
/********************************************************************/
#include <synaps/usolve/bezier/bounding.h>
#include <synaps/usolve/SturmMem.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS 
//====================================================================
template<class O, class X=double>
struct IslBzBdgSturm 
{
  typedef Interval<O> solution_t;
  typedef X precision_t;
  X eps;
  IslBzBdgSturm(): eps(1.e-6) {}
  IslBzBdgSturm(const X& e):eps(e) {}
  IslBzBdgSturm(const IslBzBdgSturm & mth): eps(mth.eps) {}

};
//--------------------------------------------------------------------
template< class O, class UPOL>
Seq<typename IslBzBdgSturm<O>::solution_t> 
solve(const UPOL& r, const IslBzBdgSturm<O> & mth, 
      const O& a, const O& b)
{
  typedef typename IslBzBdgSturm<O>::solution_t sol_t;

  Seq<O> sol;
  bezier::Isole<double> slv;
  bounding::appendsol(sol,r,mth.eps, a,b, slv,true);
  SturmMem<ZZ> SturmSlv;

  Seq<sol_t> res;
  for (  unsigned i = 0; i < sol.size(); i += 2 )
    {
      if ( slv.m_res[i/2].t || (sol[i]==sol[i+1]) )
	{
	  res.push_back( sol_t(sol[i],sol[i+1]) );
	}
      else 
	{
	  // std::cout<<"Slv on ["<<sol[i]<<","<<sol[i+1]<<std::endl;
	  // O m=(sol[i]+sol[i+1])/2;
	  // res<< solve(r, IslBzInteger<O>(), sol[i], sol[i+1]);
	  // std::cout <<"XX ["<<QQ(sol[i])<<","<<QQ(sol[i+1])<<"]"<<std::endl;
	  Seq<SturmMem<ZZ>::RO_t> sl=solve(r,SturmSlv,QQ(sol[i]),QQ(sol[i+1]));
	  // std::cout <<"XX "<<sl.size()<<std::endl;	  
 	  O d,u;
 	  for(unsigned k=0;k<sl.size();k++)
 	    {
 	      let::assign(d,sl[k].interval().inf());
 	      let::assign(u,sl[k].interval().sup());
 	      res.push_back(sol_t(d,u));
 	    }
	}
    }
  return res;
}
//--------------------------------------------------------------------
template< class O, class UPOL>
Seq<typename IslBzBdgSturm<O>::solution_t> 
solve(const UPOL& r, const IslBzBdgSturm<O> & mth)
{
  using let::assign; 
  typedef typename IslBzBdgSturm<O>::precision_t prec_t;
  O mx, Mx;
  assign(mx,bound< typename rationalof<O>::T >(r,NISN()));
  assign(Mx,bound< typename rationalof<O>::T >(r,NISP()));
  IslBzBdgSturm<O> mth_rel= mth; 
  prec_t mxd, Mxd;
  assign(mxd,mx);
  assign(Mxd,Mx);
  mth_rel.eps *= Mxd-mxd;
  return solve(r,mth_rel,mx,Mx);}
//====================================================================
__END_NAMESPACE_SYNAPS
#endif  //synaps_solve_bezier_IslBzBdgSturm_H
