/*******************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *              B. Mourrain, GALAAD, INRIA
 *   $Id: Flt.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bezier_IslBzFlt_H
#define synaps_solve_bezier_IslBzFlt_H
/********************************************************************/
#include <synaps/usolve/bezier/bounding.h>
#include <synaps/usolve/bezier/IslBzInteger.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS 
//====================================================================
template<class O, class SLV=IslBzInteger<O> >
struct IslBzFlt 
{
  typedef Interval<O> solution_t;
  double eps;
  IslBzFlt(): eps(1.e-6) {}
  IslBzFlt(const double& e):eps(e) {}
  IslBzFlt(const IslBzFlt & mth): eps(mth.eps) {}

};
//--------------------------------------------------------------------
template< class O, class SLV, class UPOL>
Seq<typename IslBzFlt<O>::solution_t> 
solve(const UPOL& r, const IslBzFlt<O,SLV> & mth, 
      const O& a, const O& b)
{
  Seq<O> sol;
  typedef typename IslBzFlt<O>::solution_t sol_t;
  Seq<sol_t> res;
  bezier::Isole<double> slv;
  bounding::appendsol(sol,r,mth.eps, a,b, slv,true);
  for (  unsigned i = 0; i < sol.size(); i += 2 )
    {
      if ( slv.m_res[i/2].t || (sol[i]==sol[i+1]) )
	{
	  res.push_back( sol_t(sol[i],sol[i+1]) );
	}
      else 
	{
	  // std::cout<<"Slv "<<r<<"\n on ["<<sol[i]<<","<<sol[i+1]<<std::endl;
	  //	  O m=(sol[i]+sol[i+1])/2;
	  res<< solve(r, SLV(), sol[i], sol[i+1]);
	  //	  res<< solve(r, SturmSlv, QQ(sol[i]), QQ(sol[i+1]));
	  //	  res<< solve(r, mth, m,sol[i+1]);
	}
    }
  return res;
}
//--------------------------------------------------------------------
template< class O, class SLV, class UPOL>
Seq<typename IslBzFlt<O>::solution_t> 
Ssolve(const UPOL& r, const IslBzFlt<O,SLV> & mth)
{
  Seq<O> sol;
  typedef typename IslBzFlt<O>::solution_t sol_t;
  Seq<sol_t> res;
  bezier::Isole<double> slv;
  bounding::appendsol(sol,r,mth.eps,slv,true);
  for (  unsigned i = 0; i < sol.size(); i += 2 )
    {
      if ( slv.m_res[i/2].t || (sol[i]==sol[i+1]) )
	{
	  //std::cout<<"A"<<std::endl;
	  res.push_back( sol_t(sol[i],sol[i+1]) );
	}
      else 
	{
	  std::cout<<"Slv on ["<<sol[i]<<","<<sol[i+1]<<"]"<<std::endl;
	  O m=(sol[i]+sol[i+1])/2;
	  res<< solve(r, mth, sol[i], m);
	  res<< solve(r, mth, m,sol[i+1]);
	}
    }

  return res;
}

//--------------------------------------------------------------------
template< class O, class SLV, class UPOL>
Seq<typename IslBzFlt<O,SLV>::solution_t> 
solve(const UPOL& r, const IslBzFlt<O,SLV> & mth)
{
  using let::assign; 
  typedef typename rationalof< typename UPOL::value_type >::T FT;
  O mx, Mx;
  assign(mx,bound<FT>(r,NISN()));
  assign(Mx,bound<FT>(r,NISP()));
  IslBzFlt<O,SLV> mth_rel= mth; 
  double mxd, Mxd;
  assign(mxd,mx);
  assign(Mxd,Mx);
  mth_rel.eps *= Mxd-mxd;
  std::cout<<"["<<mx<<","<<Mx<<"]"<<std::endl;
  return solve(r,mth_rel,mx,Mx);}
//====================================================================
__END_NAMESPACE_SYNAPS
#endif  //synaps_solve_bezier_IslBzFlt_H
