/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: Isolation.h,v 1.1 2005/09/28 06:30:41 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_BzIsole_H
#define synaps_solve_BzIsole_H
/********************************************************************/

#include <synaps/init.h>
#include <synaps/arithm/NumberTraits.h>
#include <synaps/upol/UPolDse.h>
#include <synaps/upol/Bound.h>
#include <synaps/upol/bezier1d.h>
#include <synaps/usolve/bezier/Bezier.h>
#include <synaps/usolve/bezier/subdivision.h>

//====================================================================
__BEGIN_NAMESPACE_SYNAPS 
//====================================================================
struct Isolation {};
//====================================================================
template< class O, class R, class C > inline
void AppendSol(Seq<O>& sol, const R& r, 
	       const Bezier<O,Isolation,C>& mth, bool isole_)
{
  //  typedef typename Number_traits<typename R::value_type>::FT TC;
  bezier::rep1d<C> bp(r.size()); // TC->C
  assign(bp,r);
  for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;
  bezier::Isole<C> isole;
  isole.Run( bp, mth.eps, isole_ );
  if(isole_)
    isole.GetIntervals( sol );
  else
    isole.GetMidPoints( sol );

  unsigned s = sol.size();

  for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;    
  isole.Run( bp, mth.eps, isole_ );
  if( bp[0] == 0 ) { sol.push_back(0); if ( isole_ ) sol.push_back(0); };
  if(isole_)
    isole.GetIntervals( sol );
  else
    isole.GetMidPoints(sol);

  RConvert( sol, s, Bound(r,Cauchy()) );//, isole_ );
}
//--------------------------------------------------------------------
template< class O, class R, class C > inline
void AppendSol(Seq<O>& sol, const R& r, const Bezier<O,Isolation,C>& mth, 
	       const C& a, const C& b,
	       bool isole_)
{
  //  typedef typename Number_traits<typename R::value_type>::FT TC;
  bezier::rep1d<C> bp; // TC->C
  bezier::Isole<C> isole;
  assign(bp,r,a,b);
  isole.Run( bp, mth.eps, isole_ );
  if(isole_)
    isole.GetIntervals( sol );
  else
    isole.GetMidPoints( sol );
  RConvert( sol, a,b);//, isole_ );
}
//--------------------------------------------------------------------
/* polynome sous forme monomiale: conversion dans le type R::value_type, racines dans R */
// template< class O, class R, class C > inline
// void AppendSol( Seq<O>& sol, const R& r, const Bezier<O,Isolation,C>& mth, 
// 		bool isole  )
// { 
//   AppendSol(sol, r, mth, isole, 
// 	    typename numerics::is_rounded<typename R::value_type>::result_t()); 
// };
//--------------------------------------------------------------------
/* polynome sous forme de bernstein: pas de conversion, racines donnees dans [0,1] */
template< class O, class T, class C  > inline
void AppendSol( Seq<O>& sol, const bezier::rep1d<T>& brep, 
		const Bezier<O,Isolation,C>& mth, bool isole_  )
{
  bezier::Isole<C> isole;
  isole.Run( brep, mth.eps, isole_ );
  isole.GetIntervals(sol);
}
//--------------------------------------------------------------------
/* UPOL: view on an univariate polynom                 */
/*    C: number type used in computation               */
/*    O: type of result                                */
template< class UPOL, class O, class C >
Seq<O> solve( const UPOL& p, const Bezier<O,Isolation,C>& mth )
{
  Seq<O> sol;
  AppendSol( sol, p.rep(), mth, false );
  return  sol;
};
//--------------------------------------------------------------------
template< class UPOL, class O, class C >
Seq<O> solve( const UPOL& p, const Bezier<O,Isolation,C>& mth, 
	      const O& a, const O& b)
{ 
  Seq<O> sol;
  AppendSol( sol, p, mth, a,b, false);
  return sol;
};
//--------------------------------------------------------------------
template< class UPOL, class O, class C >
Seq<O> Isole( const UPOL& p, const Bezier<O,Isolation,C>& mth )
{ 
  Seq<O> sol;
  AppendSol( sol, p, mth, true  );
  return sol;
};
//--------------------------------------------------------------------
template< class UPOL, class O, class C >
Seq<O> Isole( const UPOL& p, const Bezier<O,Isolation,C>& mth, 
	      const O& a, const O& b)
{ 
  Seq<O> sol;
  AppendSol( sol, p, mth, a,b, true);
  return sol;
};

//====================================================================
__END_NAMESPACE_SYNAPS
#endif  //synaps_solve_BzIsole_H
