/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef _synaps_usolve_bezier_SlvBzBdg_h_
#define _synaps_usolve_bezier_SlvBzBdg_h_
//===================================================================
#include <synaps/arithm/gmp.h>
#include <synaps/arithm/sign.h>
#include <synaps/usolve/bezier/IslBzBdg.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//===================================================================
template<class O> 
struct SlvBzBdg: public IslBzBdg<O> 
{
  SlvBzBdg():IslBzBdg<O>(){this->isole=false;}
  SlvBzBdg(double eps):IslBzBdg<O>(eps){this->isole=false;}

  template < typename POL ,typename C>
  void solve(Seq<O>& sol, const POL& p, const C& u,const C& v);

  template < typename POL>
  void solve(Seq<O>& sol, const POL& p);

};

//--------------------------------------------------------------------
/** Compute an approximation of the roots of @p@ in the interval @[u,v]@ within
    the precision @this->eps@ and insert them in the sequence @sol@ by
    increasing order.
**/
template<class O>
template < typename POL ,typename C>
void SlvBzBdg<O>::solve(Seq<O>& sol, const POL& p, const C& u,const C& v)
{
  using let::assign;

  unsigned sz= p.size();

  bezier::rep1d<double> ubp( p.size() );
  bezier::rep1d<double> dbp( p.size() );
  
  this->init_pol(ubp,dbp,p,p.size(),u,v);
  this->Run(ubp,dbp,this->eps*0.01,false);
  
  O x,ur,vr;
  assign(ur,u);
  assign(vr,v);

  //  O sl[2*this->nb_sol()];
  //     for(unsigned i=0;i<this->nb_sol();i++) 
  //       {
  // 	this->Convert(x,this->m_res[i].a); sl[2*i]=ur*(1-x)+vr*x;
  //  	this->Convert(x,this->m_res[i].b); sl[2*i+1]=ur*(1-x)+vr*x;
  //       }
  // Solutions should be stored before calling recursively the solver.

  Seq<O> sl;
  this->GetIntervals(sl);
  RConvert(sl,u,v);
  
  for(  unsigned i = 0; i < sl.size(); i += 2 )
    {
      if (O(sl[i+1]-sl[i]) < this->eps )
 	sol.push_back( (sl[i]+sl[i+1])/2 );
      else 
 	{
 	  //	  std::cout<<"\tRef "<<s[i]<<" "<<s[i+1]<<std::endl;
 	  this->solve(sol,p,sl[i],sl[i+1]);
 	  //sol.push_back((sl[i]+sl[i+1])/2);
 	}
    }
}

//--------------------------------------------------------------------
/** Compute an approximation of the roots of @p@ within
    the precision @this->eps@ and insert them in the sequence @sol@ by
    increasing order.
**/
template<class O>
template < typename POL>
void SlvBzBdg<O>::solve(Seq<O>& sol, const POL& p)
{
  using let::assign;

  unsigned sz= p.size();

  bezier::rep1d<double> ubp( p.size() );
  bezier::rep1d<double> dbp( p.size() );
  
  this->init_pol(ubp,dbp,p,p.size());
  this->Run(ubp,dbp,this->eps*0.01,false);
  

  //  O sl[2*this->nb_sol()];
  //     for(unsigned i=0;i<this->nb_sol();i++) 
  //       {
  // 	this->Convert(x,this->m_res[i].a); sl[2*i]=ur*(1-x)+vr*x;
  //  	this->Convert(x,this->m_res[i].b); sl[2*i+1]=ur*(1-x)+vr*x;
  //       }

  // Solutions should be stored before calling recursively the solver.
  Seq<O> sl;
  this->GetIntervals(sl);

  RConvert( sol, 0, Bound(p,Cauchy()) );

  for(  unsigned i = 0; i < sl.size(); i += 2 )
    {
      if (O(sl[i+1]-sl[i]) < this->eps )
 	sol.push_back( (sl[i]+sl[i+1])/2 );
      else 
 	{
 	  //	  std::cout<<"\tRef "<<s[i]<<" "<<s[i+1]<<std::endl;
 	  this->solve(sol,p,sl[i],sl[i+1]);
 	  //sol.push_back((sl[i]+sl[i+1])/2);
 	}
    }
}
//--------------------------------------------------------------------

template<class O, class POL,class C> 
Seq<O> solve(const POL& p, SlvBzBdg<O> mth)
{

  Seq<O> sol;
  mth.solve(sol,p,true);
  mth.solve(sol,p);
  return sol;
}
//--------------------------------------------------------------------
template<class O, class POL,class C> 
Seq<O> solve(const POL& p, SlvBzBdg<O> mth,const C& a, const C& b)
{
  Seq<O> sol;
  mth.solve(sol,p,a,b);
  return sol;
}
//===================================================================
__END_NAMESPACE_SYNAPS
#endif //_synaps_usolve_bezier_IslBzBdg_h_

