/*******************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *              B. Mourrain, GALAAD, INRIA
 *   $Id: Sleever.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bezier_SlvBzStd_H
#define synaps_solve_bezier_SlvBzStd_H
/********************************************************************/
#include <synaps/init.h>
#include <synaps/arithm/traits.h>
#include <synaps/arithm/gmp.h>
#include <synaps/arithm/Interval.h>
#include <synaps/upol/UPolDse.h>
#include <synaps/upol/bound.h>
#include <synaps/upol/bezier/rep1d.h>
#include <synaps/usolve/bezier/sbd1d.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS 
//====================================================================
/**
Class to specify the use of a subdivision solver based on the Bernstein basis representation, 
using coefficients of type @C@, and output of type @O@.
The type @X@ is the type of the number giving the precision. It is default value is @double@.
**/
template<class O, class C= double, class X=double>
struct SlvBzStd 
{
  typedef Interval<O> solution_t;
  typedef X precision_t;
  X eps;
  SlvBzStd(): eps(1.e-6) {}
  SlvBzStd(const X& e):eps(e) {}
  SlvBzStd(const SlvBzStd & mth): eps(mth.eps) {}

};
// //====================================================================
// template< class R, class O, class X ,class C > inline
// void appendsol(Seq<O>& sol, 
// 	       const R& r, 
// 	       const X& eps,
// 	       bezier::Isole<C> &  slv,
// 	       bool isole_)
// {

// }
//--------------------------------------------------------------------
template< class O, class R, class X, class C > inline
void appendsol(Seq<O>& sol, 
	       const R& r, 
	       const X& eps,
	       bezier::Isole<C>& slv, 
	       const C& a, const C& b,
	       bool isole_)
{
  //  typedef typename Number_traits<typename R::value_type>::FT TC;
  bezier::rep1d<C> bp; // TC->C
  let::assign(bp,r,a,b);
  slv.Run( bp, eps, false);
  if(isole_)
    slv.GetIntervals( sol );
  else
    slv.GetMidPoints( sol );
  RConvert( sol, a,b);
}
//--------------------------------------------------------------------
// /* polynome sous forme de bernstein: pas de conversion, racines donnees dans [0,1] */
// template< class O, class T, class C  > inline
// void appendsol( Seq<O>& sol, 
// 		const bezier::rep1d<T>& brep, 
// 		const SlvBzStd<O,C>& mth)
// {
//   bezier::Isole<C> isole;
//   isole.Run( brep, mth.eps, true);
//   isole.GetIntervals(sol);
// }
//--------------------------------------------------------------------
/**
This solver computes an approximation of the real roots of the polynomial @p@,
within the precision $\epsilon$ defined in @mth@. If there is a cluster of roots,  
within an interval of size $\epsilon$, only one root is reported.

The class @C@ specify the coefficient used during the subdivision process. It is not
necessarly the type of the coefficients of @p@, but it should be a field type. 
**/
template< class UPOL, class O, class C >
Seq<O> solve( const UPOL& p, const SlvBzStd<O,C>& mth )
{ 
  Seq<O> sol;
  bezier::Isole<C> slv;
    //  typedef typename Number_traits<typename R::value_type>::FT TC;
  unsigned v=0;
  while(p[v]==0 && v< p.size()) v++;
  bezier::rep1d<C> bp(p.size()-v); // TC->C
  let::assign(bp,p,v);
  for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;
  slv.Run( bp, mth.eps, false);
  slv.GetMidPoints( sol );


  unsigned s = sol.size();

  if( v != 0 ) 
    sol.push_back(0); 


  for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;    
  slv.Run( bp, mth.eps, false);
  slv.GetMidPoints(sol);

  RConvert( sol, s, bound<O>(p,Cauchy()) );//, isole_ );
  return sol;
};
//--------------------------------------------------------------------
template< class UPOL, class O, class C >
Seq<O> solve( const UPOL& p, const SlvBzStd<O,C>& mth, 
	      const O& a, const O& b)
{ 
  Seq<O> sol;
  bezier::Isole<C> slv;
  appendsol( sol, p, mth.eps, slv, a,b, false);
  return sol;
};
//====================================================================
__END_NAMESPACE_SYNAPS
#endif  //synaps_solve_bezier_SlvBzStd_H
