/*******************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *              B. Mourrain, GALAAD, INRIA
 *   $Id: Bounding.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bezier_bounding_H
#define synaps_solve_bezier_bounding_H
/********************************************************************/
#include <synaps/arithm/traits.h>
#include <synaps/arithm/binomial.h>
#include <synaps/upol/UPolDse.h>
#include <synaps/upol/bound.h>
#include <synaps/upol/bezier/rep1d.h>
#include <synaps/usolve/bezier/sbd1d.h>
#include <synaps/usolve/solver_base.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS 
//====================================================================
namespace bounding 
{
  /* le type porte par le polynome supporte les modes d'arrondis */
  template< class O, class R, class X >
  void appendsol( Seq<O>& sol, 
		  const R& r, 
		  const X& eps,
		  bezier::Isole<double> &  slv,
		  bool isole_)
  {
    using let::assign;
    typedef typename rationalof< typename R::coeff_t >::T rational_t;
    //    typedef typename NumberTraits<typename R::coeff_t>::FT rational_t;
    typedef double TC;
    
    std::vector<TC> ubp( r.size() );
    std::vector<TC> dbp( r.size() );
    
    double mx; assign(mx,VECTOR::max_abs(r));
    //  std::cout<<"Bounding"<<std::endl;
    //  std::copy(r.begin(),r.end(),ubp.begin());
    //  std::copy(r.begin(),r.end(),dbp.begin());
    
    // A changer: l'utilisation de binomial<unsigned> induit une limitation 
    // du degré ... 
    // (il faudrait calculer les binomiaux avec changements d'arrondis dans le type TC)
    if( mx != 0 ) 
      {
	{
	  numerics::rounding<TC> rnd( numerics::rnd_up() );
	  for ( unsigned i = 0; i < ubp.size(); i ++ )
	    {
	      assign(ubp[i],rational_t(r[i]/mx));
	      ubp[i] /= binomial<unsigned>(ubp.size()-1,i);
	    }
	};
	
	{
	  numerics::rounding<TC> rnd( numerics::rnd_dw() );
	  for ( unsigned i = 0; i < dbp.size(); i ++ )
	    {
	      assign(dbp[i],rational_t(r[i]/mx));
	      dbp[i] /= binomial<unsigned>(dbp.size()-1,i);
	    }
	};

	// VECTOR::print(std::cout,ubp)<<std::endl;
	// VECTOR::print(std::cout,dbp)<<std::endl;

	for ( unsigned k = 1; k < ubp.size(); k += 2 )
	  {
	    TC u = ubp[k];
	    ubp[k] = - dbp[k];
	    dbp[k] = - u ;
	  };
	

	//	bezier::Isole<double> isole;
	
	slv.Run(ubp,dbp,eps,isole_);

	if(isole_)
	  slv.GetIntervals( sol );
	else
	  slv.GetMidPoints( sol );
	
	unsigned s = sol.size() ;
	
	for ( unsigned k = 1; k < ubp.size(); k += 2 )
	  {
	    TC u   =  ubp[k];
	    ubp[k] = - dbp[k];
	    dbp[k] = - u;
	  };
	
	slv.Run(ubp,dbp,eps,isole_);

	if(isole_)
	  slv.GetIntervals( sol );
	else
	  slv.GetMidPoints( sol );
	RConvert( sol, s, Bound(r,Cauchy()) );
	// VECTOR::print(std::cout,sol)<<std::endl;
      }
  }
  //--------------------------------------------------------------------
  template< class O, class UPOL, class X>
  void appendsol(Seq<O>& sol, 
		 const UPOL p,
		 const X& eps,
		 const O& a, const O& b, 
		 bezier::Isole<double> &  slv,
		 bool isole_=false)
  {
    typedef typename rationalof< typename UPOL::coeff_t >::T rational_t;

    using let::assign;
    bezier::rep1d<double> ubp( p.size() );
    bezier::rep1d<double> dbp( p.size() );
    double da, db;
    //    UPolDse<rational_t> BZ(p.size(),AsSize());
    //    BEZIER::monomials2bezier(BZ,p,p.size(),a,b);

    //    UPolDse<double> dp( p.size(),AsSize());
    //      assign(da,a);
    //      assign(db,b);

    {
      numerics::rounding<double> rnd( numerics::rnd_up() );
      //  for ( unsigned i = 0; i < ubp.size(); i ++ )
      //    assign(dp[i],p[i]);
      //  assign(ubp[i],bz[i]);
      //  assign(ubp[i],bz[i],1);
      assign(db,b);
      BEZIER::monomials2bezier(ubp,p,p.size(),a,b);
    };
    {
      numerics::rounding<double> rnd( numerics::rnd_dw() );
      //for ( unsigned i = 0; i < ubp.size(); i ++ )
      //  assign(dbp[i],bz[i],-1);
      //  assign(dbp[i],bz[i]);
      assign(da,a);
      BEZIER::monomials2bezier(dbp,p,p.size(),a,b);
    };
    
    for ( unsigned i= 0; i < ubp.size(); i ++ )
      if ( dbp[i] > ubp[i] ) { 
	std::swap(dbp[i], ubp[i]);
      };

    //    VECTOR::print(std::cout,ubp)<<std::endl;
    //    VECTOR::print(std::cout,dbp)<<std::endl;

    double deps;
    assign(deps,eps);
    slv.Run(ubp,dbp,deps,isole_);
    slv.GetIntervals(sol);
    RConvert(sol,a,b);
  }
}
//====================================================================
__END_NAMESPACE_SYNAPS
#endif  //synaps_solve_bezier_bounding_H
