/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef _synaps_usolve_contfrac_SlvBdg_h_
#define _synaps_usolve_contfrac_slvBdg_h_
//===================================================================
#include <synaps/arithm/sign.h>
#include <synaps/usolve/bezier/IslBzBdg.h>
#include <synaps/usolve/bezier/SlvBzBdg.h>
#include <synaps/usolve/Aberth.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//===================================================================
struct SlvBdg : public bezier::Isole<double>
{
  double eps;

  SlvBdg():eps(1e-6){}
  SlvBdg(double e): eps(e) {}

  template < typename Poly >
  void solve(Seq<double>& sol, const Poly& r)
  {
    typedef typename Poly::value_type    RT;
    typedef typename arithm::kernelof<RT>::T kernel_t;
    typedef typename kernel_t::integer  integer_t;
    typedef typename kernel_t::rational rational_t;
    using let::assign;

    unsigned sz=r.size();
    std::vector<double> ubp( r.size() );
    std::vector<double> dbp( r.size() );
    

    rational_t mx,cf;

    assign(mx,VECTOR::max_abs(r));
    if( mx != 0 ) 
      {
	{
	  numerics::rounding<double> rnd( numerics::rnd_up() );
	  for ( unsigned i = 0; i < ubp.size(); i ++ )
	    {
	      assign(cf,r[i]);
	      cf/=mx;
	      cf/=binomial<unsigned>(ubp.size()-1,i);
	      assign(ubp[i],cf);
	    }
	};
	
	{
	  numerics::rounding<double> rnd( numerics::rnd_dw() );
	  for ( unsigned i = 0; i < dbp.size(); i ++ )
	    {
	      assign(cf,r[i]);
	      cf/=mx;
	      cf/=binomial<unsigned>(ubp.size()-1,i);
	      assign(dbp[i],cf);
	    }
	};
	for ( unsigned i= 0; i < ubp.size(); i ++ )
	  if ( dbp[i] > ubp[i] ) { 
	    std::swap(dbp[i], ubp[i]);
	  };

	//	VECTOR::print(std::cout,ubp)<<std::endl;
	//	VECTOR::print(std::cout,dbp)<<std::endl;
	
	this->Run(ubp,dbp,eps,false);
	Seq<rational_t> ls;
	this->GetIntervals(ls);
	for(unsigned i=0;i<ls.size();i+=2) 
	  {
	    ls[i]/=(1-ls[i]);
	    if(ls[i+1]!=1)
	      ls[i+1]/=(1-ls[i+1]);
	    else
	      ls[i+1]= 1;
	    
	    std::cout<<"\tSol (filter): "<<ls<<std::endl;
	    if(ls[i+1]-ls[i]>eps)
	      this->solve(sol,r,ls[i],ls[i+1]);
	  }
	std::cout<<"\tSol (filter): "<<sol<<std::endl;
      }
  }

  template < typename Poly ,typename C>
  void solve(Seq<double>& sol, const Poly& r, const C& u,const C& v)
  {
    typedef typename Poly::value_type    RT;
    typedef typename arithm::kernelof<RT>::T kernel_t;
    typedef typename kernel_t::integer  integer_t;
    typedef typename kernel_t::rational rational_t;
    using let::assign;


    std::vector<double> ubp( r.size() );
    std::vector<double> dbp( r.size() );
    
    rational_t a,b;
    double ud,vd,ad,bd;
    {
      numerics::rounding<double> rnd( numerics::rnd_dw() );
      assign(ud,u);
    }

    {
      numerics::rounding<double> rnd( numerics::rnd_up() );
      assign(vd,v);
    }
    if((v-u)>this->eps)
      {
	{
	  numerics::rounding<double> rnd( numerics::rnd_up() );
	  BEZIER::monomials2bezier(ubp,r,r.size(),ud,vd);
	};
	{
	  numerics::rounding<double> rnd( numerics::rnd_dw() );
	  BEZIER::monomials2bezier(dbp,r,r.size(),ud,vd);
	};
	this->Run(ubp,dbp,this->eps,false);
	this->GetIntervals(sol);
	for(unsigned i=0;i<this->m_res.size();i++) 
	  {
	    this->Convert(a,m_res[i].a); a*=(v-u); a+=u;
	    this->Convert(b,m_res[i].b); b*=(v-u); b+=u;
	    {
	      numerics::rounding<double> rnd( numerics::rnd_dw() );
	      assign(ad,a);
	    }
	    sol.push_back(ad);
	    {
	      numerics::rounding<double> rnd( numerics::rnd_up() );
	      assign(bd,b); 
	    }
	    sol.push_back(bd);
	  }
      }
  }

};

//===================================================================
__END_NAMESPACE_SYNAPS
#endif //_synaps_usolve_contfrac_SlvBdg_h_
