
#ifndef _synaps_usolve_contfrac_solver_H_
#define _synaps_usolve_contfrac_solver_H_

#include <stack>
#include <synaps/arithm/sign.h>
#include <synaps/upol/square_free.h>
#include <synaps/usolve.h>
#include <synaps/usolve/Aberth.h>
#include <synaps/usolve/sturm/common.h>
#include <synaps/usolve/sturm/solve_small.h>
#include <synaps/usolve/sturm/construct_root_of.h>
#include <synaps/upol/bezier/rep1d.h>
#include <synaps/upol/bezier/BEZIER.m>
#include <synaps/usolve/contfrac/IntervalData.h>
#include <synaps/usolve/contfrac/lower_bound.h>

__ALGEBRAIC_BEGIN_NAMESPACE
#define ilog2(a) mpz_sizeinbase(a.get_mpz_t(),2)

template < typename K >
void
CF_positive( const typename K::Poly& f, Seq< typename K::FIT>& RL, bool posneg, K)
{
  //  std::cout << __FUNCTION__ << std::endl; 
  typedef typename K::RT    RT;
  typedef typename K::FT    FT;
  typedef typename K::FT    rational_t;
  typedef typename K::FIT   FIT;
  typedef typename K::Poly  Poly;
  
  typedef IntervalData< RT, Poly> IntervalData;
  
  int iters  = 0;
  const RT One( 1);
  
  FT B = bound<FT>( f, Cauchy());
  unsigned long s = sign_variation( f);
  if ( s == 0 ) { return; }
  if ( s == 1 ) {
    std::cout << "Sign variation: 1"<<std::endl;;
    if ( posneg ) 
      RL.push_back( to_FIT( FT(0), B));
    else RL.push_back( to_FIT( FT(0), FT(-B)));
    return;
  }

  Poly X(2, AsSize()); X[1] = RT(1);

  std::stack< IntervalData > S;
  if ( posneg) S.push( IntervalData( 1, 0, 0, 1, f, s));
  else S.push( IntervalData( -1, 0, 0, 1, f, s));

  while ( !S.empty() ) {
    ++iters;
    IntervalData ID = S.top();
    S.pop();

    //std::cout << "Polynomial is " << ID.p << std::endl;
    // Steps 3 - 4: Choose the method for computing the lower bound 


    typename K::bound Bd;


    //    RT M = Bd.upper(ID.p);
    //    scale(ID,M);
    //    reverse(ID);
    //    translate_by_1(ID);
    //    reverse(ID);

//     Interval<rational_t> q=Bd.range(ID.p);
//     std::cout<<"Sign var: "<< sign_variation(ID.p)<<" ";
//     RT a, b;
//     if(q<0)
//       {
//  	continue;
//       }
//     else if(q!=0)
//       {
//  	a=numerator(q.lower());
//  	b=denominator(q.lower());
//  	inv_scale(ID,b);
//  	shift(ID, a);
//  	scale(ID,b);
// 	let::assign(a,rational_t(q.upper()-q.lower()));
// 	shift(ID,a);
//  	a=One;
//       }

//     std::cout<<"("<< sign_variation(ID.p)<<")"<<std::endl;;


    RT a = Bd.lower(ID.p);
    if ( a > One ) {
      scale(ID,a);
      a = One;
    }
    
    // Step 5 //
    if ( a >= One ) {
      //      shift(ID, a);
      
      if ( ID.p[0] == RT(0)) {
	RL.push_back( to_FIT( FT( ID.b, ID.d), FT( ID.b, ID.d)));
	ID.p /= X;
      }
      ID.s = sign_variation( ID.p);
      if ( ID.s == 0 ) { continue; }
      if ( ID.s == 1 ) {
	std::cout << "Sign variation: 1"<<std::endl;;
	RL.push_back( to_FIT<K>( ID.a, ID.b, ID.c, ID.d, B));
	continue;
      }
    }
    
    // Step 6
    IntervalData I1;
    translate_by_1( I1, ID);

    unsigned long r  = 0;
    
    if (I1.p[0] == RT(0)) {
      std::cout << "Zero at end point"<<std::endl;;
      RL.push_back( to_FIT( FT( I1.b, I1.d), FT( I1.b, I1.d)));
      I1.p /= X;
      r = 1;
    }
    I1.s = sign_variation( I1.p);

    IntervalData I2;
        
    I2.s = ID.s - I1.s - r;

    I2.a = ID.b;
    I2.b = ID.a + ID.b;
    I2.c = ID.d;
    I2.d = ID.c + ID.d;
    
    // Step 7;
    if ( I2.s > 1 ) {
      //p2 = p2( 1 / (x+1));
      reverse( I2.p, ID.p);
      translate_by_1( I2.p);

      if ( I2.p[0] == 0) {
	I2.p /= X;
      }
      I2.s = sign_variation( I2.p);
    }

    // Step 8
    if ( I1.s < I2.s ) {
      std::swap( I1, I2);
    }
    
    // Step 9
    if ( I1.s == 1 ) {
      std::cout << "Sign variation: 1"<<std::endl;;
      RL.push_back( to_FIT<K>( I1.a, I1.b, I1.c, I1.d, B));
    } else if ( I1.s > 1) {
      S.push( IntervalData( I1.a, I1.b, I1.c, I1.d, I1.p, I1.s));
    }

    // Step 10
    if ( I2.s == 1 ) {
      std::cout << "Sign variation: 1"<<std::endl;;
      RL.push_back( to_FIT<K>( I2.a, I2.b, I2.c, I2.d, B));
    } else if ( I2.s > 1) {
      S.push( IntervalData( I2.a, I2.b, I2.c, I2.d, I2.p, I2.s));
    }
  } // while
  std::cout << "-------------------- iters: " << iters << std::endl; 
  return;  
}

// A continued fraction algorithm that combines the Descartes method and
// Akritas' algorithm. The basic idea is to take the better of performing
// a Taylor shift or subdividing into half.
template < typename K >
void
MCF_positive( const typename K::Poly& f, Seq< typename K::FIT>& RL, bool posneg, K)
{
//  std::cout << __FUNCTION__ << std::endl; 
  typedef typename K::RT    RT;
  typedef typename K::FT    FT;
  typedef typename K::FIT   FIT;
  typedef typename K::Poly  Poly;
  
  typedef IntervalData< RT, Poly> IntervalData;
  
  int iters  = 0;
  //std::cout <<"Polynomial is "<< f << std::endl;

  FT B = bound<FT>( f, Cauchy());
  //  std::cout << "Upper bound is "<< B << " Positive or negative " << posneg << std::endl;
  unsigned long s = sign_variation( f);
  if ( s == 0 ) { return; }
  if ( s == 1 ) {
    if ( posneg ) RL.push_back( to_FIT( FT(0), B));
    else RL.push_back( to_FIT( FT(0), FT(-B)));
    return;
  }

  Poly X(2, AsSize()); X[1] = RT(1);

  std::stack< IntervalData > S;
  if ( posneg) S.push( IntervalData( 1, 0, 0, 1, f, s));
  else S.push( IntervalData( -1, 0, 0, 1, f, s));

  IntervalData ID, I1, I2;// ID is the data associated with the parent and
                          // I1, I2 are the data associated with its children

  RT lowerBound;
  FT midpoint; // The point corresponding to the midpoint of the interval 
               // with endpoints (a/c), (b/d).
  FT temp; // A temporary variable 
  unsigned long mult = 0; // multiplicity of an exact root.

  while ( !S.empty() ) {
    ++iters;
    ID = S.top();
    S.pop();
    //    std::cout <<"Transformation is " << ID.a << ":" << ID.b << ":"<< ID.c << ":" << ID.d << std::endl;
    
    // Steps 3 - 4
    lowerBound =  CauchyMinimumBound( ID.p);
    std::cout << "lower bound is "<< lowerBound << std::endl;
    
    if(ID.c == 0){// then ID.a is not equal to zero
      midpoint = FT(ID.d * B - ID.b*sign(ID.a), 2* abs(ID.a));
      //std::cout << "Midpoint = " << midpoint << std::endl;
    }else if (ID.d > ID.c ){
      midpoint = FT(ID.d, ID.c);
    }else{}

    if(midpoint < 1)  midpoint = 1;
    std::cout << "Midpoint = " << midpoint << std::endl;
    
    if(lowerBound >= midpoint){
      if ( lowerBound > 1 ) {
	UPOLDAR::scale( ID.p, ID.p, lowerBound);
	ID.a *= lowerBound;
	ID.c *= lowerBound;
	lowerBound = 1;
      }
    
      if ( lowerBound == 1 ) {
	translate_by_1( ID.p, ID.p);// If we scaled above then this is required
	ID.b += ID.a;               // to bring the origin to lowerBound
	ID.d += ID.c;
      
	if ( ID.p[0] == RT(0)) {
	  RL.push_back( to_FIT( FT( ID.b, ID.d), FT( ID.b, ID.d)));
	  ID.p /= X;
	}
	ID.s = sign_variation( ID.p);
	if ( ID.s == 0 ) { continue; }
	if ( ID.s == 1 ) {
	  RL.push_back( to_FIT<K>( ID.a, ID.b, ID.c, ID.d, B));
	  continue;
	}
      }

      translate_by_1( I1.p, ID.p);
      I1.a = ID.a;
      I1.b = ID.a + ID.b;
      I1.c = ID.c;
      I1.d = ID.c + ID.d;
    
      if (I1.p[0] == RT(0)) {
	RL.push_back( to_FIT( FT( I1.b, I1.d), FT( I1.b, I1.d)));
	I1.p /= X;
	mult = 1;
      }
      I1.s = sign_variation( I1.p);

      I2.s = ID.s - I1.s - mult;
      I2.a = ID.b;
      I2.b = ID.a + ID.b;
      I2.c = ID.d;
      I2.d = ID.c + ID.d;
    
      if ( I2.s > 1 ) {
	//p2 = p2( 1 / (x+1));
	reverse( I2.p, ID.p);
	translate_by_1( I2.p);
      }
    }else{// midpoint > lowerbound, so perform subdivision like the Descartes method
	// Construct the polynomial p(X+midpoint)
      //      std::cout << "Performing shift by midpoint " << std::endl;
      UPOLDAR::shift(I1.p, ID.p, RT(midpoint));
      I1.a = ID.a;
      I1.b = ID.a * RT(midpoint) + ID.b;
      I1.c = ID.c;
      I1.d = ID.c * RT(midpoint) + ID.d;
      mult = 0;
      //      std::cout <<"Transformation is " << I1.a << ":" << I1.b << ":"<< I1.c << ":" << I1.d << std::endl;

      if (I1.p[0] == RT(0)) {
	RL.push_back( to_FIT( FT( I1.b, I1.d), FT( I1.b, I1.d)));
	I1.p /= X;
	mult = 1;
      }
      I1.s = sign_variation( I1.p);

      I2.a = ID.b;
      I2.b = ID.b + ID.a * RT(midpoint);
      I2.c = ID.d;
      I2.d = ID.d + ID.c * RT(midpoint);
      I2.s = ID.s - I1.s - mult;

      //      std::cout <<"Transformation is " << I2.a << ":" << I2.b << ":"<< I2.c << ":" << I2.d << std::endl;
      //      std::cout << "Predicted sign variations of I2 " << ID.s - I1.s - mult << std::endl;
      
      if( I2.s > 1){// Construct the polynomial (X+1)^n p(midpoint/(X+1))
	//	std::cout << "Before scaling by " << RT(midpoint) << "poly is "<< ID.p << std::endl;
	UPOLDAR::scale( I2.p, ID.p, RT(midpoint));
	//	std::cout << "After scaling poly is "<< I2.p << std::endl;	
	//	std::cout << "Sign variations of I2 " << sign_variation(I2.p) << std::endl;
	reverse(I2.p);
	//	std::cout << "poly is " << I2.p << std::endl;
	translate_by_1(I2.p);
	//	std::cout << "After shift poly is " << I2.p << std::endl;
      }
    }// end of else

    if(I2.s > 1){
      if ( I2.p[0] == 0) {//If I2.p[0] = 0 then so is I1.p[0]; so avoid recounting
	I2.p /= X;
      }
      I2.s = sign_variation( I2.p);
    }
    //    std::cout <<"Sign variations " << I1.s << ":" << I2.s << std::endl;

    if ( I1.s < I2.s ) {
      std::swap( I1, I2);
    }
    
    if ( I1.s == 1 ) {
      RL.push_back( to_FIT<K>( I1.a, I1.b, I1.c, I1.d, B));
    } else if ( I1.s > 1) {
      S.push( IntervalData( I1.a, I1.b, I1.c, I1.d, I1.p, I1.s));
    }
    
    if ( I2.s == 1 ) {
      RL.push_back( to_FIT<K>( I2.a, I2.b, I2.c, I2.d, B));
    } else if ( I2.s > 1) {
      S.push( IntervalData( I2.a, I2.b, I2.c, I2.d, I2.p, I2.s));
    }
  } // while
  std::cout << "---------------- iters: " << iters << std::endl; 
  return;  
}



template < typename OutputIterator, typename K >
OutputIterator
CF_solve( const typename K::Poly& f, OutputIterator sol, int  mult, K)
{
  
  KERNEL_DEFS;

  Seq < FIT > isolating_intervals;
  
  // Check if 0 is a root
  int idx;
  for (idx = 0; idx <= degree( f); ++idx) {
    if ( f[idx] != 0 ) break;
  }
  
  Poly p;
  if ( idx == 0 ) { p = f; } 
  else {
    p.resize( degree( f) + 1 - idx);
    for (int j = idx; j <= degree( f); ++j) 
      p[j-idx] = f[j];
  }
  
  //std::cout << "Solving: " << p << std::endl; 

  // Isolate the negative roots
  for (int i = 1; i <= degree( p); i+=2) p[i] *= -1;
  CF_positive( p, isolating_intervals, false, K());

  //std::cout << "Ok negative" << std::endl; 

  // Is 0 a root?
  if (idx != 0) isolating_intervals.push_back( FIT( 0, 0));
  
  // Isolate the positive roots
  for (int i = 1; i <= degree( p); i+=2) p[i] *= -1;
  CF_positive( p, isolating_intervals, true, K());

//  std::cout << "Ok, positive" << std::endl; 

  sort( isolating_intervals.begin(), isolating_intervals.end(), CompareFIT());

//  std::cout << "ok sorting" << std::endl; 

  for (int i = 0 ; i < isolating_intervals.size(); ++i) {
    if ( singleton( isolating_intervals[i]) ) {
      if ( lower( isolating_intervals[i]) == 0 ) continue;
      
      Poly t( 2, AsSize());
      t[1] = denominator( isolating_intervals[i].lower());
      t[0] = - numerator( isolating_intervals[i].lower());
      
      Poly dummy;
      EUCLIDIAN::div_rem( dummy, p, t);
      p = dummy;
    }
  }
  
//  std::cout << "now p: " << p << ",  #nr: " << isolating_intervals.size() << std::endl; 

  for (int i = 0 ; i < isolating_intervals.size(); ++i) {
//    std::cout << "i: " << i << ", " << isolating_intervals[i] << std::endl; 
    if ( singleton( isolating_intervals[i]) ) {
      RO_t r = construct_root_of( lower( isolating_intervals[i]), K());
      r.set_multiplicity( mult);
      *sol++ = r;
    } else {
      RO_t r = construct_root_of( p, isolating_intervals[i], K());
      r.set_multiplicity( mult);
      *sol++ = r;
    }
  }
  
//  std::cout << "Done...." << std::endl; 
  
  return sol;
}





template < typename K,
	   typename OutputIterator > inline
OutputIterator
solve_contfrac( const typename K::Poly& h, OutputIterator sol, K)
{
  KERNEL_DEFS;
  
  // solve degree <= 4 fast
  if (UPOLDAR::degree(h) < 5) { return solve_small_degree(h, 1, sol, K()); }
  
  // Check if 0 is a root
  int zero_mult = 0;    
  if ( h[0] == RT(0)) {
    zero_mult = 1;
    RO_t r = construct_root_of( RT(0), RT(1), K());
	
    while (h[zero_mult] == RT(0)) {
      r.set_multiplicity( ++zero_mult); 
    }
    *sol++ = r;
  }
  int dg = UPOLDAR::degree( h) - zero_mult;
  Poly f( dg+1, AsSize());
  for (int i = 0; i <= dg; ++i) { f[i]  = h[i+zero_mult]; }
  
  // solve degree <= 4 fast
  if (UPOLDAR::degree(f) < 5) { return solve_small_degree(f, 1, sol, K()); }
  
  // Check if it square free
  Is_square_free is_square_free;
  if ( is_square_free( f) ) {
    
    return CF_solve( f, sol, 1, K());
  }
 
  // If you think that it is  not square free
  // Compute the square free factorization
  SturmSeq<RT>   sth;
  Seq<Poly> PL = square_free_factorization( f, sth);
    
  
  // solve for every square free part
  if (PL.size() == 1) {
    if (UPOLDAR::degree(PL[0]) < 5) { 
      return solve_small_degree( PL[0], 1, sol, K()); 
    } else { 
      return CF_solve( PL[0], sol, 1, K()); 
    }
  } else {
    for (int i = 0; i < PL.size(); ++i) {
      if (UPOLDAR::degree( PL[i]) < 5) {
	solve_small_degree( PL[i], i+1, sol, K());
      } else {
	CF_solve( PL[i], sol, i+1, K());
      }
    }	
  }
  return sol;
}


__ALGEBRAIC_END_NAMESPACE
#endif  // _synaps_usolve_contfrac_solver_H_


