
#ifndef SYNAPS_SOLVE_STURM_CONSTRUCT_ROOT_OF_H_
#define SYNAPS_SOLVE_STURM_CONSTRUCT_ROOT_OF_H_


#include <synaps/init.h>
#include <synaps/usolve/algebraic/Algebraic_config.h>




__ALGEBRAIC_BEGIN_NAMESPACE


#define KERNEL_DEFS \
  typedef typename K::RT           RT;       \
  typedef typename K::FT           FT;       \
  typedef typename K::XT           XT;       \
  typedef typename K::RL           RL;       \
  typedef typename K::FIT          FIT;      \
  typedef typename K::XIT          XIT;      \
  typedef typename K::RIT          RIT;      \
  typedef typename K::Poly         Poly;     \
  typedef typename K::RO_t         RO_t;     


template <typename K>
typename K::RO_t
construct_root_of( const typename K::FT& a, K)
{
  typedef typename K::Poly   Poly;
  typedef typename K::RO_t   RO_t;
  
  Poly f(2, AsSize());
  f[0] = -numerator( a);
  f[1] = denominator( a);
  return RO_t(f, 0, 1, a, a);
}

template <typename K>
typename K::RO_t
construct_root_of( const typename K::RT& a, K)
{
  typedef typename K::Poly   Poly;
  typedef typename K::RO_t   RO_t;
  
  Poly f(2, AsSize());
  f[0] = -a;
  f[1] = 1;
  return RO_t(f, 0, 1, a, a);
}


template <typename K>
typename K::RO_t
construct_root_of( const typename K::Poly f, const typename K::FIT& I, K)
{
  typedef typename K::RO_t   RO_t;
  return RO_t(f, lower( I), upper( I));
}




// The polynomial must be of degree 1
template <typename K>
typename K::RO_t
construct_root_of( const typename K::Poly f, unsigned mult, K)
{
  KERNEL_DEFS;
  
  assert( degree( f) == 1 );
  
  FT number( -f[0], f[1]);
  return RO_t(f, 0, 1, number, number, mult);
}



template <typename K>
typename K::RO_t
construct_root_of( const typename K::RT& c0, const typename K::RT& c1, K)
{
  KERNEL_DEFS;

  assert(c1 != 0);
  
  typedef typename K::RT     RT;
  typedef typename K::FT     FT;
  typedef typename K::Poly   Poly;
  typedef typename K::RO_t   RO_t;
  
  Poly f(2, AsSize());
  f[0] = c0;
  f[1] = c1;
  FT b = to_FT(RT(-c0), c1);
  return  RO_t(f, 0, 1, b, b);
  
}


__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_SOLVE_STURM_CONSTRUCT_ROOT_OF_H_
