/*********************************************************************
*      This file is part of the source code of SYNAPS.               *
*   Author(s): O. Ruatta,  GALAAD, INRIA                             *
*              LACO - Universit de Limoges                          *
**********************************************************************
History:
$Id: FileTools.H,v 1.1 2005/07/11 10:04:00 jppavone Exp $
**********************************************************************/

#ifndef __SYNAPS__FILETOOLS__H
#define __SYNAPS__FILETOOLS__H

#include "synaps/linalg.H"
#include "synaps/upol.H"
#include "synaps/linalg/norm.H"
#include "synaps/arithm/RealOf.H"


namespace weierstrass {

using namespace std;

template<class UPOL>
void ReadData (char* InputFile, UPOL& P, typename RealOf<typename UPOL::value_type>::TYPE& prec) {
  typedef typename UPOL::value_type value_type;
  typedef typename UPOL::size_type size_type;
  typedef typename RealOf<value_type>::TYPE real_type;
  typedef UPolDse<real_type> RUPOL;
  typedef VectDse<real_type> RVECT;

  size_type i;
  cout<<"Opening data file \n";
  ifstream infile(InputFile,ios::in);
  cout<<"Reading the number of roots or the degree\n";
  size_type d;
  infile>>d;
  cout<<"Reading the required precision\n";
  infile>>prec;
  cout<<"Reading the real part of the polynomial (monic)\n";
  RVECT RealPart(d);
  for(i = 0; i < d; i++) infile>>RealPart[i];
  cout<<"Reading the imaginary part of the polynomial \n";
  RVECT ImagPart(d);
  for(i = 0; i < d; i++) infile>>ImagPart[i];
  cout<<"Closing data file \n";
  infile.close();
  
  cout<<"Rebuilting polynomial and starting point from data\n";
  value_type V[d+1];
  for(i=0; i < d; i++) V[i]=value_type(RealPart[i],0.0) + value_type(0.0,ImagPart[i]);
  V[d] = value_type(1);
  P=UPOL(d,V);
  
  cout<<"Data ready for solving\n";
}
  
template<class VECT>
void WriteData (char* OutFile, const VECT& Z, VECT errors, unsigned int nb_iter, double temps) {
  cout<<"Generating output file. \n";
  ofstream outfile(OutFile,ios::out);
  
  cout<<"Writing output points \n";
  outfile<<Z<<endl;

  cout<<"Writing errors \n";
  outfile<<errors<<endl;

  cout<<"Writting the number of iteration \n";
  outfile<<nb_iter<<endl;

  cout<<"Writing time \n";
  outfile<<temps<<endl;
  
  outfile.close();
  cout<<"Output written \n";
}


} //namespace weierstrass

#endif //__SYNAPS__FILETOOLS__H
