 // Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
 //                     Department of Informatics and Telecommunications
 //                     University of Athens (Greece).
 // All rights reserved.
 //
 // Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

 // Partially supported by INRIA's project "CALAMATA", a
 // bilateral collaboration with National Kapodistrian University of
 // Athens.
 // Partially supported by the IST Programme of the EU as a Shared-cost
 // RTD (FET Open) Project under Contract No  IST-2000-26473
 // (ECG - Effective Computational Geometry for Curves and Surfaces)

 #include <synaps/usolve/algebraic/comparison/auto/Result_length_3.h>


__ALGEBRAIC_BEGIN_NAMESPACE

bool Result_length_3::initialize = false;

Result_length_3::MapResult Result_length_3::map_S0_p1q1;
Result_length_3::MapResult Result_length_3::map_S0_p2q2;
Result_length_3::MapResult Result_length_3::map_S1_p2q2;
Result_length_3::MapResult Result_length_3::map_S1_p1q1;
Result_length_3::MapResult Result_length_3::map_S2;

Result_length_3::Result_length_3()
{
  if (!initialize) {
    initialize = true;
    set_default_values();
  }
}

Comparison_result
Result_length_3::on_S0_p2q2(int where)
{
	 MapResult_iter it;
	 it = map_S0_p2q2.find(where);
	 if (it != map_S0_p2q2.end())
		 return to_CR(it->second);
	 else
		 return UNKNOWN;
}

Comparison_result
Result_length_3::on_S0_p1q1(int where)
{
	 MapResult_iter it;
	 it = map_S0_p1q1.find(where);
	 if (it != map_S0_p1q1.end())
		 return to_CR(it->second);
	 else
		 return UNKNOWN;
 }

Comparison_result
Result_length_3::on_S1_p2q2(int where)
{
	 MapResult_iter it;
	 it = map_S1_p2q2.find(where);
	 if (it != map_S1_p2q2.end())
		 return to_CR(it->second);
	 else
		 return UNKNOWN;
}

Comparison_result
Result_length_3::on_S1_p1q1(int where)
{
	 MapResult_iter it;
	 it = map_S1_p1q1.find(where);
	 if (it != map_S1_p1q1.end())
		 return to_CR(it->second);
	 else
		 return UNKNOWN;
}

Comparison_result
Result_length_3::on_S2(int where)
{
	 MapResult_iter it;
	 it = map_S2.find(where);
	 if (it != map_S2.end())
		 return to_CR(it->second);
	 else
		 return UNKNOWN;
}



void Result_length_3::set_default_values()
{
   map_S0_p1q1[0] = EQUAL;
   map_S0_p1q1[3] = EQUAL;

   map_S1_p1q1[1] = SMALLER;
   map_S1_p1q1[2] = SMALLER;
   map_S1_p1q1[  13] = LARGER;
   map_S1_p1q1[  14] = LARGER;
   
   map_S2[   5] = SMALLER;
   map_S2[  10] = SMALLER;
   map_S2[  22] = SMALLER;
   map_S2[  25] = SMALLER;
   map_S2[   6] = LARGER;
   map_S2[   9] = LARGER;
   map_S2[  21] = LARGER;
   map_S2[  26] = LARGER;
}


__ALGEBRAIC_END_NAMESPACE
