// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)

#include <synaps/usolve/algebraic/comparison/auto/Auto_Results.h>
#include <synaps/usolve/algebraic/comparison/auto/Result_length_4.h>

__ALGEBRAIC_BEGIN_NAMESPACE

bool Result_length_4::initialize = false;

Result_length_4::MapResult Result_length_4::map_S0_p2q2;
Result_length_4::MapResult Result_length_4::map_S0_p1q1;
Result_length_4::MapResult Result_length_4::map_S1_p2q2;
Result_length_4::MapResult Result_length_4::map_S1_p1q1;
Result_length_4::MapResult Result_length_4::map_S2_p2q2;
Result_length_4::MapResult Result_length_4::map_S2_p1q1;
Result_length_4::MapResult Result_length_4::map_S3;


Comparison_result Result_length_4::on_S0_p2q2(int where)
{
	MapResult_iter it;
	it = map_S0_p2q2.find(where);
	if (it != map_S0_p2q2.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result Result_length_4::on_S0_p1q1(int where)
{
	MapResult_iter it;
	it = map_S0_p1q1.find(where);
	if (it != map_S0_p1q1.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result Result_length_4::on_S1_p2q2(int where)
{
	MapResult_iter it;
	it = map_S1_p2q2.find(where);
	if (it != map_S1_p2q2.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result Result_length_4::on_S1_p1q1(int where)
{
	MapResult_iter it;
	it = map_S1_p1q1.find(where);
	if (it != map_S1_p1q1.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result Result_length_4::on_S2_p2q2(int where)
{
	MapResult_iter it;
	it = map_S2_p2q2.find(where);
	if (it != map_S2_p2q2.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result Result_length_4::on_S2_p1q1(int where)
{
	MapResult_iter it;
	it = map_S2_p1q1.find(where);
	if (it != map_S2_p1q1.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result Result_length_4::on_S3(int where)
{
	MapResult_iter it;
	it = map_S3.find(where);
	if (it != map_S3.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}



void Result_length_4::set_default_values()
{
	map_S0_p1q1[   0] = EQUAL;
	map_S0_p1q1[   3] = EQUAL;

	map_S2_p2q2[   2] = SMALLER;
	map_S2_p2q2[  17] = SMALLER;

	map_S2_p2q2[  14] = LARGER;
	map_S2_p2q2[  29] = LARGER;

	map_S2_p1q1[   1] = SMALLER;
	map_S2_p1q1[   5] = SMALLER;
	map_S2_p1q1[  10] = SMALLER;
	map_S2_p1q1[  50] = SMALLER;
	map_S2_p1q1[  54] = SMALLER;
	map_S2_p1q1[  57] = SMALLER;

	map_S2_p1q1[   6] = LARGER;
	map_S2_p1q1[   9] = LARGER;
	map_S2_p1q1[  13] = LARGER;
	map_S2_p1q1[  53] = LARGER;
	map_S2_p1q1[  58] = LARGER;
	map_S2_p1q1[  62] = LARGER;

	map_S3[  21] = SMALLER;
	map_S3[  25] = SMALLER;
	map_S3[  38] = SMALLER;
	map_S3[  42] = SMALLER;
	map_S3[  82] = SMALLER;
	map_S3[  86] = SMALLER;
	map_S3[  90] = SMALLER;
	map_S3[  94] = SMALLER;
	map_S3[  97] = SMALLER;
	map_S3[ 101] = SMALLER;
	map_S3[ 105] = SMALLER;
	map_S3[ 109] = SMALLER;

	map_S3[  18] = LARGER;
	map_S3[  22] = LARGER;
	map_S3[  26] = LARGER;
	map_S3[  30] = LARGER;
	map_S3[  33] = LARGER;
	map_S3[  37] = LARGER;
	map_S3[  41] = LARGER;
	map_S3[  45] = LARGER;
	map_S3[  85] = LARGER;
	map_S3[  89] = LARGER;
	map_S3[ 102] = LARGER;
	map_S3[ 106] = LARGER;
}

__ALGEBRAIC_END_NAMESPACE
