#include <iomanip>
#include <synaps/init.h>
#include <synaps/arithm/gmp.h>
#include <synaps/usolve/Algebraic.h>
#include <synaps/base/Rand.h>


#ifdef SYNAPS_WITH_GMP

using namespace ::SYNAPS;

using std::cout;
using std::cin;
using std::endl;

typedef ZZ NT;
typedef ::SYNAPS::Algebraic<NT> SOLVER;

typedef SOLVER::RT         RT;
typedef SOLVER::FT         FT;

typedef SOLVER::Poly       Poly;
//typedef SOLVER::MPoly_2_2  Poly_2;

typedef SOLVER::RO_t       RO_t;



int test_loops;

const int m = -10;
const int M = 10;


Rand<int> R;


int compare(const double& a, const double& b)
{
  double d = a - b;
  if (std::abs(d) < 1e-6) return 0;
  if (d > 0) return 1;
  return -1;
}

// Generate random integral values.
template < class RT >
RT my_rand(int m_ = m, int M_ = M)
{
  RT t = R(m_, M_);
  //	cout << "t = " << t << endl;
  return t;
}


// Generate random integral values.
int rand_bool()
{
  return R.get_rand_bool();
}

// Generate random root_of of degree 1
template < typename RT >
RO_t my_rand_root_1()
{
  return ALGEBRAIC::make_root_of_1(my_rand<RT>(), RT(1), SOLVER());
}


// Generate random root_of of degree 2
template < typename RT >
RO_t my_rand_root_2()
{
  RT a = my_rand<RT>();
  RT b = my_rand<RT>();
  RT c = my_rand<RT>();
  RT a2 = 1;
  RT a1 = -a - b;
  RT a0 = a * b;

  Poly f(3, AsSize());
  f[0] = a0;
  f[1] = a1;
  f[2] = a2;
  return solve(f, SOLVER(),  rand_bool());
}



// Generate random root_of of degree 3
template < typename RT >
RO_t my_rand_root_3()
{
  RT a = my_rand<RT>();
  RT b = my_rand<RT>();
  RT c = my_rand<RT>();
  int idx = my_rand<int>(0, 2);
  RT a3 = 1;
  RT a2 = -a - b - c;
  RT a1 = a * b + a * c + b * c;
  RT a0 =  -a * b * c;

  Poly f(4, AsSize());
  f[0] = a0;
  f[1] = a1;
  f[2] = a2;
  f[3] = 1;
  
  return solve(f, SOLVER(), idx);
}



// Generate random root_of of degree 4
template < typename RT >
RO_t my_rand_root_4()
{
  RT a = my_rand<RT>();
  RT b = my_rand<RT>();
  RT c = my_rand<RT>();
  RT d = my_rand<RT>();
  int idx = my_rand<int>(0, 3);

  RT a4 = 1;
  RT a3 = -a - b - c - d;
  RT a2 = a * b + a * c + a * d + b * c + b * d + c* d;
  RT a1 = -a * b * c - a * b * d - a * c * d - b * c * d;
  RT a0 = a * b * c * d;
  // cout << a4 << " " << a3 << " " << a2 << " " << a1 << " " << a0
  //	<< " " << " idx = " <<idx << endl;
  Poly f(5, AsSize());
  f[0] = a0;
  f[1] = a1;
  f[2] = a2;
  f[3] = a3;
  f[4] = a4;
  return solve(f, SOLVER(), idx);

}


// Compares the comparison results of two roots with
// the comparison results of their double approximations.
template < typename Root_1, typename Root_2>
bool
test_compare(const Root_1 &r1, const Root_2 &r2)
{
  typedef typename Root_1::NTR    NTR;

  if (ALGEBRAIC::compare(r1, r2) == compare(to_double(r1), to_double(r2)))
    return true;

  std::cout << "ERROR in comparing :" << std::endl << " r1 = ";
  print(std::cout, r1);
  std::cout << " [approx = " << std::setprecision(10) << to_double(r1) << " ]\n" << " r2 = ";
  print(std::cout, r2);
  std::cout << " [approx = " << std::setprecision(10) << to_double(r2) << " ]\n"
	    << " comparison gives : " << ALGEBRAIC::compare(r1, r2) << " for exact, but "
	    << compare(to_double(r1), to_double(r2))
	    << " for double approximation" << std::endl;
  return false;
}



template < typename RT >
bool
test_root_of_44()
{
  std::cout << "Testing root_of" << std::endl;
  std::cout << "Iterations = " << test_loops << std::endl;
  std::cout << "The roots are from " << m << " to " << M << std::endl;
  std::cout << "  Testing random roots of degree  4 4" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    //		cout <<"=======================\n";
    RO_t r1 = my_rand_root_4<RT>();
    RO_t r2 = my_rand_root_4<RT>();
    //		print(cout, r1); cout << endl;
    //		print(cout, r2); cout << endl;
    //	if (r1 != r1) {
    //		cout << "---> r1 = " << r1 << endl;
    //		cout << "---> comp = " << ALGEBRAIC::compare(r1, r1) << endl;
    //		cout << "---> (==) = " << (r1 == r1) << endl;
    //		cout <<"=======================\n";
    //	}
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
  }

  std::cout << "  Testing random roots of degree  4 3" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    RO_t r1 = my_rand_root_4<RT>();
    RO_t r2 = my_rand_root_3<RT>();
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
  }
  std::cout << "  Testing random roots of degree  4 2" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    RO_t r1 = my_rand_root_4<RT>();
    RO_t r2 = my_rand_root_2<RT>();
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
    assert(test_compare(r2, r1));
  }
  std::cout << "  Testing random roots of degree  4 1" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    RO_t r1 = my_rand_root_4<RT>();
    RO_t r2 = my_rand_root_1<RT>();
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
    assert(test_compare(r2, r1));
  }
  return true;
}



template < typename RT >
bool
test_root_of_33()
{
  std::cout << "Testing root_of" << std::endl;
  std::cout << "  Testing random roots of degree  3 3" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_3<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_3<RT>();
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
  }
  std::cout << "  Testing random roots of degree  3 2" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_3<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_2<RT>();
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
    assert(test_compare(r2, r1));
  }
  std::cout << "  Testing random roots of degree  3 1" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_3<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_1<RT>();
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
    assert(test_compare(r2, r1));
  }
  return true;
}




template < typename RT >
bool
test_root_of_22()
{
  std::cout << "Testing root_of" << std::endl;
  std::cout << "  Testing random roots of degree  2 2" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    //cout << "\n i = " << i << endl;
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_2<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_2<RT>();

    //cout << "r1 = " << r1 << endl;
    //cout << "r2 = " << r2 << endl;
    //if (r1 != r1) {
    //cout << "r1 = " << r1 << endl;
    //}
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
  }
  std::cout << "  Testing random roots of degree  2 1" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_2<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_1<RT>();
    assert(r1 == r1);
    assert(r2 == r2);
    assert(test_compare(r1, r2));
    assert(test_compare(r2, r1));
#if 0
    assert((r1 * RT(1)) == r1);
    ALGEBRAIC::root_of<RT> r3(r1 - RT(1));
    assert(r3 < r1);
    ALGEBRAIC::root_of<RT> r11 = r1 * RT(2);
    ALGEBRAIC::root_of<RT> r12 = r1 * RT(2);
    if (r11 != r12) {
      cout << "r1 = "; print(cout, r1); cout << r1 << endl;
      cout << "r11 = "; print(cout, r11); cout << r11 << endl;
      cout << "r12 = "; print(cout, r12); cout << r12 << endl;
    }
    assert(r11 == r12);
#endif
  }
  return true;
}

#endif //SYNAPS_WITH_GMP


int main(int argc, char **argv)
{
#ifdef SYNAPS_WITH_GMP

  srand( (unsigned)time( NULL ) );

  test_loops = argc == 2 ? atoi(argv[1]) : 10;
  bool result = true;

#if 0
  result = true;
  std::cout << "Testing root_of<ZZ>" << std::endl;
  result = result && test_root_of_22< RT >();
#endif

#if 0
  result = true;
  std::cout << "Testing root_of<ZZ>" << std::endl;
  result = result && test_root_of_33< RT >();
#endif

#if 1
  result = true;
  std::cout << "Testing root_of<ZZ>" << std::endl;
  result = result && test_root_of_44< RT >();

  //  std::cout << "Testing root_of<Quotient<MP_Float> >" << std::endl;
  // result = result &&
  //          test_root_of<root_of<CGAL::Quotient<CGAL::MP_Float> > >();
#endif
  if (result) {
    std::cout << "OK" << std::endl;
    return 0;
  } else {
    std::cout << "ERROR" << std::endl;
    return -1;
  }

#else

  std::cout <<"  Gmp is not connected"<<std::endl;
  return(77);

#endif //SYNAPS_WITH_GMP
}

