#include <synaps/init.h>
#include <iostream>
#include <synaps/arithm/gmp.h>
#include <synaps/usolve/Symbolic.h>

#ifdef SYNAPS_WITH_GMP
typedef ZZ RT;
typedef UPolDse<RT> upol_t;
typedef symbolic::real<double> real_t;

int main()
{
  using std::cin;
  using std::cout;
  using std::endl;

  upol_t f1(4, AsSize());
  f1[3] = 1;
  f1[2] = 2;
  f1[1] = -65;
  f1[0] = 150;
  real_t r1 = RealRoot<double>(f1, 1);
  cout << "r1 = " << r1 << endl;

  upol_t f2(3, AsSize());
  f2[2] = 5;
  f2[1] = -4;
  f2[0] = -9;

  real_t r2 = RealRoot<double>(f2, 0);
  cout << "r2 = " << r2 << endl;
  
  int result = compare(r1, r2);
  if (result == 1)
    cout << r1 << " > " << r2 << endl;
  else if (result == -1)
    cout << r1 << " < " << r2 << endl;
  else
    cout << r1 << " = " << r2 << endl;

  assert(compare(r2, r1)==-compare(r1,r2));

  real_t r;

  r=r1/real_t(4);
  cout <<"r1/4: "<<r<<endl;
  cout << "r1/4>1: " << compare(r,real_t(1)) << endl;

  r=r1+r2;
  cout <<"r1+r2: "<<r<<endl;
  cout << "r1+r2>0: " << compare(r,real_t(0)) << endl;
  assert(compare(r,r1)==-1);

  r=r1-r2;
  cout <<"r1-r2: "<<r<<endl;
  cout << "r1-r2>0: " << compare(r,real_t(0)) << endl;
  assert(compare(r,r1)==1);

  r=r1*r2;
  cout <<"r1*r2: "<<r<<endl;
  cout << "r1*r2>0: " << compare(r,real_t(0)) << endl;

  r=r1+r2/(r1+real_t(2));
  cout <<"r: "<<r<<endl;
  cout << "r>0: " << compare(r,real_t(0)) << endl;

  assert(compare(r1, r1)==0);

  return 0;
}

#else 
//======================================================================
#include <iostream>
int main()
{
  std::cout <<"  Gmp is not connected"<<std::endl;
  return(77);
}
#endif //SYNAPS_WITH_GMP
