#include <iostream>
#include "synaps/upol.h"
#include "synaps/usolve/bezier/Std.h"
#include "synaps/usolve/Aberth.h"
#include "synaps/usolve/Exclusion.h"
#include "synaps/usolve/Dichotomy.h"
#include "synaps/usolve/Algebraic.h"
#include "synaps/usolve/Symbolic.h"

int main()
{
  using std::cout;
  using std::cin;
  using std::endl;

  UPolDse<double> P("2*x0^7+3*x0^6+2*x0^5+1*x0^4-1*x0^3-2*x0^2-3*x0+1");
  cout <<"Testing: "<<P<<endl;
  cout <<solve(P,Bezier<double>(1e-6))<<endl;
  assert(solve(P,Bezier<double>(1e-6)).size()==3);
  //  assert(solve(P,Bezier<double,Opt>(1e-6)).size()==3);
  assert(solve(P,Exclusion(1e-6),-2.,2.).size()==3);

  //  cout <<Solve(P,Symbolic<double>(1e-6))<<endl;

  assert(solve(P,Symbolic<double>(1e-6)).size()==3);
  assert(RootOf(P,Bezier<double,Rkw>(1e-6))<0.33);
  assert(RootOf(P,Bezier<double,Fst>(1e-6))<0.33);
  //  assert(compare(RootOf(P,2,Symbolic<double>(1e-6)),1.)==1);
  //  cout<<Solve(P,Sturm<double>())<<endl;;
#ifdef SYNAPS_WITH_GMP
  assert(solve(P,SlvAberth<std::complex<double> >()).size()==7);
#endif

  UPolDse<double> Q("x0^4-2*x0^3+x0^2");
  cout <<"Testing: "<<Q<<endl;
  assert(solve(Q,Bezier<double>(1e-6)).size()==2);
  //  assert(solve(Q,Bezier<double,Opt>(1e-6)).size()==2);
  assert(solve(Q,Exclusion(1e-6),-2.,2.).size()==2);
  //  assert(Solve(Q,Symbolic<double>(1e-6)).size()==2);
  assert(RootOf(Q,Bezier<double,Rkw>(1e-6))==0);
  assert(RootOf(Q,Bezier<double,Fst>(1e-6))==0);
//  assert(compare(RootOf(Q,1,Symbolic<double>(1e-6)),-1.)==-1);
#ifdef SYNAPS_WITH_GMP
  std::cout <<solve(Q,SlvAberth<std::complex<double> >())<<std::endl;
  //  assert(solve(Q,SlvAberth<std::complex<double> >()).size()==4);
  //  assert(solve(Q,Aberth<double>()).size()==4);
#endif

  UPolDse<double> Q0("x0^2-3*x0+2");
  cout <<"Testing: "<<Q0<<endl;
  assert(solve(Q0,Bezier<double>(1e-6)).size()==2);
  //  assert(Solve(Q0,Bezier<double,Opt>(1e-6)).size()==2);
  //  assert(Solve(Q0,-2.,2.,Exclusion(1e-6)).size()==2);
  //  assert(Solve(Q0,Symbolic<double>(1e-6)).size()==2);
}

